;**********************************************************************;
;*                        D U M P A . A S M                           *;
;*--------------------------------------------------------------------*;
;*    Fonction       : Un filtre qui lit des caractres sur l'entre  *;
;*                     standard pour les ressortir sur la sortie      *;
;*                     standard sous forme de dump hexa et ASCII      *;
;*--------------------------------------------------------------------*;
;*    Auteur          : MICHAEL TISCHER                               *;
;*    Dvelopp le    :  1.08.87                                      *;
;*    Dernire modif. : 20.03.92                                      *;
;*--------------------------------------------------------------------*;
;*    Pour assembler  : MASM DUMPA;                                   *;
;*                      LINK DUMPA;                                   *;
;*                      EXE2BIN DUMPA DUMPA.COM  ou                   *;
;*                                                                    *;
;*                      TASM DUMPA                                    *;
;*                      TLINK /t DUMPA                                *;
;*--------------------------------------------------------------------*;
;*    Appel         : DUMPA [<Entre] [>Sortie]                       *;
;**********************************************************************;

;==  Constantes ========================================================

      NUL    equ 0                             ;Code ASCII caractre NUL
      BEL    equ 7                                  ;Code ASCII sonnerie
      BS     equ 8                                 ;Code ASCII Backspace
      TAB    equ 9                                ;Code ASCII Tabulateur
      LF     equ 10                                ;Code ASCII Line Feed
      CR     equ 13                          ;Code ASCII Carriage Return
      EOF    equ 26                           ;Code ASCII Fin de fichier
      ESC    equ 27                                   ;Code ASCII Escape

;== Ici commence vritablement le programme ============================

code      segment para 'CODE'             ;Dfinition du segment de code

          org 100h

          assume cs:code, ds:code, es:code, ss:code

;-- Routine initiale ---------------------------------------------------

dump      label near
          ;-- Lire 9 octets sur le priphrique d'entre standard ------

          xor  bx,bx                  ;Entre standard porte le handle 0
          mov  cx,9                                ;Charger 9 caractres
          mov  dx,offset neufoctets                   ;Adresse du buffer
          mov  ah,3Fh              ;Code de fonction pour lecture Handle
          int  21h                                 ;Appeler fonction DOS
          or   ax,ax                   ;Des caractres ont-ils t lus ?
          jne  dodump                             ;OUI --> Traiter ligne
          jmp  findump                                 ;NON --> FIN DUMP

dodump:   mov  dx,ax                    ;Ranger nombre de caractres lus

          ;-- Remplir d'espaces buffer de sortie -----------------------

          mov  cx,15                                ;15 mots (30 octets)
          mov  ax,2020h                 ;Code ASCII du " " dans AH et AL
          mov  di,offset dumpBuf  ;Augmenter adresse du buffer de sortie
          cld                              ;pour instructions de chanes
          rep  stosw                           ;Remplir buffer d'espaces

          ;-- Mettre en place buffer de sortie -------------------------

          mov  cx,dx            ;Aller chercher nombre de caractres lus
          mov  di,offset dumpBuf+31     ;Pos. codes ASCII dans le buffer
          mov  bx,offset neufoctets     ;Pointeur sur le buffer d'entre
          mov  si,offset dumpBuf         ;Pos. codes hexa dans le buffer

octetin:  mov  ah,[bx]                                    ;Lire un octet
          push si                                 ;Ranger SI sur la pile
          mov  si,offset sptab              ;Adr. table de car. spciaux
          mov  dx,offset sptext-6           ;Adr. texte de car. spciaux
sptest:   add  dx,6           ;Prochaine entre dans les textes spciaux
          lodsb            ;Charger code de table de caractres spciaux
          cmp  al,255                        ;Fin de la table atteinte ?
          je   nosp                       ;OUI --> Pas caractre spcial
          cmp  ah,al                      ;Les codes correspondent-ils ?
          jne  sptest       ;NON --> Tester prochain lment de la table

          ;-- Le code tait un caractre spcial -----------------------
          push cx                                       ;Ranger compteur
          mov  si,dx                                  ;Copier DX dans SI
          lodsb           ;Lire nombre de caractres du code de commande
          mov  cl,al            ;Transfrer nombre de caractres dans CL
          rep  movsb                     ;Copier dsignation dans buffer
          pop  cx                                      ;Retirer compteur
          pop  si                                 ;Ramener SI de la pile
          mov  al,ah                           ;Copier caractre dans AL
          jmp  short hex                             ;Calculer code hexa

nosp:     pop  si                                 ;Retirer SI de la pile
          mov  al,ah                           ;Copier caractre dans AL
          stosb                                   ;Sauver dans le buffer

hex:      mov  al,ah                          ;Code du caractre dans AL
          and  ah,1111b                  ;Masquer 4 bits du haut dans AH
          shr  al,1                        ;Dcaler AL de 4 bits en tout
          shr  al,1                                       ;sur la droite
          shr  al,1
          shr  al,1
          or   ax,3030h               ;Convertir AH et AL en codes ASCII
          cmp  al,"9"                   ;AL reprsente-t-il une lettre ?
          jbe  nobal                          ;NON --> Pas de correction
          add  al,"A"-"1"-9                                 ;Corriger AL
nobal:    cmp  ah,"9"                   ;AH reprsente-t-il une lettre ?
          jbe  hexout                         ;NON --> Pas de correction
          add  ah,"A"-"1"-9                                 ;Corriger AH
hexout:   mov  [si],ax                  ;Placer code hexa dans le buffer
          add  si,3                          ;Dsigner position suivante

          inc  bx                     ;Fixer pointeur sur prochain octet
          loop octetin                            ;Traiter octet suivant

          mov  al,219                           ;Fixer caractre spcial
          stosb

          mov  ax,LF shl 8 + CR            ;CR et LF terminent le buffer
          stosw                                      ;Ecrire dans buffer

          ;-- Envoyer Dump sur priphrique de sortie standard ---------

          mov  bx,1                   ;Sortie standard porte le handle 1
          mov  cx,di                      ;Calculer nombre de caractres
          sub  cx,offset dumpBuf                             ;transfrs
          mov  dx,offset dumpBuf                      ;Adresse du buffer
          mov  ah,40h               ;Code de fonction pour Ecrire handle
          int  21h                                 ;Appeler fonction DOS
          jmp  dump                              ;Lire 9 octets suivants

findump  label near

          mov  ax,4C00h         ;Numro fonction pour terminer programme
          int  21h                  ;Terminer programme avec code de fin

;== Donnes ============================================================

neufoctets  db 9 dup (?)                   ;Les 9 octets lus chaque fois
dumpBuf     db 30 dup (?), 219                      ;Le buffer de sortie
            db 49 dup (?)

sptab       db NUL,BEL,BS,TAB          ;Table des caractres de commande
            db LF,CR,EOF,ESC
            db 255

sptext      equ this byte                ;Textes des caractres spciaux
            db 5,"<NUL>"                                            ;NUL
            db 5,"<BEL>"                                       ;Sonnerie
            db 4,"<BS> "                                      ;Backspace
            db 5,"<TAB>"                                     ;Tabulateur
            db 4,"<LF> "                                      ;Line Feed
            db 4,"<CR> "                                ;Carriage Return
            db 5,"<EOF>"                                 ;Fin du fichier
            db 5,"<ESC>"                                         ;Escape

;== Fin ===============================================================

code      ends                                   ;Fin du segment de CODE
          end  dump
