
/**********************************************************************/
/*                           C O N F I G C                            */
/*--------------------------------------------------------------------*/
/*    Fonction        : Affiche la configuration du PC sur l'cran    */
/*                                                                    */
/*--------------------------------------------------------------------*/
/*    Auteur          : MICHAEL TISCHER                               */
/*    Dvelopp le    : 13.08.1987                                    */
/*    Dernire modif. : 19.02.1992                                    */
/*--------------------------------------------------------------------*/
/*    Modle de mmoire : SMALL                                       */
/**********************************************************************/

/*== Intgrer les fichiers Include ===================================*/

#include <dos.h>                      /* Intgrer les fichiers Header */
#include <stdio.h>

/*== Typedefs ========================================================*/

typedef unsigned char BYTE;     /* Voil comment se bricoler un OCTET */

/*== Macros ==========================================================*/

#ifdef MK_FP
  #undef MK_FP
#endif

#ifdef peekb
  #undef peekb
#endif

#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#define peekb(seg, ofs) *((BYTE far *) MK_FP(seg, ofs))

/*== Constantes ======================================================*/

#define TRUE  ( 0 == 0 )      /* Constantes pour faciliter la lecture */
#define FASLE ( 0 == 1 )      /* du texte du programme                */

/**********************************************************************/
/* CLS: Effacer l'cran et placer le curseur dans le coin suprieur   */
/*      gauche de l'cran                                             */
/* Entre : Aucune                                                    */
/* Sortie : Aucune                                                    */
/**********************************************************************/

void Cls( void )
{
 union REGS Register;         /* Registres pour appeler l'interruption*/

 Register.h.ah = 6;              /* Numro de fonction pour Scroll-UP */
 Register.h.al = 0;                                 /* 0 pour effacer */
 Register.h.bh = 7;                  /* Ecriture claire sur fond noir */
 Register.x.cx = 0;               /* Coin suprieur gauche de l'cran */
 Register.h.dh = 24;                 /* Coordonnes du coin infrieur */
 Register.h.dl = 79;                             /* droit de l'cran  */
 int86(0x10, &Register, &Register);     /* interruption vido du BIOS */

 Register.h.ah = 2;        /* Fonction pour fixer position du curseur */
 Register.h.bh = 0;                                   /* Page cran 0 */
 Register.x.dx = 0;   /* Coordonnes coin suprieur gauche de l'cran */
 int86(0x10, &Register, &Register);     /* Interruption vido du BIOS */
}

/**********************************************************************/
/* PRINTCONFIG: Affichage de la configuration d'un PC                 */
/* Entre : Aucune                                                    */
/* Sortie : Aucune                                                    */
/* Infos    : La configuration est affiche en fonction du type de PC */
/*                                                                    */
/**********************************************************************/

void PrintConfig( void )
{
 union REGS Register;        /* Registres pour appeler l'interruption */
 BYTE AT;                                         /* AT ou suprieur? */

 Cls();                                            /* Effacer l'cran */
 AT = (peekb(0xF000, 0xFFFE) == 0xFC);
 printf("CONFIGC  -  (c) 1987, 92 by Michael Tischer\n\n");
 printf("Configuration de votre PC\n");
 printf("----------------------------------------------------------\n");
 printf("Type de PC                 : ");

 switch( peekb(0xF000, 0xFFFE) )                        /* Type de PC */
  {
   case 0xFF : printf("PC\n");             /* 0xFF c'est un PC normal */
               break;
   case 0xFE : printf("XT\n");                    /* 0xFE c'est un XT */
               break;
   default   : printf("AT ou suprieur\n");       /* 0xFC c'est un AT */
               break;
  }
 printf("Mmoire RAM                : ");
 int86(0x12, &Register, &Register);       /* Lire la taille de la RAM */
 printf("%d Ko\n",Register.x.ax);                    /* et l'afficher */
 if ( AT )                                     /* Ce PC est-il un AT? */
  {                                                            /* OUI */
   Register.h.ah = 0x88;    /* n de fonction pour RAM supplmentaire */
   int86(0x15, &Register, &Register);     /* Lire la taille de la RAM */
   printf("RAM supplmentaire         : %u Ko au-dessus de 1Mo\n",
                            Register.x.ax);
  }
 int86(0x11, &Register, &Register);           /* Configuration (BIOS) */
 printf("Mode vido initial         : ");
 switch(Register.x.ax & 48)
  {
   case  0 : printf("Indfini\n");
             break;
   case 16 : printf("40*25 caractres couleur\n");
             break;
   case 32 : printf("80*25 caractres couleur\n");
             break;
   case 48 : printf("80*25 caractres mono\n");
             break;
  }
 printf("Lecteurs de disquette      : %d\n", (Register.x.ax >> 6 & 3) + 1);
 printf("Interfaces srie           : %d\n", Register.x.ax >> 9 & 0x03);
 printf("Interfaces parallle       : %d\n\n", Register.x.ax >> 14);
}

/**********************************************************************/
/**                        PROGRAMME PRINCIPAL                       **/
/**********************************************************************/

void main()
{
 PrintConfig();                          /* Afficher la configuration */
}

