/***********************************************************************
*                             C O N T C . C                            *
**--------------------------------------------------------------------**
*  Fonction         : montre comment programmer un dfilement continu  *
*                     de texte  ( Smooth Scrolling) avec une carte EGA *
*                     ou VGA                                           *
**--------------------------------------------------------------------**
*  Auteur           : MICHAEL TISCHER                                  *
*  Dvelopp le     : 26.08.1990                                       *
*  Dernire MAJ     : 14.02.1992                                       *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Compilation      : CL /AS Contc.c Contca;                           *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Compilation      : Utiliser un fichier de projet avec le contenu    *
*                     suivant                                          *
*                       Contc.c                                        *
*                       Contca.obj                                     *
**--------------------------------------------------------------------**
*  Appel            : Contc                                            *
***********************************************************************/

#include <dos.h>                                  /* Fichiers en-tte */
#include <stdarg.h>
#include <string.h>
#include <stdio.h>

#ifdef __TURBOC__                        /* Compilation avec Turbo C? */
  #define CLI()           disable()
  #define STI()           enable()
  #define outpw( p, w )   outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                  /* Non, avec QuickC 2.0 ou MSC */
  #include <conio.h>
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

#define FAST         2        /* Valeurs de SPEED dans ShowContText() */
#define MEDIUM       1
#define SLOW         0

#define COULEUR      0x5E                     /* Jaune sur fond lilas */
#define COULEUR1     0x5F                     /* Blanc sur fond lilas */
#define LARGEUR      8            /* Largeur des caractres en pixels */
#define HAUTEUR      14   /* Hauteur des caractres (lignes de trame) */
#define COLONNES     216                /* Nb de caractres  l'cran */
#define BANDSIZE     10800                      /* Taille d'une bande */
#define BANDES       3                           /* Nombre de bandes  */
#define MAXLEN       61               /* Nombre maximal de caractres */
#define STARTL       5    /* Ligne de dpart des caractres  l'cran */

#define CrtAttr      0x3C0       /* Registre du contrleur d'attribut */
#define CrtStatus    0x3DA                             /* Port d'tat */
#define CrtAdr       0x3D4             /* Port d'adresses du moniteur */

#define TRUE         (0 == 0)
#define FALSE        (0 == 1)

#define EGA          0                             /* Types de cartes */
#define VGA          1
#define NINI         2

typedef unsigned char BYTE;               /* Bricolage d'un type BYTE */
typedef unsigned int  WORD;                        /* idem pour  WORD */
typedef BYTE          BOOL;

typedef WORD   VRAM[BANDES][25][COLONNES];     /* Structure mm cran */
typedef VRAM   far *VPTR;      /* Pointeur FAR sur la mmoire d'cran */

typedef BYTE   CARDEF[256][14];     /* Structure du jeu de caractres */
typedef CARDEF far *CARPTR;         /* Pointeur sur jeu de caractres */

/*-- Fonctions externes ----------------------------------------------*/

extern void    far *getfontptr( void );     /* Fonction en assembleur */

/*-- Variables globales ----------------------------------------------*/

VPTR           vp;                 /* Pointeur sur la mmoire d'cran */

/***********************************************************************
*  SetOrigin : Fixe la partie visible de la mmoire d'cran par        *
*              programmation du contrleur vido                       *
**--------------------------------------------------------------------**
*  Entres :   Band    = Numro de la bande  afficher(1-5)            *
*              Colonne,= Numro de la colonne et de la ligne           *
*              Ligne     affiches en haut  gauche                    *
*              PixX,   = Offset en pixels                              *
*              PixY                                                    *
*  Sortie  :   nant                                                   *
***********************************************************************/

void SetOrigin( BYTE band, BYTE colonne, BYTE ligne,
                BYTE pixx, BYTE pixy)
{
 int     offset;             /* Offset de dbut de la mmoire d'cran */

 offset = ( BANDSIZE >> 1) * band + ligne * COLONNES + colonne;

 /*- Attend un retour vertical du faisceau et sa fin -----------------*/

 while ( !(( inp(CrtStatus) & 8 ) == 8 ));
 while ( !(( inp(CrtStatus) & 8 ) == 0 ));

           /* Ecrit dans les registres 0x0C et 0x0D l'offset de dbut *
            * de la mmoire d'cran                                   *
            * N'est pris en compte qu'aprs le prochain retour de     *
            * balayage vertical                                       */

 CLI();                                   /* Inhibe les interruptions */
 outpw( CrtAdr, ( offset & 0xFF00 ) + 0x0c );
 outpw( CrtAdr, ( (BYTE) offset << 8 ) + 0x0d );
 STI();                                 /* Rtablit les interruptions */

        /* Attend le dbut du retour vertical du faisceau pour fixer  *
         * le nouvel offset en pixels et la nouvelle adresse de dbut *
         * de l'cran                                                 */

 while ( !(( inp( CrtStatus ) & 8 ) == 8 ));

        /* Ecrit l'offset en pixels dans les registres 0x08 / 0x13 du *
         * contrleur d'attribut                                      */

 CLI();                                   /* Inhibe les interruptions */
 outpw( CrtAdr, ( pixy << 8 ) + 0x08 );
 (void) outp( CrtAttr, 0x13 | 0x20 );
 (void) outp( CrtAttr, pixx );
 STI();                                 /* Rtablit les interruptions */
}

/***********************************************************************
*  PrintChar : Ecrit un caractre en dehors de la zone visible de la   *
*              mmoire d'cran                                         *
**--------------------------------------------------------------------**
*  Entre  :   Caractere = Caractre  crire                          *
*              Band      = Numro de la bande (0-4)                    *
*              Colonne   = Colonne en mmoire d'cran o doit commencer*
*                          le caractre                                *
*  Info    :   Le caractre crit ne peut tre rendu visible que       *
*              par dfilement continu de l'cran .                     *
*              Le motif binaire du caractre est tir du jeu de        *
*              caractres 14*8 pixels de la ROM EGA ou VGA .           *
***********************************************************************/

void PrintChar( char caractere, BYTE band, BYTE colonne )
{
 char            ch;                         /* Pour stocker un pixel */
 BYTE            i, k,                      /* Compteurs d'itrations */
                 masque; /* Masque binaire pour dessiner le caractre */
 static CARPTR   fptr = (CARPTR) 0;       /* Jeu de caractres en ROM */

 if ( fptr == (CARPTR) 0 )               /* Pointeur dj dtermin ? */
  fptr = getfontptr();    /* Le charge avec la fonction en assembleur */

  /*- Parcourt et dessine le caractre ligne par ligne ---------------*/

 for ( i = 0; i < HAUTEUR; ++i )
 {
   masque = (*fptr)[caractere][i];    /* Motif binaire pour une ligne */
   for ( k = 0; k < LARGEUR; ++k )    /* Parcourt les diffrents bits */
   {
     ch = ( masque & 128 ) ? 219 : 32;
     (*vp)[band][STARTL+i][colonne*LARGEUR+k] =
                         (BYTE) ch+( COULEUR << 8 );
     masque <<= 1;                            /* Passe au bit suivant */
   }
 }
}

/***********************************************************************
*  IsEgaVga : Teste la prsence d'une carte EGA ou VGA                 *
**--------------------------------------------------------------------**
*  Entre  : nant                                                     *
*  Sortie  : EGA, VGA ou NINI                                          *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS    Regs;        /* Registres pour grer les interruptions */

 Regs.x.ax = 0x1a00;              /* N'existe que pour les cartes VGA */
 int86( 0x10, &Regs, &Regs );
 if( Regs.h.al == 0x1a )                  /* La fonction disponible ? */
   return VGA;
 else
 {
   Regs.h.ah = 0x12;                          /* Appelle l'option 10h */
   Regs.h.bl = 0x10;                            /* de la fonction 12h */
   int86(0x10, &Regs, &Regs );               /* Appelle le BIOS vido */
   return ( Regs.h.bl != 0x10 ) ? EGA : NINI;
 }
}

/***********************************************************************
*  ShowContText : Fait dfiler un texte sur l'cran                    *
**--------------------------------------------------------------------**
*  Entre :  dtext = Texte  faire dfiler sous forme de chane        *
*            speed = Vitesse de dfilement (SLOW, MEDIUM ou SLOW)      *
*            vc    = carte vido (EGA ou VGA)                          *
***********************************************************************/

void ShowContText( char * dtext, BYTE speed, BYTE vc )
{
  int       band,                                   /* Bande courante */
            colonne,                              /* Colonne courante */
            index,     /* Indice courant dans la chane de caractres */
            len,                      /* Longueur du texte  afficher */
            i, k,                            /* Compteur d'itrations */
            pixx;                     /* Valeur de panning horizontal */
 WORD far   *wptr;      /* Pointeur pour parcourir la mmoire d'cran */
 union REGS Regs;              /* Registres pour grer l'interruption */

 static BYTE steptable[2][3][10] =
        {
          {                             /* Pas de dfilement pour EGA */
            {   0,   1,   2,   3,   4,   5,   6,   7, 255, 255 },
            {   0,   2,   4,   6, 255, 255, 255, 255, 255, 255 },
            {   0,   4, 255, 255, 255, 255, 255, 255, 255, 255 }
          },
          {                             /* Pas de dfilement pour VGA */
            {   8,   0,   1,   2,   3,   4,   5,   6,   7, 255 },
            {   8,   2,   5, 255, 255, 255, 255, 255, 255, 255 },
            {   8,   3, 255, 255, 255, 255, 255, 255, 255, 255 }
          }
        };

 vp = MK_FP( 0xB800, 0x0000 );       /* Pointe sur la mmoire d'cran */

 /*- Remplit toute la mmoire d'cran avec des espaces ---------------*/

 for( index = 0; index < BANDES; ++index )
   for( i = 0; i < 25; ++i )
     for( k = 0; k < COLONNES; ++k )
       (*vp)[index][ i ][ k ] = ( COULEUR << 8 ) + 32;

 /*- Trace des guides horizontaux ------------------------------------*/

 for( k = 0; k < BANDES; ++k )
   for( i = 0; i < COLONNES; ++i )
   {
       (*vp)[ k ][ STARTL-2 ][ i ] =
                   (BYTE) '' + ( COULEUR1 << 8 );
       (*vp)[ k ][ STARTL + HAUTEUR + 2][ i ] =
                   (BYTE) '' + ( COULEUR1 << 8 );
   }

 /*- Retire le curseur cignotant -------------------------------------*/

 Regs.h.ah = 0x02;   /* Numro de la fonction "Set Cursor"  */
 Regs.h.bh = 0;    /* Page d'cran */
 Regs.x.dx = 0;      /* Coordonnes */
 int86( 0x10, &Regs, &Regs );

 /*- Fixe la couleur du cadre d'cran --------------------------------*/

 Regs.h.ah = 0x10;      /* Numro de la fonction "Fixer couleur cadre"*/
 Regs.h.al = 0x01;                                 /* Numro d'option */
 Regs.h.bh = COULEUR >> 4;                        /* Couleur du cadre */
 int86( 0x10, &Regs, &Regs );

 /*- Fixe  COLONNES le nbre de colonnes/ligne dans la mmoire cran -*/

 outpw( CrtAdr, ( ( COLONNES >> 1 ) << 8 ) + 0x13 );

 /*-- Ecrit le texte dfilant dans la mmoire d'cran ----------------*/

 if ( ( len = strlen( dtext ) ) > MAXLEN )    /* Chane trop longue ? */
   *(dtext + ( len = MAXLEN )) = '\0';              /* Oui, on abrge */

 for( colonne = band = index = 0; index < len; )
 {                                            /* Ecrit les caractres */
     PrintChar( *(dtext+index++), band, colonne++ );
     if ( colonne >= COLONNES / LARGEUR )    /* Changement de bande ? */
     {                                                         /* Oui */
         colonne = 0;                      /* Recommence en colonne 1 */
         ++band;                         /* Passe  la bande suivante */
         index -= 80 / LARGEUR;                /* Une page en arrire */
     }
 }

 /*-- Fait dfiler le texte de droite  gauche sur l'cran -----------*/

  for( colonne = band = 0 , i = (len - ( 80/LARGEUR )) * LARGEUR;
       i > 0;
       --i )
  {
    for( k = 0; ( pixx = steptable[vc][speed][k]) != 255 ; ++k )
      SetOrigin( band, colonne, 0, pixx, 0 );

    if( ++colonne == COLONNES - 80 )           /* Changement de bande */
    {                                                           /* Oui*/
      colonne = 0;                         /* Recommence en colonne 0 */
      ++band;                                  /* Incrmente la bande */
    }
  }

 /*- Remet 80 caractres par ligne en mmoire d'cran ----------------*/

 outpw( CrtAdr, ( 40 << 8 ) + 0x13 );

 SetOrigin( 0, 0, 0, 8, 0 );                /* Paramtrage par dfaut */

 /*- Rtablit le curseur ---------------------------------------------*/

 Regs.h.ah = 0x02;              /* Numro de la fonction "Set Cursor" */
 Regs.h.bh = 0;                                       /* Page d'cran */
 Regs.x.dx = 0;                                        /* Coordonnes */
 int86( 0x10, &Regs, &Regs );

 /*- Restaure la couleur du cadre d'cran ----------------------------*/

 Regs.h.ah = 0x10;             /* "Fixer la couleur du cadre d'cran" */
 Regs.h.al = 0x01;                              /* Numro de l'option */
 Regs.h.bh = 0;                                      /* Cadre en noir */
 int86( 0x10, &Regs, &Regs );

 /*-- Efface l'cran -------------------------------------------------*/

 for( wptr = (WORD far *) vp, i = 80*25; i-- ; )
   *wptr++ = 0x0720;
}

/*--------------------------------------------------------------------*/
/*--                       PROGRAMME PRINCIPAL                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
  BYTE     vc;                       /* Type de carte vido installe */

 if( ( vc = IsEgaVga() ) == NINI )
   printf( "TEXTE DEFILANT -  (c) 1990, 92 by MICHAEL TISCHER\n" \
           "Attention : aucune carte EGA ou VGA n'est installe !\n" );
 else
   ShowContText( "+++ La Bible PC (c) 1990-1992 Micro Application +++ "\
                 "          ", FAST, vc );
}
