/***********************************************************************
*                            D U M P C                                 *
**--------------------------------------------------------------------**
*    Fonction       : un filtre qui lit des caractres sur l'entre    *
*                     standard pour les ressortir sur la sortie        *
*                     standard sous forme de dump hexa et ASCII        *
**--------------------------------------------------------------------**
*    Auteur         : MICHAEL TISCHER                                  *
*    Dvelopp le   : 14/08/1987                                       *
*    Dernire modif.: 29/02/1992                                       *
**--------------------------------------------------------------------**
*    (MICROSOFT C)                                                     *
*    Cration       : CL /AS DUMPC.C                                   *
*    Appel          : DUMPC [<Entre] [>Sortie]                        *
**--------------------------------------------------------------------**
*    (BORLAND TURBO C)                                                 *
*    Cration       : Crer avec l'instruction COMPILE - MAKE          *
*                     (sans fichier Project) et faire excuter         *
*                     sous l'environnement DOS                         *
**********************************************************************/

/*== fichiers Include ================================================*/
#include <stdio.h>                        /* Intgrer fichiers header */
#include <dos.h>

/*== Typedefs ========================================================*/
typedef unsigned char byte;     /* Voil comment se bricoler un OCTET */

/*== Constantes ======================================================*/
#define NUL     0                            /* Code du caractre NUL */
#define BEL     7                              /* Code de la sonnerie */
#define BS      8                      /* Code de la touche Backspace */
#define TAB     9                     /* Code de la touche tabulateur */
#define LF      10                               /* Code de Line Feed */
#define CR      13                        /* Code de la touche Return */
#define ESC     27                        /* Code de la touche Escape */

/*== Macros ==========================================================*/
#define tohex(c) ( ((c)<10) ? ((c) | 48) : ((c) + 'A' - 10) )

/***********************************************************************
* GETSTDIN: lit un nombre dtermin de caractres sur le priphrique  *
*           d'entre standard et les place dans un buffer              *
* Entre  : voir plus bas                                              *
* Sortie  : Nombre de caractres lus                                   *
***********************************************************************/

unsigned int GetStdIn( char *Buffer, unsigned NombreMax )
{
  union REGS    Register;          /* Registre d'appel d'interruption */
  struct SREGS  Segments;          /* Reoit les registres de segment */

 segread( &Segments );    /* Charger contenu des registres de segment */
 Register.h.ah = 0x3F;                     /* Numro de fonction pour */
 Register.x.bx = 0;                /* Priph. d'entre std : handle 0 */
 Register.x.cx = NombreMax;                 /* Nombre d'octets  lire */
 Register.x.dx = (unsigned int) Buffer; /* Adresse d'offset du buffer */
 intdosx(&Register, &Register, &Segments);          /* int. DOS 21(h) */
 return(Register.x.ax);           /* Nombre d'octets lus  l'appelant */
}

/***********************************************************************
* STRAP: Ajouter caractre  une chane                                *
* Entre  : voir plus bas                                              *
* Sortie  : Pointeur  la suite du dernier caractre ajout            *
***********************************************************************/

char *Strap(char *String, char *Ajouter)
{
 while( *Ajouter )           /* Rpter jusqu' ce que '\0' rencontr */
   *String++ = *Ajouter++;                    /* Transfrer caractre */
 return(String);           /* Transmettre pointeur  fonction appele */
}

/***********************************************************************
* DODUMP: charge les caractres et les sort comme Dump                 *
* Entre  : Aucune                                                     *
* Sortie  : Aucune                                                     *
***********************************************************************/

void DoDump( void )

{
 char NeufOctets[9],                  /* Reoit les caractres entrs */
      DumpBuf[80],                        /* Reoit une ligne du DUMP */
      *NextAscii;  /* Dsigne prochain caractre ASCII dans le buffer */
 byte i,                                        /* Compteur de boucle */
      Nombre;                               /* Nombre d'octets entrs */

 DumpBuf[30]   = 219;           /* Caractre de sparation hexa/ASCII */
 while( (Nombre = GetStdIn(NeufOctets, 9)) != 0 )      /* Caractre ? */
 {                                                             /* Oui */
   for( i = 0; i < 30; DumpBuf[i++] = ' ')       /* Remplir d'espaces */
      ;
   NextAscii = &DumpBuf[31];                      /* Caractres ASCII */
   for( i = 0; i < Nombre; i++ )      /* Traiter ts caractres entrs */
   {
     DumpBuf[i*3]   = tohex((byte) NeufOctets[i] >> 4); /* Conv. code */
     DumpBuf[i*3+1] = tohex((byte) NeufOctets[i] & 15);    /* en hexa */
     switch (NeufOctets[i])                     /* Evaluer code ASCII */
     {
       case NUL : NextAscii = Strap(NextAscii, "<NUL>");
                  break;
       case BEL : NextAscii = Strap(NextAscii, "<BEL>");
                  break;
       case BS  : NextAscii = Strap(NextAscii, "<BS>");
                  break;
       case TAB : NextAscii = Strap(NextAscii, "<TAB>");
                  break;
       case LF  : NextAscii = Strap(NextAscii, "<LF>");
                  break;
       case CR  : NextAscii = Strap(NextAscii, "<CR>");
                  break;
       case ESC : NextAscii = Strap(NextAscii, "<ESC>");
                  break;
       case EOF : NextAscii = Strap(NextAscii, "<EOF>");
                  break;
       default  : *NextAscii++ = NeufOctets[i];
     }
   }
   *NextAscii     = 219;     /* Caractre de fin pour affichage ASCII */
   *(NextAscii+1) = '\r';       /* Carriage Return  la fin du buffer */
   *(NextAscii+2) = '\0';        /* NUL converti en LF pendant sortie */
   puts(DumpBuf);          /* Ecrire chane sur priph. de sortie std */
  }
}

/**********************************************************************/
/**                       PROGRAMME PRINCIPAL                        **/
/**********************************************************************/

void main()

{
  DoDump();                             /* Entrer et sortir caractre */
}
