/***********************************************************************
*                           D V I D E O                                *
*----------------------------------------------------------------------*
*    Fonction       : Dmonstration de l'accs direct  la             *
*                     mmoire d'cran                                  *
*----------------------------------------------------------------------*
*    Auteur         : MICHAEL TISCHER                                  *
*    Dvelopp le   : 01/10/1988                                       *
*    Dernire MAJ   : 01/02/1992                                       *
*----------------------------------------------------------------------*
*    (MICROSOFT C)                                                     *
*    Compilation    : CL /AS DVIC.C                                    *
*    Appel          : DVIC                                             *
*----------------------------------------------------------------------*
*    (BORLAND TURBO C)                                                 *
*    Compilation    : par la commande RUN de l'EDI                     *
*                     (sans fichier de projet )                        *
***********************************************************************/

/*== Fichiers en-tte ================================================*/

#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <bios.h>

/*== Typedef =========================================================*/

typedef unsigned char    BYTE;            /* Bricolage d'un type BYTE */
typedef struct velb far  *VP;    /* Pointeur FAR sur la mmoire cran */
typedef BYTE             BOOL;             /* Comme BOOLEAN en Pascal */

/*== Structures ======================================================*/

struct velb
{                       /* Dcrit une position de l'cran en 2 octets */
  BYTE caractere,                                    /* Le code ASCII */
       attribut;                          /* L'attribut correspondant */
};

/*== Macros ==========================================================*/

/*-- MK_FP forme un pointeur FAR sur un objet  partir d'une ---------*/
/*-- adresse de segment et d'une adresse d'offset --------------------*/

#ifndef MK_FP                    /* MK_FP n'a pas encore t dfini ? */
  #define MK_FP(seg, ofs) ((void far *)((unsigned long)(seg)<<16|(ofs)))
#endif

#define COULEUR(PP, FD)   ((PP << 3) + FD)

/*== Constantes ======================================================*/

#define TRUE  (0==0)          /* Constantes pour travailler avec BOOL */
#define FALSE !TRUE

/*-- Les constantes suivantes fournissent des pointeurs --------------*/
/*-- du segment des variables du BIOS  partir du segment 0x40 -------*/

#define CRT_START         ((unsigned far *) MK_FP(0x40, 0x4E))
#define ADDR_6845         ((unsigned far *) MK_FP(0x40, 0x63))

#define NORMAL            0x07              /* Attributs de caractre */
#define CLAIR             0x0f              /* pour une carte d'cran */
#define INVERSE           0x70                          /* monochrome */
#define SOULIGNE          0x01
#define CLIGNOTANT        0x80

#define NOIR              0x00           /* Attributs couleur couleur */
#define BLEU              0x01
#define VERT              0x02
#define CYAN              0x03
#define ROUGE             0x04
#define MAGENTA           0x05
#define BRUN              0x06
#define GRISCLAIR         0x07
#define GRISFONCE         0x01
#define BLEAUCLAIR        0x09
#define VERTCLAIR         0x0A
#define CYANCLAIR         0x0B
#define ROUGECLAIR        0x0C
#define MAGENTACLAIR      0x0D
#define JAUNE             0x0E
#define BLANC             0x0F

/*== Variables globales ==============================================*/

VP      vptr;              /* Premier caractre de la mmoire d'cran */

/***********************************************************************
*  Fonction         : D P R I N T                                      *
**--------------------------------------------------------------------**
*  Fonction         : Affiche une chane dans la mmoire d'cran.      *
*                                                                      *
*  Param. en entre : - COLONNE  = La colonne de sortie.               *
*                     - LIGNE    = La ligne de sortie.                 *
*                     - COULEUR  = Attribut pour les caractres.       *
*                     - STRING   = Pointeur sur la chane.             *
*  Valeur de retour : Aucune                                           *
***********************************************************************/

void dprint(BYTE colonne, BYTE ligne, BYTE couleur, char * string)
{
  register VP    lptr;                        /* Pointeur de comptage */
  register BYTE  i;                         /* Compteur de caractres */

 /*-- Pointeur sur la position de sortie dans la mmoire d'cran -----*/
  lptr = (VP) ((BYTE far *) vptr + *CRT_START) + ligne * 80 + colonne;
  for( i=0 ; *string ; ++lptr, ++i )            /* Parcourt la chane */
  {
    lptr->caractere = *(string++);          /* Transfre le caractre */
    lptr->attribut = couleur;                      /* Fixe l'attribut */
  }
}

/***********************************************************************
*  Fonction         : I N I T _ D P R I N T                            *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine le segment de la mmoire d'cran       *
*                     pour DPRINT:                                     *
*  Param. en entre : Aucun                                            *
*  Valeur de retour : Aucune                                           *
*  Infos            : L'adresse de segment de la mmoire d'cran       *
*                     est place dans la variable globale VPTR.        *
***********************************************************************/

void init_dprint()
{
  vptr = (VP) MK_FP( (*ADDR_6845 == 0x3B4) ? 0xB000 : 0xB800, 0 );
}

/***********************************************************************
*  Fonction         : C L S                                            *
**--------------------------------------------------------------------**
*  Fonction         : Efface l'cran  l'aide de DPRINT.               *
*                                                                      *
*  Param. en entre : - COULEUR    = Attribut pour les caractres.     *
*  Valeur de retour : Aucune                                           *
***********************************************************************/

void cls( BYTE couleur )
{
  static char lignevide[81] =
  { ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
    ' ',' ',' ',' ',' ','\0'
  };

  register BYTE     i;                       /* Compteur d'itrations */

  for( i=0; i<24; ++i )            /* Parcourt les diffrentes lignes */
    dprint(0, i, couleur, lignevide);      /* Affiche une ligne vierge*/
}

/***********************************************************************
*  Fonction         : N O K E Y                                        *
**--------------------------------------------------------------------**
*  Fonction         : Teste si une touche a t actionne.             *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : TRUE si une touche a t actionne, sinon        *
*                     FALSE.                                           *
***********************************************************************/

BOOL nokey()
{
#ifdef __TURBOC__                              /* Est-on en TURBO-C ? */
  return( bioskey( 1 ) == 0 );  /* OUI, teste le clavier avec le BIOS */
#else                                /* On travaille avec Microsoft C */
  return( _bios_keybrd( _KEYBRD_READY ) == 0 );  /* Test avec le BIOS */
#endif
}

/**********************************************************************/
/**                       PROGRAMME PRINCIPAL                        **/
/**********************************************************************/

void main()
{
 BYTE firstcol,          /* Couleur pour le premier carr sur l'cran */
      couleur,                       /* Couleur pour le carr courant */
      colonne,                       /* Position d'affichage actuelle */
      ligne;

  init_dprint();        /* Dtermine le segment de la mmoire d'cran */
  cls( COULEUR(NOIR, VERT) );                       /* Efface l'cran */
  dprint(22, 0, BLANC, "DVIDEO  - (c) 1988,92 by Michael Tischer");
  firstcol = NOIR;                           /* Commencer par le NOIR */

  while( nokey() )             /* Attend qu'une touche soit actionne */
  {
    if (++firstcol > BLANC)            /* Dernire couleur atteinte ? */
      firstcol = BLEU;                  /* OUI, on continue avec BLEU */
    couleur = firstcol;       /* Fixe la premire couleur sur l'cran */

  /*-- Remplit l'cran avec des pavs --------------------------------*/

    for ( colonne=0; colonne < 80; colonne += 4)
      for (ligne=1; ligne < 24; ligne += 2)
      {
        dprint( colonne, ligne,   couleur, "");
        dprint( colonne, ligne+1, couleur, "");
        couleur = ++couleur & 15;
      }
  }
}
