/***********************************************************************
*                               E M M C                                *
**--------------------------------------------------------------------**
*    Fonction       : Fournit quelques fonctions pour l'accs  la     *
*                     mmoire EMS (Expanded Memory).                   *
**--------------------------------------------------------------------**
*    Auteur         : MICHAEL TISCHER                                  *
*    Dvelopp le   : 30/08/1988                                       *
*    Dernire modif.: 30/03/1992                                       *
**--------------------------------------------------------------------**
*    (MICROSOFT C)                                                     *
*    Cration       : CL /AC EMMC.C                                    *
*                   : LINK EMMC;                                       *
*    Appel          : EMMC                                             *
**--------------------------------------------------------------------**
*    (BORLAND TURBO C)                                                 *
*    Cration       : Avec l'instruction RUN dans la ligne de menu     *
*                     (sans fichier Project)                           *
*    Infos          : Notez bien que le modle de mmoire Compact      *
*                     doit tre slectionn avec l'instruction         *
*                     Option-Compiler-Model !                          *
***********************************************************************/

/*== Intgrer les fichiers Include ===================================*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>

/*== Typedefs ========================================================*/

typedef unsigned char BYTE;           /* Nous nous bricolons un octet */
typedef unsigned int  WORD;
typedef BYTE          BOOL;                /* Comme BOOLEAN en Pascal */

/*== Macros ==========================================================*/

/*-- MK_FP compose un pointeur FAR sur un objet  partir d'une -------*/
/*-- adresse de segment et d'une adresse d'offset              -------*/

#ifdef MK_FP                             /* MK_FP  dj t dfini ? */
  #undef MK_FP
#endif
#define MK_FP(seg, ofs) ((void far *)(((unsigned long)(seg)<<16)|(ofs)))

/*-- PAGE_ADR fournit un pointeur sur la page physique X -------------*/
/*--  l'intrieur du Page Frame de la mmoire EMS       -------------*/

#define PAGE_ADR(x) ((void *) (MK_FP(ems_frame_seg() + ((x)<<10), 0)))

/*== Constantes ======================================================*/

#define TRUE    ( 0 == 0 )    /* Constantes pour travailler avec BOOL */
#define FALSE   ( 1 == 0 )

#define EMS_INT 0x67        /* N d'interruption pour l'accs  l'EMM */
#define EMS_ERR -1                     /* Est renvoy en cas d'erreur */

/*== Variables globales ==============================================*/

BYTE emm_ec;                /* Ici sont placs les codes d'erreur EMM */

/***********************************************************************
*  Fonction         : E M S _ I N S T                                  *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine si une mmoire EMS et un driver EMS    *
*                     correspondant (EMM) sont installs.              *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : TRUE si mmoire EMS installe, sinon             *
*                     FALSE.                                           *
***********************************************************************/

BOOL ems_inst( void )
{
  static char   emm_name[] = { 'E', 'M', 'M', 'X', 'X', 'X', 'X', '0' };
  union REGS    regs; /* Registres processeur pour appel interruption */
  struct SREGS  sregs;  /* Registre segment pour l'appel interruption */

   /* Mettre en place pointeur sur noms dans en-tte driver de priph.*/

  regs.x.ax = 0x3567;   /* N fct.: Rechercher vecteur interrupt 0x67 */
  intdosx( &regs, &regs, &sregs );   /* Appeler interruption DOS 0x21 */

  return (!memcmp( MK_FP(sregs.es, 10), emm_name, sizeof emm_name ));
                                                 /*TRUE si nom trouv */
}

/***********************************************************************
*  Fonction         : E M S _ N U M _ P A G E                          *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine le nombre total de pages EMS.          *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : EMS_ERR en cas d'erreur, sinon le nombre de      *
*                     pages EMS.                                       *
***********************************************************************/

int ems_num_page( void )
{
  union REGS   regs;              /* Reg. proc. pour appel interrupt. */

  regs.h.ah = 0x42;            /* N fct.: Dterminer nombre de pages */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  if( (int)(emm_ec = regs.h.ah) )    /* Une erreur est-elle apparue ? */
    return( EMS_ERR );                        /* OUI, afficher erreur */
  else                                                /* Pas d'erreur */
    return( regs.x.dx );         /* Renvoyer le nombre total de pages */
}

/***********************************************************************
*  Fonction         : E M S _ F R E E _ P A G E                        *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine le nombre de pages EMS encore libres.  *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : EMS_ERR en cas d'erreur, sinon le nombre de      *
*                     pages EMS libres.                                *
***********************************************************************/

int ems_free_page( void )
{
  union REGS  regs;   /* Registres processeur pour appel interruption */

  regs.h.ah = 0x42;            /* N fct.: Dterminer nombre de pages */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  if((int)(emm_ec = regs.h.ah))      /* Une erreur est-elle apparue ? */
    return( EMS_ERR );                        /* OUI, afficher erreur */
  else                                                /* Pas d'erreur */
    return( regs.x.bx );           /* Renvoyer nombre de pages libres */
}

/***********************************************************************
*  Fonction         : E M S _ F R A M E _ S E G                        *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine l'adresse de segment du Page Frame EMS *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : EMS_ERR en cas d'erreur, sinon adresse segment   *
*                     du Page Frame.                                   *
***********************************************************************/

WORD ems_frame_seg( void )
{
  union REGS  regs;

  regs.h.ah = 0x41;            /* N fct.: adresse segment Page Frame */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  if( (int)(emm_ec = regs.h.ah) )    /* Une erreur est-elle apparue ? */
    return( EMS_ERR );                        /* OUI, afficher erreur */
  else                                                /* Pas d'erreur */
    return( regs.x.bx );               /* Renvoyer adresse de segment */
}

/***********************************************************************
*  Fonction         : E M S _ A L L O C                                *
**--------------------------------------------------------------------**
*  Fonction         : Alloue le nombre de pages spcifi et renvoie un *
*                     Handle pour l'accs  ces pages.                 *
*  Param. en entre : PAGES : Nombre de pages  allouer                *
*                             (de 16 Ko chacune)                       *
*  Valeur Return    : EMS_ERR en cas d'erreur, sinon le Handle EMS.    *
***********************************************************************/

int ems_alloc(int pages)
{
  union REGS  regs;

  regs.h.ah = 0x43;                         /* N fct.: Allouer pages */
  regs.x.bx = pages;               /* Fixer nombre de pages  allouer */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  if( (int)(emm_ec = regs.h.ah) )    /* Une erreur est-elle apparue ? */
    return(EMS_ERR);                          /* OUI, afficher erreur */
  else                                                /* Pas d'erreur */
    return( regs.x.dx );                       /* Renvoyer Handle EMS */
}

/***********************************************************************
*  Fonction         : E M S _ M A P                                    *
**--------------------------------------------------------------------**
*  Fonction         : Calque une des pages logiques alloues sous le   *
*                     handle transmis sur la page physique du          *
*                     Page Frame.                                      *
*  Param. en entre : HANDLE: Le Handle renvoy par EMS_ALLOC.         *
*                     LOGP  : La page logique (0  n-1)                *
*                     PHYSP : La page physique (0  3)                 *
*  Valeur Return    : FALSE en cas d'erreur, sinon TRUE.               *
***********************************************************************/

BOOL ems_map(int handle, int logp, BYTE physp)
{
  union REGS  regs;

  regs.h.ah = 0x44;                         /* N fct.: Fixer Mapping */
  regs.h.al = physp;                           /* Fixer page physique */
  regs.x.bx = logp;                             /* Fixer page logique */
  regs.x.dx = handle;                             /* Fixer Handle EMS */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  return( !(emm_ec = regs.h.ah) );
}

/***********************************************************************
*  Fonction         : E M S _ F R E E                                  *
**--------------------------------------------------------------------**
*  Fonction         : Libre  nouveau la mmoire alloue sous un      *
*                     Handle.                                          *
*  Param. en entre : HANDLE: le Handle renvoy par EMS_ALLOC.         *
*  Valeur Return    : FALSE en cas d'erreur, sinon TRUE.               *
***********************************************************************/

BOOL ems_free(int handle)
{
  union REGS  regs;

  regs.h.ah = 0x45;                         /* N fct.: Librer pages */
  regs.x.dx = handle;                             /* Fixer Handle EMS */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  return( !(emm_ec = regs.h.ah) ); /* Si AH contient 0, tout va bien. */
}

/***********************************************************************
*  Fonction         : E M S _ V E R S I O N                            *
**--------------------------------------------------------------------**
*  Fonction         : Dtermine le numro de version EMM.              *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : EMS_ERR en cas d'erreur, sinon le numro de      *
*                     version EMM.                                     *
*  Infos            : Pour le n de version, 10 signifie 1.0, 11       *
*                     signifie 1.1, 34 signifie 3.4 etc.               *
***********************************************************************/

BYTE ems_version( void )
{
  union REGS  regs;

  regs.h.ah = 0x46;                /* N fct.: Dterminer version EMM */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  if( (int)(emm_ec = regs.h.ah) )    /* Une erreur est-elle apparue ? */
    return( EMS_ERR );                        /* OUI, afficher erreur */
  else             /* Pas d'erreur, n version  partir du nombre BCD */
    return( (regs.h.al & 15) + (regs.h.al >> 4) * 10 );
}

/***********************************************************************
*  Fonction         : E M S _ S A V E _ M A P                          *
**--------------------------------------------------------------------**
*  Fonction         : Sauvegarde le calquage (Mapping) entre pages     *
*                     logiques et physiques.                           *
*  Param. en entre : HANDLE: le handle renvoy par EMS_ALLOC.         *
*  Valeur Return    : FALSE en cas d'erreur, sinon TRUE.               *
***********************************************************************/

BOOL ems_save_map(int handle)
{
  union REGS  regs;

  regs.h.ah = 0x47;                   /* N fct.: Sauvegarder Mapping */
  regs.x.dx = handle;                             /* Fixer Handle EMS */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  return (!(emm_ec = regs.h.ah));  /* Si AH contient 0, tout va bien. */
}

/***********************************************************************
*  Fonction         : E M S _ R E S T O R E _ M A P                    *
**--------------------------------------------------------------------**
*  Fonction         : Rtablit un calquage entre pages logiques et     *
*                     physiques sauvegard pralablement avec          *
*                     EMS_SAVE_MAP.                                    *
*  Param. en entre : HANDLE: le handle renvoy par EMS_ALLOC.         *
*  Valeur Return    : FALSE en cas d'erreur, sinon TRUE.               *
***********************************************************************/
BOOL ems_restore_map(int handle)
{
  union REGS  regs;

  regs.h.ah = 0x48;                      /* N fct.: Rtablir Mapping */
  regs.x.dx = handle;                             /* Fixer Handle EMS */
  int86( EMS_INT, &regs, &regs );                      /* Appeler EMM */
  return( !(emm_ec = regs.h.ah) ); /* Si AH contient 0, tout va bien. */
}

/***********************************************************************
*  Fonction         : P R I N T _ E R R                                *
**--------------------------------------------------------------------**
*  Fonction         : Sort un message d'erreur EMS sur l'cran et      *
*                     termine le programme.                            *
*  Param. en entre : Aucun                                            *
*  Valeur Return    : Aucune                                           *
*  Infos            : Cette fonction ne doit tre appele que si une   *
*                     erreur s'est produite lors d'un appel antrieur  *
*                     d'une fonction de l'EMM.                         *
***********************************************************************/

void print_err( void )
{
  static char     nid[] = "non identifiable";
  static char     *err_vec[] =
  {
    "Erreur dans le driver EMS (EMM dtruit)",                /* 0x80 */
    "Erreur dans l'lectronique EMS",                         /* 0x81 */
     nid,                                                     /* 0x82 */
    "Handle EMM incorrect",                                   /* 0x83 */
    "Fonction EMS appele n'existe pas",                      /* 0x84 */
    "Plus de handles EMS disponibles",                        /* 0x85 */
    "Erreur de sauvegarde ou de reconstitution du Mapping",   /* 0x86 */
    "Plus de pages rclames qu'il n'en existe physiquement", /* 0x87 */
    "Plus de pages rclames qu'il n'en reste de libres",     /* 0x88 */
    "Zro page rclame",                                     /* 0x89 */
    "Page logique ne correspondant pas au Handle",            /* 0x8A */
    "Numro de page physique incorrect",                      /* 0x8B */
    "Zone de mmoire Mapping pleine",                         /* 0x8C */
    "Sauvegarde du Mapping dj effectue",                   /* 0x8D */
    "Reconstitution du Mapping sans sauvegarde antrieure"
   };

  printf("\nATTENTION ! Erreur lors de l'accs  la mmoire EMS\n");
  printf("         ... %s\n", (emm_ec<0x80 || emm_ec>0x8E)
                               ? nid
                               : err_vec[emm_ec-0x80]);
  exit( 1 );                 /* Terminer programme avec code d'erreur */
}

/***********************************************************************
*  Fonction        :   V R _ A D R                                     *
**--------------------------------------------------------------------**
*  Fonction      :   Fournit un pointeur sur la RAM vido              *
*  Entre        :   Aucune                                            *
*  Sortie        :   Pointeur sur la RAM vido                         *
***********************************************************************/

void *vr_adr( void )
{
  union REGS  regs;

  regs.h.ah = 0x0f;                 /* N fct.: Dterminer mode vido */
  int86(0x10, &regs, &regs);    /* Appeler interruption vido du BIOS */
  return( MK_FP(((regs.h.al==7) ? 0xb000 : 0xb800), 0) );
}

/**********************************************************************/
/**                       PROGRAMME PRINCIPAL                        **/
/**********************************************************************/

void main()
{
 int  nmbpage,                                 /* Nombre de pages EMS */
      handle,                 /* Handle pour l'accs  la mmoire EMS */
      i;                                        /* Compteur de boucle */
 WORD pageseg;                    /* Adresse de segment du Page Frame */
 BYTE emmver;                           /* Numro de version de l'EMM */

 printf("EMMC  -  (c) 1988, 92 by MICHAEL TISCHER\n\n");
 if( ems_inst() )                          /* Mmoire EMS installe ? */
 {                                                             /* Oui */
    /*-- Sortir des informations sur la mmoire EMS ------------------*/
   if( (int)(emmver = ems_version()) == EMS_ERR)     /* n de version */
     print_err();     /* Erreur : Sortir message d'erreur et terminer */
   else                                               /* Pas d'erreur */
     printf("Numro de version EMM        : %d.%d\n",
           emmver/10, emmver%10);

   if( (nmbpage = ems_num_page()) == EMS_ERR)        /* nmb. de pages */
     print_err();            /* Erreur : message d'erreur et terminer */
   else
     printf("Nombre de pages EMS          : %d (%d Ko)\n",
          nmbpage, nmbpage << 4);

   if( (nmbpage = ems_free_page()) == EMS_ERR )
     print_err();            /* Erreur : message d'erreur et terminer */
   else
     printf("... dont libres              : %d (%d Ko)\n",
          nmbpage, nmbpage << 4);

   if( (int) (pageseg = ems_frame_seg()) == EMS_ERR )
     print_err();            /* Erreur : message d'erreur et terminer */
   else
     printf("Adresse de segment du Page Frame : %X\n", pageseg);

   printf("\nUne page de la mmoire EMS va maintenant tre\n");
   printf("alloue et le contenu de l'cran va tre copi de la\n");
   printf("RAM vido dans cette page.\n");
   printf("                    ... Veuillez frapper une touche\n");
   getch();                                    /* Attendre une touche */

   /*-- Allouer une page et la calquer sur la premire page ----------*/
   /*-- logique dans le Page Frame                          ----------*/
   if( (handle = ems_alloc(1)) == EMS_ERR )
     print_err();            /* Erreur : message d'erreur et terminer */
   if( !ems_map(handle, 0, 0) )                      /* Fixer Mapping */
     print_err();            /* Erreur : message d'erreur et terminer */

   /*-- Copier 4000 octets de la RAM vido dans la mmoire EMS -------*/

   memcpy( PAGE_ADR(0), vr_adr(), 4000 );

   for( i = 0; i < 24; ++i )                         /* Vider l'cran */
     printf("\n");

   printf("L'ancien contenu de l'cran a maintenant t effac et\n");
   printf("est donc perdu. Cependant, comme il avait\n");
   printf("t sauvegard dans la mmoire EMS, il peut tre recopi\n");
   printf("dans la RAM vido  partir de l.\n");
   printf("                   ... Veuillez frapper une touche\n");
   getch();                                    /* Attendre une touche */

   /*-- Recopier le contenu de la RAM vido d'aprs la mmoire EMS ---*/
   /*-- et librer  nouveau la mmoire EMS alloue                ---*/

   memcpy( vr_adr(), PAGE_ADR(0), 4000 );           /* Recopier V-RAM */
   if( !ems_free(handle) )                         /* Librer mmoire */
     print_err();            /* Erreur : message d'erreur et terminer */
   printf("FIN");
 }
 else                          /* Le driver EMS n'a pu tre identifi */
   printf("ATTENTION ! Pas de mmoire EMS installe\n");
}
