;**********************************************************************;
;*                           H M A C A . A S M                        *;
;*--------------------------------------------------------------------*;
;*    Sujet        : Routine en assembleur  intgrer au programme C  *;
;*                     HMAC.C. Contient une routine d'interrogation   *;
;*                     de la mmoire tendue et de libration/blocage *;
;*                     du canal d'adresses A20.                       *;
;*                     Cette implmentation est destine au modle    *;
;*                     de mmoire SMALL.                              *;
;*--------------------------------------------------------------------*;
;*    Auteur          : MICHAEL TISCHER                               *;
;*    Dvelopp le    : 28.07.1990                                    *;
;*    Dernire m.  j : 28.07.1990                                    *;
;*--------------------------------------------------------------------*;
;*    assembler       : MASM HAMCA;   ou    TASM HMACA;               *;
;*                     ... et ensuite lier au programme C HMAC.C      *;
;*                         compil.                                   *;
;**********************************************************************;

;== Constantes =========================================================

KB_COMMAND equ 64h                               ;Port commandes clavier
KB_STATUS  equ 64h                                ;est aussi port d'tat
KB_DATA    equ 60h                              ;Port donnes du clavier
IB_FREE    equ 2                      ;Masque pour test d'Input Buf vide
WO_COMMAND equ 0D1h               ;Commande d'criture de la Output Port

GATE_ON    equ 11011111b                ;Code de canal adresse A20 libre
GATE_OFF   equ 11011101b               ;Code de canal adresse A20 bloqu

;== Dclarations de segments pour le programme en C ====================

IGROUP group _text                       ;Rsum des segments programmes
DGROUP group _bss, _data                    ;Rsum des segments donnes
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS' ;ce segment rcupre ttes les variables
_BSS   ends                                  ;statiques non initialises

_DATA  segment word public 'DATA'     ;Toutes les variables initialises
                                  ;globales et statiques sont regroupes
                                                        ;dans ce segment

_DATA  ends

;== Programme ==========================================================

_TEXT  segment byte public 'CODE'                  ;le segment programme

public     _HMAAvail
public     _GateA20
public     _IsA20On

;-----------------------------------------------------------------------
;-- HMAAvail : indiquera se le HMA existe
;-- Appel du C: BOOL HMAAvail( void )
;-- Valeur de retour : TRUE, si HMA disponible, sinon FALSE

_HMAAvail  proc near

      ;-- Tester d'abord si AT ou 386 ----------------------------------

      xor    ax,ax                   ;Ces instructions sont recommandes
      push   ax                      ;par Intel pour dtecter le type de
      popf                       ;processeur. Elles prouvent l'existence
      pushf                                      ;d'un 8088 ou d'un 8086
      pop    ax
      and    ax,0F000h
      cmp    ax,0F000h
      je     pashma                            ;PC normal PC ---> pashma

      ;-- AT ou 386, mais avec mmoire tendue? ------------------------

      mov    ah,88h                  ;Taille mmoire tendue par le BIOS
      int    15h                                                       ;
      cmp    ax,64                   ;Plus de 64 Ko en mmoire tendue ?
      jb     pashma                                     ;Non, aucune HMA

      mov    ax,0FFFFh                                    ;Oui, HMA prt
      ret                                        ;alors on retourne TRUE

pashma:
      xor    ax,ax                               ;Retourner 0 pour FALSE
      ret                                         ;retour au programme C

_HMAAvail  endp

;-----------------------------------------------------------------------
;-- GateA20 : Librer ou bloquer le canal d'adresses A20
;-- Appel par C: BOOL GateA20( BOOL libre )
;-- Valeur de retour: TRUE si l'opration est russie, sinon FALSE

_GateA20   proc near

            ;-- Structure pour accder facilement au paramtre ---------

sframe    struc                        ;Structure pour accder  la pile
bp0        dw ?                                               ;stocke BP
ret_adr    dw ?                          ;Adresse de retour  l'appelant
libre      dw ?                               ;bloque ou libre le canal
sframe    ends                                      ;fin de la structure

frame equ [ bp - bp0 ]             ;adresse les lments de la structure

      ;-- Macro demandant l'tat du contrleur du clavier --------------

kbc_ready  macro                              ;Contrleur clavier prt ?
      local  notready

notready:
      in     al,KB_STATUS                         ;Interroge port d'tat
      test   al,IB_FREE                             ;Tampon Input vide ?
      loopne notready           ;Non, et CX encore diffrent de zro -->

           endm

      ;-- Voici le dbut du code de la procdure -----------------------

      push   bp                                   ;Stocke BP sur la pile
      mov    bp,sp                                   ;Copier SP aprs BP

      mov    ah,11011101b                ;Cette valeur = "bloquer canal"
      cmp    frame.libre,0                               ;bloquer canal?
      je     g1                                    ;Oui, le code est OK.

      mov    ah,11011111b         ;Non, charger le code de "canal libre"

g1:   xor    cx,cx                               ;compteur pour Time Out
      cli                              ;aucune Interruption en ce moment
      kbc_ready                             ;attendre contrleur clavier
      jne    gerr                               ;Time Out? oui ---> GERR

      mov    al,WO_COMMAND          ;Envoyer le code d'accs  Output-P.
      out    KB_COMMAND,al                        ;au port d'instruction

      kbc_ready                             ;attendre contrleur clavier
      jne    gerr                               ;Time Out? oui ---> GERR

      mov    al,ah            ;Envoyer instruction de libration/blocage
      out    KB_DATA,al                              ;au port de donnes

      kbc_ready                             ;attendre contrleur clavier
      sti                                          ;Interruption active
      jne    gerr                               ;Time Out? oui ---> GERR

      mov    ax,0FFFFh                    ;Pas de Time Out, tout va bien
      pop    bp                                     ;Lire BP sur la pile
      ret

gerr: xor    ax,ax                              ;aucune bascule possible
      pop    bp                                   ;Relire BP sur la pile
      ret

_GateA20   endp

;-----------------------------------------------------------------------
;-- IsA20On : Indique si le canal d'adresses A20 est libr
;-- Appel par C: BOOL IsA20( void )
;-- Valeur de retour : TRUE, si canal libre sinon FALSE

ramptr     dd 000000000h       ;Pointeur dans la mmoire conventionnelle
extptr     dd 0FFFF0010h               ;Pointeur dans la mmoire tendue

_IsA20On   proc near

      push   ds
      push   es

      lds    si,cs:ramptr    ;Si le canal est bloqu, les deux pointeurs
      les    di,cs:extptr               ;dsignent une adresse identique

      mov    cx,64                                  ;Comparer 128 octets
      cld                 ;Si instruction de chane compter vers le haut
      repe   cmpsw                    ;compare les deux blocs de mmoire

      pop    es                                  ;Rcupre les registres
      pop    ds
      jcxz   a20off                 ;CX = 0 --> Blocs mmoire identiques

      mov    ax,0FFFFh                ;Bloc diffrent ---> A20 est actif
      ret

a20off:
      xor    ax,ax                  ;Blovc identique ---> A20 est bloqu
      ret

_IsA20On   endp

;-----------------------------------------------------------------------

_text      ends                                  ;fin du segment de code
           end                                         ;fin du programme
