/***********************************************************************
*                              L O G O C . C                           *
**--------------------------------------------------------------------**
*  Fonction          : Montre comment dfinir des jeux de caractres   *
*                      personnaliss avec une carte EGA ou VGA en      *
*                      donnant comme exemple une routine reprsentant  *
*                      un logo en mode texte                           *
**--------------------------------------------------------------------**
*  Auteur            : MICHAEL TISCHER                                 *
*  Dvelopp le      : 06.08.1990                                      *
*  Dernire MAJ      : 14.01.1992                                      *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Compilation       : CL /AS /Zp logoc.c logoca                       *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Compilation       : crer un projet avec le contenu suivant         *
*                      logoc.c                                         *
*                      logoca.obj                                      *
**--------------------------------------------------------------------**
*  Appel             : logoc                                           *
***********************************************************************/

#include <dos.h>                                 /* Fichier d'en-tte */
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>

#ifdef __TURBOC__                            /* Travail avec Turbo C? */
  #define CLI()           disable()
  #define STI()           enable()
  #define outpw( p, w )   outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                  /* Non c'est QuickC 2.0 ou MSC */
  #include <conio.h>
  #define MK_FP(seg,ofs)  ((void far *)\
                          (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()           _disable()
  #define STI()           _enable()
#endif

#define EGA       0                                /* Types de cartes */
#define VGA       1
#define NINI      2
#define MAX_CHAR  32         /* maximum de caractres redfinissables */

#define EGAVGA_SEQUENCER 0x3C4    /* Port adresses/donnes squenceur */
#define EGAVGA_MONCTR    0x3D4             /* Adr. contrleur d'cran */
#define EGAVGA_GRAPHCTR  0x3CE     /*Port adresses/donnes ctrl graph */

/*-- Dclarations de types -------------------------------------------*/

typedef unsigned char     BYTE;

                                             /* Routine en assembleur */
extern void      defchar( BYTE ascii, BYTE table, BYTE height,
              BYTE nombre, void far * buf );

/***********************************************************************
*  SetCursor : Positionne le curseur clignotant                        *
**--------------------------------------------------------------------**
*  Entre    : COLONNE = nouvelle colonne du curseur (0-79)            *
*              LIGNE   = nouvelle ligne du curseur (0-24)              *
*  Sortie    : nant                                                   *
***********************************************************************/

void SetCursor( BYTE colonne, BYTE ligne )
{
  union REGS  regs;                       /* Registres d'interruption */

  regs.h.ah = 2;                 /* Numro de la fonction "SetCursor" */
  regs.h.bh = 0;                                    /* Page d'cran 0 */
  regs.h.dh = ligne;                              /* Indique la ligne */
  regs.h.dl = colonne;                               /* et la colonne */
  int86(0x10, &regs, &regs);          /* Appelle l'interruption vido */
}

/***********************************************************************
*  PrintfAt : Affiche une chane en n'importe quel point de l'cran    *
**--------------------------------------------------------------------**
*  Entres  : COLONNE = Position d'affichage                           *
*             LIGNE                                                    *
*             COULEUR  = attribut des caractres                       *
*             STRING = Pointe sur la chane                            *
*  Sortie   : nant                                                    *
*  Info     : Cette fonction en doit tre appele qu'aprs vrification*
*             pralable de la prsence d'une carte EGA ou VGA          *
***********************************************************************/

void PrintfAt(BYTE colonne, BYTE ligne, BYTE couleur, char *string,... )
{
  va_list     parameter;        /* Liste de paramtres pour macro _VA */
  char        affichage[255],        /* buffer pour la chane formate*/
              *affptr;
  BYTE far    *vptr;                 /* Pointe sur la mmoire d'cran */

  va_start( parameter, string );         /* Conversion des paramtres */
  vsprintf( affichage, string, parameter );              /* Formatage */

  vptr = (BYTE far *) MK_FP( 0xB800, colonne * 2 + ligne * 160 );

  for( affptr = affichage; *affptr ; )          /* Parcourt la chaine */
  {
    *vptr++ = *(affptr++);      /* Caractres dans la mmoire d'cran */
    *vptr++ = couleur;                    /* et de mme les attributs */
  }
}

/***********************************************************************
*  ClrScr : Efface l'cran                                             *
**--------------------------------------------------------------------**
*  Entre : COULEUR  = attribut des caractres                         *
*  Sortie : nant                                                      *
***********************************************************************/

void ClrScr( BYTE couleur )
{
  BYTE far  *vptr;                   /* Pointe sur la mmoire d'cran */
  int       count = 2000;            /* Nombre de caractres  effacer*/

  vptr = (BYTE far *) MK_FP( 0xB800, 0 ); /* Ptr sur la mmoire cran */

  for( ; count--; )                    /* Parcourt la mmoire d'cran */
  {
    *vptr++ = ' ';              /* Ecrit le caractre et son attribut */
    *vptr++ = couleur;                     /* dans la mmoire d'cran */
  }
}

/***********************************************************************
*  SetCharWidth : Fixe la largeur des caractres pour cartes VGA       *
*                  8 ou 9 pixels                                      *
**--------------------------------------------------------------------**
*  Entre       : Largeur = largeur du caractre (8 ou 9)              *
***********************************************************************/

void SetCharWidth( BYTE largeur )
{
  union REGS      Regs;        /* Registres pour grer l'interruption */
  unsigned char   x;                           /* Variable de travail */

  Regs.x.bx = ( largeur == 8 ) ? 0x0001 : 0x0800;

  x = inp( 0x3CC ) & (255-12);     /* Passe de la rsolution de 720  */
  if( largeur == 9 )                      /* 640 pixels ou vice-versa */
    x |= 4;
  (void) outp( 0x3C2, x);

  CLI();                    /* Programme le squenceur en consquence */
  outpw( EGAVGA_SEQUENCER, 0x0100 );
  outpw( EGAVGA_SEQUENCER, 0x01 + ( Regs.h.bl << 8 ) );
  outpw( EGAVGA_SEQUENCER, 0x0300 );
  STI();

  Regs.x.ax = 0x1000;               /* Ajuste l'cran horizontalement */
  Regs.h.bl = 0x13;
  int86( 0x10, &Regs, &Regs );
}

/***********************************************************************
*  IsEgaVga : teste la prsence d'une carte EGA ou VGA                 *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
*  Sortie : l'une des constantes EGA_MONO, EGA_COLOR etc.              *
***********************************************************************/

BYTE IsEgaVga( void )
{
  union REGS      Regs;              /* Registres pour l'interruption */

  Regs.x.ax = 0x1a00;        /* La fonction 1Ah n'existe que pour VGA */
  int86( 0x10, &Regs, &Regs );
  if( Regs.h.al == 0x1a )        /* La fonction est-elle disponible ? */
    return VGA;
  else                                  /* Non, serait-ce  une EGA  ? */
  {
    Regs.h.ah = 0x12;                         /* Appelle l'option 10h */
    Regs.h.bl = 0x10;                           /* de la fonction 12h */
    int86( 0x10, &Regs, &Regs );                /* Interruption vido */
    return ( Regs.h.bl != 0x10 ) ? EGA : NINI;
  }
}

/***********************************************************************
*  BuildLogo : Dessine un logo compos de diffrents caractres        *
*              qui sont peu utiliss franais                          *
*                                                                      *
**--------------------------------------------------------------------**
*  Entre    : COLONNE    = Colonne o dbute le logo (1-80)           *
*              LIGNE      = Ligne o dbute le logo (1-25)             *
*              PROFONDEUR = Nombre de lignes de trame du logo          *
*              COULEUR    = Couleur d'affichage du logo                *
*              BUFP       = pointeur sur une tableau de pointeurs qui  *
*                           rfrencent des chanes de caractres      *
*                           reprsentant le motif du logo              *
*  Info      : - La fonction test montre comment raliser le buffer   *
*                transmettre                                           *
*              - Le logo est centr dans son bloc de caractres        *
***********************************************************************/

void BuildLogo( BYTE colonne, BYTE ligne, BYTE profondeur,
                BYTE couleur, char **bufp )
{
  static BYTE     UseChars[MAX_CHAR] =        /* caractres redfinis */
          {
            128, 129, 132, 134, 141, 142, 143, 144, 146, 149,
            152, 153, 154, 155, 156, 157, 158, 159, 160, 161,
            162, 163, 164, 165, 166, 167, 168, 171, 172, 173,
            174, 175
            };

 BYTE       cvideo;                            /* Type de carte vido */
 BYTE       chardef[16],              /* Motif binaire d'un caractre */
            charhauteur,   /* Nombre de lignes de trame par caractre */
            i, j, k, l,              /* Variables pour les itrations */
            masque,                 /* Masque pour une ligne de trame */
            largeur,                          /* Largeur de la chane */
            index,       /* Indice pour parcourir le tableau UseChars */
            dx,        /* Largeur du bloc du Logo (colonnes de texte) */
            dy,       /* Profondeur du bloc du logo (lignes de texte) */
            colonnec,                             /* colonne courante */
        lignec,                                     /* ligne courante */
            gauche,                         /* Marge gauche en pixels */
            droite,                         /* Marge droite en pixels */
            haut,                              /* Marge sup en pixels */
            bas;                               /* Marge inf en pixels */

  cvideo = IsEgaVga();                        /* Quelle carte vido ? */
  switch( cvideo )
  {
    case NINI :
      printf( "Erreur : Pas de carte EGA ou VGA installe !\n" );
      return;

    case EGA  :
      charhauteur = 14;      /* EGA: 14 lignes de trame par caractre */
      break;

    case VGA  :
      SetCharWidth( 8 );                /* 8 pixels de largeur en VGA */
      charhauteur = 16;           /* 16 lignes de trame par caractre */
      break;
  }

  largeur = strlen( *bufp );                       /* largeur du logo */
  dx = ( largeur + 7 ) / 8;                   /* Nombre de caractres */
  dy = ( profondeur + charhauteur - 1 ) / charhauteur;
  if( dx*dy > MAX_CHAR )
    printf( "Erreur : Logo trop grand dans BuildLogo !\n" );
  else
  {
    haut   = ( dy*charhauteur-profondeur ) / 2;             /* Marges */
    bas    = profondeur + haut - 1;
    gauche = ( dx*8-largeur ) / 2;
    droite = largeur + gauche - 1;

    for( index = 0, i = 0; i < dy; ++ i)
    {                                        /* Parcourt lignes texte */
      for(j = 0; j < dx; ++j, ++index )    /* Parcourt colonnes texte */
      {
        PrintfAt( colonne+j, ligne+i, couleur,  /* Affiche caractres */
                  "%c", UseChars[ index ] );

 /*-- Calcule le nouveau motif pour le caractre ---------------------*/

        for( k = 0; k < charhauteur; ++ k )  /* Parcourt lignes trame */
        {
      masque = 0;                        /* Pour l'instant masque nul */
          for( l = 0; l <= 7; ++l )            /* 8 pixels de largeur */
      {
            masque <<= 1;          /* Dcale le masque vers la gauche */
            lignec = i * charhauteur + k;
            colonnec = j * 8 + l;

            if( lignec>=haut     && lignec<=bas  &&  /* Pixel au-del */
                colonnec>=gauche && colonnec<=droite)/* de la marge ? */
              if( *(*(bufp+lignec-haut)+colonnec-gauche) != ' ' )
                masque |= 1;              /* Dessine un point du logo */
          }
          chardef[ k ] = masque;   /* motif dans le buffer caractres */
        }
    defchar( UseChars[ index ], 0, charhauteur, 1, chardef );
      }
    }
  }
}

/***********************************************************************
*  ResetLogo : recharge le jeu de caractres d'origine                 *
**--------------------------------------------------------------------**
*  Entre    : nant                                                   *
***********************************************************************/

void ResetLogo( void )
{
  union REGS    Regs;          /* Registres pour grer l'interruption */

  switch( IsEgaVga() )
  {
    case EGA :
      Regs.x.ax = 0x1101;             /* Charge  nouveau le jeu 8*14 */
      Regs.h.bl = 0;
      int86( 0x10, &Regs, &Regs );
      break;

    case VGA :
      SetCharWidth( 9 );    /* Reprsente les caractres sur 9 pixels */
      Regs.x.ax = 0x1104;                     /* recharge le jeu 8*16 */
      Regs.h.bl = 0;
      int86( 0x10, &Regs, &Regs );
   }
}

/***********************************************************************
*  Test   : exemple de fonctionnement de la procdure BuildLogo        *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************/

void Test( void )
{
  static char *MyLogo[32] =
              { "                  **                  ",
                "                 ****                 ",
                "                 ****                 ",
                "                  **                  ",
                "                                      ",
                "                                      ",
                "                                      ",
                "                 ****                 ",
                "                 ****                 ",
                "**************************************",
                "**************************************",
                "***              ****              ***",
                "**               ****               **",
                "*                ****                *",
                "                 ****                 ",
                "                 ****                 ",
                "      ********   ****   ********      ",
                "      ****  **** **** ****  ****      ",
                "      ****     ********     ****      ",
                "      ****      ******      ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      ",
                "      ****       ****       ****      " };

  int         i, j;
  BYTE        couleur;

  static BYTE NouvDef[MAX_CHAR] =             /* Caractres redfinis */
            {
            128, 129, 132, 134, 141, 142, 143, 144, 146, 149,
            152, 153, 154, 155, 156, 157, 158, 159, 160, 161,
            162, 163, 164, 165, 166, 167, 168, 171, 172, 173,
            174, 175
            };

  ClrScr( 0 );
  for( i = 0; i < 256; ++i ) /* Affiche le jeu complet des caractres */
  {
    for( j = 0; j < MAX_CHAR; ++ j )        /* nouvelle dfinition .? */
      if( NouvDef[ j ] == i )     /* Le caractre en fait-il partie ? */
        break;                            /* Oui, sortir de la boucle */
    couleur = ( j < MAX_CHAR ) ? 15 : 14;

    PrintfAt( (i % 13) * 6 + 1, i / 13, couleur, "%3d:%c", i, i );
  }

  PrintfAt( 18, 22, 14, "LOGOC  -  (c) 1990, 92 by MICHAEL TISCHER" );
  BuildLogo( 60, 21, 30, 0x3F, MyLogo );          /* dessine le logo  */
  getch();
  ResetLogo();                                      /* Efface le logo */
  ClrScr( 15 );
  SetCursor( 0, 0 );
}

/***********************************************************************
*                               PROGRAMME PRINCIPAL                    *
***********************************************************************/

void main( void )
{
  Test();
}

