/***********************************************************************
*                          M I K A D O C . C                           *
**--------------------------------------------------------------------**
*  Fonction         : Montre comment mettre en service le mode 512     *
*                     caractres des cartes EGA et VGA.                *
*                     La routine de dmonstration installe une fentre *
*                     graphique en mode texte                          *
**--------------------------------------------------------------------**
*  Auteur           : MICHAEL TISCHER                                  *
*  Dvelopp le     :  2.04.1990                                       *
*  Dernire MAJ     : 14.02.1992                                       *
***********************************************************************/

/*-- Constantes et fichiers d'en-tte --------------------------------*/

#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#ifdef __TURBOC__                         /* Compilation par Turbo C? */
  #define CLI()           disable()
  #define STI()           enable()
  #define outpw( p, w )   outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                   /* Non, par QuickC 2.0 ou MSC */
  #include <conio.h>
  #define random(x)      (rand() % ( x + 1 ))
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

#define EGAVGA_SEQUENCER 0x3C4         /* Port adr/data du squenceur */
#define EGAVGA_MONCTR    0x3D4                    /* Contrleur vido */
#define EGAVGA_GRAPHCTR  0x3CE       /* Ports adr/data du ctrlr graph */
#define CHAR_LARGEUR     8
#define CHAR_BYTES       32
#define MIKADOS          5          /* Mikados visibles simultanment */

#define TRUE             (0 == 0)
#define FALSE            (0 == 1)

#define BLACK            0x00                 /* Attributs de couleur */
#define BLUE             0x01
#define GREEN            0x02
#define CYAN             0x03
#define RED              0x04
#define MAGENTA          0x05
#define BROWN            0x06
#define LIGHTGREY        0x07
#define GREY             0x01
#define LIGHTBLUE        0x09
#define LIGHTGREEN       0x0A
#define LIGHTCYAN        0x0B
#define LIGHTRED         0x0C
#define LIGHTMAGENTA     0x0D
#define YELLOW           0x0E
#define WHITE            0x0F

/*-- Dclarations de types  ------------------------------------------*/

typedef unsigned char    BYTE;
typedef BYTE             BOOL;

typedef BYTE             PALARY[16];  /* Jeu de registres de palettes */

/*-- Variables globales ----------------------------------------------*/

BYTE far *vioptr = (BYTE far *)0xB8000000,       /* Mmoire graphique */
     far *fontptr;                  /* Pointe sur la police graphique */

BYTE CharHauteur,
     lenx;           /* Largeur en caractres de la fentre graphique */
int  xmax,       /* Coordonnes max en pixels de la fentre graphique */
     ymax;

/***********************************************************************
*  IsEgaVga : Teste la prsence d'une carte EGA ou VGA.                *
**--------------------------------------------------------------------**
*  Entre   : nant                                                    *
*  Sortie   : EGA, VGA ou NINI                                         *
***********************************************************************/

BYTE IsEgaVga( void )
{
  union REGS Regs;          /* Registres pour grer les interruptions */

  Regs.x.ax = 0x1a00;           /* La fonction 1Ah n'existe qu'en VGA */
  int86( 0x10, &Regs, &Regs );
  if( Regs.h.al == 0x1a )     /* Cette fonction est-elle disponible ? */
  {                                        /* Oui, on a une carte VGA */
    CharHauteur = 16;                /* Hauteur des caractres en VGA */
    return 1;
  }
  else
  {
    CharHauteur = 14;                /* Hauteur des caractres en EGA */
    Regs.h.ah = 0x12;                   /* Appelle l'option 10h de la */
    Regs.h.bl = 0x10;                                 /* fonction 12h */
    int86(0x10, &Regs, &Regs );     /* Dclenche l'interruption vido */
    return Regs.h.bl != 0x10;
  }
}

/***********************************************************************
*  SetCursor   : Positionne le curseur clignotant                      *
**--------------------------------------------------------------------**
*  Entres     : COLONNE = nouvelle colonne du curseur (0-79)          *
*                LIGNE = nouvelle ligne du curseur (0-24)              *
*  Sortie      : nant                                                 *
***********************************************************************/

void SetCursor( BYTE colonne, BYTE ligne )
{
  union REGS  regs;        /*  Registres pour grer les interruptions */

  regs.h.ah = 2;                /* Numro de la fonction "Set Cursor" */
  regs.h.bh = 0;                                /* Accde  la page 0 */
  regs.h.dh = ligne;                                 /* Fixe la ligne */
  regs.h.dl = colonne;                               /* et la colonne */
  int86(0x10, &regs, &regs);        /* Dclenche l'interruption vido */
}

/***********************************************************************
*  PrintfAt : Affiche une chane formate en n'importe quel point de   *
*             l'cran                                                  *
*                                                                      *
**--------------------------------------------------------------------**
*  Entres  :    COLONNE = Position d'affichage                        *
*                LIGNE                                                 *
*                COULEUR = attribut des caractres                     *
*                STRING  = Pointe sur la chane                        *
*  Sortie   : nant                                                    *
*  Info     : Cette fonction ne doit tre invoque qu'aprs            *
*             vrification pralable de la prsence d'une              *
*             carte EGA ou VGA                                         *
***********************************************************************/

void PrintfAt( BYTE colonne, BYTE ligne, BYTE couleur, char *string,...)
{
  va_list    parametre;         /* Liste de paramtres pour macro _VA */
  char       Affichage[255],        /* Buffer pour la chane formate */
             *affptr;
  BYTE far   *vptr;                /* Pointeur sur la mmoire d'cran */

  va_start( parametre, string );          /*Conversion des paramtres */
  vsprintf( Affichage, string, parametre );              /* Formatage */

  vptr = (BYTE far *) MK_FP( 0xB800, colonne * 2 + ligne * 160 );

  for( affptr = Affichage; *affptr ; )          /* Parcourt la chane */
  {
    *vptr++ = *(affptr++);           /* Ecrit dans la mmoire d'cran */
    *vptr++ = couleur;                    /* et de mme les attributs */
  }
}

/***********************************************************************
*  ClrScr : Efface l'cran                                             *
**--------------------------------------------------------------------**
*  Entre : COULEUR  = attribut des caractres                         *
*  Sortie : nant                                                      *
***********************************************************************/

void ClrScr( BYTE couleur )
{
  BYTE far  *vptr;                   /* Pointe sur la mmoire d'cran */
  int       count = 2000;           /* Nombre de caractres  effacer */

  vptr = (BYTE far *) MK_FP( 0xB800, 0 );         /* Fixe le pointeur */

  for( ; count--; )                    /* Parcourt la mmoire d'cran */
  {
    *vptr++ = ' ';         /* Ecrit les caractres en mmoire d'cran */
    *vptr++ = couleur;                    /* et de mme les attributs */
  }
}

/***********************************************************************
*  SetCharWidth : Fixe la largeur des caractres des cartes VGA        *
*                  8 ou 9 pixels                                      *
**--------------------------------------------------------------------**
*  Entre : Largeur = Largeur des caractres (8 ou 9)                  *
***********************************************************************/

void SetCharWidth( BYTE largeur )
{
  union REGS Regs;          /* Registres pour grer les interruptions */
  unsigned char x;                             /* Variable de travail */
  Regs.x.bx = ( largeur == 8 ) ? 0x0001 : 0x0800;

  x = inp( 0x3CC ) & (255-12);     /* Passe de la rsolution de 720  */
  if ( largeur == 9 )                     /* 640 pixels ou vice-versa */
    x |= 4;
  (void)outp( 0x3C2, x);

  CLI();                    /* Programme le squenceur en consquence */
  outpw( EGAVGA_SEQUENCER, 0x0100 );
  outpw( EGAVGA_SEQUENCER, 0x01 + ( Regs.h.bl << 8 ) );
  outpw( EGAVGA_SEQUENCER, 0x0300 );
  STI();

  Regs.x.ax = 0x1000;               /* Ajuste l'cran horizontalement */
  Regs.h.bl = 0x13;
  int86( 0x10, &Regs, &Regs );
}

/***********************************************************************
*  SelectMaps : Slectionne les jeux de caractres accessibles par     *
*               le bit 3 de l'attribut de caractre                    *
*---------------------------------------------------------------------**
*  Entre : MAP0 = Numro du premier jeu de caractres  (Bit 3 = 0 )   *
*           MAP1 = Numro du deuxime jeu de caractres (Bit 3 = 1 )   *
*  Info   : - avec une carte EGA on peut choisir les jeux 0  3,       *
*           - avec une carte VGA les jeux 0  7                        *
***********************************************************************/

void SelectMaps( BYTE map0, BYTE map1)
{
  union REGS   Regs;       /*  Registres pour grer les interruptions */

  Regs.x.ax = 0x1103;         /* Registre de slection des caractres */
  Regs.h.bl = ( map0 & 3 ) + ( ( map0 & 4 ) << 2 ) +
              ( ( map1 & 3 ) << 2 ) + ( ( map1 & 4 ) << 3 );
  int86( 0x10, &Regs, &Regs );    /* Dclenche l'interruption du BIOS */
}

/***********************************************************************
*  GetFontAccess : Permet d'accder directement  la deuxime zone     *
*                  de mmoire o sont stocks les jeux de caractres   *
*                  par l'adresse A000:0000                             *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
*  Info   : A l'issue de cette procdure il n'est plus possible        *
*           d'accder  la mmoire d'cran par B800:0000               *
***********************************************************************/

void GetFontAccess( void )
{
  static unsigned  SeqRegs[4] = { 0x0100, 0x0402, 0x0704, 0x0300 },
                   GCRegs[3]  = { 0x0204, 0x0005, 0x0006 };
  BYTE             i;                        /* Compteur d'itrations */

  CLI();                             /* Pas d'interruption maintenant */

  for ( i=0; i<4; ++i )         /* Charge les registres du squenceur */
    outpw( EGAVGA_SEQUENCER, SeqRegs[ i ] );

  for ( i=0; i<3; ++i )        /* Charge les registres du ctrlr graph */
    outpw( EGAVGA_GRAPHCTR, GCRegs[ i ] );

  STI();                                /* rtablit les interruptions */
}

/***********************************************************************
*  ReleaseFontAccess : Rtablit l'accs  la mmoire d'cran par       *
*                      B800:0000 mais empche en mme temps l'accs    *
*                      aux jeux de caractres                          *
*                      situs en page mmoire N 2.                    *
**--------------------------------------------------------------------**
*  Entre            : nant                                           *
***********************************************************************/

void ReleaseFontAccess( void )
{
  static unsigned    SeqRegs[4] = { 0x0100, 0x0302, 0x0304, 0x0300 },
                     GCRegs[3]  = { 0x0004, 0x1005, 0x0E06 };
  BYTE               i;                      /* Compteur d'itrations */

  CLI();                            /* Pas d'interruptions maintenant */

  for( i=0; i<4; ++i )          /* Charge les registres du squenceur */
    outpw( EGAVGA_SEQUENCER, SeqRegs[ i ] );

  for ( i=0; i<3; ++i )         /* Charge les registres du Ctrl Graph */
    outpw( EGAVGA_GRAPHCTR, GCRegs[ i ] );

  STI();                                 /*Rtablit les interruptions */
}

/***********************************************************************
*  ClearGraphArea : Efface la zone graphique en y mettant  0 les      *
*                   motifs des caractres.                             *
**--------------------------------------------------------------------**
*  Entre         : nant                                              *
***********************************************************************/

void ClearGraphArea( void )
{
  int  caract,                              /* Caractres  parcourir */
       ligne;             /* Ligne  l'intrieur de chaque caractre  */

  for( caract = 0; caract < 256; ++caract )             /* Caractres */
    for( ligne = 0; ligne < CharHauteur; ++ ligne )         /* Lignes */
      *(fontptr + caract * CHAR_BYTES + ligne) = 0;       /* Mise  0 */
}

/***********************************************************************
*  InitGraphArea  : Prpare une zone d'cran  recevoir un affichage   *
*                  graphique                                           *
**--------------------------------------------------------------------**
*  Entre         : X       = Colonne o dbute la zone (1-80)         *
*                   Y       = Ligne o dbute la zone   (1-25)         *
*                   XLEN    = Largeur de la zone en caractres         *
*                   YLEN    = Hauteur de la zone en caractres         *
*                   MAP     = Numro du jeu de caractres graphiques   *
*                   COULEUR = Couleur de la zone graphique             *
*                             (0  7 ou 0xFF)                          *
*  Info    : - Si la couleur indique est 0xFF, elle est variable      *
*              ce qui donne un effet de "mikado"                       *
*                                                                      *
***********************************************************************/

void InitGraphArea( BYTE x, BYTE y, BYTE xlen, BYTE ylen, BYTE map,
                    BYTE couleur )
{
  unsigned   offset;                      /* Offset en mmoir d'cran */
  int        colonne, ligne;                 /* Compteur d'itrations */
  BYTE       codec;                              /* Code de caractre */

  if( xlen * ylen > 256 )                       /* Zone trop grande ? */
    printf( "Erreur : La zone graphique ne doit pas englober"\
            " plus de 256   caractres !\n" );
  else
  {
    if( CharHauteur == 16 )                                   /* VGA? */
      SetCharWidth( 8 );     /* Oui, change la largeur des caractres */
    SelectMaps( 0, map );         /* Slectionne le jeu de caractres */
    xmax = xlen*CHAR_LARGEUR;            /* Coordonnes max en pixels */
    ymax = ylen*CharHauteur;
    lenx = xlen;
    fontptr = MK_FP( 0xA000, map * 0x4000 );     /* Mmoire graphique */
    GetFontAccess();       /* Autorise l'accs aux jeux de caractres */
    ClearGraphArea();                               /* Efface la zone */
    ReleaseFontAccess();     /* Rtablit l'accs  la mmoire d'cran */

    /*-- remplit la zone graphique avec des caractres ---------------*/

    codec = 0;
    for( ligne = ylen-1; ligne >= 0; --ligne )              /* Lignes */
      for ( colonne = 0; colonne < xlen; ++colonne )      /* Colonnes */
      {                    /* Fixe le code et l'attribut du caractre */
        offset = ((ligne+y) * 80 + colonne+x) << 1; /* Ofst mm cran */
        *(vioptr+offset) = codec;               /* Ecrit le caractre */
        *(vioptr+offset+1) = ( couleur == 0xff )
                                      ? ( codec % 6 ) + 1 + 8
                                      : couleur | 0x08;
        ++codec;                        /* Passe au caractre suivant */
      }
  }
}

/***********************************************************************
*  CloseGraphArea : Clture l'accs  la zone graphique                *
**--------------------------------------------------------------------**
*  Entre         : nant                                              *
***********************************************************************/

void CloseGraphArea( void )
{
  ReleaseFontAccess();         /* Libre l'accs  la mmoire d'cran */
  SelectMaps( 0, 0 );               /* Choisit le jeu de caractres 0 */
  if( CharHauteur == 16 )                                     /* VGA? */
    SetCharWidth( 9 );         /* Oui, fixe la largeur des caractres */
}

/***********************************************************************
*  SetPixel: Dessine ou efface un pixel dans la fentre graphique      *
**--------------------------------------------------------------------**
*  Entres : X,Y   = Coordonnes du pixel  (0-...)                     *
*            ON    = TRUE pour dessiner et FALSE pour effacer          *
***********************************************************************/

void SetPixel( int x, int y, BOOL on )
{
  BYTE charnum,                                /* Numro du caractre */
       linenr,                    /* Ligne  l'intrieur du caractre */
       far *bptr;

  if( ( x < xmax ) && ( y < ymax ) )             /* Coordonnes o.k.? */
  {                 /* Oui calcule le numro du caractre et la ligne */
    charnum = ((x / CHAR_LARGEUR) + (y / CharHauteur * lenx));
    linenr  = CharHauteur - ( y % CharHauteur ) - 1;
    bptr = fontptr + charnum * CHAR_BYTES + linenr;
    if( on )                                 /* Dessiner ou effacer ? */
      *bptr= *bptr | ( 1 << (CHAR_LARGEUR - 1 - ( x % CHAR_LARGEUR )));
    else
      *bptr= *bptr & !( 1 << (CHAR_LARGEUR - 1 - ( x % CHAR_LARGEUR )));
  }
}

/***********************************************************************
*  Line : Trace un segment dans la fentre graphique en appliquant     *
*         l'algorithme de Bresenham                                    *
**--------------------------------------------------------------------**
*  Entres : X1, Y1 = Coordonnes de l'origine                         *
*            X2, Y2 = Coordonnes de l'extrmit terminale             *
*            ON     = TRUE pour dessiner et FALSE pour effacer         *
***********************************************************************/

/*-- Fonction accessoire pour changer deux variables entires -------*/

void SwapInt( int *i1, int *i2 )
{
  int   dummy;

  dummy = *i2;
  *i2   = *i1;
  *i1   = dummy;
}

/*-- Procdure principale --------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BOOL on )
{
  int d, dx, dy,
      aincr, bincr,
      xincr, yincr,
      x, y;

  if( abs(x2-x1) < abs(y2-y1) )    /* Sens du parcours : axe X ou Y ? */
  {                                                          /* Par Y */
    if( y1 > y2 )                            /* y1 plus grand que y2? */
    {
      SwapInt( &x1, &x2 );                   /* Oui change X1 et X2, */
      SwapInt( &y1, &y2 );                                /* Y1 et Y2 */
    }

    xincr = ( x2 > x1 ) ?  1 : -1;          /* Fixe le pas horizontal */

    dy = y2 - y1;
    dx = abs( x2-x1 );
    d  = 2 * dx - dy;
    aincr = 2 * (dx - dy);
    bincr = 2 * dx;
    x = x1;
    y = y1;

    SetPixel( x, y, on );                 /* dessine le premier pixel */
    for( y=y1+1; y<= y2; ++y )                /* Parcourt l'axe des Y */
    {
      if( d >= 0 )
      {
        x += xincr;
        d += aincr;
      }
      else
        d += bincr;
      SetPixel( x, y, on );
    }
  }
  else                                                       /* par X */
  {
    if( x1 > x2 )                            /* x1 plus grand que x2? */
    {
      SwapInt( &x1, &x2 );                   /* Oui, change X1 et X2 */
      SwapInt( &y1, &y2 );                                /* Y1 et Y2 */
    }

    yincr = ( y2 > y1 ) ? 1 : -1;             /* Fixe le pas vertical */

    dx = x2 - x1;
    dy = abs( y2-y1 );
    d  = 2 * dy - dx;
    aincr = 2 * (dy - dx);
    bincr = 2 * dy;
    x = x1;
    y = y1;

    SetPixel( x, y, on );                 /* Dessine le premier pixel */
    for( x=x1+1; x<=x2; ++x )                 /* Parcourt l'axe des X */
    {
      if( d >= 0 )
      {
        y += yincr;
        d += aincr;
      }
      else
        d += bincr;
      SetPixel( x, y, on );
    }
  }
}

/***********************************************************************
*  SetPalCol : Dfinit une couleur dans une des 16 palettes            *
*              ou la couleur du cadre d'cran (Overscan-Color)         *
**--------------------------------------------------------------------**
*  Entre    : RegNr = Numro du registre de palette (0  15) ou 16    *
*                    pour la couleur du cadre d'cran                  *
*              Col   = Code de la couleur de 0  15                    *
***********************************************************************/

void SetPalCol( BYTE RegNr, BYTE Col )
{
  union REGS   Regs;       /*  Registres pour grer les interruptions */

  Regs.x.ax = 0x1000;          /* Option 00h de la fonction vido 10h */
  Regs.h.bh = Col;                                    /* Code couleur */
  Regs.h.bl = RegNr;   /* Numro du registre du contrleur d'attribut */
  int86( 0x10, &Regs, &Regs );      /* Dclenche l'interruption vido */
}

/***********************************************************************
*  SetPalAry : installe une nouvelle palette de 16 couleurs sans       *
*              changer la couleur du cadre d'cran.                    *
**--------------------------------------------------------------------**
*  Entre    : NewColPtr = Pointeur sur une palettes de type PALARY    *
***********************************************************************/

void SetPalAry( BYTE *NewColPtr )
{
  BYTE   i;                                  /* Compteur d'itrations */

  for( i = 0; i < 16; ++i )   /* Parcourt les 16 lments de la table */
    SetPalCol( i, NewColPtr[i] );   /* Fixe une couleur  chaque fois */
}

/***********************************************************************
*  GetPalCol: Lit le contenu d'un registre de palette                  *
**--------------------------------------------------------------------**
*  Entre : RegNr = Numro du registre de Palette (0  15) ou 16       *
*                    pour la couleur du cadre d'cran                  *
*  Sortie : Code de couleur                                            *
*  Info   : Avec EGA il n'est pas possible de lire le contenu          *
*           des registres de palette. On suppose que dans ce cas       *
*           les registres de palette sont en disposition standard      *
*           et la fonction retourne le numro correspondant.           *
***********************************************************************/

BYTE GetPalCol( BYTE RegNr )
{
 union REGS Regs;          /*  Registres pour grer les interruptions */

 if ( CharHauteur == 14 )                               /* Carte EGA? */
  return RegNr;   /*Oui, impossible de lire les registres de palettes */
 else                                                     /* Non, VGA */
 {
   Regs.x.ax = 0x1007;         /* Option 07h de la fonction vido 10h */
   Regs.h.bl = RegNr;  /* Numro du registre du contrleur d'attribut */
   int86( 0x10, &Regs, &Regs );         /* Interruption vido du BIOS */
   return Regs.h.bh;                /* Contenu du registre de palette */
  }
}

/***********************************************************************
*  GetPalAry: Lit les contenus des 16 registres de palette et les      *
*             enregistre dans une table         .                      *
**--------------------------------------------------------------------**
*  Entre : ColAryPtr = Pointeur sur une table de palettes du type     *
*                       PALARY, qui va recevoir les codes des couleurs *
***********************************************************************/

void GetPalAry( BYTE *ColAryPtr )
{
 BYTE i;                                     /* Compteur d'itrations */

 for (i = 0; i < 16; ++i )     /* parcourt les 16 lments du tableau */
  ColAryPtr[i] = GetPalCol( i );     /* lit une couleur  chaque fois */
}

/***********************************************************************
*  Mikado: Dmonstration du maniement des routines prsentes          *
*          dans ce programme                                           *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************/

void Mikado( void )
{
 typedef struct {                         /* Coordonnes d'un segment */
                 int x1, y1,
                     x2, y2;
                } LIGNE;

static PALARY NewCols =
     {             /*-- Couleurs des caractres de texte ordinaires --*/
      BLACK,                                                  /* noir */
      BLUE,                                                   /* bleu */
      GREEN,                                                  /* vert */
      RED,                                                   /* rouge */
      CYAN,                                                   /* cyan */
      MAGENTA,                                             /* magenta */
      YELLOW,                                                /* jaune */
      WHITE,                                                 /* blanc */
                   /*---------Couleurs des graphiques ----------------*/
      LIGHTBLUE,                                        /* bleu clair */
      LIGHTGREEN,                                       /* vert clair */
      LIGHTRED,                                        /* rouge clair */
          LIGHTCYAN,                                    /* cyan clair */
          LIGHTMAGENTA,                              /* magenta clair */
          BLUE,                                               /* bleu */
          YELLOW,                                            /* jaune */
          WHITE };                                           /* blanc */

 int    i, j,                                /* Compteur d'itrations */
        first,                     /* Indice du mikado le plus rcent */
        last;                      /* Indice du mikado le plus ancien */
 BOOL   clear;                            /* Pour effacer les mikados */
 LIGNE  lar[MIKADOS];                            /* Table des mikados */
 PALARY OldCols;                      /* Table des anciennes couleurs */

 GetPalAry( OldCols );            /* Dtermine les couleurs prsentes */
 SetPalAry( NewCols );                /* Installe une nouvelle palette*/
              /*TextColor( 7 );
 TextBackGround( 1 );
 GotoXY(1,1); */
 ClrScr( 0x07 );                                    /* Efface l'cran */
 for (i=0; i<25; ++i )   /* puis le remplit avec un jeu de caractres */
  for (j=0; j<80; ++j )
   PrintfAt( j, i, 0x07, "%c", 32 + (((int) i*80+j) % 224) );

 /*-- Initialise la zone graphique et fait tomber les mikados---------*/

 PrintfAt( 27,6, 0x70, "       M I K A D O       " );
 SetCursor(27,6);
 InitGraphArea( 27, 7, 25, 10, 1, 0xFF );
 GetFontAccess();            /* Assure l'accs aux jeux de caractres */

 clear = FALSE;                           /* Pour effacer les mikados */
 first = 0;                          /* Commence au dbut de la table */
 last = 0;
 do
  {                                              /* Boucle des mikados*/
   if (first == MIKADOS )                             /* Wrap-Around? */
    first = 0;
   lar[first].x1 = random( xmax-1 );                /* Cre un mikado */
   lar[first].x2 = random( xmax-1 );
   lar[first].y1 = random( ymax-1 );
   lar[first].y2 = random( ymax-1 );
   Line( lar[first].x1, lar[first].y1,               /* et le dessine */
         lar[first].x2, lar[first].y2, TRUE );
   if ( ++first == MIKADOS )                     /* faut-il effacer ? */
    clear = TRUE;
   if ( clear )                             /* On efface maintenant ? */
    {                                                          /* Oui */
     Line( lar[last].x1, lar[last].y1,
           lar[last].x2, lar[last].y2, FALSE );
     if ( ++last == MIKADOS )
      last = 0;
    }
  }
 while (!kbhit());     /* Rpte l'opration jusqu' frappe de touche */
 getch();                    /* Retire la touche du buffer du clavier */
/*-- Termine le programme --------------------------------------------*/

 CloseGraphArea();
 SetPalAry( OldCols );     /* Restaure l'ancienne palette de couleurs */
 SetCursor(0,24);
 printf( "\nLe jeu de caractres standard est  nouveau en place .\n" );
}

/*--------------------------------------------------------------------*/
/*---------------------- PROGRAMME PRINCIPAL -------------------------*/
/*--------------------------------------------------------------------*/

void main()
{
 if ( IsEgaVga() )                   /* A-t-on une carte EGA ou VGA ? */
  Mikado();                          /* Oui, c'est parti pour la dmo */
 else                /* Non, impossible de faire tourner le programme */
  printf( "Attention: aucune carte EGA ou VGA n'est installe !" );
}
