/**********************************************************************/
/*                          P L I N K C . C                           */
/*--------------------------------------------------------------------*/
/*    Fonction       : transmet des fichiers par le port parallle    */
/*--------------------------------------------------------------------*/
/*    Auteur         : Michael Tischer                                */
/*    Dvelopp le   : 27.09.1991                                     */
/*    Dernire MAJ   : 28.11.1991                                     */
/*--------------------------------------------------------------------*/
/*    Modle mmoire : SMALL                                          */
/**********************************************************************/

/*== Fichiers d'inclusion      =======================================*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <setjmp.h>
#include <string.h>
#ifdef __TURBOC__                            /* Compilateur Turbo C ? */
  #include <dir.h>            /* En-ttes des fonctions de traitement */
  #include <ctype.h>                               /* des rpertoires */
#endif

/*== Dfinitions de types ============================================*/

typedef unsigned char BYTE;                       /* Dfinit un octet */
typedef unsigned int WORD;                  /* Bricolage du type WORD */
typedef struct {                 /* En-tte de transmission des blocs */
                BYTE         Token;
                unsigned int Len;
               } BHEADER;

/*== Fonctions du module en assembleur ==============================*/

extern void IntrInstall( int far  * escape_flag,
                         WORD far * timeout_count );
extern void IntrRemove( void );
extern void EscapeDirect( int ausloesen );

/*== Constantes ======================================================*/

#define UNESEC        18                               /* Une seconde */
#define DIXSEC        182                             /* Dix secondes */
#define TO_DEFAULT    DIXSEC         /* Valeur de Time-Out par dfaut */

#define TRUE          ( 0 == 0 )
#define FALSE         ( 0 == 1 )
#define MAXBLOC      4096        /* Taille des blocs 4 Ko ( Cluster ) */

/*-- Constantes pour le protocole de transmission-------------------*/

#define ACK           0x00                     /* Accus de rception */
#define NAK           0xFF                 /* Non-Accus de rception */
#define MAX_TRY       5          /* Nombre de tentatives avant erreur */

/*-- Tokens pour la communication entre metteur et rcepteur ------*/

#define TOK_DATSTART  0                           /* Dbut de fichier */
#define TOK_DATNEXT   1                  /* Bloc suivant d'un fichier */
#define TOK_DATEND    2           /* Transmission du fichier termine */
#define TOK_FIN       3                      /* Terminer le programme */
#define TOK_ESCAPE    4/* Interruption par ESC sur ordinateur distant */

/*-- Codes pour les appels LongJump ----------------------------------*/

#define LJ_OKEMET     1        /* Tous les fichiers correctement mis */
#define LJ_OKRECEPT   2       /* Tous les fichiers correctement reus */
#define LJ_TIMEOUT    3   /* Time-out: le correspondant ne rpond pas */
#define LJ_ESCAPE     4/*Interruption par Escape sur ordinateur local */
#define LJ_REMESCAPE  5 /* Interruption Escape sur ordinateur distant */
#define LJ_DATA       6                    /* Erreur de communication */
#define LJ_NOLINK     7                             /* Pas de liaison */
#define LJ_NOPAR      8                            /* Pas d'interface */
#define LJ_PARA       9               /* Paramtres d'appel invalides */

/*== Macros ==========================================================*/
/*-- Les trois bits infrieurs du registre d'entre ne sont pas     --*/
/*-- utillss selon les systmes ils peuvent tre  1 ou 0 , ici    --*/
/*-- ils sont limins par GetB() .                                 --*/

#ifdef __TURBOC__                            /* Compilateur Turbo C ? */
  #define GetB()                         ( inportb( PortdEntree ) & 0xF8 )
  #define PutB( QuelqueChose )           outportb( PortdeSortie, QuelqueChose )
  #define DIRSTRUCT                      struct ffblk
  #define FINDFIRST( path, buf, attr )   findfirst( path, buf, attr )
  #define FINDNEXT( buf )                findnext( buf )
  #define NomFichier                      ff_name
#else                                            /* Non , Microsoft C */
  #define GetB()                         ( inp( PortdEntree ) & 0xF8 )
  #define PutB( QuelqueChose )                    outp( PortdeSortie, QuelqueChose )
  #define DIRSTRUCT                      struct         find_t
  #define FINDFIRST( path, buf, attr )   _dos_findfirst(path, attr, buf)
  #define FINDNEXT( buf )                _dos_findnext( buf )
  #define NomFichier                      name
#endif

#ifdef MK_FP                            /* Macro MK_FP dj dfinie ? */
  #undef MK_FP                                     /* Oui on l'efface */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/*== Variables globales ==============================================*/

int     PortdEntree;                      /* Adresse du port d'entre */
int     PortdeSortie;                    /* Adresse du port de sortie */
int     Escape = 0;                  /* Pas de touche ESCAPE enfonce */
WORD    Timeout = TO_DEFAULT;                   /* Valeur de Time Out */
WORD    TO_Count;                              /* Compteur de Timeout */
jmp_buf Branchement;               /* Adresse de retour pour terminer */
BYTE    *BlocBuf;                 /* Buffer de mmorisation d'un bloc */
FILE    *Fichier = NULL;  /* Variable fichier pour traiter un fichier */

/**********************************************************************/
/* GetPortAdr    : Initialise les adresses des ports d'une interface  */
/*                 parallle  l'aide des variables PortdEntre et    */
/*                 Portde Sortie                                      */
/* Entre        : NUMMER = Numo de l'interface parallle (1-4)      */
/* Sortie        : TRUE, si l'interface est valable                   */
/* Var. globales : PortdEntree/W, PortdeSortie/W                      */
/* Info          : Les adresses de base des interfaces parallles     */
/*                 (en nombre de 1  4) se trouvent dans les mots     */
/*                 mmoires commenant en 0040:0008                   */
/**********************************************************************/

int GetPortAdr( int Numero )
{  /* Lit les adresses des ports dans le segment de variables du BIOS */
 PortdeSortie = *( WORD far * ) MK_FP( 0x0040, 6 + Numero * 2 );
 if ( PortdeSortie != 0 )                   /* Interface disponible ? */
  {                                                            /* Oui */
   PortdEntree = PortdeSortie + 1;/* Adresse pour le registre d'entre*/
   return TRUE;                                       /* Pas d'erreur */
  }
 else
  return FALSE;                          /* Erreur: interface absente */
}

/**********************************************************************/
/* Port_Init     : Initialise les registres ncessaires  la          */
/*                 transmission                                       */
/* Entre        : EMETTEUR=TRUE, si metteur, FALSE, si rcepteur    */
/* Sortie        : TRUE, si les reg. ont t correctement initialiss */
/* Var. globales : PortdEntree/R, PortdeSortie/R                      */
/* Info          : La dissymtrie: envoie 00010000, attend 00000000   */
/*                 est rendue ncessaire par l'inversion du signal.   */
/*                 Normalement les registres d'entre et de sortie    */
/*                 contiennent les valeurs souhaites mais            */
/*                 l'initialisation est ncessaire lorsqu'on reprend  */
/*                 une transmission interrompue                       */
/**********************************************************************/

int Port_Init( int Emetteur )
{
 EscapeDirect( TRUE );             /* Dclenche un Time Out si Escape */
 if ( Emetteur )                      /* L'appareil est-il metteur ? */
 {
  TO_Count = Timeout * 5;       /* Initialise le compteur de Time Out */
  PutB( 0x10 );                                 /* Envoie : 00010000b */
  while ( ( GetB() != 0x00 ) && TO_Count )      /* Attend : 00000000b */
   ;
  }
 else                                     /* L'appareil est rcepteur */
  {
   TO_Count = Timeout * 5;      /* Initialise le compteur de Time Out */
   while ( ( GetB() != 0x00 ) && TO_Count )     /* Attend : 00000000b */
    ;
   PutB( 0x10 );                                /* Envoie : 00010000b */
  }
 EscapeDirect( FALSE );                   /* Si Escape pas de Timeout */
 return ( TO_Count != 0 );                 /* Initialisation termine */
}

/**********************************************************************/
/* EmetOctet     : Envoie un octet en deux parties  l'ordinateur     */
/*                 distant et teste le rsultat                       */
/* Entre        : VALEUR: octet  mettre                            */
/* Sortie        : Transmission correcte ? ( 0 = Erreur , -1 = ok )   */
/* Var. globales : Timeout/R, PortdEntrer/R, PortdeSortie/R (macros)  */
/**********************************************************************/

int EmetOctet( BYTE Valeur )
{
 BYTE Retour;                                    /* Octet rceptionn */

/*-- Emet le quartet infrieur ---------------------------------------*/

 TO_Count = Timeout;             /* Initialise le compteur de Timeout */
 PutB( Valeur & 0x0F );                /* Envoi avec mise  0 de BUSY */
 while ( ( ( GetB() & 128 ) == 0 ) && TO_Count )  /* Attend le retour */
  ;
 if ( TO_Count == 0 )                           /* Erreur de Timeout ?*/
  longjmp( Branchement, LJ_TIMEOUT );   /* Interrompt la transmission */

 Retour = ( GetB() >> 3 ) & 0x0F;                 /* Bits 3-6 en  0-3 */

/*-- Emet le quartet suprieur ---------------------------------------*/

 TO_Count = Timeout;             /* Initialise le compteur de Timeout */
 PutB( ( Valeur >> 4 ) | 0x10 );       /* Envoi avec mise  1 de BUSY */
 while ( ( ( GetB() & 128 ) != 0 ) && TO_Count )   /* Atend le retour */
  ;

 if ( TO_Count == 0 )                          /* Erreur de Timeout ? */
  longjmp( Branchement, LJ_TIMEOUT );   /* Interrompt la transmission */

 Retour = Retour | ( ( GetB() << 1 ) & 0xF0 );     /* Bits 3-6 en 4-7 */
 return ( Valeur == Retour );        /* Octet correctement transmis ? */
}

/**********************************************************************/
/* RecOctet      : Rceptionne un octet en deux parties de la part    */
/*                 d'un ordinateur distant et renvoie les parties     */
/*                 pour vrification                                  */
/* Entre        : nant                                              */
/* Sortie        : Octet reu                                         */
/* Var. globales : Timeout/R, PortdEntree/R, PortdeSortie/R (Macros)  */
/**********************************************************************/

BYTE RecOctet( void )
{
 BYTE LoNib, HiNib;                                 /* Quartets reus */

      /*-- Rceptionne le quartet infrieur et le renvoie ------------*/

 TO_Count = Timeout;             /* Initialise le compteur de Timeout */
 while ( ( ( GetB() & 128 ) == 0 ) && TO_Count )/* Attend que BUSY =1 */
  ;

 if ( TO_Count == 0 )                           /* Erreur de Timeout ?*/
  longjmp( Branchement, LJ_TIMEOUT );   /* Interrompt la transmission */

 LoNib = ( GetB() >> 3 ) & 0x0F;                 /* Bits 3-6 dans 0-3 */
 PutB( LoNib );                                            /* Renvoie */

/*-- Rceptionne le quartet suprieur et le renvoie ------------------*/

 TO_Count = Timeout;            /* Initialise le compteur de Time Out */
 while ( ( ( GetB() & 128 ) != 0 ) && TO_Count ) /* Attend que BUSY 0 */
  ;

 if ( TO_Count == 0 )                          /* Erreur de Timeout ? */
  longjmp( Branchement, LJ_TIMEOUT );   /* Interrompt la transmission */

 HiNib = ( GetB() << 1 ) & 0xF0;                   /* Bits 3-6 en 4-7 */
 PutB( ( HiNib >> 4 ) | 0x10 );            /* Renvoie et met Busy  1 */

 return( LoNib | HiNib );                            /* Octet renvoy */
}

/**********************************************************************/
/* EmetBloc  : Envoi un bloc de donnes                               */
/* Entres   : TOKEN  = Commande pour le rcepteur                    */
/*             NOMBRE = Nombre d'octets   transmettre                */
/*             DONNEES= Pointeur sur le buffer des donnes            */
/* Sortie    : nant, en cas d'erreur branchement par                 */
/*             LongJmp  la routine de traitement d'erreur            */
/**********************************************************************/

void EmetBloc( BYTE Token, int Nombre, void *Donnees )
{
 BHEADER header;            /* En-tte pour mmoriser Token et Nombre */
 BYTE    *bptr,               /* Pointe sur l'octet courant  mettre */
     RecEscape;      /* A-t-on tap ESCAPE sur l'ordinateur distant ? */
 int     ok,                                   /* Indicateur d'erreur */
     i,                                      /* Compteur d'itrations */
     try;                                 /* Nombre d'essais restants */

 if ( Escape )             /* A-t-on tap Escape sur cet ordinateur ? */
  {
   Token = TOK_ESCAPE;                  /* Oui envoie le token Escape */
   Nombre = 0;
  }

      /*-- Emission de l'en-tte -------------------------------------*/

 header.Token = Token;                         /* Construit l'en-tte */
 header.Len = Nombre;

 for ( try = MAX_TRY; try; --try )   /* Au maximum MAX_TRY tentatives */
  {
   ok = TRUE;                   /* A priori la transmission est bonne */
   for ( bptr = (BYTE *) &header, i = sizeof( header); i; --i )
     ok = ok & EmetOctet( *bptr++ );                 /* Emet un octet */

   ok = ok & EmetOctet( (BYTE) (ok ? ACK : NAK) );     /*Confirmation */
   if ( ok )                               /* Transmission correcte ? */
     break;                             /* Oui, pas d'autre tentative */
  }

 if ( try == 0 )               /* L'en-tte a-t-il pu tre transmis ? */
  longjmp( Branchement, LJ_DATA );    /* Non, arrter la transmission */

 if ( Token == TOK_ESCAPE )        /* A-t-on envoy l'avis d'ESCAPE ? */
  longjmp( Branchement, LJ_ESCAPE );  /* Oui, arrter la transmission */

   /*-- Emission du bloc de donnes proprement dit -------------------*/

 if ( Nombre )                                   /* Taille diff de 0? */
  {
   for ( try = MAX_TRY; try; -- try )/* MAX_TRY tentatives au maximum */
    {
     ok = TRUE;                 /* A priori la transmission est bonne */
     for ( bptr = (BYTE *) Donnees, i = Nombre; i; --i )
       ok = ok & EmetOctet( *bptr++ );/*Envoie octet et interroge tat*/

     ok = ok & EmetOctet( (BYTE) (ok ? ACK : NAK) );  /* Confirmation */
     if ( ok )                             /* Transmission correcte ? */
       break;                            /* Oui pas d'autre tentative */
    }
   if ( try == 0 )           /* Les donnes ont-elle t transmises ? */
    longjmp( Branchement, LJ_DATA );  /* Non, interrompt transmission */
  }

  /*-- Teste l'octet ESCAPE du rcepteur -----------------------------*/

 for ( try = MAX_TRY; try; -- try )           /* Nombre de tentatives */
  {
   RecEscape = RecOctet();               /* Dtecte un escape distant */
   if ( RecEscape == (BYTE) TRUE  ||  RecEscape == (BYTE) FALSE )
    break;                           /* Etat de la touche Escape reu */
  }
 if ( try == 0 )      /* L'tat de la touche Escape a-t-il t reu ? */
  longjmp( Branchement, LJ_DATA ); /* Non, interrompt la transmission */

 if ( RecEscape )                  /* Escape sur ordinateur distant ? */
  longjmp( Branchement, LJ_REMESCAPE );/* Oui interrompt transmission */
}

/**********************************************************************/
/* RecBloc : Rceptionne un bloc de donnes                           */
/* Entre  : TOKEN   = Pointeur sur la variable qui mmorise le token */
/*           LEN     = Pointeur sur la variable qui mmorise la long. */
/*           DONNEES = Pointeur sur le buffer des donnes             */
/* Sortie  : nant, en cas d'erreur branchement sur routine d'erreur  */
/*           par LongJmp                                              */
/* Info    : Le buffer transmis doit prvoir de la place pour MAXBLOC */
/*           pour MAXBLOC octets, la taille du bloc ne pouvant pas    */
/*           tre anticipe                                           */
/**********************************************************************/

void RecBloc( BYTE *Token, int *Len, void *Donnees )
{
 BHEADER header;                                /* Mmorise l'en-tte */
 BYTE    *bptr;       /* Pointeur courant dans le buffer de rception */
 int     ok,                                   /* Indicateur d'erreur */
     i,                                      /* Compteur d'itrations */
     try,                           /* Nombre de tentatives restantes */
     EscapeStatus;     /* Mmorise l'tat courant de la touche Escape */

 /*-- Rceptionne d'abord l'en-tte ----------------------------------*/

 for ( try = MAX_TRY; try; -- try )  /* MAX_TRY tentatives au maximum */
  {
   for ( bptr = (BYTE *) &header, i = sizeof(header); i; --i )
     *bptr++ = RecOctet( );

   if ( RecOctet() == ACK );          /* Tous les octets bien reus ? */
     break;                                 /* Oui pluss de tentative */
  }

 if ( try == 0 )                       /* En-tte correctement reu ? */
  longjmp( Branchement, LJ_DATA ); /* Non, interrompre la transmission*/

 if ( ( *Token = header.Token ) == TOK_ESCAPE )   /* Emetteur ESCAPE? */
  longjmp( Branchement, LJ_REMESCAPE );/* Oui interrompt transmission */

      /*-- L'en-tte est bon, il faut passer au bloc des donnes . ---*/

 if ( ( *Len = header.Len ) != 0 )       /* Pas de bloc des donnes ? */
  {                                                             /* si */
   for ( try = MAX_TRY; try; -- try )/* MAX_TRY tentatives au maximum */
    {
     for ( bptr = (BYTE *) Donnees, i = header.Len; i; --i )
       *bptr++ = RecOctet( );

     if ( RecOctet() == ACK );        /* Tous les octets bien reus ? */
      break;                                /* Oui, plus de tentative */
    }

  if ( try == 0 )           /* Le bloc a-t-il t correctement reu ? */
   longjmp( Branchement, LJ_DATA );/* Non, interrompt la transmission */
  }

 /*-- Envoie l'tat actuel de la touche Escape  l'ordinateur distant */

 EscapeStatus = Escape;                            /* Mmorise l'tat */
 for ( try = MAX_TRY; try; -- try )         /* Nombre de tentatives   */
  {
   if ( EmetOctet( (BYTE) (EscapeStatus != 0) ) )         /* arriv ? */
    break;                     /* Oui, plus de tentative ncessaire   */
  }

 if ( try == 0 )                /* L'tat ESC a-t-il pu tre envoy ? */
   longjmp( Branchement, LJ_DATA );/* Non, interrompt la transmission */

 if ( EscapeStatus )      /* A-t-on actionn Esc sur cet ordinateur ? */
  longjmp( Branchement, LJ_ESCAPE );/*Oui, interrompt la transmission */
}

/**********************************************************************/
/* EmetFichier : Envoie un fichier                                    */
/* Entre      : NOM = Ptr sur un buffer contenant le nom du fichier  */
/* Sortie      : nant                                                */
/**********************************************************************/

void EmetFichier( char *Nom  )
{
 int           Status;                             /* Etat d'mission */
 WORD          Lus;                            /* Nombre d'octets lus */
 unsigned long Taille;                     /* Nombre d'octets envoys */

 printf( "%-13s", Nom);
 Fichier = fopen( Nom,"rb" );                     /* Ouvre le fichier */
 EmetBloc( TOK_DATSTART, strlen(Nom)+1, Nom );       /* envoie son nom*/

          /*-- Transfre le contenu du fichier -----------------------*/

 Taille = 0;
 do
  {
   Lus = fread( BlocBuf, 1, MAXBLOC, Fichier );       /* Lit un bloc  */
   if ( Lus > 0 )                                /* Est-on  la fin ? */
    {                                                          /* Non */
     EmetBloc( TOK_DATNEXT, Lus, BlocBuf );           /* Emet le bloc */
     Taille += Lus;
     printf( "\r%-13s (%ld)", Nom, Taille );
    }
  }
 while (  Lus > 0 );
 printf( "\n" );

 EmetBloc( TOK_DATEND, 0, NULL );          /* Clture la transmission */

 fclose( Fichier );                             /* Referme le fichier */
 Fichier = NULL;                                     /* Fichier ferm */
}

/**********************************************************************/
/* RecFichier  : Rceptionne un fichier                               */
/* Entre      : nant                                                */
/* Sortie      : Dernier token reu                                   */
/**********************************************************************/

int RecFichier( void )
{
 int           Status;                           /* Etat de rception */
 WORD          AEnregistrer;                /* Taille du dernier bloc */
 unsigned long Taille;
 BYTE          Token;                            /* Token rceptionn */
 int           Len;                          /* Longueur du bloc reu */
 char          Name[13];                            /* Nom du fichier */

 RecBloc( &Token, &Len, BlocBuf );
 if ( Token == TOK_DATSTART )
  {
   strcpy( Name, BlocBuf );
   Fichier = fopen( Name, "wb" );          /* Ouvre (cre) le fichier */
   printf( "%-13s", Name );

   /*-- Rceptionne le contenu du fichier ----------------------------*/

   Taille = 0;
   do
    {
     RecBloc( &Token, &Len, BlocBuf );         /* Rceptionne un bloc */
     if ( Token == TOK_DATNEXT )      /* Bloc de donnes conscutif ? */
      {                                                         /* Ja */
       fwrite( BlocBuf, 1, Len, Fichier );              /* Enregistre */
       Taille += Len;
       printf( "\r%-13s (%ld)", Name, Taille );
      }
    }
   while ( Token == TOK_DATNEXT );
   fclose( Fichier );                           /* Referme le fichier */
   Fichier = NULL;                            /* Le fichier est ferm */
   printf( "\n" );
  }
 return Token;                            /* Retourne l'tat d'erreur */
}

/**********************************************************************/
/*                           PROGRAMME PRINCIPAL                      */
/**********************************************************************/

void main( int argc, char *argv[] )
{
 DIRSTRUCT SRec;    /* Structure pour la recherche dans un rpertoire */
 BYTE      Emetteur;   /* Mode de transmission (Emission, Rception ) */
 int       sjStatus,                              /* Code de longjmup */
       Numero,                               /* Numro de l'interface */
       i,                                    /* Compteur d'itrations */
       Trouve;                      /* Pour la recherche des fichiers */

 static char *Avis[ 9 ] =
  { "FIN: Tous les fichiers ont t correctement mis",
    "FIN: Tous les fichiers ont t correctement reus",
    "ERREUR: Time-Out, le systme distant ne rpond pas",
    "FIN: Interruption par Escape.",
    "FIN: Interruption par Escape sur l'ordinateur distant.",
    "ERREUR: Interface ou cble dfectueux",
    "ERREUR: pas de contact avec l'ordinateur distant",
    "ERREUR: L'interface indique n'existe pas !",
    "ERREUR: Paramtre inconnu ou invalide !" };

 printf( "\n\nTransmission de donnes par l'interface parallle    " );
 printf( "(c) 1991 by Michael Tischer\n" );
 printf( "=====================================================" );
 printf( "===========================\n\n" );

 if ( strcmp( argv[ 1 ], "?" ) == 0 )         /* Affiche la syntaxe ? */
  {                                                             /* oui*/
   printf( "Appel: plinkc [/Pn] [/Tnn] [Nom de Fichier]\n" );
   exit ( 0 );
  }

 sjStatus = setjmp( Branchement );               /* Adresse de retour */
 if ( sjStatus )                       /* Longjmp a-t-il t appel ? */
  {                                                            /* Oui */
   IntrRemove( );         /* Dsactive le gestionnaire d'interruption */
   if ( Fichier )                   /* Reste-t-il un fichier ouvert ? */
     fclose( Fichier );
   free( BlocBuf );                /* Libre la place mmoire alloue */
   printf( "\n\n%s\n", Avis[ sjStatus - 1 ] );
   exit( 0 );
  }

 BlocBuf = malloc( MAXBLOC );      /* Cre un buffer pour les donnes */
 IntrInstall( &Escape, &TO_Count );/* Initilialise driver interruption*/
 /*-- Fixe les paramtres par dfaut et exploite la ligne de commande */

 Emetteur = FALSE;           /* Par dfaut l'ordinateur est rcepteur */
 Numero = 1;                                  /* L'interface est LPT1 */

 for ( i = 1; i < argc; i++ )
  {
   if ( argv[i][0] == '/' )                             /* Paramtres */
    {
     switch ( toupper( argv[i][1] ) )
      {
       case 'T' : Timeout = (atol( &argv[i][2] ) * DIXSEC) / 10;
                  if ( Timeout == 0 )
           longjmp( Branchement, LJ_PARA );              /* incorrect */
                  break;
       case 'P' : Numero = argv[i][2] - 48;              /* Interface */
                  if ( Numero == 0   ||   Numero > 4 )
           longjmp( Branchement, LJ_PARA );              /* incorrect */
                  break;
       default  : longjmp( Branchement, LJ_PARA );/* paramtre inconnu*/
                  break;
      }
     argv[i][0] = '\0';                          /* Efface l'argument */
    }
   else         /* Pas de paramtre, il doit s'agir du nom du fichier */
    Emetteur = TRUE;                                      /* Emetteur */
  }

/*-- Lance la transmission -------------------------------------------*/

 if ( GetPortAdr(Numero) == FALSE )    /* L'interface existe-t-elle ? */
  longjmp( Branchement, LJ_NOPAR );                    /* Non, erreur */

 if ( Port_Init( Emetteur ) == FALSE )         /* Etablit  la liaison */
  longjmp( Branchement, LJ_NOLINK );        /* Erreur,  impossibilit */

 if ( Emetteur )                                         /* Emetteur? */
  {
   printf( "Emission vers LPT%d:\n\n", Numero );

        /*-- Transmet tous les fichiers ------------------------------*/

   for ( i = 1; i < argc; i++ )      /* Parcourt la ligne de commande */
    {
     if ( argv[i][0] != '\0' )                    /* Nom du fichier ? */
      {                                                         /* Oui*/
       Trouve = FINDFIRST( argv[i], &SRec, 0);
       while ( !Trouve )
        {
         if ( SRec.NomFichier[0] != '.' )
      EmetFichier( SRec.NomFichier );              /*Fichier transmis */
         Trouve = FINDNEXT( &SRec );
        }
      }
    }
   EmetBloc( TOK_FIN, 0 , NULL );          /* Tous les fichiers mis  */
   longjmp( Branchement, LJ_OKEMET );
  }
 else                                              /* Non, rcepteur  */
  {
   printf( "Rception sur LPT%d:\n\n", Numero );
   while ( RecFichier() != TOK_FIN )      /* Rceptionne les fichiers */
    ;                                         /* jusqu'au token final */
   longjmp( Branchement, LJ_OKRECEPT );
  }
}
