/**********************************************************************/
/*                         R E C L O C K C . C                        */
/*--------------------------------------------------------------------*/
/*    Fonction       : Montre comment fonctionne le verrouillage des  */
/*                     enregistrements avec les fonctions du module   */
/*                     NETFILEC                                       */
/*--------------------------------------------------------------------*/
/*    Modle mmoire : SMALL                                          */
/*--------------------------------------------------------------------*/
/*    Auteur         : Michael Tischer                                */
/*    Dvelopp le   : 10.02.1992                                     */
/*    Dernire MAJ   : 13.02.1991                                     */
/**********************************************************************/

#include "netfilec.c"        /* Intgre les routines orientes rseau */
#include <stdio.h>
#include <conio.h>

/*== Constantes ======================================================*/

#define NOMFICHIER "rec.dat"                /* Nom du fichier de test */
#define ENREGIS    10                      /* Nombre d'enrg . de test */

/*== Dfinitions de types ============================================*/

typedef char Test[ 160 ];              /*Type de donnes pour le test */
typedef char TestString[ 161 ];    /* Type des donnes pour affichage */

#ifndef __TURBOC__                                    /* Microsoft C? */

  #define clrscr() clearwindow( 1, 1, 80, 25 )

  /********************************************************************/
  /* Gotoxy       : Positionne le curseur                             */
  /* Entres      : Coordonnes du curseur                            */
  /* Sortie       : nant                                             */
  /********************************************************************/

  void gotoxy( int x, int y )

  {
   regs.h.ah = 0x02;          /* Numro de la fonction d'interruption */
   regs.h.bh = 0;                                          /* Couleur */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                     /* Interruption   */
  }
#endif

/**********************************************************************/
/* clearwindow  : Efface une partie de l'cran                        */
/* Entres      : cf infra                                            */
/* Sortie       : nant                                               */
/**********************************************************************/

void clearwindow( int x1, int y1, int x2, int y2 )
{
 regs.h.ah = 0x07;             /* Numro de la fonction d'interruption*/
 regs.h.al = 0x00;
 regs.h.bh = 0x07;
 regs.h.ch = y1 - 1;
 regs.h.cl = x1 - 1;
 regs.h.dh = y2 - 1;
 regs.h.dl = x2 - 1;
 int86( 0x10, &regs, &regs );                          /* Interruption*/
 gotoxy( x1, y1 );                               /* Refixe le curseur */
}

/**********************************************************************/
/* OuvreFichier : Ouvre un fichier prexistant. Sinon crer un nouveau*/
/*                fichier de test et le remplit avec les donnes      */
/*                prvues  cet effet                                 */
/* Entres      : cf infra                                            */
/* Sortie       : Fichier                                             */
/**********************************************************************/

int OuvreFichier( NFILE *Fichier )                 /* Fichier rseau  */
{
 int  i;                                                   /* Compteur*/
 Test TestEnreg;      /* Indispensable pour crer les donnes de test */

 /*-- Ouvre un fichier pour entre/sortie en mode Deny-None ----------*/

 NetReset( NOMFICHIER, FM_RW | SM_NO, sizeof( Test ), Fichier );
 if ( NetError == NE_FileNotFound )              /* Fichier inconnu ? */
 {
                  /*-- Cre le fichier et le remplit -----------------*/

  NetRewrite( NOMFICHIER, FM_RW | SM_NO, sizeof( Test ), Fichier );
  if ( NetError == NE_OK )                     /* Cration correcte ? */
  {
   if ( NetLock( Fichier, 0L, (long) ENREGIS ))/* Verrouille tous les */
                                                    /* enregistrement */
   {
    NetSeek( Fichier, 0L );         /* Pointe sur le dbut du fichier */
    for ( i = 0; i < ENREGIS; i++ )
    {
     memset( TestEnreg, 'A' + i, 160 );
     NetWrite( Fichier, TestEnreg );     /* Ecrit les donnes de test */
    }
    return NetUnLock( Fichier, 0L, (long) ENREGIS );
   }
   else
    return FALSE;                           /* Erreur de verrouillage */
  }
  else
   return FALSE;                                /* Erreur de cration */
 }
 else
  return ( NetError == 0 );                /* Ouverture sans erreur ? */
}

/**********************************************************************/
/* ResoEdit   : Dmonstration des fonctions de rseau                 */
/* Entres    : voir plus loin                                        */
/* Sortie     : Fichier                                               */
/**********************************************************************/

void ResoEdit( NFILE *TestFile )                    /* Fichier rseau */
{
 unsigned long ActRecord;                  /* Numro d'enreg. courant */
 TestString    ActEnreg;                         /* Donnes courantes */
 int           Action;                            /* Action souhaite */
 int           Status;                         /* Enreg. verrouill ? */
 char          Caractere[ 10 ];
 char          SDummy[ 50 ];                        /* Etat du rseau */
 int           Verrouille[ ENREGIS ];
 int           i;                                         /* Compteur */

/*-- AFfiche le menu -------------------------------------------------*/

 printf( "Fonctions proposes\n" );
 printf( "  1: Positionne le pointeur\n" );
 printf( "  2: Verrouille un enregistrement\n" );
 printf( "  3: Lit un enregistrement\n" );
 printf( "  4: Modifie les donnes \n" );
 printf( "  5: Ecrit un enregistrement\n" );
 printf( "  6: Dverrouille un enregistrement\n" );
 printf( "  7: Fin\n" );

 /*-- Insitialise et affiche l'tat des enregistrements --------------*/

 gotoxy( 58, 4 );
 printf( "Etat des enreg:" );
 for ( i = 0; i < ENREGIS; ++i )
  {
   Verrouille[i] = FALSE;
   gotoxy( 64, i+5 );
   printf( "%2d   libre", i );
  }

 ActRecord = 0;                             /* Enregistrement courant */
 Status = FALSE;                                    /* Non verrouill */
 memset( ActEnreg, 32, 160 );             /* Gnre des donnes vides */

 do
 {
 /*-- Affiche les informations ---------------------------------------*/

  gotoxy( 1, 16 );                            /* Position du pointeur */
  printf( "Enregistrement courant: %4li\n",  ActRecord );
  printf( "Etat          :    %s\n",
       Verrouille[ActRecord] ? "verrouill" : "libre      " );
  NetErrorMsg( NetError, SDummy );
  printf( "Etat du rseau : %4i = %s", NetError, SDummy );
  gotoxy( 1, 21 );                             /* Affiche les donnes */
  printf( "Donnes courantes :\n" );
  ActEnreg[ 160 ] = 0;
  printf( "%s", ActEnreg );

  NetSeek( TestFile, ActRecord );           /* Positionne le pointeur */
  gotoxy( 1, 13 );
  printf( "Slection:                            " );
  gotoxy( 12, 13 );
  Action = 0;
  scanf( "%i", &Action );
  switch( Action )
  {
   case 1 : gotoxy( 1, 13 );
        printf( "Nouveau numro: " );
            do
             {
              gotoxy( 22, 13 );
              printf( "                      " );
              gotoxy( 22, 13 );
              scanf( "%li", &ActRecord );
             }
            while (!( ActRecord >= 0  &&  ActRecord < ENREGIS ));
            break;

   case 2 : Status = NetLock( TestFile, ActRecord, 1L );
            if ( Status )
             {
              Verrouille[ ActRecord ] = TRUE;
              gotoxy( 64, (int) ActRecord +5 );
          printf( "%2d   verrouill", ActRecord );
             }
            break;

   case 3 : NetRead( TestFile, ActEnreg );         /* Lit les donnes */
            break;

   case 4 : gotoxy( 1, 13 );
        printf( "Nouveau caractre:" );
            scanf( "%s", Caractere );
            memset( ActEnreg, Caractere[ 0 ], 160 );
            break;

   case 5 : NetWrite( TestFile, ActEnreg ); /* Enregistre les donnes */
            break;

   case 6 : Status = NetUnLock( TestFile, ActRecord, 1L );
            if ( Status )
             {
              Verrouille[ ActRecord ] = FALSE;
              gotoxy( 64, (int) ActRecord+5 );
              printf( "%2d   libre      ", ActRecord);
             }
            break;
  }

 }
 while ( Action != 7 );
}

/**********************************************************************/
/*                           Programme principal                      */
/**********************************************************************/

void main( )

{
 NFILE Fichier;                                    /* Fichier de test */

 clrscr();
 printf( "RECLOCKC Dmo du verrouillage d'enregistrements DOS" \
         " (c) 1992 by Michael Tischer\n" );
 printf( "====================================================" \
         "===========================\n\n" );

 if ( ShareInst() )                                /* Share install? */
 {
  if ( OuvreFichier( &Fichier ) )         /* Fichier ouvert ou cr ? */
  {
   ResoEdit( &Fichier );                  /* C'est parti pour la dmo */
   NetClose( &Fichier );                          /* Ferme le fichier */
   clrscr( );
  }
  else
   printf( "\nErreur %i  l'ouverture du fichier" , NetError );
 }
 else
  printf( "\nTest impossible, SHARE doit tre install" );
}
