;**********************************************************************;
;*                         S 3 2 2 0 C A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Fonction      : contient les routines pour travailler avec les  *;
;*                    sprites dans le mode 320*200-256 de la carte VGA*;
;*--------------------------------------------------------------------*;
;*    Auteur         : MICHAEL TISCHER                                *;
;*    Dvelopp le   :  8.09.1990                                     *;
;*    Dernire MAJ   : 14.02.1992                                     *;
;*--------------------------------------------------------------------*;
;*    Modle mmoire : SMALL                                          *;
;*--------------------------------------------------------------------*;
;*    Assemblage     : MASM /mx S3220CA;    ou     TASM -mx S3220CA   *;
;*                     ... puis lier  S3220C.C                       *;
;**********************************************************************;

IGROUP group _text                   ;Regroupe les segments de programme
DGROUP group _bss,  _data              ;Regroupe les segments de donnes
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'        ;Segment des variables statiques
_BSS   ends                                            ;non initialises

_DATA  segment word public 'DATA'        ;Segment des variables globales
                                          ;et statiques non initialises

_DATA  ends

;== Constantes =========================================================

SC_INDEX       = 3c4h      ;Registre d'index du contrleur du squenceur
SC_MAP_MASK    = 2                          ;Numro du registre Map Mask
SC_MEM_MODE    = 4                   ;Numro du registre de mode mmoire
GC_INDEX       = 3ceh          ;Registre d'index du contrleur graphique
GC_GRAPH_MODE  = 5                 ;Numro du registre de mode graphique

VERT_RETRACE   = 3DAh                       ;Registre d'tat d'entre #1
PIXX           = 320                             ;Rsolution horizontale

;== Programme ==========================================================

_TEXT  segment byte public 'CODE'                  ;Segment de programme

;-- Dclarations publiques ---------------------------------------------

public    _blockmove

;-- BLOCKMOVE: Dplace un groupe de pixels dans la mmoire d'cran-----
;-- Dclaration : void blockmove( unsigned char depage, int dex,
;--                               int dey, unsigned char verspage,
;--                               int versx, int versy, BYTE largeur,
;--                               byte hauteur, byte *bmskp );

_blockmove proc near

sframe4    struc                            ;Structure d'accs  la pile
 bp4         dw ?                                           ;mmorise BP
 additif     dw ?                                       ;variable locale
 restz       dw ?
 movec       dw ?
 dataseg     dw ?
 ret_adr4    dw ?                        ;Adresse de retour  l'appelant
 depage      dw ?                                        ;Page d'origine
 dex         dw ?                                    ;Abscisse d'origine
 dey         dw ?                                   ;Ordonnes d'origine
 verspage    dw ?                                   ;Page de destination
 versx       dw ?                             ;Abscisse X de destination
 versy       dw ?                             ;Ordonne Y de destination
 largeur     dw ?                                               ;Largeur
 hauteur     dw ?                                               ;Hauteur
 bmskp       dw ?           ;Pointe sur le buffer avec le masque binaire
sframe4    ends                                     ;Fin de la structure

frame  equ [ bp - bp4 ]            ;adresse les lments de la structure

       sub   sp,8                   ;8 octets pour les variables locales

       push  bp                      ;Prpare l'adressage des paramtres
       mov   bp,sp                                  ; par le registre BP

       push  ds
       push  si
       push  di

       mov   frame.dataseg,ds

       mov   dx,GC_INDEX                   ;Lit le mode Write courant et
       mov   al,GC_GRAPH_MODE                     ; fixe le mode Write 1
       out   dx,al
       inc   dx
       in    al,dx
       push  ax                                  ;Empile le mode courant
       and   al,not 3
       or    al,1
       out   dx,al

       mov   al,4       ;DS va pointer sur le dbut de la page d'origine
       mov   cl,byte ptr frame.depage
       mul   cl
       or    al,0A0h
       xchg  ah,al
       mov   ds,ax

       mov   al,4 ;ES va pointeur sur le dbut de la page de destination
       mov   cl,byte ptr frame.verspage
       mul   cl
       or    al,0A0h
       xchg  ah,al
       mov   es,ax

       mov   ax,PIXX / 4        ;SI va pointer sur la position d'origine
       mul   frame.dey
       mov   si,frame.dex
       shr   si,1
       shr   si,1
       add   si,ax

       mov   ax,PIXX / 4   ;DI va pointer sur la position de destination
       mul   frame.versy
       mov   di,frame.versx
       shr   di,1
       shr   di,1
       add   di,ax

       mov   dh,byte ptr frame.hauteur                      ;DH = Lignes
       mov   dl,byte ptr frame.largeur                      ;DL = octets
       shr   dl,1
       shr   dl,1

       mov   bx,PIXX / 4                     ;BX = offset ligne suivante
       sub   bl,dl
       xor   ch,ch                  ;Octet haut du compteur toujours nul
       cmp   frame.bmskp,0                   ;Pas de fond  respecter  ?
       jne   mt2                          ;Si, utilise une autre routine

       push  dx                               ;Sauvegarde DX sur la pile
       mov   dx,SC_INDEX        ;Assure l'accs  tous les plans de bits
       mov   ah,0Fh
       mov   al,SC_MAP_MASK
       out   dx,ax
       pop   dx                                              ;Reprend DX

       ;-- Routine de copie pour tous les quatre plans de bits, --------
       ;-- ne respecte pas le fond -------------------------------------

mt1:   mov   cl,dl                                ;Nombre d'octets en CL

       rep   movsb                                      ;Copie une ligne
       add   di,bx                                ;DI sur ligne suivante
       add   si,bx                                         ;ainsi que SI
       dec   dh                                  ;Reste-t-il une ligne ?
       jne   mt1                                    ;Oui---> on continue
       jmp   short mtend                         ;Non, prpare la sortie

       ;-- Routine de copie pour plans de bits individuels avec --------
       ;-- exploitation du tableau de masques binaires transmis --------

mt2:   mov   byte ptr frame.restz,dh     ;Mmorise d'abord les variables
       mov   byte ptr frame.movec,dl      ;qui sont stockes sur la pile
       mov   frame.additif,bx                         ;sous forme locale

       mov   al,SC_MAP_MASK      ;Adresse le registre MAP MASK permanent
       mov   dx,SC_INDEX
       out   dx,al
       inc   dx                   ;DX pointe sur le registre de donnesx

       mov   bx,frame.bmskp            ;BX = tableau de masques binaires
       push  ds
       mov   ds,frame.dataseg
       mov   al,[bx]                            ;Charge le premier octet
       xor   ah,ah                          ;Commence avec un octet pair
       pop   ds

mt3:   mov   cl,byte ptr frame.movec              ;Nombre d'octets en CL

mt4:   out   dx,al                               ;Fixe le masque binaire
       movsb                                             ;Copie 4 octets

       inc   ah                      ;Incrmente le compteur pair/impair
       test  ah,1                                      ;Valeur impaire ?
       jne   mt5                                ;Oui, dplace le quartet

       ;-- Octet pair, passe  l'octet suivant du buffer ---------------

       inc   bx                      ;BX sur le prochain octet de masque
       push  ds
       mov   ds,frame.dataseg
       mov   al,[bx]                             ;Charge l'octet suivant
       pop   ds
       loop  mt4                      ;Passe aux quatre latches suivants
       jmp   short mt6

mt5:   shr   al,1                      ;Transfre dans le quartet faible
       shr   al,1                   ;le masque binaire de l'octet impair
       shr   al,1
       shr   al,1
       loop  mt4                       ;Pass aux quatre latches suivants

mt6:   add   di,frame.additif                     ;DI sur ligne suivante
       add   si,frame.additif                             ; ainsi que SI
       dec   byte ptr frame.restz                     ;Reste une ligne ?
       jne   mt3                                    ;Oui---> on continue

mtend: mov   dx,GC_INDEX                   ;Rtablit l'ancien mode Write
       pop   ax
       mov   ah,al
       mov   al,GC_GRAPH_MODE
       out   dx,ax

       pop   di
       pop   si
       pop   ds
       pop   bp

       add   sp,8                        ;Supprime les variables locales
       ret

_blockmove endp

;== Fin  ===============================================================

_text    ends                               ;Fin du segment de programme
         end                                ;Fin du source en assembleur
