/**********************************************************************/
/*                         T O U C H E C                              */
/*--------------------------------------------------------------------*/
/*  Fonction: Implmente une fonction qui permet de lire un caractre */
/*            au clavier en affichant l'tat des touches de bascule   */
/*            INSERT, CAPS et NUM                                     */
/*--------------------------------------------------------------------*/
/*    Auteur                : MICHAEL TISCHER                         */
/*    Dvelopp le          : 13.08.1987                              */
/*    Dernire modification : 01.01.1992                              */
/*--------------------------------------------------------------------*/
/*    Modle de mmoire : SMALL                                       */
/**********************************************************************/

/*== Fichiers d'inclusion ============================================*/

#include <dos.h>
#include <bios.h>
#include <stdio.h>

/*== Typedefs ========================================================*/

typedef unsigned char BYTE;               /* bricolage d'un type BYTE */

/*== Macros ==========================================================*/

#ifdef __TURBOC__                         /* Dfinitions pour TURBO C */

   #define GetKbKey()        ( bioskey( 0 ) )
   #define GetKbReady()      ( bioskey( 1 ) != 0 )
   #define GetKbEtat()     ( bioskey( 2 ) )

#else                   /* Dfinitions pour le compilateur Microsoft C*/

   #define GetKbKey()        ( _bios_keybrd( _KEYBRD_READ ) )
   #define GetKbReady()      ( _bios_keybrd( _KEYBRD_READY ) != 0 )
   #define GetKbEtat()     ( _bios_keybrd( _KEYBRD_SHIFTSTATUS ) )

#endif

/*== Constantes ======================================================*/

/*-- Disposition des bits dans la variable d'tat du clavier du BIOS -*/

#define SCRL  16                                   /* Bit Scroll-Lock */
#define NUML  32                                      /* Bit Num-Lock */
#define CAPL  64                                     /* Bit Caps-Lock */
#define INS  128                                        /* Bit Insert */

#define TRUE  ( 0 == 0 )                 /* Constantes pour faciliter */
#define FALSE ( 0 == 1 )                         /* la comprhension  */

#define FL      0                /* Ligne d'affichage des indicateurs */
#define FC      65            /* Colonne d'affichage des indicateurs  */
#define CoulIndic 0x70           /* Couleur indicateur = noir / blanc */

/*-- Code retourn par GETKEY pour quelques touches ------------------*/
#define BEL     7                            /* Code du signal sonore */
#define BS      8                      /* Code de la touche Backspace */
#define TAB     9                  /* Code de la touche de tabulation */
#define LF      10                      /* Code de la touche LineFeed */
#define CR      13                        /* Code de la touche Entre */
#define ESC     27                 /* Code de la touche d'chappement */
#define F1      315                            /* Touches de fonction */
#define F2      316
#define F3      317
#define F4      318
#define F5      319
#define F6      320
#define F7      321
#define F8      322
#define F9      323
#define F10     324
#define CUP     328                           /* Touches de direction */
#define CLEFT   331
#define CRIGHT  333
#define CDOWN   328

/*== Variables globales ==============================================*/

BYTE Insert,                              /* Etat de la touche INSERT */
     Num,                                    /* Etat de la touche NUM */
     Caps;                                  /* Etat de la touche CAPS */

/**********************************************************************/
/* GETPAGE : Lit la page d'cran courante                             */
/* Entres : nant                                                    */
/* Sortie  : nant                                                    */
/**********************************************************************/

BYTE GetPage( void )
{
 union REGS Registre;      /* Variables registres pour l'interruption */

 Registre.h.ah = 15;                         /* Numro de la fonction */
 int86(0x10, &Registre, &Registre); /* Dclenche l'interruption 10(h) */
 return(Registre.h.bh);         /* Numro de la page d'cran courante */
}

/**********************************************************************/
/* SETPOS : Fixe la position du curseur dans la page d'cran courante */
/* Entres : COLONNE= nouvelle colonne                                */
/*           LIGNE  = nouvelle ligne                                  */
/* Sortie  : nant                                                    */
/* Info    : La position du curseur clignotant ne se modifie          */
/*           que si la page d'cran indique est la page courante     */
/**********************************************************************/

void SetPos(BYTE Colonne, BYTE Ligne)
{
 union REGS Registre;      /* Variables registres pour l'interruption */

 Registre.h.ah = 2;                          /* Numro de la fonction */
 Registre.h.bh = GetPage();                           /* page d'cran */
 Registre.h.dh = Ligne;                              /* Ligne d'cran */
 Registre.h.dl = Colonne;                          /* Colonne d'cran */
 int86(0x10, &Registre, &Registre);    /* Appel de l'interruption 10h */
}

/**********************************************************************/
/* GETPOS : Lit la position du curseur dans la page d'cran courante  */
/* Entres : nant                                                    */
/* Sortie  : Colonne = Pointeur sur la variable colonne courante      */
/*             Ligne = Pointeur sur la variable ligne courante        */
/**********************************************************************/

void GetPos(BYTE * Colonne, BYTE * Ligne)
{
 union REGS Registre;       /* Variables registres pour l'interruption*/

 Registre.h.ah = 3;                          /* Numro de la fonction */
 Registre.h.bh = GetPage();                           /* Page d'cran */
 int86(0x10, &Registre, &Registre);     /* Appelle l'interruption 10h */
 *Colonne = Registre.h.dl;        /* Lit les rsultats de la fonction */
 *Ligne = Registre.h.dh;                        /* dans les registres */
}

/**********************************************************************/
/* WRITECHAR : Affiche un caractre avec un attribut donn  la       */
/*             position et dans la page courantes                     */
/* Entres : - CARACTERE = Code ASCII du caractre  afficher         */
/*           - Couleur   = Attribut du caractre                      */
/* Sortie : nant                                                     */
/**********************************************************************/

void WriteChar(char Caractere, BYTE Couleur)
{
 union REGS Registre;      /* Variables registres pour l'interruption */

 Registre.h.ah = 9;                          /* Numro de la fonction */
 Registre.h.al = Caractere;                   /* Caractre  afficher */
 Registre.h.bh = GetPage();                           /* Page d'cran */
 Registre.h.bl = Couleur;          /* Couleur du caractre  afficher */
 Registre.x.cx = 1;                              /* Un seul caractre */
 int86(0x10, &Registre, &Registre);   /* Dclenche l'interruption 10h */
}

/**********************************************************************/
/* WRITETEXT: affiche une chane de caractre avec un attribut        */
/*            constant  partir d'une position donne sur la page     */
/*            d'cran courante                                        */
/* Entres : - COLONNE= colonne d'affichage                           */
/*           - LIGNE  = ligne d'affichage                             */
/*           - TEXTE  = pointeur sur la chane  afficher             */
/*           - Couleur  = Attribut des caractres                     */
/* Sortie  : nant                                                    */
/* Info    : Texte est un pointeur rfrenant un vecteur de          */
/*           caractres qui contient le texte  afficher avec un '\0' */
/*           terminal                                                 */
/**********************************************************************/

void WriteTexte(BYTE Colonne, BYTE Ligne, char *Texte, BYTE Couleur)
{
 union REGS InRegistre,    /* Variables registres pour l'interruption */
	    OutRegistre;

 SetPos(Colonne, Ligne);               /* Fixe la position du curseur */
 InRegistre.h.ah = 14;                       /* Numro de la fonction */
 InRegistre.h.bh = GetPage();                         /* Page d'cran */
 while (*Texte)                              /* Afficher jusqu' '\0' */
  {
   WriteChar(' ', Couleur);                  /* Attribut du caractre */
   InRegistre.h.al = *Texte++;                /* Caractre  afficher */
   int86(0x10, &InRegistre, &OutRegistre); /* Dclencher interruption */
  }
}

/**********************************************************************/
/* CLS : Efface la page d'cran courante                              */
/* Entre : nant                                                     */
/* Sortie : nant                                                     */
/**********************************************************************/

void Cls( void )
{
 union REGS Registre;      /* Variables registres pour l'interruption */

 Registre.h.ah = 6;                /* Numro de la fonction Scroll-UP */
 Registre.h.al = 0;                                 /* 0 pour effacer */
 Registre.h.bh = 7;                              /* noir / fond blanc */
 Registre.x.cx = 0;                          /* Coin suprieur gauche */
 Registre.h.dh = 24;                           /* Coordonnes du coin */
 Registre.h.dl = 79;                               /* infrieur droit */
 int86(0x10,&Registre,&Registre);/* Dclenche interruption BIOS vido */
}

/**********************************************************************/
/* NEGFLAG : Inverse un indicateur si ncessaire et affiche le texte  */
/*          associ                                                   */
/* Entres : FLAG    = Dernier tat de l'indicateur                   */
/*           FLAGREG = Etat actuel de l'indicateur (0 = inactif)      */
/*           COLONNE = Colonne d'affichage du nom de l'indicateur     */
/*           LIGNE   = Ligne d'affichage du nom de l'indicateur       */
/*           TEXTE   = Nom de l'indicateur                            */
/* Sortie : nouvel tat de l'indicateur(TRUE = actif,FALSE = inactif) */
/**********************************************************************/

BYTE NegFlag(BYTE Flag, unsigned int FlagReg,
	     BYTE Colonne, BYTE Ligne, char * Texte)
{
 BYTE LigneCour,                                    /* Ligne courante */
      ColCour;

 if (!(Flag == (FlagReg != 0)))               /* Indicateur modifi ? */
  {                                                            /* Oui */
   GetPos(&ColCour, &LigneCour);/* Lit la position courante du curseur*/
   WriteTexte(Colonne, Ligne, Texte, (BYTE) ((Flag) ? 0 : CoulIndic));
   SetPos(ColCour, LigneCour);             /* Repositionne le curseur */
   return(Flag ^1);                /* Change le bit 0 de l'indicateur */
  }
 else return(Flag);                        /* sinon tout reste pareil */
}

/**********************************************************************/
/* GETKEY : Lit un caractre et affiche l'tat des indicateurs        */
/* Entre : nant                                                     */
/* Sortie : Code de la touche frappe                                 */
/*               < 256 : touche ordinaire                             */
/*              >= 256 : touche  code tendu                         */
/**********************************************************************/

unsigned int GetKey( void )
{
 int Touche,                                      /* Touche retourne */
     Etat;                                         /* Etat du clavier */

 do
  {
   Etat = GetKbEtat();                       /* Lit l'tat du clavier */
   Insert = NegFlag(Insert, Etat & INS, FC+9, FL, "INSERT");
   Caps = NegFlag(Caps, Etat & CAPL, FC+3, FL, " CAPS ");
   Num = NegFlag(Num, Etat & NUML, FC, FL, "NUM");
  }
 while ( !GetKbReady() );/* Recommence jusqu' dtection d'une frappe */

 Touche = GetKbKey();                                /* Lit la touche */
 return ((Touche & 255) == 0) ? (Touche >> 8) + 256 : Touche & 255;
}

/**********************************************************************/
/* INIKEY : Initialise les indicateurs des touches                    */
/* Entre : nant                                                     */
/* Sortie : nant                                                     */
/* Info   : Les indicateurs sont inverss par rapport  leur tat     */
/*          actuel pour que ce dernier puisse tre affich au         */
/*          prochain appel de GETKEY                                  */
/**********************************************************************/

void IniKey( void )
{
 int Etat;                                         /* Etat du clavier */

 Etat = GetKbEtat();                         /* Lit l'tat du clavier */
 Insert = (Etat & INS) ? FALSE : TRUE;        /* Inverse les contenus */
 Caps = (Etat & CAPL)? FALSE : TRUE;                     /* courants  */
 Num = (Etat & NUML) ? FALSE : TRUE;
 }

/**********************************************************************/
/**                         PROGRAMME PRINCIPAL                      **/
/**********************************************************************/

void main( void )
{
 unsigned int Touche;

 Cls();                                             /* Efface l'cran */
 SetPos(0,0);                             /* Curseur en haut  gauche */
 printf("TOUCHEC  -  (c) 1987, 92 by Michael Tischer\n\n");
printf("Tapez quelques caractres en activant ou dsactivant\n");
printf("les touches INSERT, CAPS ou NUM.\n");
printf("L'tat de ces touches va tre affich  tout moment\n");
printf("dans le coin suprieur droit de l'cran.\n");
printf("La frappe de <Entre> ou <F1> termine le programme...\n\n");
printf("Votre saisie : ");

 IniKey();                   /* Initialise les indicateurs du clavier */
 do
  {
   if ((Touche = GetKey()) < 256)                   /* Lit une touche */
    printf("%c", (char) Touche);  /* Affiche la touche (si ordinaire) */
  }
 while (!(Touche == CR || Touche == F1));       /* Rpte l'opration */
						  /* jusqu' F1 ou CR */
 printf("\n");
}
