;**********************************************************************;
;*                             T Y P M C A                            *;
;*--------------------------------------------------------------------*;
;*    Fonction       : Routine en assembleur  inclure dans un        *;
;*                     programme en C permettant de fixer la vitesse  *;
;*                     Typematic du clavier tendu                    *;
;*--------------------------------------------------------------------*;
;*    Auteur                : MICHAEL TISCHER                         *;
;*    Dvelopp le          : 27.08.1988                              *;
;*    Dernire modification : 27.08.1988                              *;
;*--------------------------------------------------------------------*;
;*    Assemblage     : MASM TYPMCA;    ou       TASM TYPMCA           *;
;*                     ... puis lier avec TYPMC.C                     *;
;**********************************************************************;

;== constantes =========================================================

KB_STATUS_P   equ 64h                            ;Port d'tat du clavier
KB_DATA_P     equ 60h                        ;Port de donnes du clavier

OB_FULL       equ 1                ;Bit 0 dans le port d'tat du clavier
                                  ;un caractre dans le buffer de sortie
IB_FULL       equ 2                ;Bit 1 dans le port d'tat du clavier
                                   ;un caractre dans le buffer d'entre

ACK_SIGNAL    equ 0fah            ;Signal accus de rception du clavier
SET_TYPEM     equ 0f3h                               ;Code Set Typematic

MAX_TRY       equ 3                       ;Nombre de tentatives permises

;== Dclaration des segments pour le programme en C ====================

IGROUP group _text               ;Regroupement des segments de programme
DGROUP group _bss,  _data          ;Regroupement des segments de donnes
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'        ;Segement accueillant toutes les
_BSS   ends                        ;variables statiques non initialises

_DATA  segment word public 'DATA'              ;Segement accueillant les
                                        ;variables globales et statiques
                                                           ;initialises
_DATA  ends

;== Programme===========================================================

_TEXT  segment byte public 'CODE'                  ;Segment de programme

public     _set_typm

;-----------------------------------------------------------------------
;-- SET_TYPM: Transmet la vitesse Typematic au contrleur du clavier ---
;
;-- Appel depuis C: bool set_typem( byte trate );
;-- Renvoie : TRUE, si la vitesse Typematic a pu tre fixe
;--               FALSE si une erreur est survenue

_set_typm  proc near

sframe0    struc                       ;Structure pour accder  la pile
bp0        dw ?                                             ;Mmorise BP
ret_adr0   dw ?                          ;Adresse de retour  l'appelant
trate0     dw ?                               ;Vitesse Typematic  fixer
sframe0    ends                                     ;Fin de la structure

frame      equ [ bp - bp0 ]        ;Adresse les lments de la structure

       push bp                                                ;Empile BP
       mov  bp,sp                                  ;Transfre SP dans BP

       xor  dx,dx                     ;Transmission a priori dfectueuse
       mov  ah,SET_TYPEM          ;Commande de fixation vitesse Typmatic
       cli                                     ;Inhibe les interruptions
       call send_kb                                  ;Vers le contrleur
       jne  error                                ;Erreur ? Oui --> Error

       mov  ah,byte ptr frame.trate0           ;Lit la vitesse Typematic
       call send_kb                                  ;Vers le contrleur
       jne  error                                  ;Erreur Oui --> Error

       inc  dl                               ;tout va bien ,renvoie TRUE

error: sti                                   ;Rtablit les interruptions
       mov  ax,dx                      ;Charge la valeur de retour en AX
       pop  bp                                              ;Restaure BP
       ret                               ;Rend la main au programme en C

_set_typm  endp

;-----------------------------------------------------------------------
;-- SEND_KB: envoie un octet au contrleur du clavier ------------------
;-- Entre   : AH = octet  evoyer
;-- Sortie : Indicateur de zro : 0=Erreur , 1=o.k.
;-- Registres : AX et le registres des indicateurs
;     sont modifis
;-- Info     : Routine  usage interne, ne peut tre appele
;--            qu' partir du prsent module

send_kb   proc near

      push cx                                 ;Empile tous les registres
      push bx                                   ;modifis par la routine

      mov  bl,MAX_TRY                     ;Nombre de tentatives tolres

                ;-- Attend que le contrleur puisse recevoir des donnes

skb_1:    xor  cx,cx                        ;65536 itrations au maximum
skb_2: in   al,KB_STATUS_P                ;Lit le contenu du port d'tat
       test al,IB_FULL        ; Reste-t-il un caractre dans le buffer ?
       loopne skb_2                                       ;Oui --> SKB_2

                                   ;-- Envoie un caractre au contrleur

        mov  al,ah                         ;Transfre le caractre en AL
        out  KB_DATA_P,al                  ;puis dans le port de donnes
skb_3:    in   al,KB_STATUS_P             ;Lit le contenu du port d'tat
        test al,OB_FULL              ;Rponse dans le buffer de sortie ?
        loope skb_3                                      ;Non  --> SKB_3

                 ;-- Lit et exploite la rponse du contrleur ----------

        in   al,KB_DATA_P                        ;Lit le port de donnes
        cmp  al,ACK_SIGNAL                          ;Caractre accept ?
        je   skb_end                                ;Oui--> tout va bien

          ;-- Le caractre n'a pas t accept   -----------------------

        dec  bl                        ;Dcrmente le compteur d'erreurs
        jne  skb_2                               ;Peut-on recommencer  ?
                                                           ;Oui--> SKB_2

        or   bl,1                 ;Non , Mettre  0 l'indicateur de zro
                                                 ;pour signaler l'erreur

skb_end: pop  bx                       ;Dpile les registres sauvegards
         pop  cx                                                       ;
         ret                                      ;Retourne  l'appelant

send_kb   endp

;-----------------------------------------------------------------------

_text  ends                                      ;Fin du segment de code
       end                                             ;Fin du programme
