;**********************************************************************;
;*                            V I O S C A                             *;
;*--------------------------------------------------------------------*;
;*    Fonction       : Fournit une fonction  intgrer dans un        *;
;*                     programme C et servant  dterminer le type    *;
;*                     des deux adaptateurs vido installs.          *;
;*--------------------------------------------------------------------*;
;*    Auteur       : MICHAEL TISCHER                                  *;
;*    Dvelopp le : 02.10.1988                                       *;
;*    Dernire MAJ : 14.02.1992                                       *;
;*--------------------------------------------------------------------*;
;*    Assemblage   : MASM VIOSCA;                                     *;
;*                     ... puis dition de liens avec VIOSC.C         *;
;**********************************************************************;

;== Constantes pour la structure VIOS ==================================

                                      ;-- Constantes pour la carte vido
NO_VIOS    = 0                                      ; Pas de carte vido
VGA        = 1                                               ; Carte VGA
EGA        = 2                                               ; Carte EGA
MDA        = 3                              ; Monochrome Display Adapter
HGC        = 4                                  ; Hercules Graphics Card
CGA        = 5                                  ; Color Graphics Adapter
                                           ; Constantes pour le moniteur
NO_MON     = 0                                         ; Pas de moniteur
MONO       = 1                                     ; Moniteur monochrome
COLOR      = 2                                        ; Moniteur couleur
EGA_HIRES  = 3                  ; Moniteur haute rsolution ou Multisync
ANAL_MONO  = 4                          ; Moniteur analogique monochrome
ANAL_COLOR = 5                             ; Moniteur couleur analogique
; == Dclarations de segment pour le programme C =======================

IGROUP group _text                  ; Regroupement segments de programme
DGROUP group const,_bss,  _data       ; Regroupement segments de donnes
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

CONST  segment word public 'CONST'              ; Segment des constantes
CONST  ends                                         ; pour lecture seule

_BSS   segment word public 'BSS'                 ; Segment des variables
_BSS   ends                                 ; statiques non initialises

_DATA  segment word public 'DATA'                   ; Segment de donnes

vios_tab   equ this byte

      ;-- table de conversion traitant le rsultat retourn par l'option
                              ;-- 00h de la fonction 1Ah du BIOS VGA----

           db NO_VIOS, NO_MON                       ; Pas de carte vido
           db MDA    , MONO          ; Carte MDA sur moniteur monochrome
           db CGA    , COLOR            ; Carte CGA sur moniteur couleur
           db ?      , ?                            ; Code 3 non utilis
           db EGA    , EGA_HIRES                 ; Carte EGA/moniteur HR
           db EGA    , MONO          ; Carte EGA sur moniteur monochrome
           db ?      , ?                            ; Code 6 non utilis
           db VGA    , ANAL_MONO    ; Carte VGA/moniteur mono analogique
           db VGA    , ANAL_COLOR    ; Carte VGA/moniteur couleur analog

ega_dips   equ this byte

             ;-- Table de conversion pour traiter les micro-commutateurs
                                                 ;-- DIP de la carte EGA

           db COLOR, EGA_HIRES, MONO
           db COLOR, EGA_HIRES, MONO

_DATA  ends

; == Programme =========================================================

_TEXT  segment byte public 'CODE'                 ; Segment de programme

public     _get_vios

;-----------------------------------------------------------------------
;-- GET_VIOS: Dtermine le type des cartes vido installes ------------
;-- Appel en C   : void get_vios( struct vios *vp );
;-- Dclaration  : struct vios { BYTE carte, monitor; };
;-- Valeur retourne : Aucune
;-- Infos        : Dans cet exemple, la fonction est conue pour tre
;--                intgre dans le modle SMALL


_get_vios  proc near

sframe     struc                      ; Structure pour accder  la pile
cga_possi  db ?                                        ; Variable locale
ega_possi  db ?                                                ;       "
mono_possi db ?                                                ;       "
bptr       dw ?                                              ; Reoit BP
ret_adr    dw ?                         ; Adresse de retour  l'appelant
vp         dw ?                    ; Pointeur sur la 1re structure VIOS
sframe     ends                                    ; Fin de la structure

frame      equ [ bp - cga_possi ]             ; Elments de la structure

           push bp                           ; Sauvegarde BP sur la pile
           sub  sp,3                  ; Place pour les variables locales
           mov  bp,sp                             ; Transfre SP dans BP
           push di                                ; Sauve DI sur la pile

           mov  frame.cga_possi,1                        ; Peut tre CGA
           mov  frame.ega_possi,1                        ; Peut tre EGA
           mov  frame.mono_possi,1                ; Peut tre MDA ou HGC

           mov  di,frame.vp         ; Recherche l'offset de la structure
           mov  word ptr [di],NO_VIOS              ; Aucun systme vido
           mov  word ptr [di+2],NO_VIOS          ; trouv pour le moment

           call test_vga                            ; Teste si carte VGA
           cmp  frame.ega_possi,0          ; Carte EGA encore possible ?
           je  gv1                          ; NON --> passer au test CGA

           call test_ega                            ; Teste si carte EGA
gv1:       cmp  frame.cga_possi,0          ; Carte CGA encore possible ?
           je   gv2                   ; NON --> passe au test MDA et HGC
           call test_cga                            ; Teste si carte CGA
gv2:       cmp  frame.mono_possi,0  ; Carte MDA ou HGC encore possible ?
           je   gv3                             ; NON --> Tests termins

           call test_mono                    ; Teste si carte MDA et HGC


           ;-- Dtermine la carte vido active -------------------------

gv3:       cmp  byte ptr [di],VGA               ; Carte VGA identifie ?
           je   gvi_end              ; OUI, carte active dj identifie
           cmp  byte ptr [di+2],VGA      ; Carte VGA= systme secondaire
           je   gvi_end              ; OUI, carte active dj identifie

           mov  ah,0Fh             ; Dtermine le mode vido courant par
           int  10h                               ; l'interruption vido

           and  al,7           ; Seuls les modes 0  7 sont intressants
           cmp  al,7                         ; Carte monochrome active ?
           jne  gv4                            ; Non, en mode CGA ou EGA
           ;-- La carte MDA, HGC ou EGA (mono) est active --------------

           cmp  byte ptr [di+1],MONO                     ; Moniteur mono
           je   gvi_end                             ; OUI, ordre correct
           jmp  short echange                   ; NON, intervertir ordre

           ;-- Une carte CGA ou EGA est active -------------------------

gv4:       cmp  byte ptr [di+1],MONO   ; Moniteur mono ds 1re structure
           jne  gvi_end                             ; NON, ordre correct

echange:   mov  ax,[di]               ; contenu de la premire structure
           xchg ax,[di+2]               ; Echange avec seconde structure
           mov  [di],ax

gvi_end:   pop  di                                ; Retire DI de la pile
           add  sp,3          ; Elimine les variables locales de la pile
           pop  bp                                ; Retire BP de la pile
           ret                                   ; Retour au programme C

_get_vios  endp

;-----------------------------------------------------------------------
;-- TEST_VGA: Dtermine si une carte VGA est installe

test_vga   proc near

           mov  ax,1a00h       ; Appelle l'option 00h de la fonction 1Ah
           int  10h                                            ; du BIOS
           cmp  al,1ah                            ; Fonction supporte ?
           jne  tvga_end                    ; NON --> Termine la routine

          ;-- La fonction est supporte , BL contient maintenant le code
          ;-- du systme vido actif, BH celui du systme non actif-----

           mov  cx,bx                        ; Range le rsultat dans CX
           xor  bh,bh                                       ; Met BH  0
           or   ch,ch                          ; Un seul systme vido ?
           je   tvga_1                 ; OUI --> Code du premier systme

          ;-- Convertit le code du second systme ----------------------

           mov  bl,ch                     ; Code du second systme en BL
           add  bl,bl                   ; Calcule l'offset dans la table
           mov  ax,offset DGROUP:vios_tab[bx]       ; Code dans la table
           mov  [di+2],ax           ; stocke dans la structure appelante
           mov  bl,cl       ; Remet dans BL les codes du premier systme

          ;-- Convertit le code du premier systme ---------------------

tvga_1:    add  bl,bl                   ; Calcule l'offset dans la table
           mov  ax,offset DGROUP:vios_tab[bx]       ; Code dans la table
           mov  [di],ax             ; stocke dans la structure appelante

           mov  frame.cga_possi,0     ; Le test CGA n'a plus lieu d'tre
           mov  frame.ega_possi,0                 ; Le test EGA non plus
           mov  frame.mono_possi,0                 ; Reste  tester MONO

           mov  bx,di                   ; Adresse de la structure active
           cmp  byte ptr [bx],MDA       ; Systme monochrome identifi ?
           je   do_tmono                 ; OUI --> Effectue test MDA/HGC

           add  bx,2                  ; Adresse de la structure inactive
           cmp  byte ptr [bx],MDA       ; Systme monochrome identifi ?
           jne  tvga_end                        ; NON termine la routine

do_tmono:  mov  word ptr [bx],0        ; Simule la non-identification du
                                                               ; systme
           mov  frame.mono_possi,1          ; Excute le test monochrome

tvga_end:  ret                            ; Retour au programme appelant

test_vga   endp


;-----------------------------------------------------------------------
;-- TEST_EGA: Dtermine si une carte EGA est installe

test_ega   proc near

           mov  ah,12h                                    ; Fonction 12h
           mov  bl,10h                                      ; Option 10h
           int  10h                                ; Appelle le BIOS EGA
           cmp  bl,10h                            ; Fonction supporte ?
           je   tega_end                    ; NON --> Termine la routine

         ;-- La fonction est supporte , CL contient maintenant --------
         ;-- la position des micro-commutateurs DIP de la carte EGA ----

           mov  al,cl                           ; Commutateurs DIP en AL
           shr  al,1              ; Dcale d'une position vers la droite
           mov  bx,offset DGROUP:ega_dips           ; Offset de la table
           xlat                           ; Elment AL de la table en AL
           mov  ah,al                           ; Type de moniteur en AH
           mov  al,EGA                             ; C'est une carte EGA
           call trouve             ; Inscrit les donnes dans le vecteur

           cmp  ah,MONO              ; Connexion sur  cran monochrome ?
           je   is_mono                          ; OUI --> ni MDA ou HGC

           mov  frame.cga_possi,0                 ; Carte CGA impossible
           jmp  short tega_end                     ; Terminer la routine

is_mono:   mov  frame.mono_possi,0       ; carte EGA est sur un moniteur
                                  ; monochrome, ni MDA ni HGC ne peuvent
                                                       ; tre installes

tega_end:  ret                            ; Retour au programme appelant

test_ega   endp

;-----------------------------------------------------------------------
;-- TEST_CGA: Dtermine si une carte CGA est installe

test_cga   proc near

           mov  dx,3D4h      ; Adresse de port du registre CRTC pour CGA
           call test_6845              ; Teste si 6845 CRTC est install
           jc   tega_end                       ; NON --> Termine le test
           mov  al,CGA                          ; OUI, CGA est installe
           mov  ah,COLOR         ; Toujours un moniteur couleur avec CGA
           jmp  trouve             ; Inscrit les donnes dans le vecteur

test_cga   endp

;-----------------------------------------------------------------------
;-- TEST_MONO: Teste la prsence d'une carte MDA ou HGC

test_mono  proc near

           mov  dx,3B4h        ; Adr. de port du registre CRTC pour MONO
           call test_6845              ; Teste si 6845 CRTC est install
           jc   tega_end                       ; NON --> Termine le test

           ;-- Une carte d'cran monochrome est installe --------------
           ;-- Le test suivant fait la distinction entre MDA et HGC ----

           mov  dl,0BAh                       ; Port d'tat MONO en 3BAh
           in   al,dx                             ; Teste le port d'tat
           and  al,80h                           ; Ne garde que le bit 7
           mov  ah,al                              ; et le range dans AH

           ;-- Si le contenu du bit 7 du port d'tat est modifi au ----
           ;-- cours de l'un des tests suivants, c'est qu'il s'agit ----
           ;-- d'une carte HGC                                      ----

           mov  cx,8000h                   ; 32768 itrations au maximum
test_hgc:  in   al,dx                             ; Teste le port d'tat
           and  al,80h                           ; Ne garde que le bit 7
           cmp  al,ah                 ; Son contenu a-t-il t modifi ?
           jne  is_hgc               ; Bit 7 = 1 --> c'est une carte HGC
           loop test_hgc                            ; Poursuit la boucle

           mov  al,MDA            ; Le bit 7!= 1 --> c'est une carte MDA
           jmp  set_mono                           ; Fixe les paramtres

is_hgc:    mov  al,HGC   ; Le bit 7 est devenu 1 --> c'est une carte HGC
set_mono:  mov  ah,MONO            ; MDA et HGC seulement sur cran MONO
           jmp  trouve                             ; Fixe les paramtres

test_mono  endp


;-- TEST_6845: retourne un indicateur de retenue  1 si aucun 6845
;--            ne figure  l'adresse de port en DX

test_6845  proc near

           mov  al,0Eh                          ; Appelle le registre 14
           out  dx,al      ; N registre dans le registre d'adresse CRTC
           inc  dx          ; DX maintenant sur registre de donnes CRTC

           in   al,dx              ; Recherche le contenu du registre 14
           mov  ah,al                                ; et le range en AH

           mov  al,4Fh                          ; Ecrit dans le registre
           out  dx,al                            ; une valeur quelconque

           mov  cx,100             ; Petite boucle d'attente pour que le
wait:      loop wait                                ; 6845 puisse ragir

           in   al,dx                  ; Relit le contenu du registre 14
           xchg al,ah                             ; Intervertit AH et AL
           out  dx,al                       ; Restaure l'ancienne valeur

           cmp  ah,4Fh             ; La valeur crite a-t-elle t lue ?
           je   t6845_end                      ; OUI --> Termine le test

           stc                 ; NON --> Met  1 l'indicateur de retenue

t6845_end: ret                                     ; Retour  l'appelant

test_6845  endp

;-----------------------------------------------------------------------
;-- TROUVE : enregistre dans le vecteur vido le type de carte vido
;--          (en AL) et le type de moniteur (en AH)

trouve   proc near

           mov bx,di                    ; Adresse de la structure active
           cmp word ptr [bx],0                          ; systme vido?
           je  set_data       ; NON --> Donnes dans la structure active

           add bx,2            ; OUI, adresse de la structure dsactive

set_data:  mov [bx],ax             ; Place les donnes dans la structure
           ret                            ; Retour au programme appelant

trouve   endp

;-----------------------------------------------------------------------

_text      ends                                 ; Fin du segment de code
           end                                        ; Fin du programme

