{**********************************************************************}
{*                          D I R P 1 . P A S                         *}
{*--------------------------------------------------------------------*}
{*    Fonction        : Affiche tous les fichiers d'un rpertoire     *}
{*                     quelconque y compris les sous-rpertoires et   *}
{*                     noms de volumes sur l'cran.                   *}
{*                     La demande s'effectue  travers les deux       *}
{*                     fonctions DOS $4E et $4F.                      *}
{*                     Reportez-vous galement au programme           *}
{*                     DIRP2.PAS.                                     *}
{*--------------------------------------------------------------------*}
{*    Auteur          : MICHAEL TISCHER                               *}
{*    Dvelopp le    :  8.07.1988                                    *}
{*    Dernire modif. : 15.03.1992                                    *}
{**********************************************************************}

program DIRP1;

Uses Crt, Dos;                         { Insrer les units CRT et DOS }

{-- Dclarations de type ----------------------------------------------}

type DirBufTyp = record     { Structure de donnes des fonc.$4E et $4F }
                   Reserve    : array [1..21] of char;
                   Attr       : byte;
                   Time       : integer;
                   Date       : integer;
                   Size       : longint;
                   Name       : array [1..13] of char
                 end;

     MonVec    = array[1..12] of string[3]; {Tableau contenant les mois}

{-- Constantes --------------------------------------------------------}

const FA_ReadOnly  = $01;                       { Attributs de fichier }
      FAHidden     = $02;
      FA_SysFile   = $04;
      FA_VolumeID  = $08;
      FA_Directory = $10;
      FA_Archive   = $20;
      FA_AnyFile   = $3F;

      EINTR = 14;       { Autant d'entres sont visibles simultanment }
      Mois : MonVec = ( 'Jan', 'Fev', 'Mar', 'Avr', 'Mai', 'Jun',
                        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');



{**********************************************************************}
{* FindFirst: Lire la premire entre de rpertoire                   *}
{* Entre : Aucune                                                    *}
{* Sortie : TRUE ou FALSE, selon qu'une entre a t trouve          *}
{*           ou non                                                   *}
{**********************************************************************}

function FindFirst(NomFichier : string;        { Fichiers  rechercher }
                   Attribut  : integer) : boolean; {Attribut recherch }

var Regs : Registers;
                 { Registres du processeur pour l'appel d'interruption }

begin
  NomFichier := NomFichier + #0;         { Terminer NomFichier par NUL }
  Regs.ah := $4E;           { Numro de fonction pour Search for first }
  Regs.cx := Attribut;                          { Attributs recherchs }
  Regs.ds := Seg(NomFichier);     {Adresse de segment du nom de fichier}
  Regs.dx := succ(Ofs(NomFichier)); {Adresse d'offset du nom de fichier}
  MsDos( Regs );

                                      { Appeler l'interruption DOS $21 }
  FindFirst := ( (Regs.flags and 1) = 0 )          { Tester Flag Carry }
end;

{**********************************************************************}
{* FindNext: Lire l'entre de rpertoire suivante                     *}
{* Entre : Aucune                                                    *}
{* Sortie : true ou false selon qu'une entre a t trouve           *}
{*             ou non                                                 *}
{* Infos    : Cette fonction ne peut tre appele que si l'appel de   *}
{*           FinFirst a abouti                                        *}
{**********************************************************************}

function FindNext : boolean;

var Regs : Registers;
                  { Registres du processeur pour l'appel d'interruption}

begin
  Regs.ah := $4F;            { Numro de fonction pour Search for next }
  MsDos( Regs );                      { Appeler l'interruption DOS $21 }
  FindNext := ( (Regs.flags and 1) = 0 )           { Tester Flag Carry }
end;

{**********************************************************************}
{* SetDTA: Spcifie l'adresse de DTA                                  *}
{* Entre : SEGMENT = Adresse de segment du nouveau DTA               *}
{*           OFFSET  = Adresse d'offset du nouveau DTA                *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure SetDTA(Segment,           {Nouvelle adresse de segment du DTA}
                 Offset  : integer);  {Nouvelle adresse d'offset du DTA}

var Regs : Registers;
                 { Registres du processeur pour l'appel d'interruption }

begin
  Regs.ah := $1A;        { Spcifier le numro de fonction pour le DTA }
  Regs.ds := Segment;         { Adresse de segment dans le registre DS }
  Regs.dx := Offset;            { Adresse d'offset dans le registre DX }
  MsDos( Regs );                      { Appeler l'interruption DOS $21 }
end;

{**********************************************************************}
{* PRINTDATA: Afficher les informations  propos d'une entre         *}
{* Entre : DIRBUF = Structure de donnes avec informations de fichier*}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure PrintData( DirBuf : DirBufTyp );

var Compteur : byte;

begin
  writeln;          { Fait avancer la fentre d'une ligne vers le haut }

  Compteur := 1;             { Commencer par la premire lettre du nom }
  while (DirBuf.Name[Compteur]<>#0) do           { Rpter jusqu' NUL }
   begin
    write(DirBuf.Name[Compteur]);     { Afficher les caractres du nom }
    Compteur := succ(Compteur)          { Traiter le caractre suivant }
   end;

  GotoXY(13, EINTR);
  write('', DirBuf.Size:7);
  GotoXY(21, EINTR);
  write(' ', DirBuf.Date and 31:2,' ');          { Dterminer le jour }
  write (Mois[DirBuf.Date shr 5 and 15]);           { Afficher le mois }
  write(DirBuf.Date shr 9 + 1980:5);              { Dterminer l'anne }
  GotoXY(34, EINTR);
  write('  ', DirBuf.Time shr 11:2, 'h');        { Dterminer l'heure }
  write(DirBuf.Time shr 5 and 63:2);            { Dterminer la minute }

  GotoXY(44, EINTR);
  write('');            { Trait de sparation avec le champ prcdent }
  Compteur := 1;                              { Afficher les attributs }
  while ( Compteur < 32 ) do
    begin
      if (DirBuf.Attr and Compteur) <> 0 then write('X')
                                             else write(' ');
      Compteur := Compteur shl 1;
    end;
  write('');                         { Ct droit du cadre de fentre }
end;

{**********************************************************************}
{* ConfigEcran: Configure l'cran pour la sortie                      *}
{*                   du rpertoire                                    *}
{* Entre : Aucune                                                    *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure ConfigEcran;

var Compteur : integer;                           { Compteur de boucle }

begin
 ClrScr;                                             { Effacer l'cran }
 Window(14,(20-EINTR) shr 1+1,64,(20-EINTR) shr 1 +5+EINTR);
 GotoXY(1,1);    { Curseur dans le coin suprieur gauche de la fentre }

 write('ͻ');
 write(' Nom         Taille   Date       Heure  RHSVD');
 write('Ķ');

 for Compteur := 1 to EINTR do
  write('                                             ');
 write('ͼ');

 Window(15,(20-EINTR) shr 1+4,66,(20-EINTR) shr 1 +3+EINTR);
 GotoXY(1, EINTR);          { Curseur dans coin sup. gauche de fentre }
end;

{**********************************************************************}
{* Dir: Contrle la lecture et la sortie du rpertoire                *}
{* Entre : CHEMIN     = Chemin de recherche avec masque de fichier   *}
{*           ATTRIBUT = Attribut de recherche                         *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure Dir( CHEMIN : string; Attr : byte );

var NbEntrees,                     { Nombre total des entres trouves }
    NbImage      : integer;            { Nombre d'entres dans l'image }
    Touche        : char;     { Sert  attendre l'appui sur une touche }
    DirBuf       : DirBufTyp;                  { Reoit une entre Dir }

begin
  SetDTA(Seg(DirBuf), Ofs(DirBuf));        { DirBuf est le nouveau DTA }
  clrscr;                                            { Effacer l'cran }
  ConfigEcran;       { Construire l'cran pour la sortie du rpertoire }

  NbImage := -1; { Aucune entre n'est encore affiche dans la fentre }
  NbEntrees := 0;                 { Aucune entre n'est encore trouve }
  if FindFirst( CHEMIN, Attr ) then    { Rechercher la premire entre }
    repeat
      NbEntrees := succ(NbEntrees);         { Une autre entre trouve }
      NbImage := succ(NbImage);     { Une autre entre dans la fentre }
      if NbImage = EINTR then            { La fentre est-elle pleine? }
        begin                                                    { OUI }
          Window( 14, (20-EINTR) shr 1 + 5+ EINTR,
                  66 ,(20-EINTR) shr 1 + 6+ EINTR );
          GotoXY(1, 1);           { Curseur dans ligne inf. de fentre }
          TextBackground( LightGray );                    { Fond blanc }
          TextColor( Black );                         { Ecriture noire }
          write('          Veuillez appuyer sur une touche          ');
          Touche := ReadKey;                          { Lire la touche }
          GotoXY(1, 1);     { Curseur dans coin sup. gauche de fentre }
          TextBackground( Black );                         { Fond noir }
          TextColor( LightGray );                  { Ecriture en blanc }
          write('                                                   ');
          Window(15,(20-EINTR) shr 1+4,65,(20-EINTR) shr 1 +3+EINTR);
          GotoXY(1, EINTR);       { Replacer curseur sur anc. position }
          NbImage := 0;                      { Reprendre le calcul  0 }
        end;
      PrintData( DirBuf );          { Afficher les donnes de l'entre }
    until not(FindNext);                 { Y a-t-il une autre entre ? }

  Window(14,(20-EINTR) shr 1 +5+EINTR,65,(20-EINTR) shr 1 +6+EINTR);
  GotoXY(1, 1);  { Curseur dans le coin suprieur gauche de la fentre }
  TextBackground( LightGray );                            { Fond blanc }
  TextColor( Black );                                 { Ecriture noire }
  write('                                                   ');

  GotoXY(2, 1);
  case NbEntrees of
    0 : write('Aucun fichier trouv');
    1 : write('Un fichier trouv');
    else write(NbEntrees,' Fichiers trouvs')
  end;

 Window(1, 1, 80, 25);        { Reconfigurer l'cran comme une fentre }
end;

{**********************************************************************}
{**                           PROGRAMME PRINCIPAL                    **}
{**********************************************************************}

begin
  case ParamCount of        { Ragir en fonction du nombre d'arguments }
    0  : Dir( '*.*', FA_AnyFile );    { Tous fichiers du rp. en cours }
    1  : Dir( ParamStr(1), FA_AnyFile );     { Afficher fich. spcifi }
    else writeln('Nombre de paramtres incorrect');
  end;
end.
