{**********************************************************************}
{*                          D V I P                                   *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Montre comment accder directement  la        *}
{*                     mmoire d'cran en Turbo Pascal.               *}
{*--------------------------------------------------------------------*}
{*    Auteur         : MICHAEL TISCHER                                *}
{*    Dvelopp le   :  2/10/1987                                     *}
{*    Dernire MAJ   : 14/01/1991                                     *}
{**********************************************************************}

program DVIP;

Uses Crt, Dos;                         { Intgre les units CRT et DOS }

const NORMAL        = $07;        { Dfinit les attributs de caractre }
      CLAIR         = $0f;                    { pour une carte d'cran }
      INVERSE       = $70;                                { monochrome }
      SOULIGNE      = $01;
      CLIGNOTANT    = $80;

      NOIR          = $00;                      { Attributs de couleur }
      BLEU          = $01;
      VERT          = $02;
      CYAN          = $03;
      ROUGE         = $04;
      MAGENTA       = $05;
      BRUN          = $06;
      GRISCLAIR     = $07;
      GRISFONCE     = $01;
      BLEUCLAIR     = $09;
      VERTCLAIR     = $0A;
      CYANCLAIR     = $0B;
      ROUGECLAIR    = $0C;
      MAGENTACLAIR  = $0D;
      JAUNE         = $0E;
      BLANC         = $0F;

type TextTyp = string[80];

var VSeg : word;            { Adresse de segment de la mmoire d'cran }

{**********************************************************************}
{* InitDPrint: Dtermine le segment de la mmoire d'cran pour DPrint *}
{* Entre : Aucune                                                    *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure InitDPrint;

var CRTC_PORT : word absolute $0040:0063;{ ariable dans seg. var. BIOS }

begin
  if CRTC_PORT = $3B4 then          { Adaptateur monochrome connect ? }
    VSeg := $B000           { OUI, la mmoire d'cran est en B000:0000 }
  else                       { Non, ce doit tre un adaptateur couleur }
    VSeg := $B800;               { La mmoire d'cran est en B800:0000 }
end;

{**********************************************************************}
{* DPrint: Ecrit une chane directement dans la mmoire d'cran.      *}
{* Entre : - COLONNE : colonne d'affichage                           *}
{*          - LIGNE   : ligne d'affichage                             *}
{*          - COULEUR : couleur (ou attribut) des caractres.         *}
{*          - STROUT  : chane  afficher                             *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure DPrint( Colonne, Ligne, Couleur : byte; StrOut : TextTyp);

var PAGE_OFS : word absolute $0040:$004E;   { Variable  seg. var. BIOS }
    Offset   : word;   { Pointeur sur la position d'affichage courante }
    i, j     : byte;                          { Compteurs d'itrations }
    Attribut : word;                            { Attribut d'affichage }

begin
  Offset := Ligne * 160 + Colonne * 2 + PAGE_OFS;
  Attribut := Couleur shl 8;{ Octet fort pour accs  la mmoire cran }
  i := length( StrOut );          { Dtermine la longueur de la chane }
  for j:=1 to i do                                { Parcourt la chane }
    begin { Ecrit caractre et attribut directement dans mmoire cran }
      memw[VSeg:Offset] := Attribut or ord( StrOut[j] );
      Offset := Offset + 2;  { Passe au prochain couple ASCII-attribut }
    end;
end;

{**********************************************************************}
{* Dmo   : Dmonstration du fonctionnement de DPrint.                *}
{* Entre : Aucune                                                    *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure demo;

var Colonne,                           { Position d'affichage courante }
    Ligne,
    Couleur   : integer;

begin
  TextBackGround( NOIR );                     { Colore le fond en noir }
  ClrScr;                                             { Efface l'cran }
  DPrint( 22, 0, BLANC, 'DVIP  - (c) 1988, 1992 by Michael Tischer');
  Randomize;              { Active le gnrateur de nombres alatoires }
  while not KeyPressed do  { Rpte opration jusqu' touche actionne }
    begin
      Colonne := Random( 76 );             { Colonne, ligne et couleur }
      Ligne := Random( 22 ) + 1;              { sont choisis au hasard }
      Couleur := Random( 14 ) + 1;
      DPrint( Colonne, Ligne,   Couleur, '');    { Affiche un pav }
      DPrint( Colonne, Ligne+1, Couleur, '');
    end;
  ClrScr;                                   { Efface  nouveau l'cran }
end;

{**********************************************************************}
{**                       PROGRAMME PRINCIPAL                        **}
{**********************************************************************}

begin
  InitDPrint;                      { Initialise l'affichage par DPrint }
  Demo;                         { Dmontre le fonctionnement de DPrint }
end.
