{**********************************************************************}
{*                              E M M P                               *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Fournit diffrentes fonctions permettant       *}
{*                     d'accder  la mmoire EMS ou  l'EMM.         *}
{*--------------------------------------------------------------------*}
{*    Auteur         : MICHAEL TISCHER                                *}
{*    Dvelopp le   : 30/08/1988                                     *}
{*    Dernire modif.: 30/03/1992                                     *}
{**********************************************************************}

program EMMP;

Uses Dos, CRT;                        { Intgrer les units DOS et CRT }

type  ByteBuf = array[0..1000] of byte;      { zone mmoire comme Byte }
      CharBuf = array[0..1000] of char;      { zone mmoire comme Char }
      BytePtr = ^ByteBuf;              { Pointeur sur une zone d'octet }
      CharPtr = ^CharBuf;                 { Pointeur sur une zone Char }

const EMS_INT   = $67;        { N d'interruption pour l'accs  l'EMM }
      EMS_ERR   = -1;                    { Est renvoy en cas d'erreur }
      W_EMS_ERR = $FFFF;                { Code d'erreur en format WORD }
      EmmName   : array[0..7] of char = 'EMMXXXX0';     { Nom de l'EMM }

var   EmmEC,                  { Ici sont placs les codes d'erreur EMM }
      i        : byte;                            { Compteur de boucle }
      Handle,                   { Handle pour l'accs  la mmoire EMS }
      EmmVer   : integer;                 { Numro de version de l'EMM }
      NmbPage,                                   { Nombre de pages EMS }
      PageSeg  : word;              { Adresse de segment du Page Frame }
      Touche    : char;


{**********************************************************************}
{* EmsInst: Dtermine si mmoire EMS et un EMM correspondant          *}
{*          installs.                                                *}
{* Entre : Aucune                                                    *}
{* Sortie : TRUE si mmoire EMS prsente, sinon FALSE.                *}
{**********************************************************************}

function EmsInst : boolean;

type  EmmName  = array[1..8] of char;              { Nom du driver EMS }
      EmmNaPtr = ^EmmName;                       { Pointeur sur le nom }

const Name : EmmName = 'EMMXXXX0';             { Dsignation du driver }

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
        {*-- Mettre en place ptr sur nom dans en-tte driver de priph.}

  Regs.ax := $35 shl 8 + EMS_INT; { N fct.: Rech. vec. interrupt. $67 }
  MsDos( Regs );                        { Appeler interruption DOS $21 }

  EmsInst := (EmmNaPtr(Ptr(Regs.ES,10))^ = Name);
                                                  { TRUE si nom trouv }
end;

{**********************************************************************}
{* EmsNumPage: Dtermine le nombre total de pages EMS.                *}
{* Entre : Aucune                                                    *}
{* Sortie : EMS_ERR en cas d'erreur, sinon nombre de pages EMS.       *}
{**********************************************************************}

function EmsNumPage : integer;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $42;                { N fct.: Dterminer nombre de pages }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  if( Regs.ah <>0 ) then               { Une erreur est-elle apparue ? }
    begin                                                        { Oui }
      EmmEC := Regs.ah;                      { Ranger le code d'erreur }
      EmsNumPage := EMS_ERR;                       { Afficher l'erreur }
    end
  else                                                  { Pas d'erreur }
    EmsNumPage := Regs.dx;         { Renvoyer le nombre total de pages }
end;

{**********************************************************************}
{* EmsFreePage: Dtermine le nombre de pages EMS encore libres.       *}
{* Entre : Aucune                                                    *}
{* Sortie : EMS_ERR en cas d'erreur, sinon nombre de pages EMS        *}
{*          non encore rserves.                                     *}
{**********************************************************************}

function EmsFreePage : integer;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $42;                { N fct.: Dterminer nombre de pages }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  if (Regs.ah <>0 ) then               { Une erreur est-elle apparue ? }
    begin                                                        { Oui }
      EmmEC := Regs.ah;                      { Ranger le code d'erreur }
      EmsFreePage := EMS_ERR;                      { Afficher l'erreur }
    end
  else                                                  { Pas d'erreur }
    EmsFreePage := Regs.bx;       { Renvoyer le nombre de pages libres }
end;

{**********************************************************************}
{* EmsFrameSeg: Dtermine l'adresse de segment du Page Frame.         *}
{* Entre : Aucune                                                    *}
{* Sortie : EMS_ERR en cas d'erreur, sinon adr. seg. obtenue          *}
{**********************************************************************}

function EmsFrameSeg : word;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $41;            { N fct.: Rech adr segment de Page Frame }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  if (Regs.ah <>0 ) then               { Une erreur est-elle apparue ? }
    begin                                                        { Oui }
      EmmEC := Regs.ah;                      { Ranger le code d'erreur }
      EmsFrameSeg := W_EMS_ERR;                    { Afficher l'erreur }
    end
  else                                                  { Pas d'erreur }
    EmsFrameSeg := Regs.bx;   { Renvoyer adresse segment de Page Frame }
end;

{**********************************************************************}
{* EmsAlloc: Alloue le nombre de pages spcifi et renvoie un handle  *}
{*           pour l'accs  ces pages.                                *}
{* Entre : PAGES: nombre de pages  allouer.                         *}
{* Sortie : EMS_ERR en cas d'erreur, sinon le handle.                 *}
{**********************************************************************}

function EmsAlloc( Pages : integer ) : integer;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $43;                             { N fct.: Allouer pages }
  Regs.bx := Pages;                  { Fixer nombre de pages  allouer }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  if (Regs.ah <>0 ) then               { Une erreur est-elle apparue ? }
    begin                                                        { Oui }
      EmmEC := Regs.ah;                      { Ranger le code d'erreur }
      EmsAlloc := EMS_ERR;                         { Afficher l'erreur }
    end
  else                                                  { Pas d'erreur }
    EmsAlloc := Regs.dx;                             { Renvoyer handle }
end;

{**********************************************************************}
{* EmsMap  : Calque une des pages logiques alloues sous le handle    *}
{*           transmis sur la page physique du Page Frame.             *}
{* Entre : HANDLE: Le Handle fourni par EmsAlloc.                    *}
{*          LOGP  : La page logique  calquer                         *}
{*          PHYSP : La page physique dans le Page Frame.              *}
{* Sortie : FALSE en cas d'erreur, sinon TRUE.                        *}
{**********************************************************************}

function EmsMap(Handle, LogP : integer; PhysP : byte) : boolean;

var Regs : Registers;   { Registres processeur pour appel interruption }
begin
  Regs.ah := $44;                             { N fct.: Fixer Mapping }
  Regs.al := PhysP;                              { Fixer page physique }
  Regs.bx := LogP;                                { Fixer page logique }
  Regs.dx := Handle;                                { Fixer Handle EMS }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  EmmEC := Regs.ah;                             { Ranger code d'erreur }
  EmsMap := (Regs.ah = 0)              { Renvoyer TRUE si pas d'erreur }
end;

{**********************************************************************}
{* EmsFree : Libre  nouveau la mmoire alloue sous un Handle       *}
{* Entre : HANDLE: Le Handle fourni par EmsAlloc.                    *}
{* Sortie : FALSE en cas d'erreur, sinon TRUE.                        *}
{**********************************************************************}

function EmsFree(Handle : integer) : boolean;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $45;                             { N fct.: Librer pages }
  Regs.dx := handle;                                { Fixer Handle EMS }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  EmmEC := Regs.ah;                             { Ranger code d'erreur }
  EmsFree := (Regs.ah = 0)             { Renvoyer TRUE si pas d'erreur }
end;

{**********************************************************************}
{* EmsVersion: Dtermine le numro de version de l'EMM                *}
{* Entre : Aucune                                                    *}
{* Sortie : EMS_ERR en cas d'erreur, sinon le numro de version.      *}
{*          11 signifie 1.1, 40 signifie 4.0 etc.                     *}
{**********************************************************************}

function EmsVersion : integer;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $46;                    { N fct.: Dterminer version EMM }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  if (Regs.ah <>0 ) then               { Une erreur est-elle apparue ? }
    begin                                                        { Oui }
      EmmEC := Regs.ah;                      { Ranger le code d'erreur }
      EmsVersion := EMS_ERR;                       { Afficher l'erreur }
    end
  else   { Pas d'erreur, calculer numro de version d'aprs nombre BCD }
   EmsVersion := (Regs.al and 15) + (Regs.al shr 4) * 10;
end;

{**********************************************************************}
{* EmsSaveMap: Sauvegarde calquage entre pages logiques et physiques  *}
{*             pour le Handle transmis.                               *}
{* Entre : HANDLE: Le Handle fourni par EmsAlloc.                    *}
{* Sortie : FALSE en cas d'erreur, sinon TRUE.                        *}
{**********************************************************************}

function EmsSaveMap( Handle : integer ) : boolean;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $47;                       { N fct.: Sauvegarder Mapping }
  Regs.dx := handle;                                { Fixer Handle EMS }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  EmmEC := Regs.ah;                             { Ranger code d'erreur }
  EmsSaveMap := (Regs.ah = 0)          { Renvoyer TRUE si pas d'erreur }
end;

{**********************************************************************}
{* EmsRestoreMap: Rtablit le calquage entre pages logiques et        *}
{*                physiques tel qu'il avait t sauvegard auparavant *}
{*                avec EmsSaveMap.                                    *}
{* Entre : HANDLE: Le Handle fourni par EmsAlloc.                    *}
{* Sortie : FALSE en cas d'erreur, sinon TRUE.                        *}
{**********************************************************************}

function EmsRestoreMap( Handle : integer ) : boolean;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $48;                          { N fct.: Rtablir Mapping }
  Regs.dx := handle;                                { Fixer Handle EMS }
  Intr(EMS_INT, Regs);                                   { Appeler EMM }
  EmmEC := Regs.ah;                             { Ranger code d'erreur }
  EmsRestoreMap := (Regs.ah = 0)       { Renvoyer TRUE si pas d'erreur }
end;

{**********************************************************************}
{* PrintErr: Sort un message d'erreur EMS et termine le programme.    *}
{* Entre : Aucune                                                    *}
{* Sortie : Aucune                                                    *}
{* Infos   : Cette fonction ne doit tre appele que si une erreur a  *}
{*           t signale lors d'un appel pralable d'une fonction de *}
{*           ce module.                                               *}
{**********************************************************************}

procedure PrintErr;

begin
  writeln('ATTENTION ! Erreur d''accs  la mmoire EMS');
  write('     ... ');
  if ((EmmEC<$80) or (EmmEc>$8E) or (EmmEc=$82)) then
    writeln('non identifiable')
  else
    case EmmEC of
      $80 : writeln('Erreur dans le driver EMS (EMM dtruit)');
      $81 : writeln('Erreur dans l''lectronique EMS');
      $83 : writeln('Handle EMM incorrect');
      $84 : writeln('Fonction EMS appele n''existe pas');
      $85 : writeln('Plus de handles EMS disponibles');
      $86 : writeln('Erreur de sauvegarde ou de reconstitution ',
                    'du Mapping');
      $87 : writeln('Plus de pages rclames qu''il n''en existe ',
                    'physiquement');
      $88 : writeln('Plus de pages rclames qu''il n''en reste',
                     ' de libres');
      $89 : writeln('Zro page rclame');
      $8A : writeln('Page logique ne correspondant pas au Handle');
      $8B : writeln('Numro de page physique incorrect');
      $8C : writeln('Zone de mmoire de Mapping pleine');
      $8D : writeln('Sauvegarde du Mapping dj effectue');
      $8E : writeln('Reconstituation du Mapping sans ',
                    'sauvegarde antrieure');
    end;
  Halt;                                           { Terminer programme }
end;

{**********************************************************************}
{* VrAdr: Fournit un pointeur sur la RAM vido.                       *}
{* Entre : Aucune                                                    *}
{* Sortie : Pointeur sur la RAM vido.                                *}
{**********************************************************************}

function VrAdr : BytePtr;

var Regs : Registers;   { Registres processeur pour appel interruption }

begin
  Regs.ah := $0f;                     { N fct.: Dterminer mode vido }
  Intr($10, Regs);                { Appeler interruption vido du BIOS }
  if (Regs.al = 7) then                   { Carte d'cran monochrome ? }
    VrAdr := ptr($B000, 0)               { Oui, RAM vido en B000:0000 }
  else                                     { Carte couleur, EGA ou VGA }
    VrAdr := ptr($B800, 0);                   { RAM vido en B800:0000 }
end;

{**********************************************************************}
{* PageAdr: Fournit l'adresse d'une page physique dans le Page Frame  *}
{* Entre : PAGE: Numro de page physique (0  3)                     *}
{* Sortie : Pointeur sur la page physique.                            *}
{**********************************************************************}

function PageAdr( Page : integer ) : BytePtr;

begin
  PageAdr := ptr( EmsFrameSeg + (Page shl 10), 0 );
end;

{**********************************************************************}
{**                       PROGRAMME PRINCIPAL                        **}
{**********************************************************************}

begin
  ClrScr;                                              { Vider l'cran }
  writeln('EMMP - (c) 1988, 92 by MICHAEL TISCHER',#13#10);
  if EmsInst then                            { Mmoire EMS installe ? }
    begin                                                        { Oui }
       {*-- Sortir informations sur la mmoire EMS -------------------*}

      EmmVer := EmsVersion;         { Dterminer numro de version EMM }
      if EmmVer = EMS_ERR then         { Une erreur est-elle apparue ? }
        PrintErr; { Oui, sortir message d'erreur et terminer programme }
      writeln('Numro de version EMM        : ',EmmVer div 10, '.',
              EmmVer mod 10);

      NmbPage := EmsNumPage;        { Dterminer nombre total de pages }
      if NmbPage = EMS_ERR then        { Une erreur est-elle apparue ? }
        PrintErr; { Oui, sortir message d'erreur et terminer programme }
      writeln('Nombre de pages EMS          : ', NmbPage, ' (',
               NmbPage shl 4, ' Ko)');

      NmbPage := EmsFreePage;      { Dterminer nombre de pages libres }
      if NmbPage = EMS_ERR then        { Une erreur est-elle apparue ? }
        PrintErr; { Oui, sortir message d'erreur et terminer programme }
      writeln('... dont libres              : ', NmbPage, ' (',
               NmbPage shl 4, ' Ko)');

      PageSeg := EmsFrameSeg;       { Adresse de segment du Page Frame }
      if PageSeg = W_EMS_ERR then      { Une erreur est-elle apparue ? }
        PrintErr; { Oui, sortir message d'erreur et terminer programme }
      writeln('Adresse de segment du Page Frame: ', EmsFrameSeg );

      writeln;
      writeln('On alloue maintenant une page de la mmoire EMS et');
      writeln('le contenu de l''cran est copi de la RAM vido');
      writeln('dans cette page.');
      writeln('                     ... veuillez actionner une Touche');
      Touche := ReadKey;                         { Attendre une touche }

      {*-- Allouer une page et la calquer sur la premire page  ------*}
      {*-- logique dans le Page Frame                           ------*}

      Handle := EmsAlloc( 1 );                      { Allouer une page }
      if Handle = EMS_ERR then         { Une erreur est-elle apparue ? }
        PrintErr; { Oui, sortir message d'erreur et terminer programme }
      if not(EmsMap(Handle, 0, 0)) then                { Fixer Mapping }
        PrintErr;              { Erreur : message d'erreur et terminer }

      {*-- Copier 4000 octets de la RAM vido dans la mmoire EMS ----*}

      Move(VrAdr^, PageAdr(0)^, 4000);

      ClrScr;                                          { Vider l'cran }
      while KeyPressed do                     { Lire le buffer clavier }
        Touche := ReadKey;
      writeln('L''ancien contenu de l''cran a maintenant t vid et');
      writeln('est donc dfinitivement perdu. Mais comme il avait t');
      writeln('sauvegard dans la mmoire EMS, il peut tre recopi');
      writeln('de l dans la RAM vido.');
      writeln('                     ... Veuillez actionner une Touche');
      Touche := ReadKey;                         { Attendre une touche }

        {*-- Recopier le contenu de la RAM vido d'aprs la mmoire --*}
        {*-- EMS et librer  nouveau la mmoire EMS alloue        --*}

      Move(PageAdr(0)^, VrAdr^, 4000);            { Recopier RAM vido }
      if not(EmsFree(Handle)) then                   { Librer mmoire }
        PrintErr;              { Erreur : message d'erreur et terminer }
      GotoXY(1, 15);
      writeln('FIN')
    end
 else                            { Le driver EMS n'a pu tre identifi }
   writeln('ATTENTION ! Pas de mmoire EMS installe.');
end.
