{***********************************************************************
*                              E X T P . P A S                         *
**--------------------------------------------------------------------**
*  Dmonstration de l'accs  la mmoire tendue par les fonctions     *
*  BIOS de l'interruption 15h, en grant les disques virtuels          *
**--------------------------------------------------------------------**
*  Auteur           : MICHAEL TISCHER                                  *
*  dvelopp le     : 18.05.1989                                       *
*  Dernire m.  j. : 19.02.1992                                       *
***********************************************************************}

program ExtP;

uses Dos;

{-- Variables globales ------------------------------------------------}

var RdLen    : integer;                { Taille disques virtuels en Ko }
    ExtAvail : boolean;                  { Mmoire tendue disponible? }
    ExtStart : longint;     { Adresse de la mmoire tendue : linaire }
    ExtLen   : integer;                 { Taille mmoire tendue en Ko }

{***********************************************************************
*  ExtAdrConv : Convertit un pointeur en adresse linaire de 32 bits   *
*               qui sera retourne sous forme d'un LONGINT             *
**--------------------------------------------------------------------**
*  Entre : Adr = Le pointeur  convertir                              *
*  Sortie : l'adresse convertie                                        *
***********************************************************************}

function ExtAdrConv ( Adr : pointer ) : longint;

type PTRREC = record                       { Sert  accder aux compo- }
                 Ofs : word;                  { sants d'un pointeur au }
                 Seg : word;                                   { choix }
               end;

begin
  ExtAdrConv := longint( PTRREC( Adr ).seg ) shl 4 + PTRREC( Adr ).ofs;
end;

{***********************************************************************
*  ExtCopy : Copie des donns entre deux tampons quelconque dans la    *
*            limite adressable de 16 Mo des 80286/i386/i486.           *
**--------------------------------------------------------------------**
*  Entre : Start = Adresse du tampon Start sous forme d'adresse       *
*                   linaire sur 32 bits.                              *
*            But  = Adresse du tampon But sous forme d'adresse         *
*                   linaire sur 32 bits.                              *
*            Len   = Nombre d'octets  copier                          *
*  Info    : - Le nombre d'octets  copier doit tre pair.             *
*            - L'usage de cette procdure est rserv  l'intrieur de *
*              cette Unit.                                            *
***********************************************************************}

procedure ExtCopy( Start, But : longint; Len : word );

{-- Structures de donnes pour l'accs  la mmoire tendue -----------}

type SDES = record                               { Descripteur segment }
              Longueur   : word;       { Longueur du segment en octets }
              AdrLo    : word;         { Bits 0  15 de l'adr. du segm }
              AdrHi    : byte;        { Bits 16  23 de l'adr. du segm }
              Attribut : byte;                   { Attribut du segment }
              Res      : word;                  { Rserv pour le i386 }
            end;

     GDT  = record                           { Global Descriptor Table }
              Dummy : SDES;
              GDTS  : SDES;
              Start : SDES;                             { copie de ... }
              But  : SDES;                                  { ... vers }
              Code  : SDES;
              Stack : SDES;
            end;

     LI = record                    { Sert  accder aux lments d'un }
           LoWord : word;          { LongInts reprsentant une adresse }
           HiByte : byte;                       { linaire sur 32 bits }
           dummy  : byte;
         end;

var GTab : GDT;                              { Global Descriptor Table }
    Regs : Registers;       { Registres process. pr appel interruption }
    Adr  : longint;                     { pour conversion de l'adresse }

begin
  FillChar( GTab, SizeOf( GTab ), 0 );           { Tous les champs  0 }

   {-- Construction du descripteur du segment Start -------------------}

  GTab.Start.AdrLo      := LI( Start ).LoWord;
  GTab.Start.AdrHi      := LI( Start ).HiByte;
  GTab.Start.Attribut   := $92;
  GTab.Start.Longueur   := Len;

    {-- Construction du descripteur du segment But --------------------}

  GTab.But.AdrLo      := LI( But ).LoWord;
  GTab.But.AdrHi      := LI( But ).HiByte;
  GTab.But.Attribut   := $92;
  GTab.But.Longueur   := Len;

   {-- Copie de blocs de mmoire  l'aide de la fonction $87 de   -----}
 {-- l'interruption $15 pour cassettes                            -----}

  Regs.AH := $87;                 { N de la fonction 'copier mmoire' }
  Regs.ES := seg( GTab );                             { Adresse de GDT }
  Regs.SI := ofs( GTab );                                 { vers ES:SI }
  Regs.CX := Len shr 1;                { Nbre de mots  copier dans CX }
  intr( $15, Regs );                                { Appeler fonction }
  if ( Regs.AH <> 0 ) then                                   { Erreur? }
    begin                               { Oui, AH contient code Erreur }
      writeln('Erreur en accs  la mmoire tendue (', Regs.AH,')!');
      RunError;                { Quitter programme avec Run Time Error }
    end;
end;

{***********************************************************************
*  ExtRead : Copie un certain nombre d'octets de la mmoire tendue    *
*            dans la mmoire principale.                               *
**--------------------------------------------------------------------**
*  Entre :  ExtAdr = Adresse source (linaire) dans mmoire tendue   *
*            BuPtr  = Pointeur sur le tampon cible dans mmoire princ. *
*            Len    = Nombre d'octets  copier                         *
***********************************************************************}

procedure ExtRead( ExtAdr : longint;  BuPtr : pointer;  Len : word );

begin
  ExtCopy( ExtAdr, ExtAdrConv( BuPtr ), len );
end;

{***********************************************************************
*  ExtWrite : Copie un certain nombre d'octets de la mmoire principale*
*             dans la mmoire tendue.                                 *
**--------------------------------------------------------------------**
*  Entre :  BuPtr  = Ptr sur tampon source dans mmoire principale    *
*            ExtAdr = Adresse But (linaire) en mmoire tendue        *
*            Len    = Nombre d'octets  copier                         *
***********************************************************************}

procedure ExtWrite( BuPtr : pointer;  ExtAdr : longint;  Len : word );

begin
  ExtCopy( ExtAdrConv( BuPtr ), ExtAdr, len );
end;

{***********************************************************************
*  ExtGetInfo : Retourne l'adresse de la mmoire tendue et sa taille  *
*               en tenant compte des ventuels disques virtuels de     *
*               type VDISK s'y trouvant                                *
**--------------------------------------------------------------------**
*  Entre : aucune                                                     *
*  Sortie : aucune                                                     *
*  Globals : ExtAvail/W, ExtStart/W, ExtLen/W                          *
***********************************************************************}

procedure ExtGetInfo;

type NAME_TYP    = array [1..5] of char;
type BOOT_SECTEUR = record          { Secteur boot d'un disque virtuel }
                     dummy1   : array [1..3] of byte;
                     Name     : NAME_TYP;
                     dummy2   : array [1..3] of byte;
                     BpS      : word;
                     dummy3   : array [1..6] of byte;
                     Secteurs : word;
                     dummy4   : byte;    { remplissage pr taille paire }
                   end;

const VdiskName : NAME_TYP = 'VDISK';

var BootSek : BOOT_SECTEUR;             { copie le Boot Sector suppos }
    Derniere  : boolean;                           { marque fin boucle }
    Regs    : Registers;    { Registres process. pr appel interruption }

begin
     {-- Demander la taille de la mmoire tendue et en dduire  ------}
     {-- la prsence ventuelle de mmoire tendue                  ---}

  Regs.ah := $88;  { n de fonction: "renvoyer taille mmoire tendue" }
  intr( $15, Regs );                   { Appeler interruption Cassette }
  if ( Regs.AX = 0 ) then
    begin                                         { Pas de RAM tendue }
      ExtAvail := FALSE;
      ExtLen   := 0;
      ExtStart := 0;
      exit;                                      { Retour  l'appelant }
    end;

  ExtAvail := TRUE;                       { Mmoire tendue disponible }
  ExtLen   := Regs.AX;            { copier sa taille dans une variable }

{-- Rechercher disques virtuels type VDISK ----------------------------}

  ExtStart := $100000;                               { Si 1 Mo, lancer }
  Derniere := FALSE;                            { A partir du RAM disk }
  repeat                                        { boucle interrogation }
    ExtRead( ExtStart, @BootSek, SizeOf( BootSek ) );
    with BootSek do
      if Name = VDiskName then     { secteur boot d'un disque virtuel? }
        inc( ExtStart, longint( Secteurs ) * BpS ) {Oui, aprs RAM disk}
      else
        Derniere := TRUE;             { Pas d'autres RAM disks trouvs }
  until Derniere;

    {-- Soustraire la taille des disques virtuels ---------------------}
    {-- de la mmoire tendue disponible ------------------------------}

  dec( ExtLen, integer( (ExtStart - longint($100000)) shr 10) );
end;

{***********************************************************************
*  CheckExt : Teste la continuit de la mmoire tendue libre          *
***********************************************************************}

procedure CheckExt;

var AdrTest   : longint;                        { Adresse du bloc test }
    i, j      : integer;                       { Compteur d'itrations }
    WriteBuf,                                          { Blocs de test }
    ReadBuf   : array [1..1024] of byte;
    Erreur    : boolean;                     { Pointeur Erreur mmoire }

begin
  Randomize;                  { Initialise gnrateur nbres alatoires }
  AdrTest := ExtStart;
  for i := 1 to ExtLen do           { lit la mmoire par blocs de 1 Ko }
    begin
      for j := 1 to 1024 do       { Remplir bloc avec nbres alatoires }
        WriteBuf[ j ] := Random( 255 );

      write(#13, AdrTest );    { Affiche adresse du bloc de 1 Ko test }

                    {-- Lire tampon et le copier dans ReadBuf   -------}

      ExtWrite( @WriteBuf, AdrTest, 1024 );
      ExtRead( AdrTest, @ReadBuf, 1024 );

         {-- Dfinir identit de WriteBuf et de ReadBuf  --------------}

      for j := 1 to 1024 do
        if WriteBuf[j] <> ReadBuf[j] then  { Contenu tampon identique? }
          begin                                         { Non, erreur! }
            writeln( '  Erreur! Adresse ',
                     AdrTest + longint(j-1) );
            Erreur := TRUE;
          end;

      inc( AdrTest, longint( 1024 ) );    { Positionner AdrTest sur le }
    end;                                         { bloc suivant d'1 Ko }
 writeln;
 if not( Erreur ) then                                      { Erreur ? }
   writeln( 'a baigne !' );                                     { Non }
end;

{***********************************************************************
*  P R O G R A M M E   P R I N C I P A L                               *
***********************************************************************}

begin
  writeln( #13#10'EXTDEMO - (c) 1989,92  by Michael Tischer'#13#10);
  ExtGetInfo;       { Donne disponibilit et taille de mmoire tendue }
  if ExtAvail then                        { Mmoire tendue es-tu l ? }
    begin                                                        { Oui }
      RdLen := integer( (ExtStart - longint( $100000 ) ) shr 10 );
      if ( RdLen = 0 ) then                    { RAM disks installs ? }
        begin                                                    { Non }
          writeln( 'Aucun RAM disk install. ');
          writeln( 'La mmoire tendue libre commence  la ',
                   'limite du 1er Mo. ');
        end
      else                               { Oui, il y a des RAM disks ! }

begin
          writeln( 'Un ou plusieurs RAM disks occupent ',
                    RdLen,'Ko de mmoire tendue..');
          writeln( 'La mmoire tendue libre commence ', RdLen,
                   ' Ko aprs la limite du 1er Mo. ', RdLen, RdLen);
        end;
       writeln( ' Taille de la mmoire tendue libre ',
                 ExtLen, ' Ko.');
       writeln( #13#10'Test de la continuit de la mmoire tendue',
                  ' en cours...'#13#10);
       CheckExt;
    end
  else
    writeln( 'Pas de mmoire tendue dans votre ordinateur !');
end.
