{**********************************************************************}
{*                           M C B P . P A S                          *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Permet de suivre le chanage des blocs de      *}
{*                     mmoire allous par DOS                        *}
{*--------------------------------------------------------------------*}
{*    Auteur         : Michael TISCHER                                *}
{*    Dvelopp le   : 22.08.1988                                     *}
{*    Dernire MAJ   : 20.03.1992                                     *}
{**********************************************************************}

program MCBP;

uses DOS, CRT;                         { Intgre les units DOS et CRT }

type BytePtr = ^byte;                             { Pointeur sur octet }
     Zone    = array[0..1000] of byte;       { Zone qlcq de la mmoire }
     ZonePtr = ^Zone;                              { Pointeur sur zone }
     MCB     = record                            { Memory-Control-Bloc }
         IdCode   : char;     { "M": existe un bloc qui suit, "Z": Fin }
         PSP      : word;                     { Segment du PSP associ }
         Distance : word;                 { Nombre de  paragraphes - 1 }
               end;
     MCBPtr  = ^MCB;                                { Pointeur sur MCB }
     MCBPtr2 = ^MCBPtr;              { Pointeur sur un pointeur de MCB }
     HexStr  = string[4];         { Mmorise chane hexa.  4 chiffres }

var  CvHStr  : HexStr;             { Mmorise la chane hexa convertie }

{**********************************************************************}
{* HexString: Transforme un nombre en chane hexadcimale             *}
{*             4 positions                                           *}
{* Entre : - HexVal = Valeur  convertir                             *}
{* Sortie : La chane hexa .                                          *}
{**********************************************************************}

function HexString( HexVal : word ) : HexStr;

var compteur,                                  { Compteur d'itrations }
    Nibble  : byte;                       { Quartet infrieur d'un mot }

begin
  CvHStr := 'xxxx';                                   { Cre la chane }
  for compteur:=4 downto 1 do               { Parcourt ses 4 positions }
    begin
      Nibble := HexVal and $000f;    { Ne prend en cmpt que 4 bits sup }
      if( Nibble > 9 ) then                       { Chiffre = lettre ? }
        CvHStr[ compteur ] := chr(Nibble - 10 + ord('A'))        { Oui }
      else                                      { Chiffre entre 0 et 9 }
        CvHStr[ compteur ] := chr(Nibble + ord('0'));
      HexVal := HexVal shr 4;
    end;
  HexString := CvHStr;                    { Transmet la chane trouve }
end;

{**********************************************************************}
{* FirstMCB: Retourne un pointeur sur le premier MCB.                 *}
{* Entre  : nant                                                    *}
{* Sortie : Pointeur sur le premier MCB                               *}
{**********************************************************************}

function FirstMCB : MCBPtr;

var Regs : Registers;         { Registres pour grer les interruptions }

begin
  Regs.ah := $52;           { Fonction: Lit l'adresse du DOS-Info-Bloc }
  MsDos( Regs );                { Dclenche l'innterruption $21 de DOS }

{*-- ES:(BX-4) pointe sur le premier  MCB, forme le pointeur ---------*}

  FirstMCB := MCBPtr2( ptr( Regs.ES-1, Regs.BX+12 ) )^;
end;

{**********************************************************************}
{* Dump: Affiche le dump hexa et ASCII d'une zone de mmoire          *}
{* Entre : - DPtr  = Pointeur sur la zone  dumper.                  *}
{*          - Nbr   = Nombre de lignes de 16 octets  dumper          *}
{* Sortie : nant                                                     *}
{**********************************************************************}

procedure Dump( DPtr : ZonePtr; Nbr : byte);

type HBStr = string[2];         { Mmorise un nombre hexa  2 chiffres }

var  Offset,                          { Offset dans la zone de mmoire }
     Z       : integer;                                     { Compteur }
     HexStr  : HBStr;           { Mmorise un nombre hexa pour le dump }

procedure HexByte( HByte : byte );

begin
  HexStr[1] := chr( (HByte shr 4) + ord('0') );      { Premier chiffre }
  if HexStr[1] > '9' then                        { Est-ce une lettre ? }
    HexStr[1] := chr( ord(HexStr[1]) +  7 );                     { Oui }
  HexStr[2] := chr( (HByte and 15) + ord('0') );    { Deuxime chiffre }
  if HexStr[2] > '9' then                        { Est-ce une lettre ? }
    HexStr[2] := chr( ord(HexStr[2]) +  7 );                     { Oui }
end;

begin
  HexStr := 'zz';                                { Cre la chane hexa }
  writeln;
  write('DUMP  0123456789ABCDEF        00 01 02 03 04 05 06 07 08');
  writeln(' 09 0A 0B 0C 0D 0E 0F');
  write('');
  writeln('');
  Offset := 0;              { Commence par le premier octet de la zone }
  while Nbr>0 do                         { Parcourt la boucle Nbr fois }
    begin
      write(HexString(Offset), '  ');
      for Z:=0 to 15 do                   { Traite 16 octets  la fois }
        if(Dptr^[Offset+Z] >= 32) then             { ASCII ordinaire ? }
          write( chr(Dptr^[Offset+Z]) )               { Oui, l'affiche }
        else                                                     { Non }
          write(' ');      { A la place du caractre affiche un espace }
      write('        '); { Place le curseur sur la partie hexadcimale }
      for Z:=0 to 15 do                             { Traite 16 octets }
        begin
           HexByte( Dptr^[Offset+Z] );            { Conversion en hexa }
           write(HexStr, ' ');                { Affiche la chane hexa }
        end;
      writeln;
      Offset := Offset + 16;               { Offset sur ligne suivante }
      Dec( Nbr );           { Dcrmente le nombre de lignes restantes }
    end;
  writeln;
end;

{**********************************************************************}
{* TraceMCB: Suit la chane des blocs MCB.                            *}
{* Entre  : nant                                                    *}
{* Sortie  : nant                                                    *}
{**********************************************************************}

procedure TraceMCB;

const ComSpec : array[0..7] of char = 'COMSPEC=';

var  ActMCB   : MCBPtr;
     Fin      : boolean;
     Touche   : char;
     NrMCB,                                    { Numro du MCB examin }
     Z        : integer;                                    { Compteur }
     MemPtr   : ZonePtr;

begin
  Fin    := false;
  NrMCB  := 1;                      { Le premier MCB porte le numero 1 }
  ActMCB := FirstMCB;                { Lit le pointeur sur premier MCB }
  repeat                                              { Suit la chane }
    if ActMCB^.IdCode = 'Z' then               { Dernier MCB atteint ? }
      Fin := true;                                               { Oui }
    writeln('Numro du MCB = ', NrMCB);
    writeln('Adresse du MCB= ', HexString(seg(ActMCB^)), ':',
                                HexString(ofs(ActMCB^)) );
    writeln('Adr. Mmoire  = ', HexString(succ(seg(ActMCB^))), ':',
                                HexString(ofs(ActMCB^)) );
    writeln('ID            = ', ActMCB^.IdCode);
    writeln('Adresse du PSP= ', HexString(ActMCB^.PSP), ':0000');
    writeln('Taille        = ', ActMCB^.Distance, ' paragraphes ',
        '( ', longint(ActMCB^.Distance) shl 4, ' octets)');
    write  ('Contenu       = ');

             {*-- Est-ce un environnement  ? -------------------------*}

    Z      := 0;              { Commence l'examen par le premier octet }
    MemPtr := ZonePtr(ptr(Seg(ActMCB^)+1, 0));     { Ptr sur mm. vive }
    while( (Z<=7) and (ord(ComSpec[Z]) = MemPtr^[Z]) ) do
      Inc(Z);                                { Z sur caractre suivant }
    if Z>7 then                         { A-t-on dcouvert la chane ? }
      begin                        { Oui, il s'agit d'un environnement }
        writeln('Environnement');
        MemPtr := ZonePtr(ptr(Seg(ActMCB^)+1, 0));
        if Lo(DosVersion) >= 3 then         { DOS 3.0 ou postrieure ? }
        begin                         { Oui, donne le nom du programme }
          write('Nom du progr  = ');
          Z := 0;                       { Commence par le premier octet}
          while not( (MemPtr^[Z]=0) and (MemPtr^[Z+1]=0) ) do
          Inc( Z );                              { Cherche chane vide }
          if( MemPtr^[Z+2]=1 ) and ( MemPtr^[Z+3]=0 ) then
            begin                          { Voici un nom de programme }
              Z := Z + 4;                      { Z sur le dbut du nom }
              repeat                    { Parcourt le nom du programme }
                write( chr(MemPtr^[Z]) );       { Affiche un caractre }
                Inc( Z );                { Traite le caractre suivant }
              until ( MemPtr^[Z]=0 );    { Jusqu' la fin de la chane }
              writeln;
            end
          else                 { Nom du programme impossible  trouver }
            writeln('inconnu');
        end;

        {*-- AFfiche les chanes de l'environnement ------------------*}

        writeln(#13,#10, 'Chanes de l''environnement');
        Z := 0;     { Commence par le premier octet de la zone alloue }
        while MemPtr^[Z]<>0 do            { rpte jusqu' chane vide }
        begin
          write('      ');
          repeat                                   { Affiche un chane }
            write( chr(MemPtr^[Z]) );            { Affiche un caractre}
            Inc( Z );                     { Traite le caractre suivant}
          until MemPtr^[Z]=0;                       { Fin de la chane }
          Inc( Z );             { Passe au dbut de la chane suivante }
          writeln;                                  { Termine la ligne }
        end
      end
    else                                         { Pas d'environnenent }
      begin
        {*-- S'agit-il d'un PSP ? ------------------------------------*}
        {*-- (introduit par la commande INT 20 (Code=$CD $20) ) ------*}

        MemPtr := ZonePtr(ptr(seg(ActMCB^)+1, 0));         { Init. ptr }
        if( (MemPtr^[0]=$CD) and (MemPtr^[1]=$20) ) then
                                                   { c'est bien un PSP }
          writeln('PSP (suivi d''un programme)')
        else             { La commande INT 20 n'a pas pu tre dtecte }
          begin
            writeln('non identifiable (Programme ou donnes)');
            Dump( MemPtr, 5);        { Effectue un dump de 5*16 octets }
          end;
      end;

    write('');
    writeln(' Appuyez sur une touche');
    if( not Fin ) then
      begin                                 { Pointeur sur MCB suivant }
        ActMCB := MCBPtr(ptr(seg(ActMCB^) + ActMCB^.Distance + 1, 0));
        Inc(NrMCB);                      { Incrmente le numro du MCB }
        Touche := ReadKey;
      end;
  until( Fin )                 { rpte opration jusqu'au dernier MCB }
end;

{**********************************************************************}
{**                           PROGRAMME PRINCIPAL                    **}
{**********************************************************************}

begin
  ClrScr;                                             { Efface l'cran }
  writeln( 'MCBP  -  (c) 1988, 92 by Michael TISCHER' );
  writeln;
  writeln;
  TraceMCB;                               { Parcourt la chane des MCB }
end.
