{**********************************************************************}
{*                        M E M D E M O P . P A S                     *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Montre comment DOS gre la mmoire             *}
{*--------------------------------------------------------------------*}
{*    Auteur         : Michael TISCHER                                *}
{*    Dvelopp le   : 08.10.1991                                     *}
{*    Dernire MAJ   : 20.03.1992                                     *}
{**********************************************************************}

{$M 8096, 0, 10240 }

program MEMDEMOP;

uses crt,                                        { Intgre l'unit CRT }
     dos;                                        { Intgre l'unit DOS }

{== Constantes ========================================================}

const
{-------- Numros des fonctions de l'interruption 21h------------------}

      GET_MEM      = $48;                 { Rserve de la mmoire vive }
      FREE_MEM     = $49;                  { Libre de la mmoire vive }
      CHANGE_MEM   = $4A;    { Modifie la taille d'une zone de mmoire }
      GET_STRATEGY = $5800;            { Lit la stratgie d'allocation }
      SET_STRATEGY = $5801;           { Fixe la stratgie d'allocation }
      GET_UMB      = $5802;     { Lit l'tat d'inclusion des blocs UMB }
      SET_UMB      = $5803;           { Fixe l'inclusion des blocs UMB }

{-- Stratgies de recherche pour FixeStrategie ------------------------}

      CHERCHE_ENBAS    = $00;          { premier bloc de mmoire libre }
      CHERCHE_MEILLEUR = $01;         { meilleur bloc de mmoire libre }
      CHERCHE_ENHAUT   = $02;          { dernier bloc de mmoire libre }
      DABORD_UMB       = $80;              { Chercher dans la zone UMB }
                                         { utiliser avec CHERCHE_....) }

{-------- Constantes pour FixeUMB -------------------------------------}

      UMB_NON = $00;               { Ne tient pas compte des blocs UMB }
      UMB_OUI = $01;                            { Alloue des blocs UMB }

{------- Constantes pour Demo -----------------------------------------}

      TEST_TA        = 10240-1;          { 10239 paragraphes pour test }
      TEST_TA_UMB    = 2560-1;      { 2559 paragraphes d'UMB pour test }
      TEST_TA_KB     = 160;           { Environnement de test = 160 Ko }
      TEST_TA_UMB_KB = 40;         { Environnement de test UMB = 40 Ko }
      NBREBLOC       = 26;     { Nbre d'adresses pour aff. du rsultat }

{-- Codes de touches pour saisie utilisateur --------------------------}

      ESC  = #27;                               { Interruption par ESC }
      F1   = #59;                              { Touche de fonction F1 }
      F2   = #60;                              { Touche de fonction F2 }
      F3   = #61;                              { Touche de fonction F3 }
      F8   = #66;                              { Touche de fonction F8 }
      F9   = #67;                              { Touche de fonction F9 }
      F10  = #68;                             { Touche de fonction F10 }

{== Dclarations de types =============================================}

type  BlocTyp = record           { Gestion d'un bloc de mmoire allou }
           Adresse,                                          { Segment }
           Taille : word;                       { Taille du bloc en Ko }
                 end;

{== Typisierte Constantes =============================================}

const OuiNon     : array[ false..true ] of string =
           ( 'Non ', 'Oui ' );
      SText      : array[ CHERCHE_ENBAS..CHERCHE_ENHAUT ] of string =
           ( 'Utiliser le premier bloc de mmoire libre ',
             'Utiliser le meilleur bloc de mmoire libre',
             'Utiliser le dernier bloc de mmoire libre ' );
      ChampCouleur: array[ 0..1 ] of byte   =
           ( $07, $70 );
      TexteTouche : array[ 0..3 ] of string =
           ( ' [F1]  Allouer de la mmoire',
             ' [F2]  Librer de la mmoire',
             ' [F3]  Modifier la taille ',
             ' [ESC] Fin du programme ' );

{== Variables globales=================================================}

var Regs         : Registers;              { Registres d'interruptions }
    ChampAdresse : array[ 0..1000 ] of word;       { Champ de mmoires }
    NbAdresse    : word;                           { Nombre d'adresses }
    SegConv      : word;        { Adr. bloc de test dans mmoire conv. }
    UMBSeg       : word;           { Adr. bloc de test en mmoire sup. }
    BlocChamp    : array[ 0..NBREBLOC - 1 ] of BlocTyp;      { Mmoire }

{***********************************************************************
* DOS_GetMem : Rserve de la mmoire                                   *
* Entre     : Taille mmoire souhaite en paragraphes                 *
* Sortie     : Segment du bloc de mmoire allou ,                     *
*              Nombre de paragraphes allous ou nombre maximal  des    *
*              paragraphes disponibles                                 *
***********************************************************************}

procedure DOS_GetMem(     Ta  : word;
                      var Adr : word;
                      var Res : word );

begin
  with Regs do
    begin
      ah := GET_MEM;                              { Numro de fonction }
      bx := Ta;                     { Nombre de paragraphes  rserver }
      msdos( Regs );                       { Dclenche  l'interruption }
      if( flags and fcarry = 0 ) then                 { Appel russi ? }
        begin
          Adr := regs.ax;       { Oui, restitue l'adresse et la taille }
          Res := Ta;
        end
      else                                               { Non ,erreur }
        begin
          Adr := 0;                          { pas de mmoire rserve }
          Res := bx;                      { Taille disponible maximale }
        end;
    end;
end;

{***********************************************************************
* DOS_FreeMem : Libre de la mmoire pralablement rserve            *
* Entre      : Segment de la mmoire                                  *
* Sortiee     : nant                                                  *
***********************************************************************}

procedure DOS_FreeMem( Adr : word );

begin
  with Regs do
    begin
      ah := FREE_MEM;                          { Numro de la fonction }
      es := Adr;                     { Adresse de la mmoire  librer }
      msdos( Regs );                        { Dclenche l'interruption }
    end;
end;

{***********************************************************************
* DOS_ChangeMem : Change la taille d'un bloc rserv                   *
* Entre       : Ancien segment, nvelle taille souhaite (paragraphes) *
* Sortie       : Segment du bloc allou                                *
*                 Nombre de paragraphes allous ou nombre maximal      *
*                 de paragraphes disponibles                           *
***********************************************************************}

procedure DOS_ChangeMem(     Ta  : word;
                         var Adr : word;
                         var Res : word );
begin
  with Regs do
    begin
      ah := CHANGE_MEM;                        { Numro de la fonction }
      bx := Ta;                     { Nombre de paragraphes  rserver }
      es := Adr;               { Segment du bloc de mmoire  modifier }
      msdos( Regs );                        { Dclenche l'interruption }
      if( flags and fcarry = 0 ) then                 { Appel russi ? }
        Res := Ta                            { Nouvelle taille mmoire }
      else                                               { Non, erreur }
        Res := bx;                        { Taille maximale disponible }
    end;
end;

{***********************************************************************
* LitStrategie : Lit la stratgie de gestion de la mmoire             *
* Entre       : nant                                                 *
* Sortie       : Type de stratgie                                     *
***********************************************************************}

function LitStrategie : integer;

begin
  with Regs do
    begin
      ax := GET_STRATEGY;              { Fixe le numro de la fonction }
      msdos( Regs );                        { Dclenche l'interruption }
      LitStrategie := ax;                       { Retourne la stratgie}
    end;
end;

{***********************************************************************
* LitUMB       : Lit l'tat d'inclusion des blocs UMB                  *
* Entre       : nant                                                 *
* Sortie       : Indique si les blocs UMB sont pris en compte          *
* Info          : Disponible  partir de MS-DOS version 5.0            *
***********************************************************************}

function LitUMB : integer;

begin
  with Regs do
    begin
      ax := GET_UMB;                   { Fixe le numro de la fonction }
      msdos( Regs );                        { Dclenche l'interruption }
      LitUMB := al;                                  { Retourne l'tat }
    end;
end;

{***********************************************************************
* FixeStrategie: Fixe la stratgie d'allocation de la mmoire          *
* Entre       : Nouveau type de stratgie souhaite                   *
* Sortie       : nant                                                 *
***********************************************************************}

procedure FixeStrategie( Strategie : integer );

begin
  with Regs do
    begin
      ax := SET_STRATEGY;              { Fixe le numro de la fonction }
      bx := Strategie;
      msdos( Regs );                        { Dclenche l'interruption }
    end;
end;

{***********************************************************************
* FixeUMB      : Fixe l'tat d'inclusion des blocs de mmoire UMB      *
* Entre       : Nouvel tat d'inclusion souhait                      *
* Sortie       : nant                                                 *
* Info         : Disponible  partir de la version 5.0 de MS-DOS       *
***********************************************************************}

procedure FixeUMB( UMB : integer );

begin
  with Regs do
    begin
      ax := SET_UMB;                   { Fixe le numro de la fonction }
      bx := UMB;
      msdos( Regs );                        { Dclenche l'interruption }
    end;
end;

{***********************************************************************
* AlloueMemoire   : Cre l'environnement du test                       *
* Entre          : nant                                              *
* Sortie          : nant                                              *
***********************************************************************}

procedure AlloueMemoire;

var SegAdr    : word;                  { Segment de la mmoire alloue }
    Essai     : word;                    { Taille de mmoire souhaite }
    Taille    : word;           { Taille de la zone de mmoire alloue }

begin
      {-- Alloue le  bloc de test--------------------------------------}

  FixeUMB( UMB_NON );                     { En mmoire conventionnelle }
  DOS_GetMem( TEST_TA, SegConv, Taille );             { Cherche le bloc}

  if ( SegConv = 0 ) then                                    { Erreur? }
    exit;                 { Pas de bloc de test, terminer la procdure }


  FixeUMB( UMB_OUI );             { Bloc de test en mmoire suprieure }
  FixeStrategie( CHERCHE_ENHAUT or DABORD_UMB );
  DOS_GetMem( TEST_TA_UMB, UMBSeg, Taille );
  if( UMBSeg <> 0 ) and ( UMBSeg < $A000 ) then   { Pas de blocs UMB ? }
    begin
      DOS_FreeMem( UMBSeg );                       { Libre la mmoire }
      UMBSeg := 0;                              { Absence de blocs UMB }
    end;

  {-- Alloue la mmoire conv. restante et la mmoire suprieure -------}
  {--    par blocs de 1 Ko                                          ---}

  Essai := 63;               { Essaie d'abord d'allouer 15 paragraphes }
  NbAdresse := 0;      { Initialise le nombre de blocs de 1 Ko allous }
  repeat
    DOS_GetMem( Essai, SegAdr, Taille );       { Rclame de la mmoire }
    if ( SegAdr <> 0 ) then                    { Est-elle disponible ? }
      begin
        ChampAdresse[ NbAdresse ] := SegAdr;      { Mmorise l'adresse }
        inc( NbAdresse );
      end;
  until( SegAdr = 0 );                               { Tout est allou }

    {-- 3 Libre  nouveau les blocs de test--------------------------}

  if( SegConv > 0 ) then           { Mmoire conventionnelle alloue ? }
    begin                                                        { Oui }
      DOS_FreeMem( SegConv );
      dec( SegConv );                           { MCB galement libr }
    end;
  if( UMBSeg > 0 ) then                 { Mmoire suprieure alloue ? }
    begin                                                        { Oui }
      DOS_FreeMem( UMBSeg );
      dec( UMBSeg );                            { MCB galement libr }
    end;
end;

{***********************************************************************
* LibereMemoire   : Libre la mmoire alloue par AlloueMemoire        *
* Entre          : nant                                              *
* Sortie          : nant                                              *
* Variables glob. : ChampAdresse/R                                     *
***********************************************************************}

procedure LibereMemoire;

var i : word;                                               { Compteur }

begin
  if( NbAdresse > 0 ) then             { A-t-on allou de la mmoire ? }
    for i := 0 to NbAdresse - 1 do      { Oui, la libre bloc par bloc }
      DOS_FreeMem( ChampAdresse[ i ] );
end;

{***********************************************************************
* AfficheResultat : Affiche l'occupation de la mmoire                 *
* Entre          : AVECCADRE = TRUE, si le cadre doit galement tre  *
*                                affich                               *
* Sortie          : nant                                              *
***********************************************************************}

procedure AfficheResultat( AvecCadre : boolean );

var SChamp      : array[ 0..TEST_TA_KB - 1 ] of char;
    SChamp_UMB  : array[ 0..TEST_TA_UMB_KB - 1 ] of char;
    i,j         : word;                                    { Compteurs }
    Position    : word;                          { Variable auxiliaire }
    Sdummy      : string;
    DerCara     : char;        { Mmorise le dernier caractre affich }
    CoulCour    : byte;                 { Couleur d'affichage courante }

begin
  fillchar( SChamp[ 0 ], TEST_TA_KB, #32 );       { Champs d'affichage }
  fillchar( SChamp_UMB[ 0 ], TEST_TA_UMB_KB, #32 );

  {-- Remplit le tableau de la mmoire --------------------------------}

  for i := 0 to NBREBLOC - 1 do
    begin
      if( BlocChamp[ i ].Adresse > $A000 ) then                 { UMB? }
        begin
          Position := ( BlocChamp[ i ].Adresse - UMBSeg ) div 64;
          for j := 0 to BlocChamp[ i ].Taille div 64 do
            SChamp_UMB[ Position + j ] := chr( i + 65 );
        end
      else if( BlocChamp[ i ].Adresse > 0 ) then
        begin
          Position := ( BlocChamp[ i ].Adresse - SegConv ) div 64;
          for j := 0 to BlocChamp[ i ].Taille div 64 do
            SChamp[ Position + j ] := chr( i + 65 );
        end;
    end;

          {-- Dessine le cadre du tableau -----------------------------}

  if( AvecCadre ) then
    begin
      writeln( 'Mmoire conventionnelle :' );
      writeln( '                   1         2         3         4 ' );
      writeln( '          1        0         0         0         0 ' );
      writeln( 'ͻ' );
      for i := 0 to 3 do
        begin
          str( i * 40 : 3, Sdummy );
          writeln( ' ' + Sdummy +
                   ' Ko                                         ',
                   TexteTouche[ i ] );
        end;
      writeln( 'ͼ' );
      if ( UMBSeg > 0 ) then
       begin
        writeln(#13#10'UMB:' );
        writeln('                   1         2         3         4 ');
        writeln('          1        0         0         0         0 ');
        writeln('ͻ');
        writeln('   0 KB                                         ');
        writeln('ͼ');
       end
      else
       writeln( #13#10, 'Pas d''UMB disponible' );
    end;

  {-- Affiche le tableau de la mmoire conventionnelle ----------------}

  DerCara := #0;                           { Dernier caractre affich }
  CoulCour := 0;                           { Dernire couleur affiche }
  for i := 0 to 3 do
    for j := 0 to 39 do
      begin
        if( DerCara <> SChamp[ i * 40 + j ] ) then  { Chang. couleur ? }
          begin
            CoulCour := ( CoulCour + 1 ) mod 2; { Nouveau code couleur }
            textcolor( ChampCouleur[ CoulCour ] shr 4 );
            textbackground( ChampCouleur[ CoulCour ] and $0F );
            DerCara := SChamp[ i * 40 + j ];       { Symbole d'galit }
          end;
        gotoxy( j + 11, i + 11  );              { Affiche un caractre }
        write( SChamp[ i * 40 + j ] );
      end;

  {-- AFfiche le tableau de la mmoire suprieure ---------------------}

  if ( UMBSeg > 0 ) then
    begin
      for j := 0 to 39 do
        begin
          if( DerCara <> SChamp_UMB[ j ] ) then     { Chang. couleur ? }
            begin
              CoulCour := ( CoulCour + 1 ) mod 2;       { code couleur }
              textcolor( ChampCouleur[ CoulCour ] shr 4 );
              textbackground( ChampCouleur[ CoulCour ] and $0F );
              DerCara := SChamp_UMB[  j ];         { Symbole d'galit }
            end;
          gotoxy( j + 11, 21  );
          write( SChamp_UMB[ j ] );
        end;
    end;
  textcolor( $07 );
  textbackground( $00 );
end;

{***********************************************************************
* Demo         : Dmonstration de la gestion de la mmoire             *
* Entre       : Inclusion des UMB, recherche a priori dans les UMB ,  *
*                stratgie d'allocation de la mmoire                  *
* Sortie       : nant                                                 *
* Info         : Les fonctions UMB sont disponibles qu' partir de la  *
*                version 5.0 de DOS                                    *
***********************************************************************}

procedure Demo( Avec_UMB    : boolean;
                UMB_dabord : boolean;
                Strategie  : integer );


var i        : integer;                        { Compteur d'itrations }
    Touche   : char;                                  { Touche frappe }
    Marqueur : char;     { Marqueur (A-Z) souhait pour la rservation }
    Essai    : word;                        { Taille de la rservation }
    Taille   : word;
    sdummy   : string;

begin
  {-- Initialise les champs adresse et taille -------------------------}

  for i := 0 to NBREBLOC - 1 do            { Pas encore de bloc allou }
    with BlocChamp[ i ] do
      begin
        Adresse := 0;                     { Segment du bloc de mmoire }
        Taille := 0;                                  { Taille du bloc }
      end;

  gotoxy( 1, 7 );
  AfficheResultat( TRUE );                        { Affiche le tableau }

 {-- Boucle de dmonstration ------------------------------------------}

  repeat
                        {-- Fixe la stratgie souhaite ---------------}

    if ( Avec_UMB ) then           { Exploiter la mmoire suprieure ? }
      FixeUMB( UMB_OUI )
    else
      FixeUMB( UMB_NON );

    if ( UMB_dabord ) then
      FixeStrategie( Strategie or DABORD_UMB )
    else
      FixeStrategie( Strategie );

    {-- Affiche la stratgie actuelle ---------------------------------}

    gotoxy( 1, 3 );
    writeln( ' [F8] Gestion mmoire           : ', SText[ Strategie ]);
    writeln( ' [F9] Recherche dans l''UMB      : ', OuiNon[UMB_dabord]);
    writeln( '[F10] Utilisation des blocs UMB : ', OuiNon[Avec_UMB ]);
    writeln( '--------------------------------------------------',
             '----------------------------' );

    {-- Saisie et traitement     --------------------------------------}

    repeat until keypressed;             { Attend une frappe de touche }
    Touche := readkey;                                 { Lit la touche }
    if( ( Touche = #0 ) and ( keypressed ) ) then     { T. de fonction }
     Touche := readkey;                              { Lit le 2me code }

    case Touche of
      F1 :                                 { Alloue le bloc de mmoire }
        begin
          i := -1;                         { pas encore de bloc valide }
          repeat
            gotoxy( 1, 23 );
            write( 'Quel bloc faut-il rserver [ A-Z ]   : ' );
            readln( Marqueur );
            Marqueur := upcase( Marqueur );
            if( Marqueur >= 'A' ) and ( Marqueur <= 'Z' ) then
              if( BlocChamp[ ord( Marqueur ) - 65 ].Adresse = 0 ) then
                i := ord( Marqueur ) - 65;
          until( i <> - 1 );
          write( 'Combien de Ko faut-il rserver       : ' );
          readln( Essai );

          Essai := Essai * 64 - 1;         { Conversion en paragraphes }
          DOS_GetMem( Essai, BlocChamp[ i ].Adresse,
                      BlocChamp[ i ].Taille );
          if( BlocChamp[ i ].Taille <> Essai ) then         { Erreur ? }
            begin
              str( ( BlocChamp[ i ].Taille + 1 ) div 64, sdummy );
              write( 'Il ne reste plus que ' + sdummy + 'Ko ' );
              repeat
              until keypressed;

              while keypressed do
                Touche := readkey;
              Touche := #0;
            end;
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          writeln( '                                             ' );
          write  ( '                                             ' );
          gotoxy( 1, 7 );
          AfficheResultat( FALSE );               { Affiche le tableau }
        end;

      F2 :                                             { Libre le bloc}
        begin
          i := -1;                        { Pas encore de bloc valable }
          repeat
            gotoxy( 1, 23 );
            write( 'Quel bloc faut-il librer [ A-Z ]    : ' );
            readln( Marqueur );
            Marqueur := upcase( Marqueur );
            if( Marqueur >= 'A' ) and ( Marqueur <= 'Z' ) then
              if( BlocChamp[ ord( Marqueur ) - 65 ].Adresse <> 0 ) then
                i := ord( Marqueur ) - 65;
          until ( i <> - 1 );
          DOS_FreeMem( BlocChamp[ i ].Adresse );
          BlocChamp[ i ].Adresse := 0;
          BlocChamp[ i ].Taille := 0;
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          gotoxy( 1, 7 );
          AfficheResultat( FALSE );               { Affiche le tableau }
        end;

      F3 :                               { Modifie la taille d'un bloc }
        begin
          i := -1;                         { Pas encore de bloc valide }
          repeat
            gotoxy( 1, 23 );
            write( 'De quel bloc faut-il changer la taille [ A-Z ]: ' );
            readln( Marqueur );
            Marqueur := upcase( Marqueur );
            if ( Marqueur >= 'A' ) and ( Marqueur <= 'Z' ) then
              if ( BlocChamp[ ord( Marqueur ) - 65 ].Adresse <> 0 ) then
                i := ord( Marqueur ) - 65;
          until ( i <> - 1 );
          write( 'Combien de Ko faut-il rserver :     ' );
          readln( Essai );
          Essai := Essai * 64 - 1;         { Conversion en paragraphes }
          DOS_ChangeMem( Essai, BlocChamp[ i ].Adresse, Taille );
          if( Taille <> Essai ) then                        { Erreur ? }
            begin
              str( ( Taille + 1 ) div 64, sdummy );
              write( 'Il ne reste plus que ' + sdummy + 'Ko ' );
              repeat until keypressed;
              while keypressed do
                Touche := readkey;
              Touche := #0;
            end
          else
            BlocChamp[ i ].Taille := Taille;  { Fixe une nvelle taille }
          gotoxy( 1, 23 );
          writeln( '                                             ' );
          writeln( '                                             ' );
          write  ( '                                             ' );
          gotoxy( 1, 7 );
          AfficheResultat( FALSE );               { Affiche le tableau }
        end;

      F8 :
         Strategie := ( Strategie + 1 ) mod 3;   { Change de stratgie }

      F9 :                                 { Commutation : UMB d'abord }
         UMB_dabord := not UMB_dabord;

      F10:                             { Commutation : Inclure les UMB }
         avec_UMB := not avec_UMB;
    end;
  until( Touche = ESC );
end;

{***********************************************************************
***                     Programme principal                          ***
***********************************************************************}

var StartStrategie : integer;       { Stratgie d'allocation au dpart }
    StartUMB       : integer;           { Etat des blocs UMB au dpart }
    ActUMB_oui     : boolean;       { Utiliser les blocs UMB (oui/non) }
    ActUMB_dabord  : boolean;         { Mmoire suprieure prioritaire }
    ActStrategie   : integer;        { Stratgie d'allocation actuelle }

begin
       {-- Sauvegarde les valeurs courantes ---------------------------}

  StartStrategie := LitStrategie;             { Stratgie d'allocation }
  StartUMB := LitUMB;       { Prise en compte de la mmoire suprieure }
  AlloueMemoire;                        { Cre l'environnement du test }
  FixeStrategie( Startstrategie );     { Restaure l'ancienne stratgie }
  FixeUMB( StartUMB );

  if ( SegConv = 0 ) then          { Mmoire conventionnelle alloue ? }
    begin                                 { Non, on arrte avec erreur }
      clrscr;
      writeln('MEMDEMOP : Mmoire insuffisante !');
      exit;                                               { C'est fini }
    end;

       {-- Valeurs de dpart pour la gestion de la mmoire ------------}

  ActUMB_oui := ( StartUMB = UMB_OUI );
  ActUMB_dabord := ( ( StartStrategie and DABORD_UMB ) = DABORD_UMB );
  ActStrategie := StartStrategie and ( $FF xor DABORD_UMB );

       {-- Dmonstration de la gestion de la mmoire ------------------}

  clrscr;
  writeln( 'Programme de gestion mmoire DOS               ',
             '(C) 1991, 92 by Michael TISCHER' );
  writeln( '==================================================',
           '============================' );
  Demo( ACTUMB_oui, ACTUMB_dabord, ActStrategie );

       {-- Restaure les anciennes valeurs de DOS-----------------------}

  LibereMemoire;                           { Libre la mmoire alloue }
  FixeStrategie( StartStrategie );
  FixeUMB( StartUMB );
  clrscr;
end.
