{***********************************************************************
*                            M I K A D O P                             *
**--------------------------------------------------------------------**
*  Fonction : Montre comment mettre en service le mode 512 caractres  *
*             des cartes EGA et VGA. La routine de dmonstration       *
*             installe une fentre graphique en mode texte.            *
**--------------------------------------------------------------------**
*  Auteur       : MICHAEL TISCHER                                      *
*  Dvelopp le :  2.04.1990                                           *
*  Dernire MAJ : 14.01.1991                                           *
***********************************************************************}

program MikadoP;

uses DOS, CRT;     { Inclut les units DOS et CRT }

{-- Constantes --------------------------------------------------------}

const EGAVGA_SEQUENCER = $3C4; { Port d'adresses/donnes du squenceur }
      EGAVGA_MONCTR    = $3D4;                    { Contrleur d'cran }
      EGAVGA_GRAPHCTR  = $3CE;{Port d'adr./donnes du contrleur graph.}
      CHAR_LARGEUR     = 8;
      CHAR_BYTES       = 32;
      MIKADOS          = 5; { Nombre de mikados visibles en mme temps }

{-- Dclarations de types ---------------------------------------------}

type VEL      = record         { Dcrit un couple caractre - attribut }
                  case boolean of                 { en mmoire d'cran }
                    true  : ( Caractere, Attribut : byte );
                    false : ( Contenu             : word );
                end;
     VPTR     = ^VEL;               { Pointe sur un caractre/attribut }
     VELARRAY = array [1..25,1..80] of VEL; {Reproduit la mmoire cran}
     VELARPTR = ^VELARRAY;             { Pointe sur la mmoire d'cran }

     FONT = array[0..255,0..CHAR_BYTES-1] of byte;{Reproduit une police}
     FPTR = ^font;                             { Pointe sur une police }

     PALARY = array[ 1..16] of BYTE;    { Jeu de registres de palettes }

{-- Variables globales ------------------------------------------------}

const vioptr : VELARPTR = ptr( $B800, $0000 ); { Pointe sur mm. cran }

var CharHauteur,
    lenx    : byte;        { Largeur en pixels de la fentre graphique }
    xmax,          { Coordonnes max en pixels de la fentre graphique }
    ymax    : integer;
    fontptr : fptr;               { Pointe sur la police de caractres }

procedure CLI; inline( $FA );
procedure STI; inline( $FB );

{***********************************************************************
*  IsEgaVga : Teste la prsence d'une carte EGA/VGA.                   *
*             Mmorise en mme temps le nombre de lignes de trame      *
*             par caractre dans la variable CharHauteur               *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
*  Sortie : TRUE, si carte EGA ou VGA , sinon FALSE                    *
***********************************************************************}

function IsEgaVga : boolean;

var Regs : Registers;            { Registres pour grer l'interruption }

begin
  Regs.AX := $1a00;               { La fonction 1Ah n'existe qu'en VGA }
  Intr( $10, Regs );
  if ( Regs.AL = $1a ) then     { Cette fonction est-elle disponible ? }
    begin
      IsEgaVGa := TRUE;
      CharHauteur := 16;                          { 16 lignes de trame }
    end
  else
    begin
      Regs.ah := $12;            { AppelLe l'option 10h de la fonction }
      Regs.bl := $10;
      intr($10, Regs);          { Appelle l'interruption vido du BIOS }
      IsEgaVga := ( Regs.bl <> $10 );
      CharHauteur := 14;                          { 14 lignes de trame }
    end;
end;

{***********************************************************************
*  SetCharWidth : Fixe  8 ou 9 pixels la largeur des caractres des   *
*                 cartes VGA.                                          *
**--------------------------------------------------------------------**
*  Entre: LARGEUT : Largeur des caractres ( 8 ou 9 )                 *
***********************************************************************}

procedure SetCharWidth( largeur : byte );

var Regs : Registers;            { Registres pour grer l'interruption }
    x    : byte;                                 { Variable de travail }

begin
  if ( largeur = 8 ) then Regs.BX := $0001    { BH = alignement horiz. }
                     else Regs.BX := $0800;   { BL = Valeur pour
                                             le registre du squenceur }

  x := port[ $3CC ] and not(4+8);    { Passe de la rsolution de 720  }
  if ( largeur = 9 ) then                  { 640 pixels ou vice-versa  }
    x := x or 4;
  port[ $3C2 ] := x;

  CLI;                        { Programme le squenceur en consquence }
  portw[ EGAVGA_SEQUENCER ] := $0100;
  portw[ EGAVGA_SEQUENCER ] := $01 + Regs.BL shl 8;
  portw[ EGAVGA_SEQUENCER ] := $0300;
  STI;

  Regs.AX := $1000;                   { Ajuste l'alignement de l'cran }
  Regs.BL := $13;
  intr( $10, Regs );
end;

{***********************************************************************
*  SelectMaps : Slectionne les jeux de caractres accessibles par le  *
*               bit 3 de l'attribut de caractres                      *
**--------------------------------------------------------------------**
*  Entres : MAP0 = Numro du premier jeu de caractres (bit 3 = 0 )   *
*            MAP1 = Numro du deuxime jeu de caractres (bit 3 = 1 )  *
*  Info    : - Avec une carte EGA on peut choisir les jeux 0  3,      *
*              avec une carte VGA les jeux 0  7                       *
***********************************************************************}

procedure SelectMaps( map0, map1 : byte );

var Regs : Registers;         { Registres pour grer les interruptions }

begin
  Regs.AX := $1103;{Prog. le registre de slection du jeu de caractres}
  Regs.BL := ( ( map0 and 3 ) + ( map0 and 4 ) shl 2 ) +
             ( ( map1 and 3 ) shl 2 + ( map1 and 4 ) shl 3 );
  Intr( $10, Regs ); { Appelle l'option 03 de la fonction $11 }
end;

{***********************************************************************
*  GetFontAccess : Permet d'accder directement  la deuxime zone     *
*                  de mmoire o sont stocks les jeux de caractres   *
*                  par l'adresse A000:0000                             *
**--------------------------------------------------------------------**
*  Entre  : nant                                                     *
*  Info    : - A l'issue de cette procdure, il n'est plus possible    *
*              d'accder  la mmoire d'cran par B800:0000            *
***********************************************************************}

procedure GetFontAccess;

const SeqRegs : array[1..4] of word = ( $0100, $0402, $0704, $0300 );
      GCRegs  : array[1..3] of word = ( $0204, $0005, $0406 );

var i : byte;                                  { Compteur d'itrations }

begin
  CLI;
  for i := 1 to 4 do              { Charge les registres du squenceur }
    portw[ EGAVGA_SEQUENCER ] := SeqRegs[ i ];

  for i := 1 to 3 do    { Charge les registres du contrleur graphique }
    portw[ EGAVGA_GRAPHCTR ] := GCRegs[ i ];
  STI;
end;

{***********************************************************************
*  ReleaseFontAccess : Libre l'accs  la mmoire d'cran par         *
*                      B800:0000 empchent en mme temps l'accs aux   *
*                      jeux de caractre situs en page mmoire N2    *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************}

procedure ReleaseFontAccess;

const SeqRegs : array[1..4] of word = ( $0100, $0302, $0304, $0300 );
      GCRegs  : array[1..3] of word = ( $0004, $1005, $0E06 );

var i : byte;                                  { Compteur d'itrations }

begin
  for i := 1 to 4 do   { Charge les diffrents registres du squenceur }
    portw[ EGAVGA_SEQUENCER ] := SeqRegs[ i ];
  for i := 1 to 3 do     { Charge les registres du contrleur graphique}
    portw[ EGAVGA_GRAPHCTR ] := GCRegs[ i ];
end;

{***********************************************************************
*  ClearGraphArea : Efface la zone graphique en y mettant  0 les      *
*                   motifs des caractres.                             *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************}

procedure ClearGraphArea;

var caracteres,                               { Caractres  parcourir }
    ligne    : byte;        { Lignes  l'intrieur de chaque caractre }

begin
  for caracteres := 0 to 255 do              { Parcourt les caractres }
    for ligne := 0 to CharHauteur-1 do           { Parcourt les lignes }
      fontptr^[ caracteres, ligne ] := 0;             { et les met  0 }
end;

{***********************************************************************
*  InitGraphArea : Prpare une zone d'cran  recevoir un affichage    *
*                  graphique.                                          *
**--------------------------------------------------------------------**
*  Entre : X       = Colonne o dbute la zone (1-80)                 *
*           Y       = Ligne o dbute la zone (1-25)                   *
*           XLEN    = Largeur de la zone en caractres                 *
*           YLEN    = Profondeur de la zone en caractres              *
*           MAP     = Numro du jeu de caractres graphiques           *
*           COULEUR = Couleur de la zone graphique (0  7 ou $FF)      *
*  Info   : - Si la couleur indique est $FF , elle est variable       *
*             ce qui donne une effet de "mikado"                       *
***********************************************************************}

procedure InitGraphArea( x, y, xlen, ylen, map, couleur : byte );

var colonne, ligne : integer;                  { Compteur d'itrations }
    codec         : byte;                          { Code de caractre }

begin
  if ( xlen * ylen > 256 ) then                   { Zone trop grande ? }
    writeln( 'Erreur: La zone graphique ne doit pas englober ' +
             ' plus de 256 caracteres !' )
  else
    begin
      if ( CharHauteur = 16 ) then                             { VGA ? }
       SetCharWidth( 8 );   { Oui, on change la largeur des caractres }
      SelectMaps( 0, map );         { Slectionne le jeu de caractres }
      xmax := xlen*CHAR_LARGEUR;           { Coordonnes max en pixels }
      ymax := ylen*CharHauteur;
      lenx := xlen;
      fontptr := ptr( $A000, map * $4000 ); { Pointe sur mm graphique }
      GetFontAccess;         { Autorise l'accs aux jeux de caractres }
      ClearGraphArea;                                 { Efface la zone }
      ReleaseFontAccess;       { rtablit l'accs  la mmoire d'cran }

      {-- Remplit la zone graphique avec des caractres ---------------}

      codec := 0;
      for ligne := ylen-1 downto 0 do          { Lignes de bas en haut }
        for colonne := 0 to xlen-1 do    { Colonnes de gauche  droite }
          begin              { Fixe le code et l'attribut du caractre }
            vioptr^[ligne+y,colonne+x].Caractere := codec;
            if ( couleur = $ff ) then
              vioptr^[ligne+y,colonne+x].Attribut := codec mod 6 + 1 + 8
            else
              vioptr^[ligne+y,colonne+x].Attribut := couleur or $08;
            inc( codec );                          { Caractre suivant }
          end;
    end;
end;

{***********************************************************************
*  CloseGraphArea : Clture l'accs  la zone graphique                *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************}

procedure CloseGraphArea;

begin
  ReleaseFontAccess;             { Libre l'accs  la mmoire d'cran }
  SelectMaps( 0, 0 );                 { Choisit le jeu de caractres 0 }
  if ( CharHauteur = 16 ) then                                 { VGA ? }
   SetCharWidth( 9 );            { Oui, fixe la largeur des caractres }
end;

{***********************************************************************
*  SetPixel: Dessine ou efface un pixel dans la fentre graphique      *
**--------------------------------------------------------------------**
*  Entre : X,Y   = Coordonnes du pixel (0-...)                       *
*            ON   = TRUE pour dessiner et FALSE pour effacer           *
***********************************************************************}

procedure SetPixel( x, y : integer; on : boolean );

var charnum,                                     { Numro du caractre }
    line     : byte;                 { Line  l'intrieur du caractre }

begin
  if ( x < xmax ) and ( y < ymax ) then            { Coordonnes o.k.? }
    begin             { Oui calcule le numro du caractre et la ligne }
      charnum := ((x div CHAR_LARGEUR) + (y div CharHauteur * lenx));
      line    := CharHauteur - ( y mod CharHauteur ) - 1;
      if on then                               { Dessiner ou effacer ? }
        fontptr^[charnum, line] := fontptr^[charnum, line] or
             1 shl (CHAR_LARGEUR - 1 - ( x mod CHAR_LARGEUR ) )
      else
        fontptr^[charnum, line] := fontptr^[charnum, line] and
           not( 1 shl (CHAR_LARGEUR - 1 - ( x mod CHAR_LARGEUR ) ) );
    end;
end;

{***********************************************************************
*  Line : Trace un segment dans la fentre graphique en appliquant     *
*         l'algorithme de Bresenham                                    *
**--------------------------------------------------------------------**
*  Entre : X1, Y1 = Coordonnes de l'origine (0 - ...)                *
*           X2, Y2 = Coordonnes de l'extrmit terminale              *
*           ON     = TRUE pour dessiner et FALSE pour effacer          *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; on : boolean );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Procdure accessoire pour changer deux variables entires --------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Procdure principale ----------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then{ Sens : par l'axe des X ou des Y }
    begin                                            { par l'axe des Y }
      if ( y1 > y2 ) then                        { y1 suprieur  y2 ? }
        begin
          SwapInt( x1, x2 );                  { Oui, change X1 et X2, }
          SwapInt( y1, y2 );                                { Y1 et Y2 }
        end;

      if ( x2 > x1 ) then xincr := 1          { Fixe le pas horizontal }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      SetPixel( x, y, on );                 { Dessine le premier point }
      for y:=y1+1 to y2 do                      { Parcourt l'axe des Y }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          SetPixel( x, y, on );
        end;
    end
  else                                               { par l'axe des X }
    begin
      if ( x1 > x2 ) then                     { x1 plus grand que x2 ? }
        begin
          SwapInt( x1, x2 );                   { Oui, change X1 et X2 }
          SwapInt( y1, y2 );                                { Y1 et Y2 }
        end;

      if ( y2 > y1 ) then yincr := 1            { Fixe le pas vertical }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      SetPixel( x, y, on );                 { Dessine le premier point }
      for x:=x1+1 to x2 do                      { Parcourt l'axe des X }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          SetPixel( x, y, on );
       end;
    end;
end;

{***********************************************************************
*  SetPalCol : dfinit une couleur dans une des 16 palettes            *
*              ou la couleur du cadre d'cran (Overscan-Color)         *
**--------------------------------------------------------------------**
*  Entre : RegNr = Numro du registre de palette (0  15) ou 16       *
*                   pour la couleur du cadre d'cran                   *
*           Col   = Code de couleur entre 0 et 15                      *
***********************************************************************}

procedure SetPalCol( RegNr : byte; Col : byte );

var Regs   : Registers;       { Registres pour grer les interruptions }

begin
  Regs.AX := $1000;              { Option $00 de la fonction vido $10 }
  Regs.BH := Col;                                       { Code couleur }
  Regs.BL := RegNr;      { Numro du registre du contrleur d'attribut }
  intr( $10, Regs );            { Appelle l'interruption vido du BIOS }
end;

{***********************************************************************
*  SetPalAry : Installe une nouvelle palette de 16 couleurs            *
*              sans changer la couleur du cadre d'cran.               *
**--------------------------------------------------------------------**
*  Entre : NewCol = Table de palettes du type PALARY                  *
***********************************************************************}

procedure SetPalAry( NewCol : PALARY );

var i : byte;   { Compteur d'itrations }

begin
  for i := 1 to 16 do  { Parcourt les 16 lments de la table }
    SetPalCol( i-1, NewCol[ i ] ); { Fixe une couleur }
end;

{***********************************************************************
*  GetPalCol: Dtermine le contenu d'un registre de palette            *
**--------------------------------------------------------------------**
*  Entre : RegNr = Numro du registre de palette (0  15) ou 16       *
*                   pour la couleur du cadre d'cran                   *
*  Sortie : Code de couleur                                            *
*  Info   : Avec les cartes EGA il n'est pas possible de lire le       *
*           contenu des registres de palette. On suppose que dans ce   *
*           cas les registres de palette sont en disposition standard, *
*           la fonction retourne alors le numro correspondant         *
***********************************************************************}

function GetPalCol( RegNr : byte ) : byte;

var Regs   : Registers;                { Registres pour l'interruption }

begin
  if ( CharHauteur = 14 ) then                           { Carte EGA ? }
   GetPalCol := RegNr{ Oui impossible de lire les registres de palette }
  else                                                     { Non , VGA }
    begin
      Regs.AX := $1007;          { Option $07 de la fonction vido $10 }
      Regs.BL := RegNr;  { Numro du registre du contrleur d'attribut }
      intr( $10, Regs );      { Dclenche l'interruption vido du BIOS }
      GetPalCol := Regs.BH;       { C'est ici que se trouve le contenu }
    end;                                      { du registre de palette }
end;

{***********************************************************************
*  GetPalAry : Lit les contenus des 16 registres de palette et les     *
*              transfre dans une table                                *
**--------------------------------------------------------------------**
*  Entre : ColAry = Table de palettes de type PALARY, qui va recevoir *
*                    qui va recevoir les codes des couleurs            *
***********************************************************************}

procedure GetPalAry( var ColAry : PALARY );

var i : byte;                                  { Compteur d'itrations }

begin    for i := 1 to 16 do     { Parcourt les 16 lments du tableau }
    ColAry[ i ] := GetPalCol( i-1 );   { Lit une couleur  chaque fois }
end;

{***********************************************************************
*  Mikado : Dmonstration du maniement des routines prsentes         *
*           dans ce programme                                          *
**--------------------------------------------------------------------**
*  Entre : nant                                                     *
***********************************************************************}

procedure Mikado;

type linie = record                         { Coordonnes d'un segment }
               x1, y1,
               x2, y2  : integer;
             end;

const NewCols : PALARY =
( {--------- Couleurs des caractres de texte ordinaires --------------}
          BLACK,                                                { noir }
          BLUE,                                                  { bleu}
          GREEN,                                                { vert }
          RED,                                                 { rouge }
          CYAN,                                                 { cyan }
          MAGENTA,                                           { magenta }
          YELLOW,                                               { brun }
          WHITE,                                               { blanc }
                  {------------- Couleurs des graphiques --------------}
          LIGHTBLUE,                                      { bleu clair }
          LIGHTGREEN,                                     { vert clair }
          LIGHTRED,                                      { rouge clair }
          LIGHTCYAN,                                      { cyan clair }
          LIGHTMAGENTA,                                { magenta clair }
          BLUE,                                                 { bleu }
          YELLOW,                                              { jaune }
          WHITE );                                             { blanc }

var i,                                         { Compteur d'itrations }
    first,                           { Indice du mikado le plus rcent }
    last   : integer;                { Indice du mikado le plus ancien }
    clear  : boolean;                             { Efface les mikados }
    lar    : array[1..MIKADOS] of linie;           { Table des mikados }
    OldCols: PALARY;                    { Table des anciennes couleurs }

begin
  GetPalAry( OldCols );             { Dtermine les couleurs prsentes }
  SetPalAry( NewCols );                { Installe une nouvelle palette }
  TextColor( 7 );
  TextBackGround( 1 );
  ClrScr;                                             { Efface l'cran }
  GotoXY(1,1);                           { Et le remplit de caractres }
  for i:=1 to 25*80-1 do                             { du jeu standard }
   write( chr(32 + i mod 224) );

  {-- Initialise la zone graphique et fait tomber les mikados ---------}

  GotoXY(27,6);
  TextColor( 7 );
  TextBackGround( 3 );
  write('       M I K A D O       ' );
  GotoXY(27,6);
  InitGraphArea( 27, 7, 25, 10, 1,  $FF );
  GetFontAccess;               { Assure l'accs aux jeux de caractres }

  clear := false;                           { Pour effacer les mikados }
  first := 1;                          { Commence au dbut de la table }
  last := 1;
  repeat                                          { Boucle des mikados }
    if first = MIKADOS+1 then first := 1;              { Wrap-Around ? }
    lar[first].x1 := random( xmax-1 );                { Cre un mikado }
    lar[first].x2 := random( xmax-1 );
    lar[first].y1 := random( ymax-1 );
    lar[first].y2 := random( ymax-1 );
    line( lar[first].x1, lar[first].y1,                { et le dessine }
          lar[first].x2, lar[first].y2, true );
    inc( first );                                     { Mikado suivant }
    if first = MIKADOS+1 then clear := true;       { faut-il effacer ? }
    if clear then                                       { maintenant ? }
      begin                                                      { Oui }
        line( lar[last].x1, lar[last].y1,
              lar[last].x2, lar[last].y2, false );
        inc( last );    { Mikado suivant }
        if last = MIKADOS+1 then last := 1;
      end;
  until keypressed;            { Rpte jusqu' dtection d'une frappe }

  {-- Termine le programme --------------------------------------------}

  CloseGraphArea;
  SetPalAry( OldCols );                  { Rtablit l'ancienne palette }
  GotoXY(1, 25 );
  TextColor( 7 );
  TextBackGround( 0 );
  ClrEol;
  writeln( 'Le jeu de caractres standard est  nouveau en place.');
end;

{----------------------------------------------------------------------}
{------------------------ PROGRAMME PRINCIPAL -------------------------}
{----------------------------------------------------------------------}

begin
  if IsEgaVga then                     { A-t-on une carte EGA ou VGA ? }
    Mikado                             { Oui, c'est parti pour la dmo }
  else                 { Non, impossible de faire tourner le programme }
    writeln( 'Attention: aucune carte EGA ou VGA n''est installe !' );
end.
