{**********************************************************************}
{                          P L I N K P . P A S                         }
{----------------------------------------------------------------------}
{    Fonction       : transmet des fichiers par le port parallle      }
{----------------------------------------------------------------------}
{    Auteur       : Michael Tischer                                    }
{    Dvelopp le : 27.09.1991                                         }
{    Dernire MAJ : 28.11.1991                                         }
{**********************************************************************}
{$M 65520, 0, 655360 }

uses dos, crt;                         { Intgre les units DOS et CRT }

{== Constantes ========================================================}

const UNESEC       =  18;                               { Une seconde  }
      DIXSEC       =  182;                             { Dix secondes  }
      TO_DEFAULT   =  DIXSEC;                    { Time-Out par dfaut }

      MAXBLOC     =  4096;         { Taille d'un bloc 4 Ko ( Cluster ) }

{-- Constantes pour le protocole de transmission ----------------------}

      ACK          = $00;                        { Accus de rception }
      NAK          = $FF;                    { Non accus de rception }
      MAX_TRY      = 5;              { Nombre de tentatives autorises }

{-- Tokens pour la communication entre metteur et rcepteur --------}

      TOK_DATSTART = 0;                             { Dbut de fichier }
      TOK_DATNEXT  = 1;                   { Suite d'un bloc de donnes }
      TOK_DONNEESD   = 2;           { Transmission du fichier termine }
      TOK_FIN     = 3;                              { Fin du programme }
      TOK_ESCAPE   = 4;{ Interruption par ESC sur l'ordinateur distant }

{-- Codes pour les appels LongJump ------------------------------------}

      LJ_OKEMETTEUR  = 1;        { Tous les fichiers correctement mis }
      LJ_OKRECEPT    = 2;       { Tous les fichiers correctement reus }
      LJ_TIMEOUT   = 3;     { Time-out: le correspondant ne rpond pas }
      LJ_ESCAPE    = 4;  { Interruption par Esc sur l'ordinateur local }
      LJ_REMESCAPE = 5;{ Interruption par Esc sur l'ordinateur distant }
      LJ_DATA      = 6;                      { Erreur de communication }
      LJ_NOLINK    = 7;                               { Pas de liaison }
      LJ_NOPAR     = 8;                             { Pas d'interface  }
      LJ_PARA      = 9;                 { Paramtres d'appel invalides }

{== Dfinitions de types===============================================}

type BHEADER = record            { En-tte pour transmission des blocs }
                 case boolean of
                   true  : ( Token : byte;
                             Len   : word );
                   false : ( Champ  : array[ 0..2 ] of byte );
               end;

     JMPBUF  = record              { Enregistrement des informations , }
         BP,                         { ncessites pour le branchement }
                 SP,
         CS,                  { ne pas changer l'ordre des variables ! }
                 IP  : word;
               end;

     DBloc = array[ 1..MAXBLOC ] of byte;          { Donnees d'un bloc }

{== Variables globales  ===============================================}

var InPort       : word;                    { Adresse du port d'entre }
    OutPort      : word;
    Escape       : boolean;            { Pas de touche ESCAPE enfonce }
    Timeout      : word;              { valeur de Timeout slectionne }
    TO_Count     : word;                         { Compteur de Timeout }
    Branchement  : JMPBUF;           { Adresse de retour pour terminer }
    BlocBuf      : DBLOC;              { Buffer pour mmoriser un bloc }
    Fichier      : file;    { Variable fichier pour traiter un fichier }

{== Dclaration des fonctions en assembleur ===========================}

{$L plinkpa.obj }    { Intgre le module en assembleur }

function  getb : byte; external;
procedure putb( Wert : byte ); external;
procedure intr_install(escape_flag, timeout_count : pointer); external;
procedure intr_remove; external;
procedure EscapeDirect( Declenche : boolean ); external;

{**********************************************************************}
{ SetJmp : Dtermine l'emplacement du programme o doit se brancher    }
{          LongJmp                                                     }
{----------------------------------------------------------------------}
{ Entre  : JB : Structure de donnes du type JumpBuf, qui contient    }
{                les informations ncessaires au branchement           }
{ Sortie  : NOJMP au retour de cette fonction ; toute autre valeur     }
{           lorsque le retour suit un appel  LongJmp                  }
{**********************************************************************}

{$F+} { Setjmp et Longjmp doivent tre de type FAR }

function SetJmp( var JB : JMPBUF ) : integer;

type WordP  = ^word;                             { Pointeur sur un mot }

begin
  JB.BP := WordP( ptr( SSeg, Sptr+2) )^;
  JB.IP := WordP( ptr( SSeg, Sptr+4) )^;
  JB.CS := WordP( ptr( SSeg, Sptr+6) )^;

      { SP doit rfrencer la position  partir de laquelle LongJmp va }
   { ultrieurement mmoriser le nouveau contenu de BP et l'adresse de }
                                                  { retour sur la pile }

  JB.SP := Sptr + 12 - 6 - 6;

  SetJmp := -1;     { Indique qu'il ne s'agit pas d'un appel  LongJmp }
end;

{**********************************************************************}
{ LongJmp : GOTO interprocdural, qui poursuit l'excution du          }
{           programme  la ligne o SetJmp a t appel                }
{----------------------------------------------------------------------}
{ Entre  : JB      : Buffer de branchement rempli par SetJmp          }
{           RetCode : Rsultat  renvoyer par SetJmp                   }
{**********************************************************************}

procedure LongJmp( JB : JMPBUF; RetCode : integer );

type WordP  = ^word;                             { Pointeur sur un mot }

begin
  WordP( ptr( SSeg, JB.SP   ) )^ := JB.BP;
  WordP( ptr( SSeg, JB.SP+2 ) )^ := JB.IP;
  WordP( ptr( SSeg, JB.Sp+4 ) )^ := JB.CS;

                  {-- Charge en AX le code de retour transmis ce qui --}
                                    {-- simule le rsultat de SetJmp --}

  inline( $8b / $46 / $06 );                           { mov ax,[bp+6] }

  inline( $8b / $ae / $fa / $ff );                     { mov bp,[bp-6] }

          { mov   sp,bp         ;Ces instructions sont automatiquement }
                    { pop   bp            ;gnres par le compilateur }
                       { ret   6             ;restaurant ainsi la pile }
end;

{$F-}

{**********************************************************************}
{ GetPortAdr : Initialise les adresses des ports d'une interface       }
{              parallle c'est--dire les variables InPort et OutPort  }
{ Entre        : NUMERO = Numro de l'interface parallle (1-4)       }
{ Sortie        : TRUE, si interface valide                            }
{ Var. globales : InPort/W, OutPort/W                                  }
{ Info          : Les adresses de base des interfaces parallles       }
{                 (en nombre de 1  4)  se trouvent dans les mots      }
{                 mmoire commenant en 0040:0008                      }
{**********************************************************************}

function GetPortAdr( Numero : integer ) : boolean;

begin
 {-- Lit els adresses des ports dans le segment des variables du BIOS--}
  OutPort := MemW[ $0040: 6 + Numero * 2 ];
  if ( OutPort <> 0 ) then                    { Interface disponible ? }
    begin                                                        { Oui }
      InPort := OutPort + 1;          { Adresse pour registre d'entre }
      GetPortAdr := TRUE;                         { Retour sans erreur }
    end
  else
  GetPortAdr := FALSE;                     { Erreur: interface absente }
end;

{**********************************************************************}
{ Port_Init     : Initialise les registres ncessaires  la            }
{                 transmission                                         }
{ Entres       : EMETTEUR = TRUE, si metteur, FALSE, si rcepteur    }
{ Sortie        : TRUE, si registres correctement initialiss          }
{ Var. globales : InPort/R, OutPort/R                                  }
{ Info          : La dissymtrie: envoie 00010000 , attend 00000000    }
{                 est rendue ncessaire par l'inversion du signal      }
{                 Normalement les registres d'entre et de sortie      }
{                 contiennent les valeurs souhaites mais              }
{                 l'initialisation est ncessaire lorsqu'on reprend une}
{                 transmission interrompue.                            }
{**********************************************************************}

function Port_Init( Emetteur : boolean ) : boolean;

begin
  EscapeDirect( TRUE );              { Dclenche un Time out si Escape }
  if ( Emetteur ) then                 { L'ordinateur est-il metteur? }
    begin
      TO_Count := Timeout * 5;    { Initialise le compteur de Time Out }
      PutB( $10 );                                { Envoie : 00010000b }
      while ( ( GetB <> $00 ) and ( TO_Count > 0 ) ) do     { Attend 0 }
        ;
    end
  else                                    { L'ordinateur est rcepteur }
    begin
      TO_Count := Timeout * 5;    { Initialise le compteur de Time Out }
      while ( ( GetB <> $00 ) and ( TO_Count > 0 ) ) do     { Attend 0 }
        ;
      PutB( $10 );                                 { Envoie: 00010000b }
    end;
  EscapeDirect( FALSE );                    { Si Escape pas de Timeout }
  Port_Init := ( TO_Count > 0 );             { Initialisation termine }
end;

{**********************************************************************}
{ EmetOctet     : Envoie un octet en deux parties  l'ordinateur       }
{                 distant et teste le rsultat                         }
{ Entres       : VALEUR = octet  mettre                             }
{ Sortie        : Transmission correcte ? ( 0 = Erreur , -1 = ok )     }
{ Var. globales : Timeout/R, InPort/R, OutPort/R (dans Macros)         }
{**********************************************************************}

function EmetOctet( Wert : byte ) : boolean;

var Retour : byte;                                 { Octet rceptionn }

begin
         {-- Emet le quartet infrieur --------------------------------}

  TO_Count := Timeout;            { Initialise le compteur de Time Out }
  PutB( Wert and $0F );                  { Envoi avec mise  0 de BUSY }
  while ( ( ( GetB and 128 ) = 0 ) and ( TO_Count > 0 ) ) do
    ;
  if ( TO_Count = 0 ) then                      { Erreur de Time Out ? }
    longjmp( Branchement, LJ_TIMEOUT );   { Interrompt la transmission }

  Retour := ( GetB shr 3 ) and $0F;                  { Bits 3-6 en 0-3 }

  {-- Emet le quartet suprieur ---------------------------------------}

  TO_Count := Timeout;            { Initialise le compteur de Time Out }
  PutB( ( Wert shr 4 ) or $10 );        { Envoie avec mise  1 de BUSY }
  while ( ( ( GetB and 128 ) <> 0 ) and ( TO_Count > 0 ) ) do

  if ( TO_Count = 0 ) then                         { Erreur de Timeout }
    longjmp( Branchement, LJ_TIMEOUT );   { Interrompt la transmission }

  Retour := Retour or ( ( GetB shl 1 ) and $F0 );    { Bits 3-6 en 4-7 }
  EmetOctet :=  ( Wert = Retour );     { Octet correctement transmis ? }
end;

{**********************************************************************}
{ RecOctet      : Rceptionne un octet en deux parties de la part d'un }
{                 ordinateur distant et renvoie les parties pour       }
{                 vrification                                         }
{ Entres       : aucune                                               }
{ Sortie        : octet reu                                           }
{ Var. globales : Timeout/R, InPort/R, OutPort/R ( Macros )            }
{**********************************************************************}

function RecOctet : byte;

var LoNib,
    HiNib : byte;                                     { Quartets reus }

begin
  {-- Rceptionne et renvoie le quartet infrieur ---------------------}

  TO_Count := Timeout;            { Initialise le compteur de Time Out }
  while ( ( ( GetB and 128 ) = 0 ) and ( TO_Count > 0 ) ) do
    ;

  if ( TO_Count = 0 ) then                      { Erreur de Time Out ? }
    longjmp( Branchement, LJ_TIMEOUT );   { Interrompt la transmission }

  LoNib := ( GetB shr 3 ) and $0F;                   { Bits 3-6 en 0-3 }
  PutB( LoNib );                              { Retour  l'expditeur  }

  {-- Rceptionne et renvoie le quartet suprieur ---------------------}

  TO_Count := Timeout;             { Initialise le compteur de Timeout }
  while ( ( ( GetB and 128 ) <> 0 ) and ( TO_Count > 0 ) ) do
    ;

  if ( TO_Count = 0 ) then                      { Erreur de Time Out ? }
    longjmp( Branchement, LJ_TIMEOUT );   { Interrompt la transmission }

  HiNib := ( GetB shl 1 ) and $F0;                   { Bits 3-6 en 4-7 }
  PutB( ( HiNib shr 4 ) or $10 );     { Renvoie en let
      Busy  1 }

  RecOctet := ( LoNib or HiNib );                         { Octet reu }
end;

{**********************************************************************}
{ EmetBloc : Emmet un bloc de donnes                                  }
{ Entre   : TOKEN  = Commande pour le rcepteur                       }
{            NOMBRE = Nombre d'octets  transmettre                    }
{            DPTR   = Pointeur sur le buffer des donnes               }
{ Sortie   : nant, en cas d'erreur branchement par LongJmp  la       }
{            routine de traitement d'erreur                            }
{**********************************************************************}

procedure EmetBloc( Token  : byte;
                    Nombre : word;
                    Dptr   : pointer );

var header     : BHEADER;     { En-tte pour mmorise Token et  Nombre }
    RecEscape : byte;{ Entre la touche ESC sur l'ordinateur distant ? }
    ok         : boolean;                        { Indicateur d'erreur }
    i          : word;                         { Compteur d'itrations }
    try        : word;               { Nombre de tentatives restantes  }
    Donnees      : ^DBloc;              { Pointeur sur bloc de donnes }

begin
  if ( Escape ) then         { A-t-on tap Escape sur cet ordinateur ? }
    begin
      Token := TOK_ESCAPE;             { Oui, mettre  le token Escape }
      Nombre := 0;
    end;

       {-- Emission de l'en-tte --------------------------------------}

  header.Token := Token;                         { Construit l'en-tte }
  header.Len := Nombre;

  try := MAX_TRY;
  repeat                               { AU maximum MAX_TRY tentatives }
    ok := TRUE;                   { A priori la transmission est bonne }
    for i := 0 to 2 do
      ok := ok and EmetOctet( Header.Champ[ i ] );     { Emet un octet }
    if ( ok ) then
      ok := ok and EmetOctet( ACK )                     { Confirmation }
    else
      ok := ok and EmetOctet( NAK );                    { Confirmation }
    if ( not ok ) then                                      { Erreur ? }
      dec( try );                               { Oui, autre tentative }
  until ( ( ok ) or ( try =  0 ) );

  if ( try = 0 ) then            { L'en-tte a-t-il pu tre transmis ? }
    longjmp( Branchement, LJ_DATA ); { Non, interrompt la transmission }

  if ( Token = TOK_ESCAPE ) then     { A-t-on envoy l'avis d'ESCAPE ? }
    longjmp( Branchement, LJ_ESCAPE );  { Oui, interrompt transmission }

    {-- Emission du bloc de donnes proprement dit---------------------}

  if ( Nombre > 0 ) then                         { Taille diff de 0  ? }
    begin
      Donnees := DPTR;
      try := MAX_TRY;
      repeat
    ok := TRUE;                   { A priori la transmission est bonne }
        for i := 1 to Nombre do
          ok := ok and EmetOctet( Donnees^[ i ] );
        if ( ok ) then
      ok := ok and EmetOctet( ACK )                     { Confirmation }
        else
      ok := ok and EmetOctet( NAK );
    if ( not ok ) then                                      { Erreur ? }
      dec( try );                            { Oui, nouvelle tentative }
      until ( ( ok ) or ( try =  0 ) );
      if ( try = 0 ) then { Le bloc de donnes transmis correctement ? }
    longjmp( Branchement, LJ_DATA ); { Non, interrompt la transmission }
    end;

  {-- Teste l'octet ESCAPE du rcepteur  ------------------------------}

  try := MAX_TRY;
  repeat
    RecEscape:=RecOctet;                   { Dtecte un Escape distant }
    dec( try );
  until ( ( RecEscape = byte( true ) ) or
          ( RecEscape = byte( false ) ) );

  if ( try = 0 ) then                { Etat de la touche Escape reu ? }
    longjmp( Branchement, LJ_DATA ); { Non, interrompt la transmission }

  if ( RecEscape = byte( true ) ) then { Esc. sur ordinateur distant ? }
    longjmp( Branchement, LJ_REMESCAPE );{ Oui interrompt transmission }
end;

{**********************************************************************}
{ RecBloc : Rceptionne un bloc de donnes                             }
{ Entres : TOKEN  = Pointeur sur la variable qui mmorise le token    }
{           LEN    = Pointeur sur la variable qui mmorise la longueur }
{           DONNEES  = Pointeur sur le buffer des donnes              }
{ Sortie  : nant, en cas d'erreur branchement sur routine d'erreur    }
{           par LongJmp                                                }
{ Info    : Le buffer translis doit prvoir de la place pour           }
{           MAXBLOC octets, la taille du bloc ne pouvant               }
{           tre anticipe                                             }
{**********************************************************************}

procedure RecBloc( var Token : byte;
                         var Len   : word;
                             Dptr  : pointer );

var header       : BHEADER;             { En-tte pour Token et Nombre }
    ok           : boolean;                      { Indicateur d'erreur }
    i            : word;                       { Compteur d'itrations }
    try          : word;              { Nombre de tentatives restantes }
    EscapeStatus : boolean;
    ByteBuffer   : byte;
    Donnees        : ^DBloc;            { Pointeur sur bloc de donnes }

begin
       {-- Rceptionne d'abord l'en-tte ------------------------------}

  try := MAX_TRY;
  repeat
    for i:= 0 to 2 do
      Header.Champ[ i ] := RecOctet;

    ByteBuffer := RecOctet;
    if ( ByteBuffer <> ACK ) then      { Tous les octets bien reus  ? }
      dec( try );                     { Oui, pas de nouvelle tentative }
  until ( ( try = 0 ) or ( ByteBuffer = ACK ) );

  if ( try = 0 ) then                    { En-tte correctement reu ? }
    longjmp( Branchement, LJ_DATA );  { Non Interrompt la transmission }

  Token := Header.Token;
  Len := Header.Len;
  if ( Token = TOK_ESCAPE ) then                    { Emetteur ESCAPE? }
    longjmp( Branchement, LJ_REMESCAPE );{ Oui interrompt transmission }

          {-- L'en-tte est bon, il faut passer au bloc des donnes. --}

  if ( Len > 0 ) then                       { Pas de bloc de donnes ? }
    begin                                                          { si}
      Donnees := Dptr;
      try := MAX_TRY;

      repeat                      { Rceptionne le bloc octet par octet}
        for i := 1 to len do
         Donnees^[ i ] := RecOctet;

         ByteBuffer := RecOctet;
     if ( ByteBuffer <> ACK ) then      { Tous les octets bien reus ? }
       dec( try );                    { Oui, pas de nouvelle tentative }
      until ( ( try = 0 ) or ( ByteBuffer = ACK ) );

      if ( try = 0 ) then              { Le bloc a-t-il t bien reu? }
    longjmp( Branchement, LJ_DATA ); { Non, interrompt la transmission }
    end;

 {-- Envoie  l'ordinateur distant l'tat actuel de la touche Escape --}

  EscapeStatus := Escape;                            { Mmorise l'tat }

  try := MAX_TRY;
  repeat
    dec( try );
  until ( EmetOctet( byte( EscapeStatus ) ) or ( try = 0 ) );

  if ( try = 0 ) then             { L'tat ESC a-t-il pu tre envoy ? }
    longjmp( Branchement, LJ_DATA ); { Non, interrompt la transmission }

  if ( EscapeStatus ) then {A-t-on actionn Escape sur cet ordinateur ?}
    longjmp( Branchement, LJ_ESCAPE );  { Oui, interrompt transmission }
end;

{**********************************************************************}
{ EmetFichier : Emet un fichier                                        }
{ Entres     : NAME = Nom de fichier                                  }
{ Sortie      : nant                                                  }
{**********************************************************************}

procedure EmetFichier( Name : string );

var Status  : word;                                  { Etat d'mission }
    Lus : word;                                  { Nombre d'octets lus }
    Taille : longint;                        { Nombre d'octets envoys }

begin
  write( copy( Name + '                   ', 1, 13 ) );
  assign( Fichier, Name );
  reset( Fichier, 1 );
  EmetBloc( TOK_DATSTART, length( Name ) + 1, @Name ); { Envoie le nom }

  {-- Transmet le contenu du fichier ----------------------------------}

  Taille := 0;
  repeat
    Blockread( Fichier, BlocBuf, MAXBLOC, Lus );         { Lit un bloc }
    if ( Lus > 0 ) then                                    { Termin ? }
      begin               {                                      { Non }
    EmetBloc( TOK_DATNEXT, Lus, @BlocBuf );           { Envoie le bloc }
        inc( Taille, Lus );
        write( #13, copy( Name + '             ', 1, 13 ),
               '(', Taille, ')' );
      end;
  until (  Lus = 0 );
  writeln;

  EmetBloc( TOK_DONNEESD, 0, NIL );          { Clture la transmission }

  close( Fichier );                               { Referme le fichier }
end;

{**********************************************************************}
{ RecFichier : Rceptionne un fichier                                  }
{ Entres    : nant                                                   }
{ Sortie     : Dernier token reu                                      }
{**********************************************************************}

function RecFichier : word;

var Status      : word;                            { Etat de rception }
    AEnregistrer : word;                      { Taille du dernier bloc }
    Taille     : longint;
    Token       : byte;                                   { Token reu }
    Len         : word;                               { Longueur reue }
    i           : word;                        { Compteur d'itrations }
    Name        : string[ 13 ];                       { Nom du fichier }

begin
  RecBloc( Token, Len, @BlocBuf );
  if ( Token = TOK_DATSTART ) then
    begin
      for i := 0 to BlocBuf[ 1 ] do
        Name[ i ] := chr( BlocBuf[ i + 1 ] );
      assign( Fichier, Name );
      rewrite( Fichier, 1 );
      write( copy( Name + '                   ', 1, 13 ) );

    {-- Rceptionne le contenu du fichier -----------------------------}

      Taille := 0;
      repeat
    RecBloc( Token, Len, @BlocBuf );             { Rceptionne un bloc }
    if ( Token = TOK_DATNEXT ) then      { Bloc de donnes conscutif ?}
          begin                                                  { Oui }
        Blockwrite( Fichier, BlocBuf, Len );              { Enregistre }
            inc( Taille, Len );
            write( #13, copy( Name + '             ', 1, 13 ),
                   '(', Taille, ')' );
          end;
      until ( TOKEN <> TOK_DATNEXT );
      close( Fichier );                           { Referme le fichier }
      writeln;
    end;
  RecFichier := Token;                      { Retourne l'tat d'erreur }
end;

{**********************************************************************}
{                             PROGRAMME PRINCIPAL                      }
{**********************************************************************}

const Avis : array[ 0..8 ] of string =
          ( 'FIN: Tous les fichiers ont t correctement mis.',
        'FIN: Tous les fichiers ont t correctement reus.',
        'ERREUR: Time-Out, le systme distant ne rpond pas.',
        'FIN: Interruption par Escape.',
        'FIN: Interruption par Escapr sur l''ordinateur distant.',
        'ERREUR: Interface ou cble dfectueux !',
        'ERREUR: Pas de contact avec l''ordinateur distant.',
        'ERREUR: L''interface indique n''existe pas !',
        'ERREUR: Paramtre inconnu ou invalide !' );

var SRec     : SearchRec;{ Structure pour recherche dans un rpertoire }
    Emetteur   : boolean;{ Mode de transmission (Emetteur, rcepteur ) }
    sjStatus : integer;                             { Code de longjump }
    Numero,                                    { Numro de l'interface }
    i,                                                      { Compteur }
    Trouve : byte;                    { Pour la recherche des fichiers }
    dummy    : integer;
    argv     : array[ 1..10 ] of string;                  { Paramtres }


begin
  write( #13#10'Transmission de donnes par l''interface parallle' );
  writeln( '   (c) 1991 by Michael Tischer' );
  write( '=================================================' );
  writeln( '==============================' );

  Escape := false;
  Timeout := TO_DEFAULT;

  if ( paramstr( 1 ) = '?' ) then           { Affiche juste la syntaxe }
    begin                                                        { Oui }
      writeln( 'Appel: plinkc [/Pn] [/Tnn] [Nom de fichier]' );
      halt( 0 );
    end;

  sjStatus := setjmp( Branchement ); { Adresse de retour pour terminer }
  if ( sjStatus > 0 ) then                          { Longjmp appel ? }
    begin                                                        { Oui }
      Intr_remove;    { dsactive le gestionnaire
     d'interruption }
      writeln( #13#10#13#10, Avis[ sjStatus - 1 ] );
      halt( 0 );
    end;

  Intr_Install( @Escape, @TO_Count );{Initialise le driver d'interrupt.}

 {-- Fixe les paramtres par dfaut et exploite la ligne de commande --}

 Emetteur := FALSE;            { Par dfaut l'ordinateur est rcepteur }
 Numero := 1;                                    { et l'interface LPT1 }

 for i := 1 to paramcount do
   begin
     argv[ i ] := paramstr( i );             { mmorise les paramtres }
     if ( argv[ i, 1 ] = '/' ) then
       begin
         case ( upcase( argv[ i, 2 ] ) ) of
           'T' : begin
                   delete( argv[ i ], 1, 2 );
                   val( argv[ i ], Timeout, dummy );
                   Timeout := ( Timeout * DIXSEC ) div 10;
                   if ( Timeout = 0 ) then
             longjmp( Branchement, LJ_PARA );              { incorrect }
                 end;
           'P' : begin
           Numero := ord( argv[ i, 3 ] ) - 48;             { Interface }
                   if ( ( Numero = 0 ) or ( Numero > 4 ) ) then
             longjmp( Branchement, LJ_PARA );              { incorrect }
                 end;
       else longjmp( Branchement, LJ_PARA );                 { inconnu }
         end;
     argv[ i ] := '';                              { Efface l'argument }
       end
     else                                { Doit tre un nom de fichier }
       Emetteur := TRUE;                                    { Emetteur }
   end;

 {-- Dmarre la transmission ------------------------------------------}

 if ( not GetPortAdr( Numero ) ) then    { L'interface existe-t-elle ? }
   longjmp( Branchement, LJ_NOPAR );                     { Non, erreur }

 if ( not Port_Init( Emetteur ) ) then            { Etablit la liaison }
   longjmp( Branchement, LJ_NOLINK );             { Erreur, impossible }

 if ( Emetteur ) then                                      { Emetteur? }
   begin
     writeln( 'Emission vers LPT', Numero, #13#10 );

     {-- Transmet tous les fichiers -----------------------------------}

     for i := 1 to paramcount do       { Parcourt la ligne de commande }
       begin
     if ( argv[ i ] <> '' ) then                    { Nom de fichier ? }
           begin                                                 { Oui }
             findfirst( argv[ i ], AnyFile, SRec);
         while ( DosError = 0 ) do                 { Tant qu'il y en a }
               begin
                 if ( SRec.Attr <> Directory ) then
           EmetFichier( SRec.Name );             { Transmet le fichier }
                 findnext( SRec );
               end;
           end;
       end;
     EmetBloc( TOK_FIN, 0 , NIL );            { Tous les fichiers mis }
     longjmp( Branchement, LJ_OKEMETTEUR );
   end
 else                                                 { Non, Rcepteur }
   begin
     writeln( 'Rception sur LPT', Numero, #13#10 );
     while ( RecFichier <> TOK_FIN ) do     { Rceptionne les fichiers }
       ;                                       { jusqu'au token de FIN }
     longjmp( Branchement, LJ_OKRECEPT );
   end;
end.
