{**********************************************************************}
{*                         R A W _ C O O K P                          *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Offre deux fonctions permettant de faire       *}
{*                     passer un driver de priphrique de caractre  *}
{*                     en mode RAW ou en mode COOKED.                 *}
{*--------------------------------------------------------------------*}
{*    Auteur         : MICHAEL TISCHER                                *}
{*    Dvelopp le   : 16/08/1987                                     *}
{*    Dernire modif.: 28/03/1992                                     *}
{**********************************************************************}

program RAWCOOKP;

Uses Crt, Dos;                            { Intgrer units CRT et DOS }

const ENTSTAND = 0;                  { handle 0 : priph. d'entre std }
      SORSTAND = 1;                 { handle 1 : priph. de sortie std }

var Touche : char;     { Sert uniquement au programme de dmonstration }

{**********************************************************************}
{* GetMode: lit l'attribut d'un driver de priphrique                *}
{* Entre : le handle transmis doit tre reli au priphrique       *}
{*          appeler                                                   *}
{* Sortie : L'attribut de priphrique                                *}
{**********************************************************************}

function GetMode(Handle : integer) : integer;

var Regs : Registers;                    { Reg. d'appel d'interruption }

begin
 Regs.ah := $44;             { Numro de fonction pour IOCTL: Get Mode }
 Regs.bx := Handle;
 MsDos( Regs );                      { Appeler interruption du DOS $21 }
 GetMode := Regs.dx             { Transmettre attribut de priphrique }
end;

{**********************************************************************}
{* SetRaw: fait passer un driver de priphrique en mode RAW          *}
{* Entre : le handle transmis doit tre reli au priphrique       *}
{*          appeler.                                                  *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure SetRaw(Handle : integer);

var Regs : Registers;                    { Reg. d'appel d'interruption }

begin
 Regs.ax := $4401;           { Numro de fonction pour IOCTL: Set Mode }
 Regs.bx := Handle;
 Regs.dx := GetMode(Handle) and 255 or 32;      { Nvl attr. de priph. }
 MsDos( Regs );                      { Appeler interruption du DOS $21 }
end;

{**********************************************************************}
{* SetCooked: fait passer un driver de caractre en mode COOKED       *}
{* Entre : le handle transmis doit tre reli au priphrique       *}
{*          appeler.                                                  *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure SetCooked(Handle : integer);

var Regs : Registers;                    { Reg. d'appel d'interruption }

begin
 Regs.ax := $4401;           { Numro de fonction pour IOCTL: Set Mode }
 Regs.bx := Handle;
 Regs.dx := GetMode(Handle) and 223;      { Nouvel attribut de priph. }
 MsDos( Regs );                      { Appeler interruption du DOS $21 }
end;

{**********************************************************************}
{* SortieTest: sort une chane de test 1000 fois sur le priphrique  *}
{*             de sortie standard                                     *}
{* Entre : Aucune                                                    *}
{* Sortie : Aucune                                                    *}
{**********************************************************************}

procedure SortieTest;

var Regs     : Registers;                { Reg. d'appel d'interruption }
    Compteur : integer;                           { Variable de boucle }
    Test     : string[9];                 { La chane de test  sortir }

begin
 Test    := 'Test.... ';
 Regs.bx := SORSTAND;  { Sortir sur le priphrique de sortie standard }
 Regs.cx := 9;                                  { Nombre de caractres }
 Regs.ds := Seg(Test);                   { Adresse de segment du texte }
 Regs.dx := Ofs(Test)+1;                   { Adresse d'offset du texte }
 for Compteur := 1 to 1000 do             { Sortir la chane 1000 fois }
  begin
    Regs.ah := $40;            { Numro de fonction pour crire handle }
    MsDos( Regs );                   { Appeler interruption du DOS $21 }
   end;
  writeln;
end;

{**********************************************************************}
{*                         PROGRAMME PRINCIPAL                        *}
{**********************************************************************}

begin
  ClrScr;                                                { Vider cran }
  writeln('RAWCOOKP (c) 1987, 92 by Michael Tischer'#13#10);
  writeln('Le driver de console est maintenant en mode RAW.',
          ' Les touches de commande,');
  writeln('comme <CTRL-S> par exemple ne seront pas identifies lors',
           ' de la sortie.'#13#10'Faites-en donc l''essai pendant les',
           ' sorties suivantes !');
  writeln('Veuillez frapper une touche pour commencer...');
  Touche := ReadKey;                             { Attendre une touche }

  SetRaw(ENTSTAND);       { Faire passer driver de console en mode RAW }
  SortieTest;                        { Sortir chane de test 1000 fois }

  ClrScr;                                              { Vider l'cran }
    while KeyPressed do                      { Vider le buffer clavier }
    Touche := ReadKey;                                   { Lire touche }
  writeln('Le driver de console se trouve maintenant en mode COOKED.');
  writeln('Les touches de commande comme <CTRL-S> par exemple sont');
  writeln('identifies lors de la sortie et traites en consquence.');
  writeln('Veuillez frapper une touche pour commencer...');

  Touche := ReadKey;                             { Attendre une touche }
  SetCooked(ENTSTAND);            { Faire passer driver en mode COOKED }
  SortieTest;                     { Sortir la chane de test 1000 fois }
end.
