{**********************************************************************}
{*                        R E C L O C K P                             *}
{*--------------------------------------------------------------------*}
{*    Fonction     : Dmonstration des fonctions de verrouillage      *}
{*                   d'enregistrements DOS                            *}
{*--------------------------------------------------------------------*}
{*    Auteur       : Michael Tischer                                  *}
{*    Dvelopp le : 19.09.1991                                       *}
{*    Dernire MAJ : 24.09.1991                                       *}
{**********************************************************************}

program RecLockP;

uses Crt, Dos,                        { Intgrer les units CRT et DOS }
     NetFileP;                            { Intgrer l'unit de rseau }

const NomFichier = 'Rec.dat';         { Nom de fichier du fichier test }

type  Test     = array[ 1..160 ] of char;    { Type de donnes du test }
      TestFile = file of Test;

var Fichier : TestFile;                                 { Fichier test }

{**********************************************************************}
{* CreerEnreg : Crer des enregistrements pour un test                *}
{* Entre     : Caractres pour l'enregistrement                      *}
{* Sortie     : Donnes de test                                       *}
{**********************************************************************}

procedure CreerEnreg(     Caractere : char;
                       var Enreg    : test );

var i : word;                                     { Compteur de boucle }

begin
  for i := 1 to 160 do
    Enreg[ i ] := Caractere;
end;

{**********************************************************************}
{* OuvrirFichier : Ouvrir un fichier rseau existant. Sinon crer un  *}
{*                 nouveau fichier test et complter le fichier       *}
{*                 avec des donnes de test                           *}
{* Entre        : Fichier                                            *}
{* Sortie        : Fichier                                            *}
{**********************************************************************}

function OuvrirFichier( var Fichier : testfile ) : boolean;

var i        : word;                              { Compteur de boucle }
    TestChaine : Test;{ Ncessaire pour la cration de donnes de test }

begin
{-- Ouvrir un fichier pour l'entre et la sortie en mode Deny-None ----}

  NetReset( NomFichier, fm_rw or sm_no, sizeof( Test ), Fichier );
  if ( NetError = NE_FileNotFound ) then         { Fichier inexistant? }
    begin

{-- Crer un fichier et complter par des donnes de test -------------}

      NetRewrite( NomFichier, fm_rw or sm_no, sizeof( Test ), Fichier );
      if ( NetError = 0 ) then                    { Crer sans erreur? }
        begin
          if NetLock( Fichier, 0, 26 ) then    { Verrouiller 26 enreg. }
            begin
              NetSeek( Fichier, 0 );{ Pointeur sur le dbut du fichier }
              for i := 1 to 26 do
                begin
                  CreerEnreg( chr( ord( 'Z' ) + 1 - i ), TestChaine );
                  NetWrite( Fichier, TestChaine );{Ecrire donnes test }
                end;
              OuvrirFichier := NetUnlock( Fichier, 0, 26 );
            end
          else
            OuvrirFichier := false;      { Erreur lors du verrouillage }
        end
      else
        OuvrirFichier := false;{ Erreur lors de la cration du fichier }
    end
  else
    OuvrirFichier := ( NetError = 0 );             { Ouvrir ou erreur? }
end;

{**********************************************************************}
{* NetzEdit   : Dmonstration des fonctions de rseau                 *}
{* Entre     : Fichier                                               *}
{* Sortie     : Fichier                                               *}
{**********************************************************************}

procedure NetzEdit( var Fichier : TestFile );

var NumEnreg : longint;               { Numro d'enregistrement actuel }
    EnregAct   : Test;                         { Enregistrement actuel }
    Action    : byte;                               { Action souhaite }
    Etat    : boolean;                    { Enregistrement verrouill? }
    Caractere   : char;

begin
{-- Afficher le menu --------------------------------------------------}

  writeln( #13#10'Fonctions possibles' );
  writeln( '  1: Positionner le pointeur de fichier.' );
  writeln( '  2: Verrouiller l''enregistrement' );
  writeln( '  3: Lire l''enregistrement' );
  writeln( '  4: Modifier l''enregistrement' );
  writeln( '  5: Ecrire dans l''enregistrement' );
  writeln( '  6: Lever l''interdiction' );
  writeln( '  7: Fin' );

  NumEnreg := 0;                               { Enregistrement actuel }
  Etat := false;                       { Enregistrement non verrouill }
  CreerEnreg( #32, EnregAct );          { Crer un enregistrement vide }

  repeat
 {-- Afficher les informations ----------------------------------------}

    gotoxy( 1, 16 );     { Afficher la position du pointeur de fichier }
    writeln( 'Enregistrement actuel: ',  NumEnreg : 4 );
    write( 'Etat          : ' );
    if Etat then
      writeln( 'Verrouill' )
    else
      writeln( 'Libre    ' );
    Writeln( 'Statut rseau  : ', NetError: 4, '  = ',
       copy( NetErrorMsg( NetError ) + '                   ', 1, 30 ) );
    gotoxy( 1, 21 );               { Afficher l'enregistrement de test }
    writeln( 'Enregistrement actuel:' );
    writeln( EnregAct );

    NetSeek( Fichier, NumEnreg ); { Positionner le pointeur de fichier }

    gotoxy( 1, 13 );
    write( 'Choix:                            ' );
    gotoxy( 10, 13 );
    readln( Action );
    case Action of
      1 : begin
            gotoxy( 1, 13 );
            write( 'Nouveau numro d''enregistrement: ' );
            readln( NumEnreg );
            Etat := false;             { Enregistrement non verrouill }
            CreerEnreg( #32, EnregAct )
          end;
      2 : Etat := Etat or NetLock( Fichier, NumEnreg, 1 );
      3 : NetRead( Fichier, EnregAct );        { Lire l'enregistrement }
      4 : begin
            gotoxy( 1, 13 );
            write( 'Nouveau caractre: ' );
            readln( Caractere );
            CreerEnreg( Caractere, EnregAct );
          end;
      5 : NetWrite( Fichier, EnregAct );{ Ecrire dans l'enregistrement }
      6 : Etat := Etat and not NetUnlock( Fichier, NumEnreg, 1 );
    end;
  until ( Action = 7 );
end;

{**********************************************************************}
{*               P R O G R A M M E  P  R I N C I P A L                *}
{**********************************************************************}

begin
  clrscr;
  writeln( 'Dmo des fonctions de verrouillage de fichiers DOS',
           '  (C) 1991 by Michael Tischer' + paramstr( 1 ) );
  writeln( '====================================================',
           '===========================' );

  if ( ShareInst ) then                    { Programme Share install? }
    begin
      if OuvrirFichier( Fichier ) then       { Fichier ouvert ou cr? }
        begin
          NetzEdit( Fichier ); { Dmonstration des fonctions de rseau }
          NetClose( Fichier );                     { Fermer le fichier }
        end
      else
        writeln( #13#10'Erreur lors de l''ouverture du fichier rseau, ' +
                       'Numro d''erreur: ', NetError );
      ClrScr;
    end
  else
    writeln( #13#10'Test non possible, Share doit tre install' );
end.
