{***********************************************************************
*                           S 3 2 4 0 P . P A S                        *
**--------------------------------------------------------------------**
*  Fonction : Monte comment travailler avec des sprites dans le mode   *
*             graphique VGA 320*200 avec 256 couleurs et deux pages    *
*             d'cran. Ce programme utilise les routines en assembleur *
*             des modules V3240PA.ASM et S3240PA.ASM                   *
**--------------------------------------------------------------------**
*  Auteur       : MICHAEL TISCHER                                      *
*  Dvelopp le : 12.09.1990                                           *
*  Dernire MAJ : 14.01.1991                                           *
***********************************************************************}

program S3240P;

uses dos, crt;

{-- Rfrences externes aux routines en assembleur -------------------}

{$L v3240pa}   { Intgre le module en assembleur }

procedure init320400; external;
procedure setpix( x, y : integer; couleur : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{$L s3240pa}   { Intgre le module en assembleur }

procedure CopyPlane2Buf( bufptr : pointer;
                         page   : byte;
                         dex,
                         dey   : integer;
                         largeur,
                         hauteur  : byte    ); external;

procedure CopyBuf2Plane( bufptr : pointer;
                         page   : byte;
                         ax,
                         ay  : integer;
                         largeur,
                         hauteur  : byte;
                         bg     : boolean ); external;

{-- Constantes --------------------------------------------------------}

const MAXX = 319;                              { Coordonnes maximales }
      MAXY = 399;

      OUT_LEFT   = 1;     { Indicateurs de collision dans SpriteMove() }
      OUT_TOP    = 2;
      OUT_RIGHT  = 4;
      OUT_BOTTOM = 8;
      OUT_NO     = 0;                               { Pas de collision }

{-- Dclarations de types ---------------------------------------------}

type PIXBUF = record          { Informations pour GetVideo et PutVideo }
        bitptr : array[0..3] of pointer;         {Ptr sur plans de bits}
        oparl  : array[0..3] of byte;           { Nb d'octets  copier }
        hauteur: byte;                              { Nombre de lignes }
                                {-- Ici les octets des plans de bits --}
              end;
     PIXPTR = ^PIXBUF;              { Pointeur sur un buffer de pixels }

     SPLOOK = record                               { Image d'un sprite }
        largeur,                                      { Largeur totale }
        hauteur  : byte;                 { Hauteur en lignes de pixels }
        pixbp  : PIXPTR;                 { Pointeur sur bloc de pixels }
          end;
     SPLP = ^SPLOOK;                          { Pointeur sur une image }

     SPID = record                        { Descripteur de sprite (ID) }
          splookp : SPLP;                       { Pointeur sur l'image }
          x, y    : array [0..1] of integer;      { Coord pages 0 et 1 }
          fondptr   : array [0..1] of PIXPTR;      { Ptr sur le buffer }
        end;                                                 { du fond }
     SPIP = ^SPID;             { Pointeur sur le descripteur du sprite }

     PTRREC = record   { Pour dcomposer un pointeur ou un entier long }
                ofs,
                seg : word;
              end;

     BYTEAR = array[0..10000] of byte;  { Pour adresser les diffrents }
     BARPTR = ^BYTEAR;                                     { buffers   }

{***********************************************************************
*  IsVga : Teste la prsence d'une carte VGA.                          *
**--------------------------------------------------------------------**
*  Entre  : nant                                                     *
*  Sortie  : TRUE ou FALSE                                             *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;                   { Registres pour l'interruption}

begin
  Regs.AX := $1a00;               { La fonction 1Ah n'existe qu'en VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Affiche un caractre en mode graphique                  *
**--------------------------------------------------------------------**
*  Entre : caractere = le caractre  afficher                        *
*           x, y      = Coordonnes du coin sup gauche                 *
*           cc        = Couleur du caractre                           *
*           cf        = Couleur du fond                                *
*  Info  : Le caractre est dessin dans une matrice de 8*8 pixels     *
*          sur la base du jeu de caractres 8*8 en ROM                 *
***********************************************************************}

procedure PrintChar( caractere : char; x, y : integer; cc, cf : byte );

type CARADEF = array[0..255,0..7] of byte;{ Structure jeu de caractres}
     CARAPTR = ^CARADEF;             { Pointe sur le jeu de caractres }

var  Regs  : Registers;       { Registres pour grer les interruptions }
     ch    : char;                                { Pixel du caractre }
     i, k,                                     { Compteur d'itrations }
     Masque : byte;        { Masque binaire pour dessiner le caractre }

const fptr : CARAPTR = NIL;   { Pointe sur le jeu de caractres en ROM }

begin
  if fptr = NIL then             { A-t-on dj dtermin ce pointeur ? }
    begin                                                         { Non}
      Regs.AH := $11;                 { Appelle l'option $1130 de la   }
      Regs.AL := $30;                 { fonction vido du BIOS         }
      Regs.BH := 3;      { pour obtenir un pointeur sur le jeu 8*8     }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );           { Compose le pointeur }
    end;

  if ( cf = 255 ) then                       { Caractre transparent ? }
    for i := 0 to 7 do { Oui ne dessine que les pixels du premier plan }
      begin
        Masque := fptr^[ord(caractere),i];{Motif binaire pour une ligne}
        for k := 0 to 7 do
          begin
            if ( Masque and 128 <> 0 ) then        { Pixel  dessiner? }
              setpix( x+k, y+i, cc );                            { Oui }
            Masque := Masque shl 1;
          end;
      end
  else                                     { Non, tient compte du fond }
    for i := 0 to 7 do                           { Parcourt les lignes }
      begin
        Masque := fptr^[ord(caractere),i];{Motif binaire pour une ligne}
        for k := 0 to 7 do
          begin
            if ( Masque and 128 <> 0 ) then           { Premier plan ? }
              setpix( x+k, y+i, cc )                             { Oui }
            else
              setpix( x+k, y+i, cf );                      { Non, fond }
            Masque := Masque shl 1;
         end;
      end;
end;

{***********************************************************************
*  Line : Trace un segment dans la fentre graphique en appliquant     *
*        l'algorithme de Bresenham                                     *
**--------------------------------------------------------------------**
*  Entre : X1, Y1  = Coordonnes de l'origine (0 - ...)               *
*           X2, Y2  = Coordonnes de l'extrmit terminale             *
*           COULEUR = couleur du segment                               *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; couleur : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Procdure accessoire pour changer deux variables entires --------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Procdure principale --------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then    { Parcours : par axe X ou Y ? }
    begin                                            { par l'axe des Y }
      if ( y1 > y2 ) then                         { y1 suprieur  y2? }
        begin
      SwapInt( x1, x2 );                      { Oui, change X1 et X2, }
      SwapInt( y1, y2 );                                    { Y1 et Y2 }
        end;

      if ( x2 > x1 ) then xincr := 1          { Fixe le pas horizontal }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      Setpix( x, y, couleur );              { Dessine le premier point }
      for y:=y1+1 to y2 do                      { Parcourt l'axe des Y }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          Setpix( x, y, couleur );
        end;
    end
  else                                               { par l'axe des X }
    begin
      if ( x1 > x2 ) then                      { x1 plus grand que x2? }
        begin
      SwapInt( x1, x2 );                       { Oui, change X1 et X2 }
      SwapInt( y1, y2 );                                    { Y1 et Y2 }
        end;

      if ( y2 > y1 ) then yincr := 1            { Fixe le pas vertical }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      Setpix( x, y, couleur );              { Dessine le premier point }
      for x:=x1+1 to x2 do                      { Parcourt l'axe des X }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          Setpix( x, y, couleur );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint : Affiche une chane formate sur l'cran graphique       *
**--------------------------------------------------------------------**
*  Entres : X, Y   = Coordonnes de dpart (0-...)                    *
*            CC     = Couleur des caractres                           *
*            CF     = Couleur du fond (255 = transparent)              *
*            STRING = Chane avec indications de formatage             *
***********************************************************************}

procedure GrafPrint( x, y : integer; cc, cf : byte; strt : string );

var i : integer;                               { Compteur d'itrations }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, cc, cf );       { Affiche par printchar }
     inc( x, 8 );               { x  la position du caractre suivant }
   end;
end;


{***********************************************************************
*  GetVideo : Charge le contenu d'une zone rectangulaire de la mmoire *
*             d'cran dans un buffer                                   *
**--------------------------------------------------------------------**
*  Entres : PAGE     = Page d'cran  (0 ou  1)                        *
*            X1, Y1   = Coordonnes de dpart                          *
*            LARGEUR  = Largeur de la zone rectangulaire en pixels     *
*            HAUTEUR  = Hauteur de la zone rectangulaire en pixels     *
*            BUFPTR   = Pointeur sur un buffer de pixels qui va        *
*                       mmoriser les informations                     *
*  Sortie  : Pointeur sur le buffer de pixels qui contient la zone     *
*            indique                                                  *
*  Info    : Si on donne au paramtre BUFPTR la valeur NIL , un        *
*            nouveau buffer de pixels est allou sur le tas et retourn*
*            Ce buffer peut tre transmis lors d'un nouvel appel si    *
*            l'ancien contenu est effaable et si la taille de la zone *
*            n'a pas chang                                            *
***********************************************************************}

function GetVideo( page : byte; x1, y1 : integer;
                   largeur, hauteur : byte; bufptr : PIXPTR ) : PIXPTR;

var i,                                         { Compteur d'itrations }
    plancour,                                   { Plan de bits courant }
    sb,                        { Plan de bits des coordonnes de dbut }
    eb,                          { Plan de bits des coordonnes de fin }
    b,                          { Nombre d'octets dans un plan de bits }
    am  : byte;  { Nombre d'octet au milieu des deux groupes de quatre }
    rptr : pointer;                    { Pointeur courant sur l'offset }

begin
  if ( bufptr = NIL ) then                   { Pas de buffer transmis ?}
    getmem( bufptr, sizeof(PIXBUF) + largeur*hauteur);{ Non, on alloue }

   {-- Calcule le nombre d'octets par plan de bits --------------------}

  am := ( ( (x1+largeur-1) and not(3) ) -  {Nombre d'octets au milieu  }
        ( (x1+4) and not(3) ) ) div 4;
  sb := x1 mod 4;                               { Plan de bit de dbut }
  eb := (x1+largeur-1) mod 4;                     { Plan de bit de fin }

  rptr := ptr( seg(bufptr^), ofs(bufptr^) + sizeof( PIXBUF ));

  {-- Parcourt les quatre plans de bits -------------------------------}

  for i:=0 to 3 do
    begin
      plancour := (sb+i) mod 4;
      b := am;                  { Nombre de base des octets  copier   }
      if ( plancour >= sb ) then
                        { galement dans le bloc des quatre du dbut ? }
    inc( b );                 { Oui, ajoute un octet dans ce plan      }
      if ( plancour <= eb ) then
                       { galement dans le bloc des quatre de la fin ? }
    inc( b );                      { Oui, ajoute un octet dans ce plan }
      bufptr^.bitptr[i] := rptr;
                    { Mmorise dans le buffer un pointeur sur le dbut }
      bufptr^.oparl[i] := b;             { Mmorise le nombre d'octets }
      CopyPlane2Buf( rptr, page, x1+i,               { Lit le contenu  }
             y1, b, hauteur );                             { des plans }
      inc( PTRREC(rptr).ofs, b * hauteur );  { Positionne le pointeur  }
                                  { sur le plan suivant dans le buffer }
    end;
  bufptr^.hauteur := hauteur;                    { mmorise la hauteur }

  GetVideo := bufptr; { Renvoie  l'appelant le pointeur sur le buffer }
end;

{***********************************************************************
*  PutVideo : Rcrit dans la mmoire d'cran le contenu d'une zone    *
*             d'cran rectangulaire pralablement sauvegarde par      *
*             GetVideo                                                 *
**--------------------------------------------------------------------**
*  Entre : BUFPTR = Pointeur renvoy par GetVideo et rfrenant un   *
*                    buffer de pixels                                  *
*           PAGE   = Page d'cran (0 ou 1)                             *
*           X1, Y1 = Coordonnes de dbut                              *
*           BG     = Indique si les pixels du fond (code couleur 255)  *
*                     doivent tre crits dans la mmoire d'cran      *
*  Info   : Le buffer de pixels n'est pas effac par cette procdure   *
*           cette tche tant remplie par FreeBixBuf()                 *
***********************************************************************}

procedure PutVideo( bufptr : PIXPTR; page : byte; x1, y1 : integer;
                    bg     : boolean );

var plancour,                                   { Plan de bits courant }
    hauteur : byte;

begin
  hauteur := bufptr^.hauteur;                     { Hauteur de la zone }
  for plancour:=0 to 3 do          { Parcourt les quatre plans de bits }
    CopyBuf2Plane( bufptr^.bitptr[plancour], page, x1+plancour,
                   y1, bufptr^.oparl[plancour], hauteur, bg );
end;

{***********************************************************************
*  FreePixBuf : Efface un buffer de pixels allou sur le tas par       *
*              GetVideo                                                *
**--------------------------------------------------------------------**
*  Entre : BUFPTR  = Pointeur renvoy par Getvideo et rfrenant     *
*                     un buffer de pixels                              *
*           LARGEUR = Largeur de la zone rectangulaire en pixels       *
*           HAUTEUR = Hauteur de la zone rctangulaire en pixels        *
***********************************************************************}

procedure FreePixBuf( bufptr : PIXPTR; largeur, hauteur : byte );

begin
  freemem( bufptr, sizeof( PIXBUF ) + largeur*hauteur );
end;

{***********************************************************************
*  CreateSprite : Cre un sprite  l'aide d'un motif de pixels         *
*                 pralablement compil                                *
**--------------------------------------------------------------------**
*  Entre  : SPLOOKP = Pointeur sur la structure de donnes produite   *
*                      par CompileSprite()                             *
*  Sortie  : Pointeur sur la structure du sprite cre                 *
***********************************************************************}

function CreateSprite( splookp : SPLP ) : SPIP;

var spidp : SPIP;                { Pointe sur la structure du sprite   }

begin
  new( spidp );           { Alloue de la mmoire pour la structure     }
  spidp^.splookp := splookp;

 {-- Cre deux buffers de fond dans lesquels GetVideo va sauvegarder   }
                                   {-- des zones de la mmoire d'cran }

  spidp^.fondptr[0] := GetVideo( 0, 0, 0, splookp^.largeur,
                               splookp^.hauteur, NIL );
  spidp^.fondptr[1] := GetVideo( 0, 0, 0, splookp^.largeur,
                               splookp^.hauteur, NIL );
  CreateSprite := spidp;   { Renvoie un ptr sur la structure du sprite }
end;

{***********************************************************************
*  CompileSprite : Cre le motif binaire d'un sprite  l'aide d'une    *
*                 dfinition connue au moment de l'excution           *
**--------------------------------------------------------------------**
*  Entre  : BUFP      = Pointeur sur un tableau de pointeurs          *
*                        rfrenant des chanes de caractres qui     *
*                        reprsente le motif du sprite                 *
*            HAUTEUR   = Hauteur du sprite et nombre de chanes de     *
*                        caractres                                    *
*            PAGE      = Page graphique pour construire le sprite      *
*            CLR       = Caractre ASCII associ  la plus petite coul.*
*            COULEURPP = Premier code de couleur pour CLR              *
*  Info    : Les sprites sont dessins  partir du bord gauche de la   *
*            ligne indique                                            *
***********************************************************************}

function CompileSprite( var buf; hauteur, page : byte;
            clr: char; couleurpp : byte ) : SPLP;

type BYPTR  = ^byte;                           { Pointeur sur un octet }

var  largeur,                { Longueur des chanes = largeur du motif }
     c,                                        { Mmorise un caractre }
     couleur,                                     { Couleur d'un pixel }
     i, k, l  : byte;                          { Variables d'itration }
     splookp  : SPLP;{ Ptr sur la structure de l'image du sprite cre }
     bptr     : barptr; {Sert  adresser le buffer de l'image du sprite}
     pbptr    : PIXPTR;                   { Mmorise le fond du sprite }

begin
  {-- Cre la structure de l'image du sprite et la remplit ------------}

  new( splookp );
  bptr := @buf;                         { Pointe sur le buffer du logo }
  largeur := bptr^[0]; { Lit la longueur des chanes = largeur du logo }
  splookp^.largeur := largeur;
  splookp^.hauteur  := hauteur;

      {-- Construit le sprite dans la page indique  partir de (0,0)--}

  setpage( page );                          { Fixe la page de dessin   }
  pbptr := GetVideo( page, 0, 0, largeur, hauteur, nil );{ Lit le fond }

  for i := 0 to hauteur-1 do                     { Parcourt les lignes }
    for k := 0 to largeur-1 do                 { Parcourt les colonnes }
      begin
    c := bptr^[i*(largeur+1)+k+1];                       { Lit un pixel}
    if ( c = 32 ) then                                        { Fond ? }
      setpix( k, i, 255 )            { Oui, choisit le code couleur 255}
    else                                  { Non, met la couleur prvue }
          setpix( k, i, couleurpp+(c-ord(clr)) );
      end;

  {-- Lit le sprite dans le buffer et restaure le fond    -------------}

  splookp^.pixbp := GetVideo( page, 0, 0, largeur, hauteur, NIL );
  PutVideo( pbptr, page, 0, 0, false );
  FreePixBuf( pbptr, largeur, hauteur );            { Libre le buffer }

  CompileSprite := splookp; { Renvoie une ptr sur le buffer de l'image }
end;

{***********************************************************************
*  PrintSprite : Affiche un sprite dans une page donne                *
**--------------------------------------------------------------------**
*  Entre  : SPIDP = Pointeur sur la structure du sprite               *
*            PAGE  = Page concerne  (0 ou 1)                          *
***********************************************************************}

procedure PrintSprite( spidp : SPIP; page : byte );

begin
  PutVideo( spidp^.splookp^.pixbp,
                page, spidp^.x[page], spidp^.y[page], true );
end;

{***********************************************************************
*  GetSpriteBg : Lit le fond du sprite et le mmorise  l'emplacement  *
*                prvu                                                 *
**--------------------------------------------------------------------**
*  Entres : SPIDP = Pointeur sur la structure du sprite               *
             PAGE  = Page d'o est tir le fond (0 ou 1)               *
***********************************************************************}

procedure GetSpriteBg( spidp : SPIP; page : BYTE );

var dummy : PIXPTR;

begin
 dummy := GetVideo( page, spidp^.x[page],  spidp^.y[page],
                    spidp^.splookp^.largeur, spidp^.splookp^.hauteur,
                    spidp^.fondptr[page] );
end;

{***********************************************************************
*  RestoreSpriteBg : Rtablit dans la page d'origine le fond d'un      *
*                    sprite sauvegard au pralable                    *
**--------------------------------------------------------------------**
*  Entre  : SPIDP = Pointeur sur la structure du sprite               *
*            PAGE  = Page o doit tre recopi le fond (0 ou 1)        *
***********************************************************************}

procedure RestoreSpriteBg( spidp : SPIP; page : BYTE );

begin
 PutVideo( spidp^.fondptr[page], page,
           spidp^.x[page],  spidp^.y[page], false );
end;

{***********************************************************************
*  MoveSprite : Dplace un sprite dans sa page                         *
**--------------------------------------------------------------------**
*  Entres : SPIDP  = Pointeur sur la structure du sprite              *
*            PAGE   = Page o doit tre recopi le fond (0 ou 1)       *
*            DELTAX = Dplacement dans les directions X etY            *
*            DELTAY                                                    *
*  Sortie  : Indicateur de collision, cf constantes OUT_...            *
***********************************************************************}

function MoveSprite( spidp : SPIP; page : byte;
                     deltax, deltay : integer    ) : byte;

var nouvx, nouvy : integer;          { Nouvelles coordonnes du sprite }
    out        : byte; { Indique une collision avec le bord de l'cran }

begin
                   {-- Dcale l'abscisse X et dtecte les collisions --}

 nouvx := spidp^.x[page] + deltax;
 if ( nouvx  < 0 ) then
   begin
     nouvx := 0 - deltax - spidp^.x[page];
     out := OUT_LEFT;
   end
 else
   if ( nouvx > MAXX - spidp^.splookp^.largeur ) then
     begin
       nouvx := (2*(MAXX+1))-nouvx-2*(spidp^.splookp^.largeur);
       out := OUT_RIGHT;
     end
  else
     out := OUT_NO;

                      {  Dcale l'ordonne Y et dtecte les collisions }

 nouvy := spidp^.y[page] + deltay;
 if ( nouvy < 0 ) then                              { Bord suprieur ? }
   begin                              { Oui , deltay doit tre ngatif }
     nouvy := 0 - deltay - spidp^.y[page];
     out := out or OUT_TOP;
   end
 else
   if ( nouvy + spidp^.splookp^.hauteur > MAXY+1  ) then   { Bord inf ?}
     begin                             { Oui, deltay doit tre positif }
       nouvy := (2*(MAXY+1))-nouvy-2*(spidp^.splookp^.hauteur);
       out := out or OUT_BOTTOM;
     end;

{Ne fixe une nouvelle position que si elle est diffrente de l'ancienne}

 if ( nouvx <> spidp^.x[page] ) or  ( nouvy <> spidp^.y[page] ) then
   begin                                           { Nouvelle position }
     RestoreSpriteBg( spidp, page );                { Restaure le fond }
     spidp^.x[page] := nouvx;     { Mmorise les nouvelles coordonnes }
     spidp^.y[page] := nouvy;
     GetSpriteBg( spidp, page );                 { Lit le nouveau fond }
     PrintSprite( spidp, page );{ Dessine sprite dans la page indique }
   end;

 MoveSprite := out;
end;

{***********************************************************************
*  SetSprite : Place le sprite  une position donne                   *
**--------------------------------------------------------------------**
*  Entres : SPIDP  = Pointeur sur la structure du sprite              *
*            x0, y0 = Coordonnes du sprite en page 0                  *
*            x1, y1 = Coordonnes du sprite en page 1                  *
*  Info    : Cette fonction doit tre dclenche avant le premier      *
*            appel  MoveSprite()                                      *
***********************************************************************}

procedure SetSprite( spidp : SPIP; x0, y0, x1, y1 : integer );

begin
 spidp^.x[0] := x0;       { Mmorise les coordonnes dans la structure }
 spidp^.x[1] := x1;
 spidp^.y[0] := y0;
 spidp^.y[1] := y1;

 GetSpriteBg( spidp, 0 );                      { Lit le fond du sprite }
 GetSpriteBg( spidp, 1 );                            { en pages 0 et 1 }
 PrintSprite( spidp, 0 );                           { Dessine le sprite}
 PrintSprite( spidp, 1 );                          { en pages 0 et 1   }
end;

{***********************************************************************
*  Demo : Dmontre l'usage des diffrentes fonctions de ce module      *
***********************************************************************}

procedure Demo;

const VaisseauMontant :array [1..20] of string[32] =
                ( '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ',
                  '             GGBBGG             ',
                  '            GBBCCBBG            ',
                  '           GBBBCCBBBG           ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  ' G       GBBBBBBBBBBBBG       G ',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '       DDDD          DDDD       '  );

const VaisseauDescendant :array [1..20] of string[32] =
                ( '       DDDD          DDDD       ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  ' G       GBBBBBBBBBBBBG       G ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  '           GBBBCCBBBG           ',
                  '            GBBCCBBG            ',
                  '             GGBBGG             ',
                  '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ' );

  NBSPR = 6;                                      { Nombre de sprites  }
  LARGEUR = 37;          { Nombre de caractres de l'avis de copyright }
  HAUTEUR  = 6;                     { Nombre de lignes de ce mme avis }
  SX     = (MAXX-(LARGEUR*8)) div 2;            { Abscisse de dpart X }
  SY     = (MAXY-(HAUTEUR*8)) div 2;            { Ordonne de dpart Y }

type SPRITE = record                         { Pour grer les sprites  }
                spidp : SPIP;            { Pointe sur l'identificateur }
                deltax,      { Dplacement horizontal pour pages 0 et 1}
                deltay : array [0..1] of integer;{ Dplacement vertical}
              end;

var sprites    : array [1..nbspr] of SPRITE;
    page,                                              { Page courante }
    lc,                                            { Caractre courant }
    out        : byte;            { Mmorise l'indicateur de collision }
    x, y, i,                                  { Compteurs d'itrations }
    dx, dy     : integer;                               { Dplacements }
    vaisseauupp,
    vaisseaudnp : SPLP;                     { Pointeur sur les sprites }
    ch         : char;
begin

  Randomize;          { Initialise le gnrateur de nombres alatoires }

           {-- Construit les motifs binaires des sprites --------------}

  vaisseauupp := CompileSprite( VaisseauMontant,   20, 0, 'A', 1 );
  vaisseaudnp := CompileSprite( VaisseauDescendant, 20, 0, 'A', 1 );

       {-- Remplit de caractres les deux pages graphiques ------------}

  for page := 0 to 1 do
    begin
      setpage( page );
      lc := 0;
      y := 0;
      while ( y < (MAXY+1)-8 ) do
        begin
          x := 0;
          while ( x < (MAXX+1)-8 ) do
            begin
              PrintChar( chr(lc and 127), x, y, lc mod 255, 0 );
              inc( lc );
              inc( x, 8 );
            end;
          inc( y, 12 );
        end;

     {-- Affiche l'avis de copyright ----------------------------------}

      Line( SX-1, SY-1, SX+LARGEUR*8, SY-1, 15 );
      Line( SX+LARGEUR*8, SY-1, SX+LARGEUR*8, SY+HAUTEUR*8,15 );
      Line( SX +LARGEUR*8, SY+HAUTEUR*8, SX-1, SY+HAUTEUR*8, 15 );
      Line( SX-1, SY+HAUTEUR*8, SX-1, SY-1, 15 );
      GrafPrint( SX, SY,    15, 4,
                   '                                     '  );
      GrafPrint( SX, SY+8,  15, 4,
                   ' S3240P.PAS - (c) 1990 by M. TISCHER ' );
      GrafPrint( SX, SY+16, 15, 4,
                   '                                     '  );
      GrafPrint( SX, SY+24, 15, 4,
                   '  Dmonstration de sprites dans le   ' );
      GrafPrint( SX, SY+32, 15, 4,
                   '    mode 320*400 de la carte VGA     '  );
      GrafPrint( SX, SY+40, 15, 4,
                   '                                     '  );
    end;

    {-- Fabrique les diffrents sprites -------------------------------}

  for i := 1 to NBSPR do
    begin
      sprites[ i ].spidp := CreateSprite( vaisseauupp );
      repeat                                { Choisir les dplacements }
        dx := 0;
        dy := random(10) - 5;
      until ( dx <> 0 ) or ( dy <> 0 );

      sprites[ i ].deltax[0] := dx * 2;
      sprites[ i ].deltay[0] := dy * 2;
      sprites[ i ].deltax[1] := dx * 2;
      sprites[ i ].deltay[1] := dy * 2;

      x := ( (320 div NBSPR) * (i-1) ) + ((320 div NBSPR)-40) div 2;
      y := random( 200 - 40 );
      SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
    end;

{-- Dplace les sprites et les fait rebondir aux extrmits de l'cran }

  page := 1;                                      { Commence en page 1 }
  while ( not keypressed ) do{Une frappe de touche interrompt la boucle}
    begin
      showpage( 1 - page );                     { Affiche l'autre page }

      for i := 1 to nbspr do                    { Parcourt les sprites }
        begin          { Daplace les sprites et dtecte les collisions }
          out := MoveSprite( sprites[i].spidp, page,
                             sprites[i].deltax[page],
                             sprites[i].deltay[page] );
          if ( ( out and OUT_TOP ) <> 0 ) or{ Contact en haut/en bas ? }
             ( ( out and OUT_BOTTOM ) <> 0 ) then
            begin
                   {-- Oui, inverse le dplacement et change l'image --}

              sprites[i].deltay[page] := -sprites[i].deltay[page];
              if ( ( out and OUT_TOP ) <> 0 ) then
                sprites[i].spidp^.splookp := vaisseaudnp
              else
                sprites[i].spidp^.splookp := vaisseauupp;
            end;
          if ( ( out and OUT_LEFT ) <> 0 ) or{Contact  gauche/droite ?}
             ( ( out and OUT_RIGHT ) <> 0 ) then
            sprites[i].deltax[page] := -sprites[i].deltax[page];
        end;
      page := (page+1) and 1;           { Passe de 1  0 et vice versa }
    end;

  ch := readkey;                         { Attend une frappe de touche }
end;

{----------------------------------------------------------------------}
{--                         PROGRAMME PRINCIPAL                      --}
{----------------------------------------------------------------------}

begin
 if ( IsVga ) then                      { Dispose-t-on d'une carte VGA }
   begin                                                { Oui, on y va }
     init320400;                        { Initialise le mode graphique }
     Demo;
     Textmode( CO80 );                         { Ratblit le mode texte }
   end
 else
   writeln( 'S3240P.PAS  -  (c) 1990 by MICHAEL TISCHER'#13#10#10 +
            'Attention! Ce programme ncessite une carte VGA ');
end.

