{**********************************************************************}
{*                         S O U R I S . P A S                        *}
{*--------------------------------------------------------------------*}
{*    Fonction       : Fournit diverses fonctions pour le travail     *}
{*                     avec la souris                                 *}
{*--------------------------------------------------------------------*}
{*    Auteur         : MICHAEL TISCHER                                *}
{*    Dvelopp le   : 21/04/1989                                     *}
{*    Dernire modif.: 23/04/1989                                     *}
{*--------------------------------------------------------------------*}
{*    Le module assembleur sourisp.obj doit se trouver dans le        *}
{*                     rpertoire courant sinon le chemin complet     *}
{*                     doit tre spcifier lors de son intgration.   *}
{**********************************************************************}

uses Dos;                                         { Intgrer unit DOS }

{$L sourispa} { Intgrer le module assembleur }

{== Dclaration des fonctions externes ================================}

{$F+}                                         { C'est une fonction FAR }
procedure AssHand ; external;        { Le gestionnaire d'vnements en }
{$F-}                   { assembleur ne plus obliger des fonctions FAR }

{== Constantes ========================================================}

const

{-- Codes d'vnement -------------------------------------------------}

      EV_MOU_MOVE     =   1;                         { Souris dplace }
      EV_LEFT_PRESS   =   2;       { Bouton gauche de la souris appuy }
      EV_LEFT_REL     =   4;      { Bouton gauche de la souris relch }
      EV_RIGHT_PRESS  =   8;        { Bouton droit de la souris appuy }
      EV_RIGHT_REL    =  16;       { Bouton droit de la souris relch }
      EV_MOU_ALL      =  31;              { Tous les vnements souris }

      LBITS           =   6;            { EV_LEFT_PRESS or EV_LEFT_REL }
      RBITS           =  24;          { EV_RIGHT_PRESS or EV_RIGHT_REL }

      AUCUNE_ZONE    = 255;    { Curseur de la souris pas dans zone xy }

      PtrSameChar     = $00ff;                        { Mme caractre }
      PtrSameCol      = $00ff;                          { Mme couleur }
      PtrInvCol       = $7777;                      { Couleur inverse }
      PtrSameColz     = $807f;              { Mme couleur clignotante }
      PtrInvColz      = $F777;          { Couleur inverse clignotante }

      ET             = 0;{ Combinaisons d'vnements pour MouEventWait }
      OU            = 1;

      ZV              = #13#10;                        { Saut de ligne }

{== Dclarations de type ==============================================}

type  FCTPTR   = longint;                 { Adresse d'une fonction FAR }
      PTRVIEW  = longint;        { Masque pour le curseur de la souris }
      ZONE  = record                    { Dcrit une zone de la souris }
                   x1,          { Coordonnes du coin suprieur gauche }
                   y1,                 { et du coin infrieur droit de }
                   x2,                             { la zone spcifie }
                   y2      : byte;
                   PtrMask : PTRVIEW;  { Masque pour le curseur souris }
                 end;
      TABLEAUZONES = array [0..100] of ZONE;
      PTRZONE   = ^TABLEAUZONES;
      PTRREC   = record                           { Sert  accder aux }
                   Ofs : word;                { lments d'un pointeur }
                   Seg : word;                            { quelconque }
                 end;
      PTRVREC  = record                           { Sert  accder aux }
                   ScreenMask : word;          { lments d'un PTRVIEW }
                   CursorMask : word;
                 end;
      BUFZONE   = array [0..10000] of byte;           { Buffer de zone }
      PTRBZ    = ^BUFZONE;            { Pointeur sur un buffer de zone }

{== Variables globales ================================================}

var  NmbZones,                                       { Nombre de zones }
     TLine,                                { Nombre de lignes de texte }
     TCol     : byte;                    { Nombre de colonnes de texte }
     MouAvail : boolean;              { Est TRUE, si souris disponible }
     OldPtr,              { Ancienne apparence du curseur de la souris }
     StdPtr   : PTRVIEW;{ Masque pour le curseur de la souris standard }
     BufPtr   : PTRBZ;  { Pointeur sur buffer pour identification zone }
     PtrZonAct: PTRZONE;         { Pointeur sur vecteur de zone actuel }
     BLen     : integer;        { Longueur du buffer de zone en octets }
     ExitOld  : pointer;      { Pointeur sur l'ancienne procdure Exit }

{ Variables dfinies lors de chaque appel du gestionnaire de la souris }

     ZonSou,                                 { Zone de souris actuelle }
     MouCol,                   { Colonne de la souris (cran de texte) }
     MouRow   : byte;            { Ligne de la souris (cran de texte) }
     MouEvent : integer;                         { Masque d'vnements }

{-- Variables qui ne sont dfinies par le gestionnaire de la souris ---}
{-- que lorsqu'intervient un vnement attendu                      ---}

     ZonEv,                      { Zone dans laquelle figure la souris }
     EvCol,                                     { Colonne de la souris }
     EvRow : byte;                                { Ligne de la souris }

{**********************************************************************}
{*  MouPtrMask: Compose les masques curseur et cran  partir d'un    *}
{*              masque de bits pour le caractre et pour la couleur   *}
{**------------------------------------------------------------------**}
{*  Entre : Caractere = masque bits pour les masques curseur et      *}
{*                       cran concernant le caractre                *}
{*           Couleur   = masque bits pour les masques curseur et      *}
{*                       cran concernant la couleur du caractre     *}
{*  Sortie : les masques curseur et cran sous forme d'une valeur du  *}
{*           type PtrView                                             *}
{*  Infos :  pour Caractere et Couleur peuvent tre transmises les    *}
{*           constantes PtrSameChar, PtrSameCol, PtrSameColz,         *}
{*           PtrInvCol et PtrInvColz ainsi que les rsultats des      *}
{*           fonctions PtrDifChar et PtrDifCol                        *}
{**********************************************************************}

function MouPtrMask( Caractere, Couleur  : word ) : PTRVIEW;

var Mask : PTRVIEW;             { Les masques curseur et cran  crer }

begin
  PTRVREC( Mask ).ScreenMask := ( ( Couleur and $ff ) shl 8 ) +
                                ( Caractere and $ff );
  PTRVREC( Mask ).CursorMask := ( Couleur and $ff00 ) + ( Caractere shr 8 );
  MouPtrMask := Mask;           { Renvoyer masque au programme d'appel }
end;

{**********************************************************************}
{*  PtrDifChar: cre la partie caractre des masques curseur et cran *}
{*              concernant le caractre                               *}
{**------------------------------------------------------------------**}
{*  Entre : Code ASCII du caractre devant tre le curseur souris    *}
{*  Sortie : Masques curseur et cran pour ce caractre               *}
{*  Infos :  Le rsultat de la fonction devra tre retrait  l'aide  *}
{*           de la fonction MouPtrMask                                *}
{**********************************************************************}

function PtrDifChar( Caractere : byte ) : word;

begin
  PtrDifChar := Caractere shl 8;
end;

{**********************************************************************}
{*  PtrDifCol: Cre la partie caractre des masques curseur et cran  *}
{*             concernant la couleur du curseur de la souris          *}
{**------------------------------------------------------------------**}
{*  Entre : Couleur du caractre devant tre le curseur de la souris *}
{*  Sortie : Masques curseur et cran pour cette couleur              *}
{*  Infos  : Le rsultat de la fonction devra tre retrait  l'aide  *}
{*           de la fonction MouPtrMask                                *}
{**********************************************************************}

function PtrDifCol( Couleur : byte ) : word;

begin
  PtrDifCol := Couleur shl 8;
end;

{**********************************************************************}
{*  MouDefinePtr: transmet au driver de la souris les masques curseur *}
{*                et cran qui dfinissent dsormais l'apparence du   *}
{*                curseur de la souris                                *}
{**------------------------------------------------------------------**}
{*  Entre : Masque = les masques curseur et cran sous forme d'un    *}
{*                    paramtre du type PTRVIEW                       *}
{*  Infos :  - le paramtre Masque doit avoir t cr  l'aide de la *}
{*             fonction MouPtrMask                                    *}
{*           - les 16 bits de plus fort poids de Masque reprsentent  *}
{*             le masque cran, les 16 bits faibles le masque curseur *}
{**********************************************************************}

procedure MouDefinePtr( Mask : PTRVIEW );

var Regs : Registers;    { Registres du processeur pour l'appel d'int. }

begin
  if OldPtr <> Mask then { Modification par rapport au dernier appel ? }
    begin                                                        { Oui }
      Regs.AX := $000a;          { N fct pour "Set text pointer type" }
      Regs.BX := 0;                           { Fixer curseur logiciel }
      Regs.CX := PTRVREC( Mask ).ScreenMask; { Mot faible : masque AND }
      Regs.DX := PTRVREC( Mask ).CursorMask;   { Mot fort : masque XOR }
      Intr( $33, Regs);               { Appeler le driver de la souris }
      OldPtr := Mask;                  { Ranger le nouveau masque bits }
    end;
end;

{**********************************************************************}
{*  MouEventHandler: Est appel par le driver de la souris  travers  *}
{*                   la routine assembleur AssHand ds qu'intervient  *}
{*                   un vnement concernant la souris                *}
{**------------------------------------------------------------------**}
{*  Entre : EvFlags  = le masque Event                               *}
{*           ButState = tat actuel des boutons de la souris          *}
{*           X, Y     = position actuelle du curseur de la souris     *}
{*                      concernant l'cran de texte                   *}
{**********************************************************************}

procedure MouEventHandler( EvFlags, ButState, x, y : integer );

var NouZon : byte;                        { Numro de la nouvelle zone }

begin

  MouEvent := MouEvent and not(1);                  { Masquer le bit 0 }
  MouEvent := MouEvent or ( EvFlags and 1 );         { Copier le bit 0 }

  if ( EvFlags and LBITS ) <> 0 then {Bouton gauche relch ou appuy ?}
    begin                                                        { Oui }
      MouEvent := MouEvent and not( LBITS );       { Masque tat actuel}
      MouEvent := MouEvent or ( EvFlags and LBITS );  { Incruster tat }
    end;

  if ( EvFlags and RBITS ) <> 0 then { Bouton droit relch ou appuy ?}
    begin                                                        { Oui }
      MouEvent := MouEvent and not( RBITS );       { Masque tat actuel}
      MouEvent := MouEvent or ( EvFlags and RBITS );  { Incruster tat }
    end;

  MouCol := x;                { Convertir colonne en colonnes de texte }
  MouRow := y;                    { Convertir ligne en lignes de texte }

  {-- Dterminer zone dans laquelle figure la souris et examiner   ----}
  {-- si la zone a t modifie depuis le dernier appel du         ----}
  {-- gestionnaire. Dans ce cas, l'apparence du curseur de la      ----}
  {-- souris doit tre redfinie.                                  ----}

  NouZon := BufPtr^[ MouRow * TCol + MouCol ];          { Retirer zone }
  if NouZon <> ZonSou then                           { Nouvelle zone ? }
    begin                                                        { Oui }
      if NouZon = AUCUNE_ZONE then            { En dehors d'une zone ? }
        MouDefinePtr( StdPtr )    { Oui, curseur de la souris standard }
      else                                          { Non, zone connue }
        MouDefinePtr( PtrZonAct^[ NouZon ].PtrMask );
    end;
  ZonSou := NouZon;      { Ranger numro de zone dans variable globale }
end;

{**********************************************************************}
{*  MouIBufFill: stocke le code d'une zone de la souris  l'intrieur *}
{*               de la mmoire de zone interne du module              *}
{**------------------------------------------------------------------**}
{*  Entre : x1, y1 = coin suprieur gauche de la zone de la souris   *}
{*           x2, y2 = coin infrieur droit de la zone de la souris    *}
{*           Code   = le code de zone                                 *}
{**********************************************************************}

procedure MouIBufFill( x1, y1, x2, y2, Code : byte );

var Index   : integer;                       { Dsigne le champ trait }
    Colonne,                                      { Compteur de boucle }
    Ligne   : byte;

begin
  for Ligne:=y1 to y2 do            { Parcourir les diffrentes lignes }
    begin
      Index := Ligne  * TCol + x1;         { Premier index de la ligne }
      for Colonne:=x1 to x2 do    { Parcourir les colonnes de la ligne }
        begin
          BufPtr^[ Index ] := Code;                     { Stocker code }
          inc( Index );                { Fixer index sur champ suivant }
        end;
    end;
end;

{**********************************************************************}
{*  MouDefZone : permet d'enregistrer diffrentes zones de l'cran,   *}
{*               qui seront identifies comme zones de souris         *}
{*               distinctes et pourront tre dotes d'un curseur de   *}
{*               la souris particulier                                *}
{**------------------------------------------------------------------**}
{*  Entre : Nombre = Nombre de zones de l'cran                      *}
{*           ZPtr   = Pointeur sur le tableau dans lequel les diff-  *}
{*                    rentes zones sont chacune dcrites par une      *}
{*                    structure du type ZONE                          *}
{*  Infos :  - le code AUCUNE_ZONE est attribu aux zones de l'cran  *}
{*             restes libres                                         *}
{*           - lorsque le curseur de la souris pntre dans l'une des *}
{*             zones souris inscrites dans le tableau, le             *}
{*             gestionnaire d'vnements commute immdiatement sur    *}
{*             la forme du curseur de la souris dans cette zone       *}
{**********************************************************************}

procedure ZoneMouDef( Nombre : byte; PtrZ : PTRZONE );

var ActZon,                               { Numro de la zone actuelle }
    Zone : byte;                                  { Compteur de boucle }

begin
  PtrZonAct := PtrZ;                     { Ranger pointeur sur vecteur }
  NmbZones := Nombre;                             { et nombre de zones }
  FillChar( BufPtr^, BLen, AUCUNE_ZONE );{Tous les lments=AUCUNE_ZONE}
  for Zone:=0 to nombre-1 do         { Parcourir les diffrentes zones }
   with PtrZ^[ Zone ] do
     MouIBufFill( x1, y1, x2, y2, Zone );

  {-- Redfinir le curseur de la souris -------------------------------}
  ActZon := BufPtr^[ MouRow * TCol + MouCol ];          { Retirer zone }
  if ActZon = AUCUNE_ZONE then                { En dehors d'une zone ? }
    MouDefinePtr( StdPtr )        { Oui, curseur de la souris standard }
  else                                              { Non, zone connue }
    MouDefinePtr( PtrZ^[ ActZon ].PtrMask );
end;

{**********************************************************************}
{*  MouEventWait: attend l'intervention d'un vnement dtermin      *}
{*                concernant la souris                                *}
{**------------------------------------------------------------------**}
{*  Entre : TYP        = type de combinaison entre les diffrents    *}
{*                        vnements                                  *}
{*           WAIT_EVENT = masque bits spcifiant les vnements      *}
{*                        attendre                                    *}
{*  Sortie : masque bits de l'vnement intervenu                     *}
{*  Infos :  - WAIT_EVENT peut tre cr par combinaison OU des       *}
{*             diverses constantes telles que EV_MOU_MOVE,            *}
{*             EV_LEFT_PRESS, etc.                                    *}
{*           - les constantes ET et OU peuvent tre spcifies comme  *}
{*             type de combinaison. Si vous optez pour ET, la         *}
{*             fonction ne rendra la main au programme d'appel qu'une *}
{*             fois que tous les vnements seront intervenus. Avec   *}
{*             OU, l'intervention d'un au moins des vnements        *}
{*             spcifis suffira.                                     *}
{**********************************************************************}

function MouEventWait( Typ : BYTE; WaitEvent : integer ) : integer;

var ActEvent : integer;
    Ligne,
    Colonne  : byte;
    Fin      : boolean;

begin
   Colonne := MouCol;       { Ranger la position actuelle de la souris }
   Ligne   := MouRow;
   Fin     := false;

  repeat
    {-- Attendre jusqu' ce qu'un des vnements se produise ----------}

    if Typ = ET then                                { Combinaison ET ? }
      repeat            { Oui, tous les vnements doivent se produire }
        ActEvent := MouEvent;            { Rechercher vnement actuel }
      until ActEvent = WaitEvent
    else                                              { Combinaison OU }
      repeat                     { Un vnement au moins doit survenir }
        ActEvent := MouEvent;            { Rechercher vnement actuel }
      until ( ActEvent and WaitEvent ) <> 0;

    ActEvent := ActEvent and WaitEvent;{Laisser que les bits vnements}

    {-- Si on attend un dplacement de la souris, l'vnement n'est -- }
    {-- accept que si le curseur de la souris a t dplac vers   -- }
    {-- une autre ligne et/ou une autre colonne de l'cran de texte  - }

    if ( ( (WaitEvent and EV_MOU_MOVE) <> 0 )  and
          ( Colonne = MouCol ) and  ( Ligne = MouRow ) ) then
      begin                 { Souris dplace mais mme position cran }
        ActEvent := ActEvent and not( EV_MOU_MOVE ); { Masquer bit move }
        Fin := ( ActEvent <> 0);         { Reste-t-il des vnements ? }
      end
    else                                         { Evnement intervenu }
      Fin := TRUE;
  until Fin;

  EvCol := MouCol;                       { Conserver position et zone  }
  EvRow := MouRow;                       { actuelles de la souris dans }
  ZonEv := ZonSou;                       { des variables globales      }

  MouEventWait := ActEvent;
end;

{**********************************************************************}
{*  MouISetEventHandler: installe un gestionnaire d'vnements appel *}
{*                       par le driver de la souris lorsque survien-  *}
{*                       nent certains vnements souris.             *}
{**------------------------------------------------------------------**}
{*  Entre : EVENT = masque bits dcrivant les vnements dont        *}
{*                   l'intervention doit entraner un appel du        *}
{*                   gestionnaire d'vnements                        *}
{*           FPTR  = Pointeur sur gestionnaire vn. du type FCTPTR   *}
{*  Infos :  - EVENT peut tre cr par combinaison OU des diverses   *}
{*             constantes telles que EV_MOU_MOVE, EV_LEFT_PRESS etc.  *}
{*           - le gestionnaire d'vn. doit tre une procdure FAR et *}
{*             ne modifier aucun des reg. processeur reus            *}
{**********************************************************************}

procedure MouISetEventHandler( Event : integer; PtrC : FCTPTR );

var Regs : Registers; { Registres du processeur pour appel d'interrup. }

begin
  Regs.AX := $000C;                 { N fct. pour "Set Mouse Handler" }
  Regs.CX := event;                      { Charger masque d'vnements }
  Regs.DX := PTRREC( PtrC ).Ofs;    { Adresse d'offset du gestionnaire }
  Regs.ES  := PTRREC( PtrC ).Seg; { Adresse de segment du gestionnaire }
  Intr( $33,  Regs );                 { Appeler le driver de la souris }
end;

{**********************************************************************}
{*  MouIGetX: Dtermine la colonne de texte dans laquelle figure le   *}
{*            curseur de la souris                                    *}
{**------------------------------------------------------------------**}
{*  Sortie : Colonne du curseur de la souris dans l'cran de texte    *}
{**********************************************************************}

function MouIGetX : byte;

var Regs : Registers; { Registres du processeur pour appel d'interrup. }

begin
  Regs.AX := $0003;               { N fct.: pour "Get mouse position" }
  Intr( $33,  Regs );                 { Appeler le driver de la souris }
  MouIGetX := Regs.CX shr 3;           { Convertir colonne et renvoyer }
end;

{**********************************************************************}
{*  MouIGetY: Dterminer ligne de texte du curseur de la souris       *}
{**------------------------------------------------------------------**}
{*  Sortie : Ligne du curseur de la souris dans l'cran de texte      *}
{**********************************************************************}

function MouIGetY : byte;

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  Regs.AX := $0003;               { N fct.: pour "Get mouse position" }
  Intr( $33, Regs );                  { Appeler le driver de la souris }
  MouIGetY := Regs.DX shr 3;             { Convertir ligne et renvoyer }
end;

{**********************************************************************}
{*  MouShowMouse: Place le curseur de la souris sur l'cran           *}
{**------------------------------------------------------------------**}
{*  Infos : Les appels de MouShowMouse et MouHidemOuse doivent        *}
{*          s'quilibrer pour avoir un effet                          *}
{**********************************************************************}

procedure MouShowMouse;

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  Regs.AX := $0001;                       { N fct.: pour "Show Mouse" }
  Intr( $33, Regs );                  { Appeler le driver de la souris }
end;

{**********************************************************************}
{*  MouHideMouse: Elimine le curseur de la souris de l'cran          *}
{**------------------------------------------------------------------**}
{*  Infos : Les appels de MouShowMouse et MouHideMouse doivent        *}
{*          s'quilibrer pour avoir un effet                          *}
{**********************************************************************}

procedure MouHideMouse;

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  Regs.AX := $0002;                        { N fct. pour "Hide Mouse" }
  Intr( $33, Regs);                   { Appeler le driver de la souris }
end;

{**********************************************************************}
{*  MouSetMoveArea: Fixe la zone de dplacement pour le curseur de    *}
{*                  la souris                                         *}
{**------------------------------------------------------------------**}
{*  Entre :  x1, y1 = Coordonnes coin suprieur gauche de la zone   *}
{*            x2, y2 = Coordonnes coin infrieur droit de la zone    *}
{*  Infos :  - Les ordonnes se rapportent  l'cran de texte et non  *}
{*              l'cran graphique virtuel du driver de la souris     *}
{**********************************************************************}

procedure MouSetMoveArea( x1, y1, x2, y2 : byte );

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  Regs.AX := $0008;               { N fct. pour "Set vertical Limits" }
  Regs.CX := integer( y1 ) shl 3;            { Conversion vers l'cran }
  Regs.DX := integer( y2 ) shl 3;               { virtuel de la souris }
  Intr( $33, Regs );                  { Appeler le driver de la souris }
  Regs.AX := $0007;             { N fct. pour "Set horizontal limits" }
  Regs.CX := integer( x1 ) shl 3;            { Conversion vers l'cran }
  Regs.DX := integer( x2 ) shl 3;               { virtuel de la souris }
  Intr( $33, Regs );                  { Appeler le driver de la souris }
end;

{**********************************************************************}
{*  MouSetSpeed: Fixe le rapport entre le dplacement de la souris et *}
{*               le mouvement du curseur de la souris qui doit en     *}
{*               rsulter                                             *}
{**------------------------------------------------------------------**}
{*  Entre : XSpeed = vitesse dans le sens des X                      *}
{*           YSpeed = vitesse dans le sens des Y                      *}
{*  Infos :  - Les deux paramtres sont exprims en units            *}
{*             mickey / 8 points cran                                *}
{**********************************************************************}

procedure MouSetSpeed( XSpeed, YSpeed : integer );

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  Regs.AX := $000f;             { N fct. "Set mickeys to pixel ratio" }
  Regs.CX := XSpeed;
  Regs.DX := YSpeed;
  Intr( $33, Regs);                   { Appeler le driver de la souris }
end;

{**********************************************************************}
{*  MouMovePtr: Amne le curseur de la souris dans un emplacement     *}
{*              dtermin de l'cran                                  *}
{**------------------------------------------------------------------**}
{*  Entre : COL = la nouvelle colonne d'cran du curseur souris      *}
{*           ROW = la nouvelle ligne d'cran du curseur souris        *}
{*  Infos :  - Les ordonnes se rapportent  l'cran de texte et non  *}
{*              l'cran graphique virtuel du driver de la souris     *}
{**********************************************************************}

procedure MouMovePtr( Col, Row : byte );

var Regs   : Registers;{Registres du processeur pour appel d'interrupt.}
    NouZon : byte;           { Zone dans laquelle la souris est amene }

begin
  Regs.AX := $0004;             { N fct. "Set mouse pointer position" }
  MouCol := col;                            { Stocker coordonnes dans }
  MouRow := row;                              { des variables globales }
  Regs.CX := integer( col ) shl 3;          { Convertir coordonnes et }
  Regs.DX := integer( row ) shl 3; {stocker dans des variables globales}
  Intr( $33, Regs );                  { Appeler le driver de la souris }

  NouZon := BufPtr^[ Row * TCol + Col ];                { Retirer zone }
  if NouZon <> ZonSou then                           { Nouvelle zone ? }
    begin                                                        { Oui }
      if NouZon = AUCUNE_ZONE then            { En dehors d'une zone ? }
        MouDefinePtr( StdPtr )    { Oui, curseur de la souris standard }
      else                                          { Non, zone connue }
        MouDefinePtr( PtrZonAct^[ NouZon ].PtrMask );
    end;
  ZonSou := NouZon;      { Ranger numro de zone dans variable globale }
end;

{**********************************************************************}
{*  MouSetDefaultPtr: Dfinit l'apparence du curseur de la souris     *}
{*                    pour les zones de l'cran qui n'ont pas t     *}
{*                    dsignes explicitement comme des zones souris  *}
{**------------------------------------------------------------------**}
{*  Entre : standard = masques curseur et cran pour curseur souris  *}
{*  Infos :  - Le paramtre transmis doit avoir t cr avec la      *}
{*             fonction MouPtrMask                                    *}
{**********************************************************************}

procedure MouSetDefaultPtr( Standard : PTRVIEW );

begin
  StdPtr := Standard;       { Ranger masque bits dans variable globale }

  {-- Si la souris ne figure actuellement dans aucune zone, on     ----}
  {-- commute directement sur nouvelle apparence                   ----}

  if ZonSou = AUCUNE_ZONE then                    { Dans aucune zone ? }
   MouDefinePtr( Standard );                                     { Non }
end;

{**********************************************************************}
{*  MouEnd: Est appele pour terminer le travail avec les fonctions   *}
{*          et procdures du module de la souris                      *}
{**------------------------------------------------------------------**}
{*  Infos : - La procdure n'a pas besoin d'tre appele explicite-   *}
{*            ment par le programme d'application car la fonction     *}
{*            MouInit la dfinit comme procdure Exit                 *}
{**********************************************************************}

{$F+}     { Doit tre FAR pour permettre un appel comme procdure EXIT }

procedure  MouEnd;

var Regs : Registers;{ Registres du processeur pour appel d'interrupt. }

begin
  MouHideMouse;             { Eliminer curseur de la souris de l'cran }
  Regs.AX := 0;              { Rinitialisation du driver de la souris }
  Intr( $33, Regs);                   { Appeler le driver de la souris }

  FreeMem( BufPtr, BLen );         { Librer  nouveau mmoire alloue }

  ExitProc := ExitOld;            { Installer  nouveau procdure Exit }
end;

{$F-}                                      { Plus de procdures FAR }

{**********************************************************************}
{*  MouInit: Dirige le travail avec les diffrentes fonctions et      *}
{*           procdures du module de la souris et initialise les      *}
{*           diffrentes variables                                    *}
{**------------------------------------------------------------------**}
{*  Entre : Colonnes = Nombre de colonnes de l'cran                 *}
{*           Lignes   = Nombre de lignes de l'cran                   *}
{*  Sortie : TRUE si un driver de la souris est install, sinon FALSE *}
{*  Infos :  - Cette fonction doit tre la premire des diffrentes   *}
{*             procdures et fonctions de ce module  tre appele    *}
{*             par un programme d'application                         *}
{**********************************************************************}

function MouInit( Colonnes, Lignes : byte ) : boolean;

var Regs : Registers; { Registres du processeur pour appel d'interrup. }

begin
  TLine := Lignes;                    { Stocker nombre de lignes et de }
  TCol  := Colonnes;            { colonnes dans des variables globales }

  ExitOld := ExitProc;           { Ranger adresse de la procdure Exit }
  ExitProc := @MouEnd;           { Dfinir MouEnd comme procdure Exit }

  {-- Allouer et remplir buffer pour les zones souris -----------------}

  BLen := TLine * TCol;            { Nombre de caractres dans l'cran }
  GetMem( BufPtr, BLen );             { Allouer buffer de zone interne }
  MouIBufFill( 0, 0, TCol-1, TLine-1, AUCUNE_ZONE );

  Regs.AX := 0;                      { Initialiser driver de la souris }
  Intr( $33, Regs );                  { Appeler le driver de la souris }
  MouInit := ( Regs.AX <> 0 );        { Driver de la souris install ? }

  MouSetMoveArea( 0, 0, TCol-1, TLine-1 ); { Fixer zone de dplacement }

  MouCol   := MouIGetX;                      { Charger position souris }
  MouRow   := MouIGetY;             { actuelle dans variables globales }
  ZonSou   := AUCUNE_ZONE;     { Curseur de la souris dans aucune zone }
  MouEvent := EV_LEFT_REL or EV_RIGHT_REL;{ Pas bouton de souris appuy}
  StdPtr   := MouPtrMask( PTRSAMECHAR, PTRINVCOL );   { Curseur souris }
  OldPtr   := PTRVIEW( 0 );

 {-- Installer le gestionnaire d'vnements assembleur "AssHand" ------}
 MouISetEventHandler( EV_MOU_ALL, FCTPTR(@AssHand) );

end;

{***********************************************************************
*                   P R O G R A M M E   P R I N C I P A L              *
***********************************************************************}

const Zones : array[0..4] of ZONE =           { Les zones de la souris }
  (
   ( x1:  0; y1:  0; x2: 79; y2:  0 ),                 { Ligne du haut }
   ( x1:  0; y1:  1; x2:  0; y2: 23 ),             { Colonne de gauche }
   ( x1:  0; y1: 24; x2: 78; y2: 24 ),                  { Ligne du bas }
   ( x1: 79; y1:  1; x2: 79; y2: 23 ),             { Colonne de droite }
   ( x1: 79; y1: 24; x2: 79; y2: 24 )           { Coin infrieur droit }
  );

var Dummy : integer;              { Reoit le rsultat de MouEventWait }

begin
  {-- Fixer curseur de la souris pour diffrentes zones de la souris --}
  Zones[ 0 ].PtrMask := MouPtrMask( PtrDifChar($18), PtrInvCol);
  Zones[ 1 ].PtrMask := MouPtrMask( PtrDifChar($1b), PtrInvCol);
  Zones[ 2 ].PtrMask := MouPtrMask( PtrDifChar($19), PtrInvCol);
  Zones[ 3 ].PtrMask := MouPtrMask( PtrDifChar($1a), PtrInvCol);
  Zones[ 4 ].PtrMask := MouPtrMask( PtrDifChar($58), PtrDifCol($40));

  writeln(#13#10,'SOURIS - (c) 1989 by MICHAEL TISCHER'#13#10);
  if MouInit( 80, 25 ) then          { Initialiser module de la souris }
   begin           { Tout va bien, un driver de la souris est install }
    writeln('Si vous dplacez le curseur de la souris sur',ZV,
            'l''cran, surtout le long des bords, vous constaterez',ZV,
            'que l''apparence du curseur de la souris se modifie en',ZV,
            'fonction de sa position.',ZV,ZV,
            'Pour mettre fin  cette dmo, amenez le curseur de la',ZV,
            'souris dans le coin infrieur droit de l''cran et appuyez'
        ,ZV,'alors simultanment sur les boutons gauche et droit de',ZV,
            'la souris.' );

    MouSetDefaultPtr( MouPtrMask( PtrDifChar( $DB ), PtrDifCol( 3 ) ) );
    ZoneMouDef( 5, @Zones );                              { Df. zones }
    MouShowMouse;          { Afficher curseur de la souris sur l'cran }

    {-- Attendre que boutons gauche et droit soient appuys en      ---}
    {-- mme temps et que le curseur de la souris se trouve  ce    ---}
    {-- moment dans la zone 4                                       ---}

    repeat                                            { Boucle de test }
     Dummy := MouEventWait( ET, EV_LEFT_PRESS or EV_RIGHT_PRESS );
    until ZonEv = 4;
   end
  else                { Pas de souris ou pas de driver souris install }
    writeln('Aucun driver souris n''est install !');
end.


