;**********************************************************************;
;*                       V 1 6 C O L P A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Fonction : contient diverses routines pour travailler dans les  *;
;*               modes graphiques  16 couleurs des cartes EGA et VGA *;
;*--------------------------------------------------------------------*;
;*    Auteur         : MICHAEL TISCHER                                *;
;*    Dvelopp le   :  5.12.1990                                     *;
;*    Dernire MAJ   : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    Assemblage     : MASM /mx V16COLPA    ou     TASM -mx V16COLPA  *;
;**********************************************************************;

;== Constantes =========================================================

SC_INDEX       = 3c4h     ;Registre d'index du contrleur du squenceur.
SC_MAP_MASK    = 2                          ;Numro du registre Map-Mask
SC_MEM_MODE    = 4                   ;Numro du registre de mode mmoire

GC_INDEX       = 3ceh          ;Registre d'index du contrleur graphique
GC_FN_SELECT   = 3          ;Numro du registre de slection de fonction
GC_READ_MAP    = 4                          ;Numro du registre Read Map
GC_GRAPH_MODE  = 5                 ;Numro du registre de mode graphique
GC_MISCELL     = 6                            ;Numro du registre divers
GC_BIT_MASK    = 8                 ;Numro du registre de masque binaire

CRTC_INDEX     = 3d4h            ;Registre d'index du contrleur d'cran
CC_MAX_SCAN    = 9     ;Numro du registre du maximum de lignes balayes
CC_START_HI    = 0Ch                        ;Numro du registre Hi_Start
CC_UNDERLINE   = 14h                 ;Numro du registre de soulignement
CC_MODE_CTRL   = 17h             ;Nummer du registre de contrle de mode

DAC_WRITE_ADR  = 3C8h                                 ;Adresse DAC Write
DAC_READ_ADR   = 3C7h                                  ;Adresse DAC Read
DAC_DATA       = 3C9h                          ;Registres de donnes DAC

VERT_RETRACE   = 3DAh                    ;Registre d'tat des entres #1

;== Segment de donnes =================================================

DATA   segment word public       ; initialiser au moment de l'excution

vio_seg    dw (?)                           ;Segment vido page courante
largeurl   dw (?)               ;Largeur d'une ligne de pixels en octets
pageofs    dw (?)                     ;Offset de la page dans le segment

DATA   ends

;== Programme ==========================================================

CODE       segment byte public                    ; Segment de programme

       assume cs:code, ds:data

;-- Dclarations publiques ---------------------------------------------

public    init640480                  ;initialise le mode 640*480 pixels
public    init640350                  ;initialise le mode 640*350 pixels
public    init640200                  ;initialise le mode 640*200 pixels
public    init320200                  ;initialise le mode 320*200 pixels
public    setpix                                       ;Dessine un pixel
public    getpix                              ;Lit la couleur d'un pixel
public    showpage                               ;Affiche la page 0 ou 1
public    setpage                    ;Fixe la page pour setpix et getpix

;-----------------------------------------------------------------------
;-- INIT640350: initialise le mode graphique EGA 640*350 en 16 couleurs
;-- Appel depuis TP: init640350;

init640350 proc near

           mov   al,10h                            ;Installe le mode 10h
           mov   cx, 28000 / 16                       ;Offset de la page

init16:    mov   bx,640/8                           ;Largeur de la ligne

init:      mov   largeurl,bx                        ;Mmorise la largeur
           mov   pageofs,cx   ;Mmorise offset de page pour adr. de seg.

           xor   ah,ah               ;Appelle la fonction 00h pour fixer
           int   10h                                            ;le mode

           mov   vio_seg,0A000h     ;Fixe le segment de la mmoire cran
           ret                                             ;Rend la main

init640350 endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- INIT640480 : Initialise le mode graphique VGA 640*480 en 16 couleurs
;-- Appel depuis TP: init640480;

init640480 proc near

       mov   al,12h                               ;Installe le mode 12hn

             ;-- L'offset de la page n'est pas intressant car une seule
                                              ;-- page est reprsentable

       jmp   init16

init640480 endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- INIT640200 : initialise le mode graphique EGA 640*200 en 16 couleurs
;-- Appel depuis TP: init640200;

init640200 proc near

       mov   al,0Eh                                ;Installe le mode 0Eh
       mov   bx,640/8                                ;Largeur des lignes
       mov   cx,( 64000 / 4 ) / 16                    ;Offset de la page
       jmp   init16

init640200 endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- INIT320200 : initialise le mode graphique EGA 320*200 en 16 couleurs
;-- Appel depuis TP: init320200;

init320200 proc near

       mov   al,0Dh                                ;Installe le mode 0Dh
       mov   bx,320/8                                ;Largeur des lignes
       mov   cx,( 32000 / 4 ) / 16                    ;Offset de la page
       jmp   init

init320200 endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- SETPIX: Dessine un pixel dans une couleur donne
;-- Appel depuis TP: setpix( x , y : integer; couleur : byte );

setpix     proc near

sframe     struc                            ;Structure d'accs  la pile
bp0        dw ?                                             ;Mmorise BP
ret_adr0   dw ?                          ;Adresse de retour  l'appelant
couleur    dw ?                                                 ;Couleur
y0         dw ?                                              ;Ordonne Y
x0         dw ?                                              ;Abscisse X
sframe     ends                                     ;Fin de la structure

frame      equ [ bp - bp0 ]        ;Adresse les lments de la structure

           push  bp                  ;Prpare l'adressage des paramtres
           mov   bp,sp                                           ;par BP

  ;--Calcule d'abord l'offset en mmoire d'cran et le dcalage       --

       mov   ax,frame.y0                            ;Charge l'ordonne Y
       mov   dx,largeurl              ;Multiplie par la largeur de ligne
       mul   dx
       mov   bx,frame.x0                            ;Charge l'abscisse Y
       mov   cl,bl                ;Mmorise l'octet faible pour dcalage

       shr   bx,1                             ;Divise l'abscisse X par 8
       shr   bx,1
       shr   bx,1
       add   bx,ax                ;Ajoute offset provenant de la multipl

       and   cl,7                  ;Calcule masque binaire  partir de X
       xor   cl,7
       mov   ah,1
       shl   ah,cl

       mov   dx,GC_INDEX                 ;Accde au contrleur graphique
       mov   al,GC_BIT_MASK       ;Charge masque bin. dans reg. masquage
       out   dx,ax

       mov   ax,(02h shl 8) + GC_GRAPH_MODE              ;mode Write 2 &
       out   dx,ax                                          ;mode Read 0

       mov   ax,vio_seg            ;Charge en ES le segment de mm cran
       mov   es,ax                                         ;avec la page

       mov   al,es:[bx]                        ;Charge le registre latch
       mov   al,byte ptr frame.couleur         ;Fixe la couleur du point
       mov   es:[bx],al                  ;Rcrit dans le registre latch

           ;-- Rtablit les valeurs par dfaut dans les registres du  --
                           ;-- contrleur graphqiue qui ont t modifis

       mov   ax,(0FFh shl 8 ) + GC_BIT_MASK
       out   dx,ax

       mov   ax,(00h shl 8) + GC_GRAPH_MODE
       out   dx,ax

       pop   bp
       ret   6                             ;Retour au programme appelant
                                       ;Retire les paramtres de la pile

setpix     endp                                        ;Fin de procdure

;-----------------------------------------------------------------------
;-- GETPIX : Dtermine la couleur d'un pixel
;-- Appel depuis TP: x := getpix( x , y : integer );

getpix     proc near

sframe1    struc                            ;Structure d'accs  la pile
bp1        dw ?                                             ;Mmorise BP
ret_adr1   dw ?                          ;Adresse de retour  l'appelant
y1         dw ?                                              ;Ordonne Y
x1         dw ?                                              ;Abscisse X
sframe1    ends                                     ;Fin de la structure

frame      equ [ bp - bp1 ]        ;adresse les lments de la structure

       push  bp                      ;Prpare l'adressage des paramtres
       mov   bp,sp                                              ; par BP

        ; Calcule d'abord l'offset en mmoire d'cran et le dcalage ---

       mov   ax,frame.y1                            ;Charge l'ordonne Y
       mov   dx,largeurl              ;Multiplie par la largeur de ligne
       mul   dx
       mov   si,frame.x1                            ;Charge l'abscisse X
       mov   cx,si                               ;et prpare le dcalage

       shr   si,1                                     ;Divise X par huit
       shr   si,1
       shr   si,1
       add   si,ax            ;Ajoute offset en provenance de la multipl

       and   cl,7               ;Calcule le masque binaire  partir de X
       xor   cl,7
       mov   ch,1
       shl   ch,cl

       mov   ax,vio_seg     ;Charge en ES le segment de la mmoire cran
       mov   es,ax                                         ;avec la page

       mov   dx,GC_INDEX                 ;Accde au contrleur graphique
       mov   ax,(3 shl 8)+ GC_READ_MAP           ;Lit d'abord le plan #3
       xor   bl,bl                                                     ;

gp1:       out   dx,ax                           ;Indique le plan  lire
           mov   bh,es:[si]            ;Lit le contenu du registre latch
           and   bh,ch                ;Ne conserve que le pixel concern
           neg   bh                              ;Fixe le bit 7 du pixel
           rol   bx,1     ;Effectue une rot bit 7 de BH dans bit 1 de BL

           dec   ah               ;Prpare le traitement du plan suivant
           jge   gp1                 ;sup ou gal  0 ? ---> on continue

           mov   al,bl                    ;Rsultat de la fonction en AL

           pop   bp
           ret   4                    ;Retourne  l'appelant en retirant
                                             ; les paramtres de la pile

getpix     endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- SETPAGE : Slectionne la page concerne par les appels aux fonctions
;--           setpix et getpix
;-- Appel depuis TP: setpage( page : byte );

setpage    proc near

       pop   cx                              ;Dpile l'adresse de retour
       pop   ax                                           ;et l'argument

       push  cx             ;Empile  nouveau la seule adresse de retour
       mul   pageofs        ;Multiplie numro de page par offset de page

       add   ax,0A000h                        ;Ajoute le segment de base
       mov   vio_seg,ax            ;Mmorise la nouvelle adr. de segment

       ret                            ;Retourne  l'appelant, l'argument
                                       ;ayant dj t retir de la pile

setpage    endp                                     ;Fin de la procdure

;-----------------------------------------------------------------------
;-- SHOWPAGE : Affiche l'une des pages d'cran disponibles
;-- Appel depuis TP: showpage( page : byte );

showpage   proc near

       pop   cx                              ;Dpile l'adresse de retour
       pop   ax                                    ;ainsi que l'argument

       push  cx             ;Empile  nouveau la seule adresse de retour
       mul   pageofs        ;Multiplie le numro de la page par l'offset
       mov   cl,4                          ;de la page et le tout par 16
       shl   ax,cl

       mov   bl,al                     ;Mmorise l'octet de poids faible

       mov   dx,CRTC_INDEX                ;Adresse le contrleur d'cran
       mov   al,CC_START_HI                ;Affiche d'abord l'octet fort
       out   dx,ax
       inc   al                                     ;puis l'octet faible
       mov   ah,bl
       out   dx,ax

                 ;-- Attend le dbut d'un rafrachissement d'cran -----

       mov   dx,VERT_RETRACE         ;Attend la fin du balayage vertical
sp3:   in    al,dx
       test  al,8
       jne   sp3

sp4:   in    al,dx                  ;puis le dbut du retour du faisceau
       test  al,8
       je    sp4

       ret                                        ;Retourne  l'appelant

showpage   endp                                     ;Fin de la procdure

;== Fin  ===============================================================

CODE   ends                                      ;Fin du segment de code
       end                                             ;Fin du programme
