{***********************************************************************
*                            V D A C P . P A S                         *
**--------------------------------------------------------------------**
*  Fonction         : Montre comment programmer les registres DAC      *
*                     dans les 256 couleurs du mode graphique de la    *
*                     carte VGA. Le programme utilise les routines     *
*                     en assembleur du module V3240PA.ASM              *
**--------------------------------------------------------------------**
*  Auteur           : MICHAEL TISCHER                                  *
*  Dvelopp le     : 02.01.1991                                       *
*  Dernire MAJ     : 14.01.1991                                       *
***********************************************************************}

program VDACP;

uses dos, crt;

{-- Dclarations de types ---------------------------------------------}

type DACREG = record                          { Dcrit un registre DAC }
                case integer of
                 0 : ( Rouge, Vert, Bleu : BYTE );    { Composants RGB }
                 1 : ( RGB : array[ 1..3] of BYTE );
              end;
     DACARRAY = array [0..255] of DACREG;          {Table DAC complte }

{-- Rfrences externes aux routines en assembleur  -------------------}

{$L v3240pa}        { Intgre le module en assembleur }

procedure init320400; external;
procedure setpix( x, y : integer; couleur : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{-- Constantes --------------------------------------------------------}

const MAXX = 319;                              { Coordonnes maximales }
      MAXY = 399;

      LARGEUR  = 10;          { Largeur d'un bloc de couleur en pixels }
      HAUTEUR = 20;           { Hauteur d'un bloc de couleur en pixels }
      DISTANCE = 2;                         { Distance entre les blocs }
      LARGEURT = 16 * LARGEUR + ( 15 * DISTANCE );     { Largeur totale}
      HAUTEURT  = 16 * HAUTEUR + ( 15 * DISTANCE );   { Hauteur totale }
      STARTX  = ( MAXX - LARGEURT ) div 2;      { Coin bloc sup gauche }
      STARTY  = ( MAXY - HAUTEURT ) div 2;

{***********************************************************************
*  IsVga   : Teste la prsence d'une carte VGA.                        *
**--------------------------------------------------------------------**
*  Entre  : nant                                                     *
*  Sortie  : TRUE ou FALSE                                             *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;                   { Registres pour l'interruption}

begin
  Regs.AX := $1a00;               { La fonction 1Ah n'existe qu'en VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Affiche un caractre en mode graphique                  *
**--------------------------------------------------------------------**
*  Entre : caractere = le caractre  afficher                        *
*           x, y      = Coordonnes du coin sup gauche                 *
*           cc        = Couleur du caractre                           *
*           cf        = Couleur du fond                                *
*  Info   : Le caractre est dessin dans une matrice de 8*8 pixels    *
*           sur la base du jeu de caractres 8*8 en ROM                *
***********************************************************************}

procedure PrintChar( caractere : char; x, y : integer; cc, cf : byte );

type CARADEF = array[0..255,0..7] of byte; {Structure jeu de caractres}
     CARAPTR = ^CARADEF;             { Pointe sur le jeu de caractres }

var  Regs  : Registers;       { Registres pour grer les interruptions }
     ch    : char;                                { Pixel du caractre }
     i, k,                                     { Compteur d'itrations }
     Masque : byte;         { Masque binaire pour dessiner le caractre}

const fptr : CARAPTR = NIL;   { Pointe sur le jeu de caractres en ROM }

begin
  if fptr = NIL then             { A-t-on dj dtermin ce pointeur ? }
    begin                                                         { Non}
      Regs.AH := $11;                   { Appelle l'option $1130 de la }
      Regs.AL := $30;                         { fonction vido du BIOS }
      Regs.BH := 3;          { pour obtenir un pointeur sur le jeu 8*8 }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );            { Compose le pointeur}
    end;

  if ( cf = 255 ) then                       { Caractre transparent ? }
    for i := 0 to 7 do  { Oui ne dessine que les pixels du premier plan}
      begin
        Masque := fptr^[ord(caractere),i];{Motif binaire pour une ligne}
        for k := 0 to 7 do
          begin
            if ( Masque and 128 <> 0 ) then        { Pixel  dessiner? }
              setpix( x+k, y+i, cc );                            { Oui }
            Masque := Masque shl 1;
          end;
      end
  else                                     { Non, tient compte du fond }
    for i := 0 to 7 do                           { Parcourt les lignes }
      begin
        Masque := fptr^[ord(caractere),i];{Motif binaire pour une ligne}
        for k := 0 to 7 do
          begin
            if ( Masque and 128 <> 0 ) then           { Premier plan ? }
              setpix( x+k, y+i, cc )                             { Oui }
            else
              setpix( x+k, y+i, cf );                       { Non, fond}
            Masque := Masque shl 1;
         end;
      end;
end;

{***********************************************************************
*  Line : Trace un segment dans la fentre graphique en appliquant     *
*         l'algorithme de Bresenham                                    *
**--------------------------------------------------------------------**
*  Entre : X1, Y1  = Coordonnes de l'origine (0 - ...)               *
*           X2, Y2  = Coordonnes de l'extrmit terminale             *
*           COULEUR = couleur du segment                               *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; couleur : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Procdure accessoire pour changer deux variables entires --------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Procdure principale --------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then     {Parcours : par axe X ou Y ? }
    begin                                            { par l'axe des Y }
      if ( y1 > y2 ) then                        { y1 suprieur  y2 ? }
        begin
      SwapInt( x1, x2 );                      { Oui, change X1 et X2, }
      SwapInt( y1, y2 );                                    { Y1 et Y2 }
        end;

      if ( x2 > x1 ) then xincr := 1          { Fixe le pas horizontal }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      Setpix( x, y, couleur );              { Dessine le premier point }
      for y:=y1+1 to y2 do                      { Parcourt l'axe des Y }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          Setpix( x, y, couleur );
        end;
    end
  else                                               { par l'axe des X }
    begin
      if ( x1 > x2 ) then                     { x1 plus grand que x2 ? }
        begin
      SwapInt( x1, x2 );                       { Oui, change X1 et X2 }
      SwapInt( y1, y2 );                                    { Y1 et Y2 }
        end;

      if ( y2 > y1 ) then yincr := 1            { Fixe le pas vertical }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      Setpix( x, y, couleur );              { Dessine le premier point }
      for x:=x1+1 to x2 do                      { Parcourt l'axe des X }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          Setpix( x, y, couleur );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint : Affiche une chane formate sur l'cran graphique       *
**--------------------------------------------------------------------**
*  Entres: X, Y   = Coordonnes de dpart (0-...)                     *
*           CC     = Couleur des caractres                            *
*           CF     = Couleur du fond (255 = transparent)               *
*           STRING = Chane avec indications de formatage              *
***********************************************************************}

procedure GrafPrint( x, y : integer; cc, cf : byte; strt : string );

var i : integer;                               { Compteur d'itrations }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, cc, cf );       { Affiche par printchar }
     inc( x, 8 );               { x  la position du caractre suivant }
   end;
end;


{***********************************************************************
*  GetDac: Dtermine les contenus d'un certain nombre de registres DAC *
**--------------------------------------------------------------------**
*  Entres : FIRST = Numro du premier registre (0-255)                *
*            NBR   = Nombre de registres DAC                           *
*            BUF   = buffer qui doit recevoir les contenus des         *
*                    registres DAC. Ce doit tre une variable du type  *
*                    DACREG ou un tableau de variables de ce type      *
*  Info    : Le buffer transmis doit comporter trois octets par        *
*            registre DAC  lire (pour la composante rouge, verte et   *
*            bleue de chaque couleur)                                  *
***********************************************************************}

procedure GetDac( First, Nbr : integer; var Buf );

var Regs : Registers;             { Registres pour grer l'interruption}

begin
  Regs.AX := $1017;             { Numro de la fonction et de l'option }
  Regs.BX := First;                    { Numro du premier registre DAC}
  Regs.CX := Nbr;                     { Nombre des registres  charger }
  Regs.ES := seg( Buf );            { Charge un pointeur sur le buffer }
  Regs.DX := ofs( Buf );
  intr( $10, Regs );                  { Dclenche l'interruption vido }
end;

{***********************************************************************
*  SetDac  : Charge un certain nombre de registres DAC                 *
**--------------------------------------------------------------------**
*  Entres : FIRST = Numro du premier registre DAC  (0-255)           *
*            NBR   = Nombre de registres                               *
*            BUF   = Buffer, d'o sont tires les valeurs  transfrer *
*                    dans les registres DAC. Il doit s'agir d'une      *
*                    variable de type DACREG ou d'un tableau de        *
*                    variables de ce type                              *
*  Info    : cf GetDac                                                 *
***********************************************************************}

procedure SetDac( First, Nbr : integer; var Buf );

var Regs : Registers;            { Registres pour grer l'interruption }

begin
  Regs.AX := $1012;             { Numro de la fonction et de l'option }
  Regs.BX := First;                   { Numro du premier registre DAC }
  Regs.CX := Nbr;                      { Nombre de registres  charger }
  Regs.ES := seg( Buf );            { Charge un pointeur sur le buffer }
  Regs.DX := ofs( Buf );
  intr( $10, Regs );          { Dclenche l'interruption vido du BIOS }
end;

{***********************************************************************
*  PrintDac : Affiche le contenu d'un registre DAC et rgle la couleur *
*             dans le registre DAC 255                                 *
**--------------------------------------------------------------------**
*  Entres : DREG    = Registre DAC                                    *
*            NUM     = Numro de ce registre                           *
*            COULEUR = Couleur d'affichage                             *
***********************************************************************}

procedure PrintDac( DReg : DACREG; Num, Couleur : BYTE );

var numstr,                           { Chane pour numro de registre }
    rstr,                               { Chane pour composante rouge }
    vstr,                               { Chane pour composante verte }
    bstr : string[3];                   { Chane pour composante bleue }

begin
  SetDac( 255, 1, DReg );           { Couleur dans le registre DAC 255 }
  str( Num : 3, numstr );     { Convertit en chanes couleurs et numro}
  str( DReg.Rouge : 2, rstr );
  str( DReg.Vert : 2, vstr );
  str( DReg.Bleu : 2, bstr );
  GrafPrint( 60, MAXY-10, Couleur, 0, 'DAC:' + numstr + '  R:' + rstr +
                                    '  V:' + vstr + '  B:' + bstr);
end;

{***********************************************************************
*  Encadre : Trace un cadre autour d'un champ de couleur               *
**--------------------------------------------------------------------**
*  Entres : X       = Abscisse X du champ de couleur (0-15)           *
*            Y       = Ordonne Y du champ de couleur (0-15)           *
*            COULEUR = Couleur du cadre                                *
*  Info    : L'paisseur du cadre est de 1 pixel indpendamment        *
*            de la distance qui spare les champs.                     *
***********************************************************************}

procedure Encadre( X, Y, Couleur : BYTE );

var sx, sy,                                  { Coin sup gauche du cadre}
    ex, ey : integer;                        { Coin inf droit du cadre }

begin
  sx := STARTX + X * (LARGEUR + DISTANCE) - 1;{calcule les coordonnes }
  ex := sx + LARGEUR + 1;                                   { du cadre }
  sy := STARTY + Y * (HAUTEUR + DISTANCE) - 1;
  ey := sy + HAUTEUR + 1;
  Line( sx, sy, ex, sy, Couleur );                  { Dessine le cadre }
  Line( ex, sy, ex, ey, Couleur );
  Line( ex, ey, sx, ey, Couleur );
  Line( sx, ey, sx, sy, Couleur );
end;

{***********************************************************************
*  ChangeDacReg : Modifie le contenu d'un registre DAC en mmoire et   *
*                 dans la table DAC de la carte vido, puis l'affiche  *
*                 sur l'cran                                          *
**--------------------------------------------------------------------**
*  Entres : DREG  = Registre DAC  modifier                           *
*            NUM   = Numro du registre DAC                            *
*            COMP  = Numro de la composante  modifier    (1-3)       *
*                    1 = Rouge, 2 = Vert, 3 = Bleu                     *
*            INCR  = Incrment pour cette composante                   *
***********************************************************************}

procedure ChangeDacReg( var DReg : DACREG; Num, Comp : BYTE;
                            Incr : integer );

begin
  inc( DReg.RGB[ Comp ], Incr );             { Incrmente la composante}
  if DReg.RGB[ Comp ] > 63 then                           { Sup  63 ? }
    DReg.RGB[ Comp ] := 0;                            { Oui, remet  0 }
  SetDac( Num, 1, DReg );                     { Charge le registre DAC }
  PrintDac( DReg, Num, 15 );               { Affiche le nouveau contenu}
end;

{***********************************************************************
*  Demo : Prsente la programmation des registres DAC et le systme de *
*         couleur de la carte VGA                                      *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
***********************************************************************}

procedure Demo;

var x,  y,
    ix, jx,
    iy, jy,
    k,  f  : integer;                          { Compteur d'itrations }
    ch     : char;                                            { Touche }
    dacbuf : DACARRAY;                            { Table DAC complte }
    DReg   : DACREG;                            { Registre DAC courant }

begin
{-- Dessine l'cran ---------------------------------------------------}

  SetPage( 0 );                                     { Traite la page 0 }
  ShowPage( 0 );                                   { Affiche la page 0 }
  GetDac( 0, 256, dacbuf );             { Charge la table DAC complte }

  GrafPrint( 10, 0, 255, 0,
             'VDACP  -  (c) 1991 by MICHAEL TISCHER' );

{-- Construit le bloc de 16*16 champs de couleur ----------------------}

  iy := STARTY;                          { Point de dpart sur l'cran }
  jy := STARTY + HAUTEUR - 1;
  f  := 0;
  for y := 0 to 15 do                { Parcourt les 16 lignes de blocs }
    begin
      ix := STARTX;
      jx := STARTX + LARGEUR - 1;
      for x := 0 to 15 do           { Parcourt les 16 colonnes de blocs}
        begin
          for k := iy to jy do   { Dessine les blocs avec des segments }
            Line( ix, k, jx, k, f );
          inc( ix, LARGEUR + DISTANCE );       { Prochain bloc  droite}
          inc( jx, LARGEUR + DISTANCE );
          inc( f );                                 { Couelur suivante }
        end;
      inc( iy, HAUTEUR + DISTANCE );               { Position suivante }
      inc( jy, HAUTEUR + DISTANCE );
    end;

{-- Lit les entres de l'utilisateur et ragit en consquence ---------}
  ix := 0;               { Commence en haut  gauche avec la couleur 0 }
  iy := 0;
  jx := 0;
  jy := 0;
  k := 0;
  GetDac( 0, 1, DReg );                             { Lit la couleur 0 }
  Encadre( 0, 0, 15 );                              { Encadre le champ }
  PrintDac( DReg, 0, 15 );                     { et affiche le contenu }
  repeat
    ch := ReadKey;                                 { Attend une frappe }
    if ( ch <> #0 ) then                             { Code tendu ? ? }
      case ch of                                    { Non, on exploite }
        'r' : ChangeDacReg( DReg, k, 1, +1 );            { r = Rouge + }
        'v' : ChangeDacReg( DReg, k, 2, +1 );             { v = Vert + }
        'b' : ChangeDacReg( DReg, k, 3, +1 );             { b = Bleu + }
        'R' : ChangeDacReg( DReg, k, 1, -1 );            { R = Rouge - }
        'V' : ChangeDacReg( DReg, k, 2, -1 );             { V = Vert - }
        'B' : ChangeDacReg( DReg, k, 3, -1 );             { B = Bleu - }
        ' ' : begin             { Space = rtablit la valeur d'origine }
                DReg := dacbuf[ k ];
                ChangeDacReg( DReg, k, 1, 0 );
              end;
      end
    else                                                 { Code tendu }
      case ReadKey of
        #72 : if ( iy = 0 ) then                { Curseur vers le haut }
                jy := 15
              else
                jy := iy - 1;

        #80 : if ( iy = 15 ) then                { Curseur vers le bas }
                jy := 0
              else
                jy := iy + 1;

        #75 : if ( ix = 0  ) then                   { Curseur  gauche }
                jx := 15
              else
                jx := ix - 1;

        #77 : if ( ix = 15 ) then                   { Curseur  droite }
                jx := 0
              else
                jx := ix + 1;
      end;

    if ( ix <> jx ) or ( iy <> jy ) then         { Nouvelle position ? }
      begin                                                      { Oui }
        Encadre( ix, iy, 0 );                  { Enlve l'ancien cadre }
        Encadre( jx, jy, 15 );                { Trace un nouveau cadre }
        ix := jx;               { Mmorise le nouveau champ de couleur }
        iy := jy;
        k := iy*16+ix;                           { Calcule son  numro }
        GetDac( k, 1, DReg );                  { Charge le registre DAC}
        PrintDac( DReg, k, 15 );                        { et l'affiche }
      end;
  until ch = #13;            { On recommence jusqu' frappe de <Entre>}

  SetDac( 0, 256, dacbuf );                     { restaure la table DAC}
end;

{----------------------------------------------------------------------}
{--                        Programme principal                     ----}
{----------------------------------------------------------------------}

begin
 if IsVga then                                { A-t-on une carte VGA ? }
   begin                                            { Oui, c'est parti }
     init320400;                        { Initialise le mode graphique }
     Demo;
     Textmode( CO80 );                        { Rtablit le mode texte }
   end
 else
   writeln( 'VDACP  -  (c) 1991 by MICHAEL TISCHER'#13#10#10 +
            'Attention ce programme exige une carte VGA ' +
            '.'#13#10 );
end.
