{***********************************************************************
*                            X M S P . P A S                           *
**--------------------------------------------------------------------**
*  Sujet        : Dmonstration de l'accs  la mmoire tendue et de  *
*                 la zone High Memory  l'aide des fonctions XMS telles*
*                 qu'elles sont appliques par exemple avec le pilote  *
*                 de priphriques HIMEM.SYS.                          *
**--------------------------------------------------------------------**
*  Auteur        : MICHAEL TISCHER                                     *
*  Dvelopp le  : 27.07.1990                                          *
*  Dernire MAJ  : 29.03.1992                                          *
***********************************************************************}

program XMSP;

uses Crt, Dos;              { Pour appel interruption et accs clavier }

const ERR_NOERR          = $00;                         { Pas d'erreur }
      ERR_NOTIMPLEMENTED = $80;            { Fonction appele inconnue }
      ERR_VDISKFOUND     = $81;               { Ram disk VDISK dtect }
      ERR_A20            = $82;      { Erreur sur canal d'adresses A20 }
      ERR_GENERAL        = $8E;    { Erreur pilote  caractre gnral }
      ERR_UNRECOVERABLE  = $8F;                  { Erreur irratrapable }
      ERR_HMANOTEXIST    = $90;                      { HMA introuvable }
      ERR_HMAINUSE       = $91;                     { HMA dj appele }
      ERR_HMAMINSIZE     = $92;              { Taille HMA insuffisante }
      ERR_HMANOTALLOCED  = $93;                     { HMA non affecte }
      ERR_A20STILLON     = $94;    { Canal d'adresses A20 encore actif }
      ERR_OUTOMEMORY     = $A0;   { Plus de mmoire tendue disponible }
      ERR_OUTOHANDLES    = $A1;        { Ts les identifs XMS sont pris }
      ERR_INVALIDHANDLE  = $A2;              { Identificateur invalide }
      ERR_SHINVALID      = $A3;       { Identificateur source invalide }
      ERR_SOINVALID      = $A4;             { Dcalage source invalide }
      ERR_DHINVALID      = $A5;  { Identificateur destination invalide }
      ERR_DOINVALID      = $A6;        { Dcalage destination invalide }
      ERR_LENINVALID     = $A7; { Longueur invalide pour fonction Move }
      ERR_OVERLAP        = $A8;                { Recouvrement interdit }
      ERR_PARITY         = $A9;                     { Erreur de parit }
      ERR_EMBUNLOCKED    = $AA;                 { UMB n'est pas bloqu }
      ERR_EMBLOCKED      = $AB;                { UMB est encore bloqu }
      ERR_LOCKOVERFLOW   = $AC;   { Dbordement du compteur de blocage }
      ERR_LOCKFAIL       = $AD;        { L'UMB ne peut pas tre bloqu }
      ERR_UMBSIZETOOBIG  = $B0;            { Plus petit UMB disponible }
      ERR_NOUMBS         = $B1;                { Plus d'UMB disponible }
      ERR_INVALIDUMB     = $B2;{ L'adresse du segment UMB est invalide }

type XMSRegs = record                    { Informations pour appel XMS }
                 AX,               { Seuls les registres AX, BX, DX et }
                 BX,                { SI sont requis selon la fonction }
                 DX,                 { appele, il faut donc une autre }
                 SI,                              { adresse de segment }
                 Segment : word
               end;

{-- Variables globales ------------------------------------------------}

var XMSPtr : pointer;     { Pointeur sur Extended Memory Manager (XMM) }
    XMSErr : BYTE;            { Code d'erreur de la dernire opration }

{***********************************************************************
* XMSInit : Initialise les routines d'appel des fonctions XMS          *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Sortie : TRUE si pilote XMS identifi sinon FALSE                    *
* Info    : - L'appel de cette fonction doit prcder celui de toutes  *
*             les autres procdures et fonctions manant de ce         *
*             programme.                                               *
***********************************************************************}

function XMSInit : boolean;

var Regs : Registers;             { Registre pour appel d'interruption }
    xr : XMSRegs;

begin
  Regs.AX := $4300;             { Dtecte disponibilit du XMS Manager }
  intr( $2F, Regs );
  if ( Regs.AL = $80 ) then                     { Dtect XMS-Manager? }
    begin                                                        { Oui }
      Regs.AX := $4310;               { Retourner point d'accs au XMM }
      intr( $2F, Regs );
      XMSPtr := ptr( Regs.ES, Regs.BX );  { Copie adresse ds var glob. }
      XMSErr := ERR_NOERR;                      { Pas d'erreur apparue }
      XMSInit := true;             { Trouv handler, module initialis }
    end
  else                                { XMS handler n'est pas install }
   XMSInit := false;
end;

{***********************************************************************
* XMSCall : Routine gnrale d'appel d'une fonction XMS                *
**--------------------------------------------------------------------**
* Entre : NumFonc = Numro de la fonction XMS appelante               *
*            XRegs = Structure de registres pour appel de fonction     *
* Info    : - Avant l'appel de cette procdure, charger excusivement   *
*             les registres effectivement ncessaires  l'appel de la  *
*             fonction.                                                *
*           - Aprs l'appel de la fonction XMS, les contenus des       *
*             registres du processeur sont copis dans les composants  *
*             correspondants de la structure renvoye.                 *
*           - Le premier appel de cette procdure doit tre prcd    *
*             d'un appel en bon ordre de la procdure XMSInit.         *
***********************************************************************}

procedure XMSCall( NumFonc : byte; var XRegs : XMSRegs );

begin
  inline ( $8C / $D9 /                          { mov    cx,ds        *}
           $51 /                                { push   cx           *}
           $C5 / $BE / $04 / $00 /              { lds    di,[bp+0004] *}
           $8A / $66 / $08 /                    { mov    ah,[bp+0008] *}
           $8B / $9D / $02 / $00 /              { mov    bx,[di+0002] *}
           $8B / $95 / $04 / $00 /              { mov    dx,[di+0004] *}
           $8B / $B5 / $06 / $00 /              { mov    si,[di+0006] *}
           $8E / $5D / $08 /                    { mov    ds,[di+08]   *}
           $8E / $C1 /                          { mov    es,cx        *}
           $26 / $FF / $1E / XMSPtr /           { call   es:[XMSPTr]  *}
           $8C / $D9 /                          { mov    cx,ds        *}
           $C5 / $7E / $04 /                    { lds    di,[bp+04]   *}
           $89 / $05 /                          { mov    [di],ax      *}
           $89 / $5D / $02 /                    { mov    [di+02],bx   *}
           $89 / $55 / $04 /                    { mov    [di+04],dx   *}
           $89 / $75 / $06 /                    { mov    [di+06],si   *}
           $89 / $4D / $08 /                    { mov    [di+08],cx   *}
           $1F                                  { pop    ds           *}
        );

     {-- Tests codes erreur -------------------------------------------}

  if ( XRegs.AX = 0 ) and ( XRegs.BX >= 128 ) then
    begin
      XMSErr := Lo(XRegs.BX)             { Erreur, stocker code erreur }
      {
       .
       .
       .
         emplacement ventuel pour un autre traitement d'erreur
       .
       .
       .
      }
    end
  else
    XMSErr := ERR_NOERR;                     { Pas d'erreur, a baigne }
end;

{***********************************************************************
* XMSQueryVer: Renvoie le n de version de l'XMS et autres informations*
*              sur l'tat                                              *
**--------------------------------------------------------------------**
* Entre : VerNr = Stocke n de version aprs appel de la fonction     *
*                  (Format: 235 = 2.35)                                *
*           RevNr = Stocke n de rvision aprs appel de la fonction   *
* Sortie : TRUEsi un HMA est disponible sinon FALSE                    *
***********************************************************************}

function XMSQueryVer( var VerNr, RevNr : integer ): boolean;

var XR : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 0, XR );
  VerNr := Hi(XR.AX)*100 + ( Lo(XR.AX) shr 4 ) * 10 +
           ( Lo(XR.AX) and 15 );
  RevNr := Hi(XR.BX)*100 + ( Lo(XR.BX) shr 4 ) * 10 +
           ( Lo(XR.BX) and 15 );
  XMSQueryVer := ( XR.DX = 1 );
end;

{***********************************************************************
* XMSGetHMA : Retourner  l'appelant le droit d'accs  la HMA.        *
**--------------------------------------------------------------------**
* Entre : LenB = Nombre d'octets  allouer                            *
* Info    : Les programmes rsidents devraient rserver exclusivement  *
*           la mmoire strictement requise. Par contre, donner $FFFF   *
*           aux applications.                                          *
* Sortie : TRUE si la HMA a pu tre rendue disponible sinon FALSE;     *
***********************************************************************}

function XMSGetHMA( LenB : word ) : boolean;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XR.DX := LenB;                    { Stocke longueur dans registre DX }
  XmsCall( 1, Xr );                            { Appel fonction XMS #1 }
  XMSGetHMA := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSReleaseHMA : Libre l'HMA et permet ainsi sa transmission        *
*                 d'autres programmes.                                 *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Info    : - Appeler cette procdure avant de quitter un programme    *
*             si la HMA a t alloue par un appel de XMSGetHMA pour   *
*             pouvoir la transmettre aux programmes appels plus tard  *
*           - L'appel de cette procdure entrane la perte des         *
*             donnes stockes dans la HAM.                            *
***********************************************************************}

procedure XMSReleaseHMA;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 2, Xr );                            { Appel fonction XMS #2 }
end;

{***********************************************************************
* XMSA20OnGlobal: Libre le canal d'adresses A20, permettant l'accs   *
*                 direct  la HMA..                                    *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Info    : - La libration du canal d'adresses A20 est relativement   *
*             lente sur de nombreux ordinateurs. Veillez  ne pas      *
*             abuser de l'usage de cette procdure.                    *
***********************************************************************}

procedure XMSA20OnGlobal;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 3, Xr );                            { Appel fonction XMS #3 }
end;

{***********************************************************************
* XMSA20OffGlobal: Pendant de la procdure XMSA20OnGlobal, celle-ci    *
*                  bloque  nouveau le canal d'adresses A20, inter-    *
*                  sant l'accs direct  la HMA.                       *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Info    : - Appelez toujours cette procdure avant de quitter un     *
*             programme si le canal d'adresses A20 a t libr par    *
*             un appel de XMSA20OnGlobal.                              *
***********************************************************************}

procedure XMSA20OffGlobal;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 4, Xr );                            { Appel fonction XMS #4 }
end;

{***********************************************************************
* XMSA20OnLocal: Voir XMSA20OnGlobal                                   *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Info    : - Cette procdure locale se distingue de sa variante       *
*             globale par la libration du canal: elle n'est possible  *
*             que si elle n'a pas t effectue par un appel prcdent *
***********************************************************************}

procedure XMSA20OnLocal;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 5, Xr );                            { Appel fonction XMS #5 }
end;

{***********************************************************************
* XMSA20OffLocal : Voir XMSA29OffGlobal                                *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Info    : - Cette procdure locale se distingue de sa variante       *
*             globale par la libration du canal: elle n'est possible  *
*             que si elle n'a pas t effectue par un appel prcdent *
***********************************************************************}

procedure XMSA20OffLocal;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 6, Xr );                            { Appel fonction XMS #6 }
end;

{***********************************************************************
* XMSIsA20On : Retourne l'tat du canal d'adresse A20                  *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
* Sortie : TRUE si le canal d'adresses A20 est libr sinon  FALSE.    *
***********************************************************************}

function XMSIsA20On : boolean;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 7, Xr );                            { Appel fonction XMS #7 }
  XMSIsA20On := ( Xr.AX = 1 );               { AX = 1 ---> canal libre }
end;

{***********************************************************************
* XMSQueryFree : Renvoie la mmoire tendue disponible et la taille du *
*                plus gros bloc libre                                  *
**--------------------------------------------------------------------**
* Entre : TotalLibre: Stocke la taille totale de l'EM libre.          *
*           MaxBl  : Stocke la taille du plus grand bloc libre.        *
* Info    : - Les deux valeurs sont en Ko                              *
*           - La taille de la HMA n'est pas comptabilise mme si elle *
*             n'a t affecte  aucun programme.                      *
***********************************************************************}

procedure XMSQueryFree( var TotalLibre, MaxBl : integer );

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  XmsCall( 8, Xr );                            { Appel fonction XMS #8 }
  TotalLibre := Xr.AX;                  { La taille totale est dans AX }
  MaxBl   := Xr.DX;                     { La mmoire libre est dans DX }
end;

{***********************************************************************
* XMSGetMem : Alloue un bloc de mmoire tendue (EMB)                  *
**--------------------------------------------------------------------**
* Entre : LenKB : Taille en Ko du bloc interrog                      *
* Sortie : Identificateur pour accs au bloc ou 0, si aucun bloc n'a   *
*          pu tre allou. Un code d'erreur se trouve alors dans la    *
*          variable globale XMSErr.                                    *
***********************************************************************}

function XMSGetMem( LenKb : integer ) : integer;

var Xr : XMSRegs;                 { Registre pour communiquer avec XMS }

begin
  Xr.DX := LenKB;           { Stockons la longueur dans le registre DX }
  XmsCall( 9, Xr );                            { Appel fonction XMS #9 }
  XMSGetMem := Xr.DX                       { Retourne l'identificateur }
end;

{***********************************************************************
* XMSFreeMem : Libre un bloc de mmoire tendue (EMB) prcdemment    *
*              allou                                                  *
**--------------------------------------------------------------------**
* Entre : Handle : L'identificateur pour accder au bloc. Il a t    *
*                   obtenu en appelant XMSGetMem.                      *
* Info    : - Le contenu de l'EMB est dfinitivement dtruit par cet   *
*             appel, l'identificateur devient invalide.                *
*           - Avant de quitter un programme, librez  l'aide de cette *
*             procdure toutes les zones prcdemment alloues pour    *
*             pouvoir les allouer aux programmes suivants.             *
***********************************************************************}

procedure XMSFreeMem( Handle : integer );

var Xr : XMSRegs;                 { Registre pour communiquer avec XMS }

begin
  Xr.DX := Handle;      { Copions l'identificateur dans le registre DX }
  XmsCall( 10, Xr );                          { Appel fonction XMS #10 }
end;

{***********************************************************************
* XMSCopy : : Copie des zones de mmoire entre la mmoire tendue et la*
*           mmoire conventionnelle ou  l'intrieur de ces deux       *
*           groupes de mmoire.                                        *
**--------------------------------------------------------------------**
* Entre : HandleOrig  : Identificateur du bloc de mmoire  dplacer  *
*          OffsetOrig  : Offset dans ce bloc,  partir duquel le       *
*                        dplacement sera effectu.                    *
*          HandleDest :  Identificateur du bloc de mmoire cible       *
*          OffsetDest :  Offset dans le bloc cible.                    *
*          LenW       :  Nombre de mots dplacs                       *
* Info    : - Pour utiliser la mmoire normale dans cette opration,   *
*           donnez la valeur 0  l'identificateur ("Handle") et pour   *
*           l'offset, le segment et l'adresse de l'offset dans sa      *
*           forme habituelle (offset avant le segment).                *
***********************************************************************}

procedure XMSCopy( HandleOrig  : integer; OffsetOrig  : longint;
                   HandleDest : integer; OffsetDest : longint;
                   LenW       : longint  );

type EMMS = record                    { Structure Move mmoire tendue }
              LenB    : longint;             { Nbre d'octets  dcaler }
              SHandle : integer;               { Identificateur source }
              SOffset : longint;                     { Dcalage source }
              DHandle : integer;                  { Handle destination }
              DOffset : longint;                { Dcalage destination }
            end;

var Xr : XMSRegs;                 { Registre de communication avec XMS }
    Mi : EMMS;                                         { Stocke l'EMMS }

begin
  with Mi do                           { Commencer par prparer l'EMMS }
    begin
      LenB := 2 * LenW;
      SHandle := HandleOrig;
      SOffset := OffsetOrig;
      DHandle := HandleDest;
      DOffset := OffsetDest
    end;

  Xr.Si      := Ofs( Mi );             { Adresse de l'offset de l'EMMS }
  Xr.Segment := Seg(Mi);                { Adresse du segment de l'EMMS }
  XmsCall( 11, Xr );                          { Appel fonction XMS #11 }
end;

{***********************************************************************
* XMSLock : Interdit tout dcalage d'un bloc de mmoire tendue par    *
*           l'XMM et retourne son adresse absolue.                     *
**--------------------------------------------------------------------**
* Entre : Handle : Identificateur du bloc de mmoire retourn lors de *
*          l'appel prcdent de XMSGetMem.                             *
* Sortie : Adresse linaire du bloc de mmoire.                        *
***********************************************************************}

function XMSLock( Handle : integer ) : longint;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := Handle;                                   { Handle de l'EMB }
  XmsCall( 12, Xr );                          { Appel fonction XMS #12 }
  XMSLock := longint(Xr.DX) shl 16 + Xr.BX;  { Calcule adr.sur 32 bits }
end;

{***********************************************************************
* XMSUnlock : Libre  nouveau un bloc de mmoire tendu pour une      *
*             opration de dcalage.                                   *
**--------------------------------------------------------------------**
* Entre : Handle : Identificateur de la zone de mmoire retourn lors *
           d'un appel prcdent de XMSGetMem.                          *
***********************************************************************}

procedure XMSUnLock( Handle : integer );

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := Handle;                                   { Handle de l'EMB }
  XmsCall( 13, Xr );                          { Appel fonction XMS #13 }
end;

{***********************************************************************
* XMSQueryInfo : Retourne diverses informations sur un bloc de         *
*                mmoire tendue pralablement allou.                 *
**--------------------------------------------------------------------**
* Entre :  Handle : Identificateur de la zone de mmoire              *
*           Lock   : Variable de stockage du compteur de Lock          *
*           LenKB  : Variable de stockage de la longueur du bloc en Ko *
*           FreeH  : Variable de stockage du nombre d'identificateurs  *
*                    restant libres.                                   *
* Info    : Cette procdure ne permet pas de connatre l'adresse d'un  *
*           bloc. Utilisez la fonction XMSLock pour cette information. *
***********************************************************************}

procedure XMSQueryInfo( Handle    : integer; var Lock, LenKB : integer;
                        var FreeH : integer );

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := Handle;                                   { Handle de l'EMB }
  XmsCall( 14, Xr );                          { Appel fonction XMS #14 }
  Lock  := Hi( Xr.BX );                            { Lit les registres }
  FreeH := Lo( Xr.BX );
  LenKB := Xr.DX;
end;

{***********************************************************************
* XMSRealloc : Agrandit ou rduit la taille d'un bloc de mmoire       *
*              tendue allou par XMSGetMem                            *
**--------------------------------------------------------------------**
* Entre :  Handle   : Identificateur de la zone de mmoire            *
*           NewLenkB : Nouvelle taille du bloc, en Ko                  *
* Sortie :  TRUE si la taille du bloc a t modifie sinon FALSE       *
* Info   :  Ce bloc ne doit pas tre verrouill!                       *
***********************************************************************}

function XMSRealloc( Handle, NewLenkB : integer ) : boolean;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := Handle;                                   { Handle de l'EMB }
  Xr.BX := NewLenkB;        { La nouvelle longueur dans le registre BX }
  XmsCall( 15, Xr );                          { Appel fonction XMS #15 }
  XMSRealloc := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSGetUMB : Alloue un bloc de Upper Memory (UMB)                     *
**--------------------------------------------------------------------**
* Entre :  LenPara : Taille de la zone alloue en paragraphes de 16   *
*                     octets chacun.                                   *
*           Seg     : Variable de stockage de l'adresse du segment de  *
*                     l'UMB allou (si tout va bien)                   *
*           MaxPara : Variable contenant la taille du plus gros bloc   *
*                     UMB en cas d'chec.                              *
* Sortie : TRUE si un UMB a t allou sinon FALSE                     *
* Info    : Attention! Cette fonction n'est pas compatible avec tous   *
*           les pilotes XMS. Elle est extrmement dpendante du        *
*           matriel.                                                  *
***********************************************************************}

function XMSGetUMB( LenPara          : integer;
                    var Seg, MaxPara : word ) : boolean;

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := LenPara;                       { Longueur demande selon DX }
  XmsCall( 16, Xr );                          { Appel fonction XMS #16 }
  Seg := Xr.BX;                        { Retourne l'adresse du segment }
  MaxPara := Xr.DX;                        { Longueur du plus gros UMB }
  XMSGetUMB := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSFreeUMB : Libre un UMB allou par XMSGetUMB.                     *
**--------------------------------------------------------------------**
* Entre  : Seg : Adresse du segment de l'UMB  librer                *
* Info    : Attention! Cette fonction n'est pas compatible avec tous   *
*           les pilotes XMS. Elle est extrmement dpendante du        *
*           matriel.                                                  *
***********************************************************************}

procedure XMSFreeUMB( var Seg : word );

var Xr : XMSRegs;                 { Registre de communication avec XMS }

begin
  Xr.DX := Seg;                 { Adresse du segment de l'UMB selon DX }
  XmsCall( 17, Xr );                          { Appel fonction XMS #17 }
end;

{----------------------------------------------------------------------}
{-- Procdures de test et de dmonstration                           --}
{----------------------------------------------------------------------}

{***********************************************************************
* HMATest : Teste la disponibilit de l'HMA et dmontre son maniement. *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
***********************************************************************}

procedure HMATest;

type HMAR = array [1..65520] of BYTE;                 { Le tableau HMA }
     HMARPTR = ^HMAR;                    { Pointeur sur le tableau HMA }

var ch   : char;                             { pour interroger touches }
    A20  : boolean;              { Etat courant du canal d'adresse A20 }
    hmap : HMARPTR;                               { Pointeur sur l'HMA }
    i,                                         { Compteur d'itrations }
    err  : word;                   { Nombre d'erreurs d'accs  la HMA }

begin
  write( 'Test HMA : Veuillez taper une touche pour lancer ' +
         'le test...' );
  ch := ReadKey;
  writeln( #10 );

   {-- Allouer HMA et tester chaque adresse de la mmoire -------------}

  if ( XMSGetHMA( $FFFF) ) then               { Contrlons-nous l'HMA? }
    begin                                                        { Oui }
      A20 := XMSIsA20On;                      { Donner l'tat du canal }
      If ( A20 = FALSE ) then                      { Canal A20 libre ? }
        XMSA20OnGlobal;                   { Non, le librer maintenant }

      hmap := HMARPTR(Ptr( $FFFF, $0010 ));         { Pointeur sur HMA }

      err := 0;                       { Jusque l encore aucune erreur }
      for i := 1 to 65520 do        { Tester chaque adresse sparment }
        begin
          write( #13, 'Cellule mmoire : ', i );
          hmap^[i] := i mod 256;               { Ecrire dans l'adresse }
          if ( hmap^[i] <> i mod 256 ) then             { et la relire }
            begin                                            { Erreur! }
              writeln( ' ERREUR!' );
              inc( err );
            end;
        end;

      XMSReleaseHMA;                                  { Librons l'HMA }
      If ( A20 = FALSE ) then   { Est-ce que le canal A20 tait libre? }
        XMSA20OffGlobal;                            { Non, librons-le }

      writeln( #13 );
      if ( err = 0 ) then                { Analyse du rsultat du test }
        writeln( 'HMA ok, aucune cellule de la mmoire dfectueuse.')
      else
        writeln( 'ATTENTION! ', err, ' cellules dfectueuses dans',
                 ' l''HMA ! ');
    end
  else
   writeln( 'ATTENTION! Accs impossible  l''HMA.' );
end;

{***********************************************************************
* EMBTest : Teste la mmoire tendue et montre l'appel de plusieurs    *
*           fonctions XMS                                              *
**--------------------------------------------------------------------**
* Entre : aucune                                                      *
***********************************************************************}

procedure EMBTest;

type BAR = array [1..1024] of BYTE;      { tableau "Byte array" d'1 Ko }
     BARPTR = ^BAR;             { Pointeur sur le tableau "Byte array" }

var ch      : char;                          { Pour interroger touches }
    Adr     : longint;                              { Adresse de l'EMB }
    barp    : BARPTR;                     { Pointeur sur tampon d'1 Ko }
    i, j,                                      { Compteur d'itrations }
    err,                           { Nombre d'erreurs en accs  l'HMA }
    Handle,                      { Identificateur pour l'accs  l'EMB }
    TotalLibre,                   { Taille de toute la mmoire tendue }
    MaxBl   : integer;                          { Plus gros bloc libre }

begin
  write( 'Test EMB : Veuillez taper une touche pour ' +
         'lancer le test...' );
  ch := ReadKey;
  writeln( #10 );

  XMSQueryFree( TotalLibre, MaxBl ); { Ret. taille tot. de mm. tend. }
  writeln( 'Taille totale de la mmoire tendue (avec HMA) : ',
            TotalLibre, ' KB' );
  writeln( '                  Dont le plus gros bloc libre : ',
            MaxBl, ' KB' );

  TotalLibre := TotalLibre - 64;   { Calcul taille effective sans HMA. }
  if ( MaxBl >= TotalLibre ) then              { Valeur vraisemblable? }
    MaxBl := MaxBl - 64;                                         { Non }

  if ( MaxBl <> 0 ) then                    { Encore assez de mmoire? }
    begin                                                        { Oui }
      Handle := XMSGetMem( MaxBl );
      writeln( MaxBl, ' Ko allous.' );
      writeln( 'Identificateur = ', Handle );
      Adr := XMSLock( Handle );                  { Retourner l'adresse }
      XMSUnlock( Handle );                         { Supprimer blocage }
      writeln( 'Adresse de dpart = ', Adr, ' (', Adr div 1024, ' Ko)');

      GetMem( barp, 1024 );                    { Tampon sur Turbo Heap }
      err := 0;                       { Jusque l encore aucune erreur }

      {-- Vrifier l'EMB Ko aprs Ko ----------------------------------}

      for  i := 0 to MaxBl-1 do
        begin
          write( #13, 'Test Ko: ', i+1 );
          FillChar( barp^, 1024, i mod 255 );
          XMSCopy( 0, longint(barp), Handle, longint(i)*1024, 512 );
          FillChar( barp^, 1024, 255 );
          XMSCopy( Handle, longint(i)*1024, 0, longint(barp), 512 );

          {-- Compare le tampon recopi avec le rsultat attendu    ---}

          j := 1;
          while ( j <= 1024 ) do
            if ( barp^[j] <> i mod 255 ) then
              begin                                          { Erreur! }
                writeln( ' ERREUR!' );
                inc( err );
                j := 1025;
              end
           else               { Pas d'erreur, cellule mmoire suivante }
            inc( j );
        end;

      writeln( #13 );
      if ( err = 0 ) then                { Analyse du rsultat du test }
        writeln( 'EMB ok, aucun des blocs d''1 Ko vrifis n''tait ' +
                 'dfectueux.')
      else
        writeln( 'ATTENTION! ', err, ' blocs d''1 Ko dfectueux' +
                 ' dans l''EMB' );

      FreeMem( barp, 1024 );                       { Librer le tampon }
      XMSFreeMem( Handle );                            { Librer l'EMB }
   end;
end;

{***********************************************************************
*                 P R O G R A M M E   P R I N C I P A L                *
***********************************************************************}

var VerNr,
    RevNr  : integer;

begin
  ClrScr;
  writeln( 'XMSP - (c) 1990, 92 by MICHAEL TISCHER' );
  writeln;
  if XMSInit then
   begin
     if XMSQueryVer( VerNr, RevNr ) then
      writeln( 'Accs possible  l''HMA.' )
     else
      writeln( 'Aucun accs  l''HMA.' );
     writeln( 'N de version XMS : ', VerNr div 100,
               '.', VerNr mod 100 );
     writeln( 'N de rvision   : ', RevNr div 100,
              '.', RevNr mod 100 );
     writeln;
     HMATest;                                               { Test HMA }
     writeln;
     EMBTest;                                   { Test mmoire tendue }
   end
  else
    writeln( 'Aucun pilote XMS install!');
end.

