/***********************************************************************
*                          V 8 0 6 0 C . C                             *
**--------------------------------------------------------------------**
*  Fonction          : Montre comment exploiter le mode 800*600 16     *
*                      couleurs de la carte Super VGA. Ce programme    *
*                      accde aux routines en assembleur du module     *
*                      V8060CA.ASM                                     *
**--------------------------------------------------------------------**
*  Auteur        : MICHAEL TISCHER                                     *
*  Dvelopp le  : 14.01.1991                                          *
*  Dernire MAJ  : 14.02.1992                                          *
**--------------------------------------------------------------------**
*  Modle mmoire   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Compilation       : CL /AS v8060c.c v8060ca                         *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Compilation       : Utiliser un projet avec le contenu suivant      *
*                       v8060c.c                                       *
*                       v8060ca.obj                                    *
**--------------------------------------------------------------------**
*  Appel           : v8060c                                            *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>
#include <io.h>
#include <stdio.h>
#include <conio.h>

/*-- Dclarations de types -------------------------------------------*/

typedef unsigned char BYTE;

/*-- Rfrences externes aux routines en assembleur ------------------*/

extern int  init800600( void );
extern void setpix( int x, int y, unsigned char couleur);
extern BYTE getpix( int x, int y );
extern void far * getfontptr( void );

/*-- Dclarations dpendantes du compilateur -------------------------*/

#ifndef __TURBOC__
  #define random(x) ( rand() % (x+1) )          /* Fonction alatoire */
#endif

/*-- Constantes ------------------------------------------------------*/

#define MAXX       799                       /* Coordonnes maximales */
#define MAXY       599
#define NBLINE     2500                           /* Nombre de lignes */
#define XDISTANCE  40       /* Distance entre le rectangle et le bord */
#define YDISTANCE  30
#define X1         ( 2 * XDISTANCE )      /* Coordonnes du rectangle */
#define Y1         ( 2 * YDISTANCE )
#define X2         ( MAXX-XDISTANCE )
#define Y2         ( MAXY-YDISTANCE )
#define XRAND      random( X2 - X1 - 1 ) + X1 + 1      /* Coordonnes */
#define YRAND      random( Y2 - Y1 - 1 ) + Y1 + 1       /* alatoires */

/***********************************************************************
*  IsVga: Teste la prsence d'une carte VGA.                           *
**--------------------------------------------------------------------**
*  Entre : nant                                                      *
*  Sortie : 0 si pas de carte VGA, sinon -1                            *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS  Regs;             /* Registres pour grer l'interruption */

 Regs.x.ax = 0x1a00;            /* La fonction 1AH n'existe qu'en VGA */
 int86( 0x10, &Regs, &Regs );
 return ( Regs.h.al == 0x1a );               /* Est-elle disponible ? */
}
/***********************************************************************
*  PrintChar : Ecrit un caractre en dehors de la zone visible         *
*              de la mmoire d'cran                                   *
**--------------------------------------------------------------------**
*  Entre  :   caractere = caractre  afficher                        *
*              x, y    = Coordonnes du coin suprieur gauche          *
*              cc      = Couleur du caractre                          *
*              cf      = Couleur du fond                               *
*  Info    : Le caractre est dessin dans une matrice de 8*8 pixels - *
*            sur la base du jeu de caractres 8*8 en ROM               *
***********************************************************************/

void PrintChar( char caractere, int x, int y, BYTE cc, BYTE cf )
{
 typedef BYTE   CARDEF[256][8];     /* Structure du jeu de caractres */
 typedef CARDEF far *CARPTR;       /* Pointe sur un jeu de caractres */

 BYTE     i, k,                              /* Compteur d'itrations */
          masque;        /* Masque binaire pour dessiner le caractre */

 static   CARPTR fptr = (CARPTR) 0;       /* Jeu de caractres en ROM */

 if( fptr == (CARPTR) 0 )      /* A-t-on dj dtermin ce pointeur ? */
   fptr = getfontptr();      /* Non, dtermine avec fonct. assembleur */

  /*- Dessine le caractre pixel par pixel ---------------------------*/

 if( cf == 255 )                           /* Caractre transparent ? */
   for( i = 0; i < 8; ++i )    /* Oui, dessine que pixels du 1er plan */
   {
     masque = (*fptr)[caractere][i];         /* Motif bin. pour ligne */
     for( k = 0; k < 8; ++k, masque <<= 1 )  /* Parcourt les colonnes */
       if( masque & 128 )                       /* Pixel  dessiner ? */
         setpix( x+k, y+i, cc );                               /* Oui */
   }
 else                                     /* Non dessine chaque pixel */
   for ( i = 0; i < 8; ++i )                   /* Parcourt les lignes */
   {
     masque = (*fptr)[caractere][i];         /* Motif bin. pour ligne */
     for( k = 0; k < 8; ++k, masque <<= 1 )  /* Parcourt les colonnes */
       setpix( x+k, y+i, (BYTE) (( masque & 128 ) ? cc : cf) );
   }
}

/***********************************************************************
*  Line: Trace un segment dans la fentre graphique en appliquant      *
*        l'algorithme de Bresenham                                     *
**--------------------------------------------------------------------**
*  Entres : X1, Y1 = Coordonnes de l'origine                         *
*            X2, Y2 = Coordonnes de l'extrmit terminale             *
*            COULEUR = couleur du segment                              *
***********************************************************************/

/*-- Fonction accessoire pour changer deux variables entires -------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;  *i2   = *i1;  *i1   = dummy;
}

/*-- Procdure principale --------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE couleur )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if( abs(x2-x1) < abs(y2-y1) )     /* Sens du parcours : axe X ou Y ? */
 {                                                           /* Par Y */
   if( y1 > y2 )                             /* y1 plus grand que y2? */
   {
     SwapInt( &x1, &x2 );                    /* Oui change X1 et X2, */
     SwapInt( &y1, &y2 );                                 /* Y1 et Y2 */
   }

   xincr = ( x2 > x1 ) ?  1 : -1;           /* Fixe le pas horizontal */

   dy    = y2 - y1;
   dx    = abs( x2-x1 );
   d     = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x     = x1;
   y     = y1;

   setpix( x, y, couleur );               /* dessine le premier pixel */
   for(y=y1+1; y<= y2; ++y )                  /* Parcourt l'axe des Y */
   {
     if( d >= 0 )
     {
       x += xincr;
       d += aincr;
     }
     else
       d += bincr;
     setpix( x, y, couleur );
   }
 }
 else                                                        /* par X */
 {
   if( x1 > x2 )                             /* x1 plus grand que x2? */
   {
     SwapInt( &x1, &x2 );                    /* Oui, change X1 et X2 */
     SwapInt( &y1, &y2 );                                 /* Y1 et Y2 */
   }

   yincr = ( y2 > y1 ) ? 1 : -1;              /* Fixe le pas vertical */

   dx    = x2 - x1;
   dy    = abs( y2-y1 );
   d     = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x     = x1;
   y     = y1;

   setpix( x, y, couleur );               /* Dessine le premier pixel */
   for( x=x1+1; x<=x2; ++x )                  /* Parcourt l'axe des X */
   {
     if( d >= 0 )
     {
       y += yincr;
       d += aincr;
     }
     else
       d += bincr;
     setpix( x, y, couleur );
   }
 }
}

/***********************************************************************
*  GrafPrintf: Affiche une chane formate sur l'cran graphique       *
**--------------------------------------------------------------------**
*  Entres : X, Y   = Coordonnes de dpart (0 - ...)                  *
*            CC     = Couleur des caractres                           *
*            CF     = Couleur du fond  (255 = transparent)             *
*            STRING = Chane avec indications de formatage             *
*            ...    = Expressions comme pour printf                    *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE cc, BYTE cf, char * string, ... )
{
 va_list  parameter;     /* Liste de paramtres pour les macros VA_...*/
 char     affichage[255],           /* Buffer pour la chane formate */
          *cp;

 va_start( parameter, string );           /* Convertit les paramtres */
 vsprintf( affichage, string, parameter );                 /* Formate */
 for( cp = affichage; *cp; ++cp, x+= 8 )         /* Affiche la chane */
   PrintChar( *cp, x, y, cc, cf );          /* formate par PrintChar */
}

/**********************************************************************
*  DrawAxis: Draws axes from the left and top borders on the screen.  *
**-------------------------------------------------------------------**
*  Input   : STEPX = Increment for X-axis                             *
*            STEPY = Increment for Y-axis                             *
*            FG    = Foreground color                                 *
*            BK    = Background color (255 = transparent)             *
**********************************************************************/

void DrawAxis( int stepx, int stepy, BYTE fg, BYTE bk )
{
 int  x, y;                                     /* boucle coordonnes */

 Line( 0, 0, MAXX, 0, fg );                          /* Dessine axe X */
 Line( 0, 0, 0, MAXY, fg );                          /* Dessine axe Y */

 for( x = stepx; x < MAXX; x += stepx )              /* Echelle axe X */
 {
   Line( x, 0, x, 5, fg );
   GrafPrintf( x < 100 ? x - 8 : x - 12, 8, fg, bk, "%d", x );
 }

 for( y = stepy; y < MAXY; y += stepy )              /* Echelle axe Y */
 {
   Line( 0, y, 5, y, fg );
   GrafPrintf( 8, y-4, fg, bk, "%3d", y );
 }
}


/***********************************************************************
*  Demo: Dmontre l'usage des diffrentes fonctions de ce module       *
***********************************************************************/

void Demo( void )
{
 int  i;                                     /* Compteur d'itrations */

 DrawAxis( 30, 20, 15, 255 );                     /* Dessine des axes */

 GrafPrintf( X1, MAXY-10, 15, 255,
             "V8060C.C  -  (c) by MICHAEL TISCHER" );

 Line( X1, Y1, X1, Y2, 15 );                  /* Encadre le rectangle */
 Line( X1, Y2, X2, Y2, 15 );
 Line( X2, Y2, X2, Y1, 15 );
 Line( X2, Y1, X1, Y1, 15 );

   /*-- Dessine des segments alatoires dans le rectangle ------------*/

 for( i = NBLINE; i > 0 ; --i )
   Line( XRAND, YRAND, XRAND, YRAND, (BYTE) (i % 16) );
}

/*--------------------------------------------------------------------*/
/*--                    PROGRAMME PRINCIPAL                         --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS  regs;

 printf( "V8060C.C  - (c) 1991, 92 by MICHAEL TISCHER\n\n" );
 if( IsVga() )                      /* Dispose-t-on d'une carte VGA ? */
 {                   /* Oui, mais le mode 800*600 est-il accessible ? */
   if( init800600() )
   {                                                   /* a marche ! */
     Demo();                                       /* Excute la dmo */
     getch();                          /* Attend une frappe de touche */
     regs.x.ax = 0x0003;                    /* Rtablit le mode texte */
     int86( 0x10, &regs, &regs );
   }
   else
   {
     printf( "Attention! Le mode 800*600 n'a pas pu tre initialis\n");
     exit( 1 );
   }
 }
 else
 {                                           /* Non, pas de carte VGA */
   printf( "Attention! Ce programme ncessite une carte VGA !");
   exit(1);                                   /* Termine le programme */
 }
}
