/**********************************************************************/
/*                            V I D E O C                             */
/*--------------------------------------------------------------------*/
/*    Fonction        : fournit quelques fonctions qui exploitent     */
/*         l'interruption vido du BIOS et qui ne sont pas encore     */
/* intgres dans les bibliothques du compilateur C de Microsoft ou  */
/* de Borland                                                         */
/*--------------------------------------------------------------------*/
/*    Auteur       : MICHAEL TISCHER                                  */
/*    Dvelopp le : 13.08.1987                                       */
/*    Dernire MAJ : 13.02.1992                                       */
/*--------------------------------------------------------------------*/
/*    (MICROSOFT C)                                                   */
/*    Compilation    : CL /AS VIDEOC.C                                */
/*    Appel          : VIDEOC                                         */
/*--------------------------------------------------------------------*/
/*    (BORLAND TURBO C)                                               */
/*    Compilation    : par la commande RUN de l'EDI                   */
/*                     (sans fichier de projet                        */
/**********************************************************************/

/*== Fichiers d'inclusion ============================================*/

#include <dos.h>                                /* Fichiers d'en-tte */
#include <io.h>
#include <stdio.h>

/*== Constantes ======================================================*/

#define NORMAL         0x07                   /* Dfinit les attributs*/
#define CLAIR          0x0F               /* pour une carte monochrome*/
#define INVERSE        0x70
#define SOULIGNE       0x01
#define CLIGNOTANT     0x80

/**********************************************************************/
/* GETVIDEOMODE : Dtermine le mode vido et divers autres paramtres */
/* Entre :  Modevideo  = Pointe sur le mode vido courant            */
/*           NOMBRE     = Pointe sur le nombre de colonnes            */
/*           PAGE       = Pointe sur la page d'cran courante         */
/* Sortie  : nant                                                    */
/**********************************************************************/

void GetVideoMode( int *ModeVideo, int *Nombre, int *Page)
{
 union REGS Register;                /* Registres pour l'interruption */

 Register.h.ah = 15;                         /* Numro de la fonction */
 int86(0x10, &Register, &Register);    /* Dclenche l'interruption 10h*/
 *ModeVideo = Register.h.al;                   /* Numro du mode vido*/
 *Nombre = Register.h.ah;           /* Nombre de caractres par ligne */
 *Page = Register.h.bh;         /* Numro de la page d'cran courante */
}

/**********************************************************************/
/* SETCURSORTYPE : Dfinit l'aspect du cursur clignotant              */
/* Entre : Debut  = Extrmit suprieure du curseur                  */
/*          Fin    = Extrmit infrieure du curseur                  */
/* Sortie : nant                                                     */
/* Info   : Les paramtres peuvent tre compris entre 0 et 13 pour    */
/*           une carte monochrome, 0 et 7 pour une carte couleur      */

/**********************************************************************/

void SetCursorType(int Debut, int Fin)
{
 union REGS Register;                 /* Registres pour l'interruption*/

 Register.h.ah = 1;                          /* Numro de la fonction */
 Register.h.ch = Debut;        /* Ligne de l'extrmit sup du curseur */
 Register.h.cl = Fin;          /* Ligne de l'extrmit inf du curseur */
 int86(0x10, &Register, &Register);             /* Interruption 10(h) */
}

/**********************************************************************/
/* SETCURSORPOS : Position du curseur dans la page d'cran indique   */
/*                                                                    */
/* Entre : PAGE    = Page d'cran                                    */
/*          COLONNE = Colonne                                         */
/*          LIGNE   = Ligne                                           */
/* Sortie : nant                                                     */
/* Info   : La position du curseur clignotant ne change  l'issue de  */
/*          cet appel que si la page d'cran indique est la page     */
/*          d'cran courante                                          */
/**********************************************************************/

void SetCursorPos( int Page, int Colonne, int Ligne)
{
 union REGS Register;                 /* Registres pour l'interruption*/

 Register.h.ah = 2;                           /* Numro de la fonction*/
 Register.h.bh = Page;                                /* Page d'cran */
 Register.h.dh = Ligne;                                      /* Ligne */
 Register.h.dl = Colonne;                                  /* Colonne */
 int86(0x10, &Register, &Register); /* Dclenche l'interruption 10(h) */
}

/**********************************************************************/
/* GETCURSORPOS : Dtermine la position courante du curseur dans la   */
/*                page d'cran ainsi que les limites de son aspect    */
/* Entre :  PAGE    = Page d'cran                                   */
/*           COLONNE = Pointe sur la colonne courante                 */
/*           LIGNE   = Pointe sur la ligne courante                   */
/*           DEBUT   = Pointe sur la ligne de trame reprsentant      */
/*                     l'extrmit suprieure du curseur.             */
/*           FIN     = Pointe sur la ligne de trame reprsentant      */
/*                     l'extrmit infrieure du curseur              */
/* Sortie : nant                                                     */
/**********************************************************************/

void GetCursorPos( int Page, int *Colonne,
                   int *Ligne, int *Debut, int *Fin)
{
 union REGS Register;                /* Registres poour l'interruption*/

 Register.h.ah = 3;                          /* Numro de la fonction */
 Register.h.bh = Page;                                /* Page d'cran */
 int86(0x10, &Register, &Register); /* Dclenche l'interruption 10(h) */
 *Colonne = Register.h.dl;                   /* Prlve les rsultats */
 *Ligne  = Register.h.dh;                       /* dans les registres */
 *Debut = Register.h.ch;                            /* et les affecte */
 *Fin   = Register.h.cl;                              /* aux variables*/
}

/**********************************************************************/
/* SETDISPLAYPAGE: Affiche la page d'cran demande                   */
/* Entre : PAGE = Page d'cran                                       */
/* Sortie : nant                                                     */
/**********************************************************************/

void SetDisplayPage( int Page )
{
 union REGS Register;                /* Registres pour l'interruption */

 Register.h.ah = 5;                          /* Numro de la fonction */
 Register.h.al = Page;                                /* Page d'cran */
 int86(0x10, &Register, &Register); /* Dclenche l'interruption 10(h) */
}

/**********************************************************************/
/* SCROLLUP : Fait dfiler une zone d'cran d'une ou plusieurs        */
/*           lignes vers le haut ou provoque son effacement           */
/* Entre  : NOMBRE    = Nombre de lignes  faire dfiler             */
/*           COULEUR   = Couleur ou attribut des lignes vierges       */
/*           COLONNEHG = Colonne du coin sup gauche de la zone        */
/*           LIGNEHG   = Ligne du coin sup gauche de la zone          */
/*           COLONNEBD = Colonne du coin infrieur droit de la zone   */
/*           LIGNEBD  = Ligne du coin infrieur droit de la zone      */
/* Sortie  : nant                                                    */
/* Info    : Si on prend 0 comme paramtre, la zone d'cran           */
/*           est remplie avec des lignes vierges                      */
/**********************************************************************/

void ScrollUp( int Nombre, int Couleur, int ColonneHG,
               int LigneHG, int ColonneBD, int LigneBD )
{
 union REGS Register;                /* Registres pour l'interruption */

 Register.h.ah = 6;                          /* Numro de la fonction */
 Register.h.al = Nombre;                          /* Nombre de lignes */
 Register.h.bh = Couleur;                /* Couleur des lignes vierges*/
 Register.h.ch = LigneHG;                  /* Fixe les coordonnes de */
 Register.h.cl = ColonneHG;                             /* la fentre */
 Register.h.dh = LigneBD;                            /* de dfilement */
 Register.h.dl = ColonneBD;
 int86(0x10, &Register, &Register); /* Dclenche l'interruption 10(h) */
}

/**********************************************************************/
/* SCROLLDOWN : Fait dfiler une zone d'cran d'une ou de plusieurs   */
/*              lignes vers le bas ou provoque son effacement         */
/* Entre :  NOMBRE    = Nombre de lignes  faire dfiler             */
/*           COULEUR   = Couleur ou attribut des lignes vierges       */
/*           COLONNEHG = Colonne du coin sup gauche de la zone        */
/*           LIGNEHG   = Ligne du coin sup gauche de la zone          */
/*           COLONNEBD = Colonne du coin inf droit de la zone         */
/*           LIGNEBD   = Ligne du coin inf droit de la zone           */
/* Sortie : nant                                                     */
/* Info    : Si le paramtre est 0, remplie de lignes vierges         */
/**********************************************************************/

void ScrollDown( int Nombre, int Couleur, int ColonneHG,
                 int LigneHG, int ColonneBD, int LigneBD )
{
 union REGS Register;                /* Registres pour l'interruption */

 Register.h.ah = 7;                          /* Numro de la fonction */
 Register.h.al = Nombre;                          /* Nombre de lignes */
 Register.h.bh = Couleur;               /* Couleur des lignes vierges */
 Register.h.ch = LigneHG;               /* Fixe les coordonnes de la */
 Register.h.cl = ColonneHG;                  /* fentre de dfilement */
 Register.h.dh = LigneBD;
 Register.h.dl = ColonneBD;
 int86(0x10, &Register, &Register); /* Dclenche l'interruption 10(h) */
}

/**********************************************************************/
/* GETCHAR : lit un caractre et son attribut  une position donne   */
/*           de la page d'cran                                       */
/* Entre :  PAGE      = Page d'cran concerne                       */
/*           COLONNE   = Colonne du caractre                         */
/*           LIGNE     = Ligne du caractre                           */
/*           Caractere = Pointe sur le caractre trait               */
/*           COULEUR   = Pointe sur l'attribut (ou la couleur)        */
/* Sortie : nant                                                     */
/**********************************************************************/

void GetChar( int Page, int Colonne, int Ligne,
              char *Caractere, int *Couleur)
{
 union REGS Register;                 /* Registres pour l'interruption*/
 int Dummy;                  /* Pour variables accessoires ou inutiles*/
 int PageCour;                               /* page d'cran courante */
 int LigneCour;                                     /* Ligne courante */
 int ColonneCour;                                 /* Colonne courante */

 GetVideoMode(&Dummy, &Dummy, &PageCour);    /* Page d'cran courante */
 GetCursorPos( PageCour, &ColonneCour, &LigneCour,   /* Pos courante  */
              &Dummy, &Dummy);                          /* du curseur */
 SetCursorPos(Page, Colonne, Ligne);         /* Positionne le curseur */
 Register.h.ah = 8;                           /* Numro de la fonction*/
 Register.h.bh = Page;                                /* Page d'cran */
 int86(0x10, &Register, &Register); /* dclenche l'interruption 10(h) */
 *Caractere = Register.h.al;  /* Lit les rsultats dans les registres */
 *Couleur = Register.h.ah;            /* et les affecte aux variables */
 SetCursorPos(PageCour, ColonneCour, LigneCour); /* Curseur anc. pos. */
}

/**********************************************************************/
/* WRITECHAR : Affiche un caractre avec son attribut                 */
/*              une position donne de la page d'cran indique      */
/* Entre :   PAGE      = Page d'cran concerne                      */
/*            CARACTERE = Caractre  afficher                        */
/*            COULEUR   = Attribut ou couleur du caractre            */
/* Sortie : nant                                                     */
/**********************************************************************/

void WriteChar( int Page, char Caractere, int Couleur)
{
 union REGS Register;                /* Registres pour l'interruption */

 Register.h.ah = 9;                          /* Numro de la fonction */
 Register.h.al = Caractere;                   /* Caractre  afficher */
 Register.h.bh = Page;                                /* Page d'cran */
 Register.h.bl = Couleur;          /* Couleur du caractre  afficher */
 Register.x.cx = 1;                             /* Un seul exemplaire */
 int86(0x10, &Register, &Register); /* dclenche l'interruption 10(h) */
}

/**********************************************************************/
/* WRITETEXT : Affiche une chane de caractre d' une certaine couleur*/
/*              une position donne de la page d'cran indique      */
/* Entre  :  PAGE     = page d'cran concerne                       */
/*            COLONNE  = Colonne o dbute la chane                  */
/*            LIGNE    = Ligne o dbute la chane                    */
/*            COULEUR  = Attribut ou couleur des caractres           */
/*            TEXT     = Pointeur sur la chane                       */
/* Sortie  : nant                                                    */
/* Info    : Text pointe sur un tableau de caractres qui contient le */
/*           texte  afficher et se termine par le caractre nul '\0' */
/**********************************************************************/

void WriteText( int Page, int Colonne, int Ligne, int Couleur,
                char *Text)
{
 union REGS InRegister,              /* Registres pour l'interruption */
            OutRegister;

 SetCursorPos(Page, Colonne, Ligne);         /* Positionne le curseur */

 InRegister.h.ah = 14;                       /* Numro de la fonction */
 InRegister.h.bh = Page;                              /* Page d'cran */
 while (*Text)                       /* Affiche le texte jusqu' '\0' */
  {
   WriteChar(Page, ' ', Couleur);             /* Couleur du caractre */
   InRegister.h.al = *Text++;                            /* Caractre */
   int86(0x10, &InRegister, &OutRegister);            /* Interruption */
  }
}

/**********************************************************************/
/* CLEARSCREEN : Efface les 80*25 caractres de l'cran et            */
/*               positionne le curseur en haut  gauche               */
/* Entre : nant                                                     */
/* Sortie : nant                                                     */
/**********************************************************************/

void ClearScreen( void )
{
 int PageCour;                               /* Page d'cran courante */
 int Dummy;                                       /* Variable fantme */

 ScrollUp(0, NORMAL, 0, 0, 79, 24);           /* Effacer tout l'cran */
 GetVideoMode(&Dummy, &Dummy, &PageCour);    /* Page d'cran courante */
 SetCursorPos(PageCour, 0, 0);               /* Positionne le curseur */
}

/**********************************************************************/
/**                    PROGRAMME PRINCIPAL                           **/
/**********************************************************************/

void main()

{
 int i, j, k, l;                             /* Variables d'itration */
 char Fleche[3];                  /* Nombre de flches en format ASCII*/

 ClearScreen();                                     /* Efface l'cran */
 for (i = 1; i < 25; i++)               /* Parcourt toutes les lignes */
  for (j = 0; j < 80; j++)                  /* et toutes les colonnes */
   {
    SetCursorPos(0, j, i);                   /* Positionne le curseur */
    WriteChar(0, i*80+j&255, NORMAL);         /* Affiche un caractre */
   }
 ScrollDown(0, NORMAL, 5, 8, 19, 22);          /* Efface la fentre 1 */
 WriteText(0, 5, 8, INVERSE, "   Fentre 1   ");
 ScrollDown(0, NORMAL, 60, 2, 74, 16);         /* Efface la fentre 2 */
 WriteText(0, 60, 2, INVERSE, "   Fentre 2   ");
 WriteText(0, 30, 12, INVERSE | CLIGNOTANT, " >>>LA BIBLE PC<<< ");
 WriteText(0, 0, 0, INVERSE, "                      il reste ");
 WriteText(0, 40, 0, INVERSE,"flches  tracer");
 for (i = 49; i >= 0 ; i--)                       /* Trace 50 flches */
  {
   sprintf(Fleche, "%2d", i); /* Conversion du nombre en chane ASCII */
   WriteText(0, 37, 0, INVERSE, Fleche);         /* affiche le nombre */
   for (j = 1; j < 16; j+= 2)         /* Flche compose de 16 lignes */
    {
     for (k = 0; k < j; k++)       /* Fabrique une ligne de la flche */
      {
       SetCursorPos(0, 12-(j>>1)+k, 9);           /* Flche fentre 1 */
       WriteChar(0, '*', CLAIR);
       SetCursorPos(0, 67-(j>>1)+k, 16);          /* Flche fentre 2 */
       WriteChar(0, '*', CLAIR);
      }
     ScrollDown(1, NORMAL, 5, 9, 19, 22); /* Fait dfiler la fentre 1*/
     ScrollUp(1, NORMAL, 60, 3, 74, 16);  /* Fait dfiler la fentre 2*/
     for (l = 0; l < 4000 ; l++)                  /* Boucle d'attente */
      ;
    }
   }
 ClearScreen();                                     /* Efface l'cran */
}
