{**********************************************************************}
{*                            V I D E O P                             *}
{*--------------------------------------------------------------------*}
{*    Fonction      : fournit  quelques fonctions qui exploitent      *}
{*                    l'interruption vido du BIOS et qui ne sont pas *}
{*                    encore intgres dans Turbo Pascal              *}
{*--------------------------------------------------------------------*}
{*    Auteur         : MICHAEL TISCHER                                *}
{*    Dvelopp le   : 10.07.1987                                     *}
{*    Dernire MAJ   : 14.01.1991                                     *}
{**********************************************************************}

program VIDEOP;

Uses Crt, Dos;                          { Inclut les units CTR et DOS }

const NORMAL        = $07;                     { Dfinit les attributs }
      CLAIR          = $0f;                          { pour les cartes }
      INVERSE        = $70;                        { cran monochromes }
      SOULIGNE = $01;
      CLIGNOTANT       = $80;

type TextTyp = string[80];

var  i,                                   { Variables pour les boucles }
     j,
     k,
     l       : integer;
     IString : string[2];                  { Nombre de flches traces }

{**********************************************************************}
{* GetVideoMode : Dtermine le mode vido courant et divers autres    *}
{*                paramtres                                          *}
{* Entre       : nant                                               *}
{* Sorties      : Les variables cites plus bas sont mises  jour     *}
{**********************************************************************}

procedure GetVideoMode(var ModeVideo,   { Numro du mode vido courant }
               Nombre,                  { Nombre de colonnes par ligne }
 Page       : integer);                        { page d'cran courante }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := $0F;                               { Numro de la fonction }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
 ModeVideo := Regs.al;                          { Numro du mode vido }
 Nombre := Regs.ah;                   { Nombre de caractres par ligne }
 Page := Regs.bh;                 { Numro de la page d'cran courante }
end;

{**********************************************************************}
{* SetCursorType : Dfinit l'aspect du curseur clignotant             *}
{* Entre        : Voir plus bas                                      *}
{* Sortie        : nant                                              *}
{* Info          : Les paramtres peuvent tre compris entre 0 et 13  *}
{*                 pour une carte monochrome, 0 et 7 pour une carte   *}
{*                 couleur                                            *}
{**********************************************************************}

procedure SetCursorType(Debut,             { Ligne de dbut du curseur }
            Fin    : integer);               { Ligne de fin du curseur }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 1;                                 { Numro de la fonction }
 Regs.ch := Debut;            { Fixe l'extrmit suprieure du curseur }
 Regs.cl := Fin;              { Fixe l'extrmit infrieure du curseur }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
end;

{**********************************************************************}
{* SetCursorPos : Fixe la position du curseur dans la page            *}
{*                d'cran indique                                    *}
{* Entre       : Voir plus bas                                       *}
{* Sortie       : nant                                               *}
{* Info         : La position du curseur clignotant ne change        *}
{*                l'issue de cet appel que si la page d'cran indique*}
{*                est la page d'cran courante                        *}
{**********************************************************************}

procedure SetCursorPos(Page,                  { Page d'cran concerne }
               Colonne,                  { nouvelle colonne du curseur }
               Ligne   : integer);         { nouvelle ligne du curseur }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 2;                                 { Numro de la fonction }
 Regs.bh := Page;                                       { Page d'cran }
 Regs.dh := Ligne;                              { Fixe les coordonnes }
 Regs.dl := Colonne;                                      { de l'cran }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
end;

{**********************************************************************}
{* GetCursorPos : Dtermine la position du curseur dans la page       *}
{*                d'cran ainsi que les limites de son aspect         *}
{* Entre       : Voir plus bas                                       *}
{* Sortie       : Les variables cites plus bas sont mises  jour     *}
{* Info         : Les limites du curseur sont indpendantes de la     *}
{*                page cran indique                                 *}
{**********************************************************************}

procedure GetCursorPos(Page : integer;                  { Page d'cran }
               var Colonne,                       { Colonne du curseur }
               Ligne,                               { Ligne du curseur }
               Debut,                { Extrmit suprieure du curseur }
               Fin    : integer);               { Extrmit infrieure }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 3;                                 { Numro de la fonction }
 Regs.bh := Page;                                       { Page d'cran }
 Intr($10, Regs);              {Dclenche l'interruption vido du BIOS }
 Colonne := Regs.dl;                           { Prlve les rsultats }
 Ligne := Regs.dh;                                { dans les registres }
 Debut := Regs.ch;                                    { et les affecte }
 Fin := Regs.cl;                                       { aux variables }
end;

{**********************************************************************}
{* SetDisplayPage : Fixe la page d'cran  afficher                   *}
{* Entre         : Voir plus bas                                     *}
{* Sortie         : nant                                             *}
{**********************************************************************}

procedure SetDisplayPage(Page : integer);      { nouvelle page d'cran }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 5;                      { Numro de la fonction "Set Page" }
 Regs.al := Page;                                       { Page d'cran }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
end;

{**********************************************************************}
{* ScrollUp : Fait dfiler une zone d'cran d'une ou plusieurs        *}
{*            lignes vers le haut ou provoque son effacement          *}
{* Entre   : Voir plus bas                                           *}
{* Sortie   : nant                                                   *}
{* Info     : Si on prend 0 comme paramtre, la zone d'cran est      *}
{*            remplie avec des lignes vierges                         *}
{**********************************************************************}

procedure ScrollUp(Nombre,          { Nombre de lignes  faire dfiler }
           Couleur,                 { Attribut pour les lignes vierges }
           ColonneHG,                        { Colonne coin sup gauche }
           LigneHG,                            { Ligne coin sup gauche }
           ColonneBD,                   { Colonne coin infrieur droit }
           LigneBD   : integer);          { Ligne coin infrieur droit }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 6;                                 { Numro de la fonction }
 Regs.al := Nombre;                 { Nombre de lignes  faire dfiler }
 Regs.bh := Couleur;                      { Couleur des lignes vierges }
 Regs.ch := LigneHG;         { Fixe les coordonnes du coin sup gauche }
 Regs.cl := ColonneHG;
 Regs.dh := LigneBD;   { Fixe les coordonnes du coins infrieur droit }
 Regs.dl := ColonneBD;
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
end;

{**********************************************************************}
{* ScrollDown : Fait dfiler une zone d'cran d'une ou de plusieurs   *}
{*              lignes vers le bas ou provoque son effacement         *}
{* Entre     : Voir plus bas                                         *}
{* Sortie     : nant                                                 *}
{* Info       : Si on prend 0 comme paramtre, la zone d'cran est    *}
{*              remplie avec des lignes vierges                       *}
{**********************************************************************}

procedure ScrollDown(Nombre,        { Nombre de lignes  faire dfiler }
             Couleur,               { Attribut pour les lignes vierges }
             ColonneHG,                { Colonne coin suprieur gauche }
             LigneHG,                    { Ligne coin suprieur gauche }
             ColonneBD,                 { Colonne coin infrieur droit }
             LigneBD : integer);          { Ligne coin infrieur droit }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah := 7;                                 { Numro de la fonction }
 Regs.al := Nombre;                 { Nombre de lignes  faire dfiler }
 Regs.bh := Couleur;                      { Couleur des lignes vierges }
 Regs.ch := LigneHG;                            { Fixe les coordonnes }
 Regs.cl := ColonneHG;                      { du coin suprieur gauche }
 Regs.dh := LigneBD;                            { Fixe les coordonnes }
 Regs.dl := ColonneBD;                       { du coin infrieur droit }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
end;

{**********************************************************************}
{* GetChar : Lit un caractre et son attribut  une position donne   *}
{*           de la page d'cran                                       *}
{* Entre  : Voir plus bas                                            *}
{* Sortie  : Voir plus bas                                            *}
{**********************************************************************}

procedure GetChar(Page,                       { Page d'cran concerne }
          Colonne,                                   { Colonne d'cran }
          Ligne       : integer;                       { Ligne d'cran }
          var Caractere : char;                         { Caractre et }
          var Couleur   : integer);                     { son attribut }

var Regs     : Registers;              { Registres pour l'interruption }
    ColonneCour,                                    { Colonne courante }
    LigneCour,                                        { Ligne courante }
    PageCour,                                  { Page d'cran courante }
    Dummy    : integer;       { pour variables accessoires ou inutiles }

begin
 GetVideoMode(Dummy, Dummy, PageCour);{Dtermine la page cran courante}
 GetCursorPos(PageCour, ColonneCour, LigneCour, Dummy, Dummy);
   { lit la position courante du curseur dans la page d'cran courante }
 SetCursorPos(Page, Colonne, Ligne);           { Positionne le curseur }

 Regs.ah := 8;    { Numro de fonction pour lire caractre et attribut }
 Regs.bh := Page;                                       { Page d'cran }
 Intr($10, Regs);             { Dclenche l'interruption vido du BIOS }
 Caractere := chr(Regs.al);                  { Code ASCII du caractre }
 Couleur := Regs.ah;                           { Attribut du caractre }
 SetCursorPos(PageCour, ColonneCour, LigneCour);
end;                       { Ramne le curseur  son ancienne position }

{**********************************************************************}
{* WRITECHAR : Affiche un caractre dans une couleur donne  la      *}
{*             pos. courante du curseur dans la page cran indique   *}
{* Entre    : Voir plus bas                                          *}
{* Sortie    : nant                                                  *}
{* Info      : Dans le traitement de l'affichage les caractres de    *}
{*             contrle comme <Entre> sont considrs comme des      *}
{*             caractres ordinaires                                  *}
{**********************************************************************}

procedure WriteChar(Page   : integer;         { Page d'cran concerne }
                    Caractere : char;        { Code ASCII du caractre }
                    Couleur   : integer);               { son attribut }

var Regs : Registers;                  { Registres pour l'interruption }

begin
 Regs.ah:=9;        { Numro de la fonction pour afficher un caractre }
 Regs.al := ord(Caractere);             { Code du caractre  afficher }
 Regs.bh := Page;                               { Fixe la page d'cran }
 Regs.bl := Couleur;                    { Fixe la couleur du caractre }
 Regs.cx := 1;                         { Un seul exemplaire  afficher }
 Intr($10, Regs);                     { Dclenche l'interruption vido }
end;

{**********************************************************************}
{* WriteText: Affiche une chane de caractres  la position indique *}
{*            de la page d'cran                                      *}
{* Entre   : Voir plus bas                                           *}
{* Sortie   : nant                                                   *}
{* Info     : Les caractres de contrle (par ex <Entre>) sont       *}
{*            traits comme tels                                      *}
{**********************************************************************}

procedure WriteText(Page,                     { Page d'cran concerne }
                    Colonne,           { Colonne o dbute l'affichage }
                    Ligne,              { Ligne, o dbute l'affichage }
                    Couleur : integer;  { Couleur pour tous caractres }
                    Text   : TextTyp);              { Texte  afficher }

var Regs : Registers;                  { Registres pour l'interruption }
    Compteur  : integer;                       { Compteur d'itrations }

begin
 SetCursorPos(Page, Colonne, Ligne);           { Positionne le curseur }
 for Compteur:=1 to length(Text) do            { Traite successivement }
  begin                                    { les diffrents caractres }
   WriteChar(Page, ' ', Couleur);       { Couleur en position courante }
   Regs.ah := 14;            { Numro de la fonction "sortie tltype" }
   Regs.al := ord(Text[compteur]);      { Code du caractre  afficher }
   Regs.bh := Page;                                     { Page d'cran }
   Intr($10, Regs);           { Dclenche l'interruption vido du BIOS }
  end;
end;

{**********************************************************************}
{**                      Programme principal                          *}
{**********************************************************************}

begin
 ClrScr;                                              { Efface l'cran }
 for i := 1 to 24 do                      { Parcourt les lignes 1  24 }
  for j := 0 to 79 do                         { et toutes les colonnes }
   begin
    SetCursorPos(0, j, i);                     { Positionne le curseur }
    WriteChar(0, chr(i*80+j and 255), NORMAL);  { affiche un caractre }
   end;
 ScrollDown(0, NORMAL, 5, 8, 19, 22);            { Efface la fentre 1 }
 WriteText(0, 5, 8, INVERSE, '   Fentre 1   ');
 ScrollDown(0, NORMAL, 60, 2, 74, 16);           { Efface la fentre 2 }
 WriteText(0, 60, 2, INVERSE, '   Fentre 2   ');
 WriteText(0, 30, 12, INVERSE or CLIGNOTANT, ' >>> BIBLE PC <<< ');
 WriteText(0, 0, 0, INVERSE, '                            Il reste '+
                         '   flches  tracer                        ');
 for i := 49 downto 0 do                         { 50 flches au total }
  begin
   Str(i:2, IString);                         { Transforme i en chane }
   WriteText(0, 37, 0, INVERSE, IString);
   j := 1;                     { Chaque flche se compose de 16 lignes }
   while j <= 15 do
    begin
      k := 0;
      while k < j do                 { Fabrique une ligne de la flche }
       begin
        SetCursorPos(0, 12-(j shr 1)+k, 9);         { Flche fentre 1 }
        WriteChar(0, '*', CLAIR);
        SetCursorPos(0, 67-(j shr 1)+k, 16);        { Flche fentre 2 }
        WriteChar(0, '*', CLAIR);
        k := succ(k);
       end;
      ScrollDown(1, NORMAL, 5, 9, 19, 22); { Fait dfiler la fentre 1 }
      ScrollUp(1, NORMAL, 60, 3, 74, 16);  { Fait dfiler la fentre 2 }
      for l := 0 to 8000 do                         { Boucle d'attente }
       ;
      j := j+2;
    end;
  end;
 ClrScr;                                              { Efface l'cran }
end.
